/* Generated by Cython 0.27.3 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "depends": [],
        "language": "c++",
        "name": "psychxr.ovr.math",
        "sources": [
            "psychxr/ovr/math.pyx"
        ]
    },
    "module_name": "psychxr.ovr.math"
}
END: Cython Metadata */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_27_3"
#define CYTHON_FUTURE_DIVISION 0
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (0 && PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#if PY_VERSION_HEX < 0x030700A0 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject **args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject **args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : PyInstanceMethod_New(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef __cplusplus
  #error "Cython files generated with the C++ option must be compiled with a C++ compiler."
#endif
#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #else
    #define CYTHON_INLINE inline
  #endif
#endif
template<typename T>
void __Pyx_call_destructor(T& x) {
    x.~T();
}
template<typename T>
class __Pyx_FakeReference {
  public:
    __Pyx_FakeReference() : ptr(NULL) { }
    __Pyx_FakeReference(const T& ref) : ptr(const_cast<T*>(&ref)) { }
    T *operator->() { return ptr; }
    T *operator&() { return ptr; }
    operator T&() { return *ptr; }
    template<typename U> bool operator ==(U other) { return *ptr == other; }
    template<typename U> bool operator !=(U other) { return *ptr != other; }
  private:
    T *ptr;
};

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif


#define __PYX_ERR(f_index, lineno, Ln_error) \
{ \
  __pyx_filename = __pyx_f[f_index]; __pyx_lineno = lineno; __pyx_clineno = __LINE__; goto Ln_error; \
}

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__psychxr__ovr__math
#define __PYX_HAVE_API__psychxr__ovr__math
#include "OVR_Math.h"
#include "ios"
#include "new"
#include "stdexcept"
#include "typeinfo"
#include <stdint.h>
#include "OVR_ErrorCode.h"
#include "OVR_CAPI.h"
#include <math.h>
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
#define __Pyx_PyBool_FromLong(b) ((b) ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False))
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c));
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "psychxr\\ovr\\math.pyx",
  "stringsource",
};

/*--- Type declarations ---*/
struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i;
struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei;
struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti;
struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f;
struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort;
struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf;
struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef;
struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f;
struct __pyx_t_7psychxr_3ovr_8ovr_capi_ovrVector4f;
typedef struct __pyx_t_7psychxr_3ovr_8ovr_capi_ovrVector4f __pyx_t_7psychxr_3ovr_8ovr_capi_ovrVector4f;

/* "ovr_capi.pxd":695
 * 
 * # additional types
 * ctypedef struct ovrVector4f:             # <<<<<<<<<<<<<<
 *     float x
 *     float y
 */
struct __pyx_t_7psychxr_3ovr_8ovr_capi_ovrVector4f {
  float x;
  float y;
  float z;
  float w;
};

/* "psychxr/ovr/math.pxd":4
 * cimport ovr_capi
 * 
 * cdef class ovrVector2i:             # <<<<<<<<<<<<<<
 *     cdef ovr_capi.ovrVector2i* c_data
 *     cdef ovr_capi.ovrVector2i  c_ovrVector2i
 */
struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i {
  PyObject_HEAD
  ovrVector2i *c_data;
  ovrVector2i c_ovrVector2i;
};


/* "psychxr/ovr/math.pxd":8
 *     cdef ovr_capi.ovrVector2i  c_ovrVector2i
 * 
 * cdef class ovrSizei:             # <<<<<<<<<<<<<<
 *     cdef ovr_capi.ovrSizei* c_data
 *     cdef ovr_capi.ovrSizei  c_ovrSizei
 */
struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei {
  PyObject_HEAD
  ovrSizei *c_data;
  ovrSizei c_ovrSizei;
};


/* "psychxr/ovr/math.pxd":12
 *     cdef ovr_capi.ovrSizei  c_ovrSizei
 * 
 * cdef class ovrRecti:             # <<<<<<<<<<<<<<
 *     cdef ovr_capi.ovrRecti* c_data
 *     cdef ovr_capi.ovrRecti  c_ovrRecti
 */
struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti {
  PyObject_HEAD
  ovrRecti *c_data;
  ovrRecti c_ovrRecti;
};


/* "psychxr/ovr/math.pxd":16
 *     cdef ovr_capi.ovrRecti  c_ovrRecti
 * 
 * cdef class ovrVector3f:             # <<<<<<<<<<<<<<
 *     cdef ovr_math.Vector3f* c_data
 *     cdef ovr_math.Vector3f  c_Vector3f
 */
struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f {
  PyObject_HEAD
  OVR::Vector3f *c_data;
  OVR::Vector3f c_Vector3f;
};


/* "psychxr/ovr/math.pxd":20
 *     cdef ovr_math.Vector3f  c_Vector3f
 * 
 * cdef class ovrFovPort:             # <<<<<<<<<<<<<<
 *     cdef ovr_capi.ovrFovPort* c_data
 *     cdef ovr_capi.ovrFovPort  c_ovrFovPort
 */
struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort {
  PyObject_HEAD
  ovrFovPort *c_data;
  ovrFovPort c_ovrFovPort;
};


/* "psychxr/ovr/math.pxd":24
 *     cdef ovr_capi.ovrFovPort  c_ovrFovPort
 * 
 * cdef class ovrQuatf:             # <<<<<<<<<<<<<<
 *     cdef ovr_math.Quatf* c_data
 *     cdef ovr_math.Quatf  c_Quatf
 */
struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf {
  PyObject_HEAD
  OVR::Quatf *c_data;
  OVR::Quatf c_Quatf;
};


/* "psychxr/ovr/math.pxd":28
 *     cdef ovr_math.Quatf  c_Quatf
 * 
 * cdef class ovrPosef:             # <<<<<<<<<<<<<<
 *     cdef ovr_math.Posef* c_data
 *     cdef ovr_math.Posef  c_Posef
 */
struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef {
  PyObject_HEAD
  OVR::Posef *c_data;
  OVR::Posef c_Posef;
};


/* "psychxr/ovr/math.pxd":32
 *     cdef ovr_math.Posef  c_Posef
 * 
 * cdef class ovrMatrix4f:             # <<<<<<<<<<<<<<
 *     cdef ovr_math.Matrix4f* c_data
 *     cdef ovr_math.Matrix4f  c_Matrix4f
 */
struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f {
  PyObject_HEAD
  OVR::Matrix4f *c_data;
  OVR::Matrix4f c_Matrix4f;
};


/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely((Py_TYPE(obj) == type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* None.proto */
static CYTHON_INLINE void __Pyx_RaiseUnboundLocalError(const char *varname);

/* None.proto */
static CYTHON_INLINE int __Pyx_div_int(int, int);

/* UnaryNegOverflows.proto */
#define UNARY_NEG_WOULD_OVERFLOW(x)\
        (((x) < 0) & ((unsigned long)(x) == 0-(unsigned long)(x)))

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* KeywordStringCheck.proto */
static int __Pyx_CheckKeywordStrings(PyObject *kwdict, const char* function_name, int kw_allowed);

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* GetModuleGlobalName.proto */
static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name);

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* GetNameInClass.proto */
static PyObject *__Pyx_GetNameInClass(PyObject *nmspace, PyObject *name);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* None.proto */
#include <new>

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CppExceptionConversion.proto */
#ifndef __Pyx_CppExn2PyErr
#include <new>
#include <typeinfo>
#include <stdexcept>
#include <ios>
static void __Pyx_CppExn2PyErr() {
  try {
    if (PyErr_Occurred())
      ; // let the latest Python exn pass through and ignore the current one
    else
      throw;
  } catch (const std::bad_alloc& exn) {
    PyErr_SetString(PyExc_MemoryError, exn.what());
  } catch (const std::bad_cast& exn) {
    PyErr_SetString(PyExc_TypeError, exn.what());
  } catch (const std::bad_typeid& exn) {
    PyErr_SetString(PyExc_TypeError, exn.what());
  } catch (const std::domain_error& exn) {
    PyErr_SetString(PyExc_ValueError, exn.what());
  } catch (const std::invalid_argument& exn) {
    PyErr_SetString(PyExc_ValueError, exn.what());
  } catch (const std::ios_base::failure& exn) {
    PyErr_SetString(PyExc_IOError, exn.what());
  } catch (const std::out_of_range& exn) {
    PyErr_SetString(PyExc_IndexError, exn.what());
  } catch (const std::overflow_error& exn) {
    PyErr_SetString(PyExc_OverflowError, exn.what());
  } catch (const std::range_error& exn) {
    PyErr_SetString(PyExc_ArithmeticError, exn.what());
  } catch (const std::underflow_error& exn) {
    PyErr_SetString(PyExc_ArithmeticError, exn.what());
  } catch (const std::exception& exn) {
    PyErr_SetString(PyExc_RuntimeError, exn.what());
  }
  catch (...)
  {
    PyErr_SetString(PyExc_RuntimeError, "Unknown exception");
  }
}
#endif

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE size_t __Pyx_PyInt_As_size_t(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);


/* Module declarations from 'libcpp' */

/* Module declarations from 'psychxr.ovr.ovr_math' */

/* Module declarations from 'libc.stdint' */

/* Module declarations from 'psychxr.ovr.ovr_errorcode' */
static CYTHON_INLINE int __pyx_f_7psychxr_3ovr_13ovr_errorcode_OVR_SUCCESS(ovrResult); /*proto*/

/* Module declarations from 'psychxr.ovr.ovr_capi' */

/* Module declarations from 'libc.math' */

/* Module declarations from 'psychxr.ovr.math' */
static PyTypeObject *__pyx_ptype_7psychxr_3ovr_4math_ovrVector2i = 0;
static PyTypeObject *__pyx_ptype_7psychxr_3ovr_4math_ovrSizei = 0;
static PyTypeObject *__pyx_ptype_7psychxr_3ovr_4math_ovrRecti = 0;
static PyTypeObject *__pyx_ptype_7psychxr_3ovr_4math_ovrVector3f = 0;
static PyTypeObject *__pyx_ptype_7psychxr_3ovr_4math_ovrFovPort = 0;
static PyTypeObject *__pyx_ptype_7psychxr_3ovr_4math_ovrQuatf = 0;
static PyTypeObject *__pyx_ptype_7psychxr_3ovr_4math_ovrPosef = 0;
static PyTypeObject *__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f = 0;
static CYTHON_INLINE PyObject *__Pyx_carray_to_py_float(float *, Py_ssize_t); /*proto*/
static CYTHON_INLINE PyObject *__Pyx_carray_to_tuple_float(float *, Py_ssize_t); /*proto*/
static CYTHON_INLINE PyObject *__Pyx_carray_to_py_float___5b_4_5d_(float (*)[4], Py_ssize_t); /*proto*/
static CYTHON_INLINE PyObject *__Pyx_carray_to_tuple_float___5b_4_5d_(float (*)[4], Py_ssize_t); /*proto*/
#define __Pyx_MODULE_NAME "psychxr.ovr.math"
extern int __pyx_module_is_main_psychxr__ovr__math;
int __pyx_module_is_main_psychxr__ovr__math = 0;

/* Implementation of 'psychxr.ovr.math' */
static PyObject *__pyx_builtin_staticmethod;
static PyObject *__pyx_builtin_TypeError;
static PyObject *__pyx_builtin_range;
static const char __pyx_k_a[] = "a";
static const char __pyx_k_b[] = "b";
static const char __pyx_k_d[] = "d";
static const char __pyx_k_f[] = "f";
static const char __pyx_k_h[] = "h";
static const char __pyx_k_v[] = "v";
static const char __pyx_k_w[] = "w";
static const char __pyx_k_x[] = "x";
static const char __pyx_k_y[] = "y";
static const char __pyx_k_z[] = "z";
static const char __pyx_k_GL[] = "GL";
static const char __pyx_k_at[] = "at";
static const char __pyx_k_eq[] = "__eq__";
static const char __pyx_k_up[] = "up";
static const char __pyx_k__18[] = "*";
static const char __pyx_k_abs[] = "abs";
static const char __pyx_k_dot[] = "dot";
static const char __pyx_k_eye[] = "eye";
static const char __pyx_k_max[] = "max";
static const char __pyx_k_min[] = "min";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_yfov[] = "yfov";
static const char __pyx_k_zero[] = "zero";
static const char __pyx_k_zfar[] = "zfar";
static const char __pyx_k_align[] = "align";
static const char __pyx_k_angle[] = "angle";
static const char __pyx_k_apply[] = "apply";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_znear[] = "znear";
static const char __pyx_k_aspect[] = "aspect";
static const char __pyx_k_c_data[] = "c_data";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_length[] = "length";
static const char __pyx_k_lookAt[] = "lookAt";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_rotate[] = "rotate";
static const char __pyx_k_up_tan[] = "up_tan";
static const char __pyx_k_GLfloat[] = "GLfloat";
static const char __pyx_k_inverse[] = "inverse";
static const char __pyx_k_ortho2d[] = "ortho2d";
static const char __pyx_k_scaling[] = "scaling";
static const char __pyx_k_align_to[] = "align_to";
static const char __pyx_k_down_tan[] = "down_tan";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_identity[] = "identity";
static const char __pyx_k_inverted[] = "inverted";
static const char __pyx_k_is_equal[] = "is_equal";
static const char __pyx_k_left_tan[] = "left_tan";
static const char __pyx_k_multiply[] = "multiply";
static const char __pyx_k_rotation[] = "rotation";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_OpenGL_GL[] = "OpenGL.GL";
static const char __pyx_k_TypeError[] = "TypeError";
static const char __pyx_k_getXBasis[] = "getXBasis";
static const char __pyx_k_getYBasis[] = "getYBasis";
static const char __pyx_k_getZBasis[] = "getZBasis";
static const char __pyx_k_length_sq[] = "length_sq";
static const char __pyx_k_normalize[] = "normalize";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_right_tan[] = "right_tan";
static const char __pyx_k_rotationX[] = "rotationX";
static const char __pyx_k_rotationY[] = "rotationY";
static const char __pyx_k_rotationZ[] = "rotationZ";
static const char __pyx_k_setXBasis[] = "setXBasis";
static const char __pyx_k_setYBasis[] = "setYBasis";
static const char __pyx_k_setZBasis[] = "setZBasis";
static const char __pyx_k_to_return[] = "to_return";
static const char __pyx_k_to_vector[] = "to_vector";
static const char __pyx_k_tolerance[] = "tolerance";
static const char __pyx_k_transform[] = "transform";
static const char __pyx_k_normalized[] = "normalized";
static const char __pyx_k_perspective[] = "perspective";
static const char __pyx_k_translation[] = "translation";
static const char __pyx_k_staticmethod[] = "staticmethod";
static const char __pyx_k_inverseRotate[] = "inverseRotate";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_psychxr_ovr_math[] = "psychxr.ovr.math";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_psychxr_ovr_math_pyx[] = "psychxr\\ovr\\math.pyx";
static const char __pyx_k_no_default___reduce___due_to_non[] = "no default __reduce__ due to non-trivial __cinit__";
static PyObject *__pyx_n_s_GL;
static PyObject *__pyx_n_s_GLfloat;
static PyObject *__pyx_n_s_OpenGL_GL;
static PyObject *__pyx_n_s_TypeError;
static PyObject *__pyx_n_s__18;
static PyObject *__pyx_n_s_a;
static PyObject *__pyx_n_s_abs;
static PyObject *__pyx_n_s_align;
static PyObject *__pyx_n_s_align_to;
static PyObject *__pyx_n_s_angle;
static PyObject *__pyx_n_s_apply;
static PyObject *__pyx_n_s_aspect;
static PyObject *__pyx_n_s_at;
static PyObject *__pyx_n_s_b;
static PyObject *__pyx_n_s_c_data;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_d;
static PyObject *__pyx_n_s_dot;
static PyObject *__pyx_n_s_down_tan;
static PyObject *__pyx_n_s_eq;
static PyObject *__pyx_n_s_eye;
static PyObject *__pyx_n_s_f;
static PyObject *__pyx_n_s_getXBasis;
static PyObject *__pyx_n_s_getYBasis;
static PyObject *__pyx_n_s_getZBasis;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_n_s_h;
static PyObject *__pyx_n_s_identity;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_inverse;
static PyObject *__pyx_n_s_inverseRotate;
static PyObject *__pyx_n_s_inverted;
static PyObject *__pyx_n_s_is_equal;
static PyObject *__pyx_n_s_left_tan;
static PyObject *__pyx_n_s_length;
static PyObject *__pyx_n_s_length_sq;
static PyObject *__pyx_n_s_lookAt;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_max;
static PyObject *__pyx_n_s_min;
static PyObject *__pyx_n_s_multiply;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_kp_s_no_default___reduce___due_to_non;
static PyObject *__pyx_n_s_normalize;
static PyObject *__pyx_n_s_normalized;
static PyObject *__pyx_n_s_ortho2d;
static PyObject *__pyx_n_s_perspective;
static PyObject *__pyx_n_s_psychxr_ovr_math;
static PyObject *__pyx_kp_s_psychxr_ovr_math_pyx;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_n_s_right_tan;
static PyObject *__pyx_n_s_rotate;
static PyObject *__pyx_n_s_rotation;
static PyObject *__pyx_n_s_rotationX;
static PyObject *__pyx_n_s_rotationY;
static PyObject *__pyx_n_s_rotationZ;
static PyObject *__pyx_n_s_scaling;
static PyObject *__pyx_n_s_setXBasis;
static PyObject *__pyx_n_s_setYBasis;
static PyObject *__pyx_n_s_setZBasis;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_n_s_staticmethod;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_to_return;
static PyObject *__pyx_n_s_to_vector;
static PyObject *__pyx_n_s_tolerance;
static PyObject *__pyx_n_s_transform;
static PyObject *__pyx_n_s_translation;
static PyObject *__pyx_n_s_up;
static PyObject *__pyx_n_s_up_tan;
static PyObject *__pyx_n_s_v;
static PyObject *__pyx_n_s_w;
static PyObject *__pyx_n_s_x;
static PyObject *__pyx_n_s_y;
static PyObject *__pyx_n_s_yfov;
static PyObject *__pyx_n_s_z;
static PyObject *__pyx_n_s_zero;
static PyObject *__pyx_n_s_zfar;
static PyObject *__pyx_n_s_znear;
static int __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i___cinit__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, int __pyx_v_x, int __pyx_v_y); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_1x___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self); /* proto */
static int __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_1x_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, int __pyx_v_value); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_1y___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self); /* proto */
static int __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_1y_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, int __pyx_v_value); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_2asTuple(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_4asList(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_6__eq__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_8__ne__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_10__add__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_a, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_12__iadd__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_14__sub__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_a, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_16__isub__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_18__neg__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_20__mul__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_a, PyObject *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_22__imul__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, PyObject *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_24__truediv__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_a, PyObject *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_26__itruediv__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, PyObject *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_28min(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_a, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_30max(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_a, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_32clamped(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, int __pyx_v_max_mag); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_34isEqual(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_b, int __pyx_v_tolerance); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_36compare(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_b, int __pyx_v_tolerance); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_38__getitem__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, int __pyx_v_idx); /* proto */
static int __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_40__setitem__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, int __pyx_v_idx, int __pyx_v_val); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_42entrywiseMultiply(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_44dot(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_46angle(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_48lengthSq(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_50length(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_52distanceSq(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_54distance(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_56isNormalized(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_58normalize(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_60normalized(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_62lerp(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_b, int __pyx_v_f); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_64projectTo(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_66isClockwise(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_68__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_70__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_7psychxr_3ovr_4math_8ovrSizei___cinit__(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_self, int __pyx_v_w, int __pyx_v_h); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrSizei_1w___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_self); /* proto */
static int __pyx_pf_7psychxr_3ovr_4math_8ovrSizei_1w_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_self, int __pyx_v_value); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrSizei_1h___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_self); /* proto */
static int __pyx_pf_7psychxr_3ovr_4math_8ovrSizei_1h_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_self, int __pyx_v_value); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrSizei_2asTuple(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrSizei_4asList(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrSizei_6__eq__(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrSizei_8__ne__(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrSizei_10__add__(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_a, struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrSizei_12__iadd__(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrSizei_14__sub__(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_a, struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrSizei_16__isub__(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrSizei_18__neg__(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrSizei_20__mul__(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_a, PyObject *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrSizei_22__imul__(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_self, PyObject *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrSizei_24__truediv__(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_a, PyObject *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrSizei_26__itruediv__(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_self, PyObject *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrSizei_28min(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_a, struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrSizei_30max(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_a, struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrSizei_32area(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrSizei_34toVector(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrSizei_36asVector(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrSizei_38__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrSizei_40__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_7psychxr_3ovr_4math_8ovrRecti___init__(CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_args, CYTHON_UNUSED PyObject *__pyx_v_kwargs); /* proto */
static int __pyx_pf_7psychxr_3ovr_4math_8ovrRecti_2__cinit__(struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_self, PyObject *__pyx_v_args, CYTHON_UNUSED PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrRecti_1x___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_self); /* proto */
static int __pyx_pf_7psychxr_3ovr_4math_8ovrRecti_1x_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_self, int __pyx_v_value); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrRecti_1y___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_self); /* proto */
static int __pyx_pf_7psychxr_3ovr_4math_8ovrRecti_1y_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_self, int __pyx_v_value); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrRecti_1w___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_self); /* proto */
static int __pyx_pf_7psychxr_3ovr_4math_8ovrRecti_1w_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_self, int __pyx_v_value); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrRecti_1h___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_self); /* proto */
static int __pyx_pf_7psychxr_3ovr_4math_8ovrRecti_1h_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_self, int __pyx_v_value); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrRecti_4asTuple(struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrRecti_6asList(struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_self); /* proto */
static Py_ssize_t __pyx_pf_7psychxr_3ovr_4math_8ovrRecti_8__len__(CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrRecti_10getPos(struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrRecti_12getSize(struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrRecti_14setPos(struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_pos); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrRecti_16setSize(struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_size); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrRecti_18__eq__(struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrRecti_20__ne__(struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrRecti_22__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrRecti_24__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f___init__(CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_args, CYTHON_UNUSED PyObject *__pyx_v_kwargs); /* proto */
static int __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_2__cinit__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, PyObject *__pyx_v_args, CYTHON_UNUSED PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_1x___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self); /* proto */
static int __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_1x_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, float __pyx_v_value); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_1y___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self); /* proto */
static int __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_1y_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, float __pyx_v_value); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_1z___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self); /* proto */
static int __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_1z_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, float __pyx_v_value); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_4as_tuple(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_6ctypes___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self); /* proto */
static Py_ssize_t __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_6__len__(CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_8zero(); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_10__eq__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_12__ne__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_14__add__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_a, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_16__iadd__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_18__sub__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_a, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_20__isub__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_22__neg__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_24__mul__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_a, PyObject *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_26__imul__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, PyObject *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_28__truediv__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_a, PyObject *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_30__itruediv__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, PyObject *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_32min(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_a, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_34max(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_a, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_36clamped(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, float __pyx_v_max_mag); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_38isEqual(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_b, float __pyx_v_tolerance); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_40compare(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_b, float __pyx_v_tolerance); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_42__getitem__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, int __pyx_v_idx); /* proto */
static int __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_44__setitem__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, int __pyx_v_idx, float __pyx_v_val); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_46entrywiseMultiply(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_48dot(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_50cross(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_52angle(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_54lengthSq(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_56length(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_58distanceSq(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_60distance(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_62isNormalized(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_64normalize(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_66normalized(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_68lerp(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_b, float __pyx_v_f); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_70projectTo(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_72projectToPlane(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_normal); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_74__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_76__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_7psychxr_3ovr_4math_10ovrFovPort___cinit__(struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *__pyx_v_self, CYTHON_UNUSED float __pyx_v_up_tan, CYTHON_UNUSED float __pyx_v_down_tan, CYTHON_UNUSED float __pyx_v_left_tan, CYTHON_UNUSED float __pyx_v_right_tan); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_10ovrFovPort_5UpTan___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *__pyx_v_self); /* proto */
static int __pyx_pf_7psychxr_3ovr_4math_10ovrFovPort_5UpTan_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *__pyx_v_self, float __pyx_v_value); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_10ovrFovPort_7DownTan___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *__pyx_v_self); /* proto */
static int __pyx_pf_7psychxr_3ovr_4math_10ovrFovPort_7DownTan_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *__pyx_v_self, float __pyx_v_value); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_10ovrFovPort_7LeftTan___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *__pyx_v_self); /* proto */
static int __pyx_pf_7psychxr_3ovr_4math_10ovrFovPort_7LeftTan_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *__pyx_v_self, float __pyx_v_value); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_10ovrFovPort_8RightTan___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *__pyx_v_self); /* proto */
static int __pyx_pf_7psychxr_3ovr_4math_10ovrFovPort_8RightTan_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *__pyx_v_self, float __pyx_v_value); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_10ovrFovPort_2max(struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *__pyx_v_a, struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_10ovrFovPort_4min(struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *__pyx_v_a, struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_10ovrFovPort_6__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_10ovrFovPort_8__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf___cinit__(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self, PyObject *__pyx_v_args); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_1x___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self); /* proto */
static int __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_1x_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self, float __pyx_v_value); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_1y___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self); /* proto */
static int __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_1y_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self, float __pyx_v_value); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_1z___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self); /* proto */
static int __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_1z_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self, float __pyx_v_value); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_1w___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self); /* proto */
static int __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_1w_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self, float __pyx_v_value); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_2asTuple(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_4asList(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self); /* proto */
static Py_ssize_t __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_6__len__(CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_8__neg__(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_10identity(); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_12__eq__(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_14__ne__(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_16__add__(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_a, struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_18__iadd__(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_20__sub__(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_a, struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_22__isub__(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_24__mul__(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_a, PyObject *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_26__imul__(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self, PyObject *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_28__truediv__(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_a, float __pyx_v_s); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_30__itruediv__(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self, float __pyx_v_s); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_32isEqual(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_b, float __pyx_v_tolerance); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_34abs(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self, float __pyx_v_v); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_36imag(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_38lengthSq(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_40length(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_42distance(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_q); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_44distanceSq(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_q); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_46dot(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_q); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_48angle(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_q); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_50isNormalized(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_52normalize(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_54normalized(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_56conj(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_58align(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_align_to, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_v); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_60rotate(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_v); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_62inverseRotate(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_v); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_64inverted(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_66inverse(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_68invert(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_70__invert__(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_72__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_74__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_7psychxr_3ovr_4math_8ovrPosef___cinit__(struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *__pyx_v_self, PyObject *__pyx_v_args, CYTHON_UNUSED PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrPosef_8rotation___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *__pyx_v_self); /* proto */
static int __pyx_pf_7psychxr_3ovr_4math_8ovrPosef_8rotation_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrPosef_11translation___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *__pyx_v_self); /* proto */
static int __pyx_pf_7psychxr_3ovr_4math_8ovrPosef_11translation_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrPosef_2rotate(struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_v); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrPosef_4inverseRotate(struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_v); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrPosef_6translate(struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_v); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrPosef_8transform(struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_v); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrPosef_10inverseTransform(struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_v); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrPosef_12transformNormal(struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_v); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrPosef_14inverseTransformNormal(struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_v); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrPosef_16apply(struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_v); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrPosef_18__mul__(struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *__pyx_v_a, struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrPosef_20inverted(struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrPosef_22normalized(struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrPosef_24normalize(struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrPosef_26__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrPosef_28__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f___cinit__(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self, PyObject *__pyx_v_args); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_1M___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_2identity(); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_4setIdentity(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_6setXBasis(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_v); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_8getXBasis(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_6xBasis___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self); /* proto */
static int __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_6xBasis_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self, PyObject *__pyx_v_v); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_10setYBasis(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_v); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_12getYBasis(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_6yBasis___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self); /* proto */
static int __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_6yBasis_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self, PyObject *__pyx_v_v); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_14setZBasis(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_v); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_16getZBasis(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_6zBasis___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self); /* proto */
static int __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_6zBasis_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self, PyObject *__pyx_v_v); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_6ctypes___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_18__eq__(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_20__ne__(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_22__add__(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_a, struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_24__iadd__(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_26__sub__(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_a, struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_28__isub__(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_30multiply(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_d, struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_a, struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_32__mul__(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_a, PyObject *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_34__imul__(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self, PyObject *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_36__truediv__(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_a, float __pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_38__itruediv__(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self, float __pyx_v_b); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_40transform(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_v); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_42transposed(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_44transpose(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_46determinant(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_48adjugated(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_50inverted(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_52invert(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_54invertedHomogeneousTransform(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_56invertHomogeneousTransform(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_58translation(PyObject *__pyx_v_v); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_60setTranslation(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_v); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_62getTranslation(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_64scaling(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_v); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_66distance(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_m2); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_68rotationX(float __pyx_v_angle); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_70rotationY(float __pyx_v_angle); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_72rotationZ(float __pyx_v_angle); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_74lookAt(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_eye, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_at, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_up); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_76perspective(float __pyx_v_yfov, float __pyx_v_aspect, float __pyx_v_znear, float __pyx_v_zfar); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_78ortho2d(float __pyx_v_w, float __pyx_v_h); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_80__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_82__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_7psychxr_3ovr_4math_ovrVector2i(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_7psychxr_3ovr_4math_ovrSizei(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_7psychxr_3ovr_4math_ovrRecti(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_7psychxr_3ovr_4math_ovrVector3f(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_7psychxr_3ovr_4math_ovrFovPort(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_7psychxr_3ovr_4math_ovrQuatf(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_7psychxr_3ovr_4math_ovrPosef(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_7psychxr_3ovr_4math_ovrMatrix4f(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_float_0_0;
static PyObject *__pyx_float_1_0;
static PyObject *__pyx_int_3;
static PyObject *__pyx_int_16;
static PyObject *__pyx_tuple_;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__5;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__7;
static PyObject *__pyx_tuple__8;
static PyObject *__pyx_tuple__9;
static PyObject *__pyx_tuple__10;
static PyObject *__pyx_tuple__11;
static PyObject *__pyx_tuple__12;
static PyObject *__pyx_tuple__13;
static PyObject *__pyx_tuple__14;
static PyObject *__pyx_tuple__15;
static PyObject *__pyx_tuple__16;
static PyObject *__pyx_tuple__17;
static PyObject *__pyx_tuple__19;
static PyObject *__pyx_tuple__21;
static PyObject *__pyx_tuple__23;
static PyObject *__pyx_tuple__25;
static PyObject *__pyx_tuple__27;
static PyObject *__pyx_tuple__29;
static PyObject *__pyx_tuple__31;
static PyObject *__pyx_tuple__33;
static PyObject *__pyx_tuple__35;
static PyObject *__pyx_tuple__37;
static PyObject *__pyx_tuple__39;
static PyObject *__pyx_tuple__41;
static PyObject *__pyx_tuple__43;
static PyObject *__pyx_tuple__45;
static PyObject *__pyx_tuple__47;
static PyObject *__pyx_tuple__49;
static PyObject *__pyx_tuple__51;
static PyObject *__pyx_tuple__53;
static PyObject *__pyx_tuple__55;
static PyObject *__pyx_tuple__57;
static PyObject *__pyx_tuple__59;
static PyObject *__pyx_codeobj__20;
static PyObject *__pyx_codeobj__22;
static PyObject *__pyx_codeobj__24;
static PyObject *__pyx_codeobj__26;
static PyObject *__pyx_codeobj__28;
static PyObject *__pyx_codeobj__30;
static PyObject *__pyx_codeobj__32;
static PyObject *__pyx_codeobj__34;
static PyObject *__pyx_codeobj__36;
static PyObject *__pyx_codeobj__38;
static PyObject *__pyx_codeobj__40;
static PyObject *__pyx_codeobj__42;
static PyObject *__pyx_codeobj__44;
static PyObject *__pyx_codeobj__46;
static PyObject *__pyx_codeobj__48;
static PyObject *__pyx_codeobj__50;
static PyObject *__pyx_codeobj__52;
static PyObject *__pyx_codeobj__54;
static PyObject *__pyx_codeobj__56;
static PyObject *__pyx_codeobj__58;
static PyObject *__pyx_codeobj__60;

/* "psychxr/ovr/math.pyx":7
 * 
 * cdef class ovrVector2i:
 *     def __cinit__(self, int x=0, int y=0):             # <<<<<<<<<<<<<<
 *         self.c_data = &self.c_ovrVector2i
 * 
 */

/* Python wrapper */
static int __pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_v_x;
  int __pyx_v_y;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_x,&__pyx_n_s_y,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_x);
          if (value) { values[0] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_y);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(0, 7, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    if (values[0]) {
      __pyx_v_x = __Pyx_PyInt_As_int(values[0]); if (unlikely((__pyx_v_x == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 7, __pyx_L3_error)
    } else {
      __pyx_v_x = ((int)0);
    }
    if (values[1]) {
      __pyx_v_y = __Pyx_PyInt_As_int(values[1]); if (unlikely((__pyx_v_y == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 7, __pyx_L3_error)
    } else {
      __pyx_v_y = ((int)0);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 0, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 7, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i___cinit__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_self), __pyx_v_x, __pyx_v_y);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i___cinit__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, int __pyx_v_x, int __pyx_v_y) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "psychxr/ovr/math.pyx":8
 * cdef class ovrVector2i:
 *     def __cinit__(self, int x=0, int y=0):
 *         self.c_data = &self.c_ovrVector2i             # <<<<<<<<<<<<<<
 * 
 *         self.c_data.x = x
 */
  __pyx_v_self->c_data = (&__pyx_v_self->c_ovrVector2i);

  /* "psychxr/ovr/math.pyx":10
 *         self.c_data = &self.c_ovrVector2i
 * 
 *         self.c_data.x = x             # <<<<<<<<<<<<<<
 *         self.c_data.y = y
 * 
 */
  __pyx_v_self->c_data->x = __pyx_v_x;

  /* "psychxr/ovr/math.pyx":11
 * 
 *         self.c_data.x = x
 *         self.c_data.y = y             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __pyx_v_self->c_data->y = __pyx_v_y;

  /* "psychxr/ovr/math.pyx":7
 * 
 * cdef class ovrVector2i:
 *     def __cinit__(self, int x=0, int y=0):             # <<<<<<<<<<<<<<
 *         self.c_data = &self.c_ovrVector2i
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":14
 * 
 *     @property
 *     def x(self):             # <<<<<<<<<<<<<<
 *         return self.c_data.x
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_1x_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_1x_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_1x___get__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_1x___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "psychxr/ovr/math.pyx":15
 *     @property
 *     def x(self):
 *         return self.c_data.x             # <<<<<<<<<<<<<<
 * 
 *     @x.setter
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->c_data->x); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":14
 * 
 *     @property
 *     def x(self):             # <<<<<<<<<<<<<<
 *         return self.c_data.x
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.x.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":18
 * 
 *     @x.setter
 *     def x(self, int value):             # <<<<<<<<<<<<<<
 *         self.c_data.x = value
 * 
 */

/* Python wrapper */
static int __pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_1x_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_value); /*proto*/
static int __pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_1x_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_value) {
  int __pyx_v_value;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_value); {
    __pyx_v_value = __Pyx_PyInt_As_int(__pyx_arg_value); if (unlikely((__pyx_v_value == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 18, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.x.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_1x_2__set__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_self), ((int)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_1x_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, int __pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "psychxr/ovr/math.pyx":19
 *     @x.setter
 *     def x(self, int value):
 *         self.c_data.x = value             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __pyx_v_self->c_data->x = __pyx_v_value;

  /* "psychxr/ovr/math.pyx":18
 * 
 *     @x.setter
 *     def x(self, int value):             # <<<<<<<<<<<<<<
 *         self.c_data.x = value
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":22
 * 
 *     @property
 *     def y(self):             # <<<<<<<<<<<<<<
 *         return self.c_data.y
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_1y_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_1y_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_1y___get__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_1y___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "psychxr/ovr/math.pyx":23
 *     @property
 *     def y(self):
 *         return self.c_data.y             # <<<<<<<<<<<<<<
 * 
 *     @y.setter
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->c_data->y); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 23, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":22
 * 
 *     @property
 *     def y(self):             # <<<<<<<<<<<<<<
 *         return self.c_data.y
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.y.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":26
 * 
 *     @y.setter
 *     def y(self, int value):             # <<<<<<<<<<<<<<
 *         self.c_data.y = value
 * 
 */

/* Python wrapper */
static int __pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_1y_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_value); /*proto*/
static int __pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_1y_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_value) {
  int __pyx_v_value;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_value); {
    __pyx_v_value = __Pyx_PyInt_As_int(__pyx_arg_value); if (unlikely((__pyx_v_value == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 26, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.y.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_1y_2__set__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_self), ((int)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_1y_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, int __pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "psychxr/ovr/math.pyx":27
 *     @y.setter
 *     def y(self, int value):
 *         self.c_data.y = value             # <<<<<<<<<<<<<<
 * 
 *     def asTuple(self):
 */
  __pyx_v_self->c_data->y = __pyx_v_value;

  /* "psychxr/ovr/math.pyx":26
 * 
 *     @y.setter
 *     def y(self, int value):             # <<<<<<<<<<<<<<
 *         self.c_data.y = value
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":29
 *         self.c_data.y = value
 * 
 *     def asTuple(self):             # <<<<<<<<<<<<<<
 *         return self.c_data.x, self.c_data.y
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_3asTuple(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_3asTuple(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("asTuple (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_2asTuple(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_2asTuple(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("asTuple", 0);

  /* "psychxr/ovr/math.pyx":30
 * 
 *     def asTuple(self):
 *         return self.c_data.x, self.c_data.y             # <<<<<<<<<<<<<<
 * 
 *     def asList(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->c_data->x); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 30, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->c_data->y); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 30, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 30, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":29
 *         self.c_data.y = value
 * 
 *     def asTuple(self):             # <<<<<<<<<<<<<<
 *         return self.c_data.x, self.c_data.y
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.asTuple", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":32
 *         return self.c_data.x, self.c_data.y
 * 
 *     def asList(self):             # <<<<<<<<<<<<<<
 *         return list(self.c_data.x, self.c_data.y)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_5asList(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_5asList(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("asList (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_4asList(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_4asList(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("asList", 0);

  /* "psychxr/ovr/math.pyx":33
 * 
 *     def asList(self):
 *         return list(self.c_data.x, self.c_data.y)             # <<<<<<<<<<<<<<
 * 
 *     def __eq__(self, ovrVector2i b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->c_data->x); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->c_data->y); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)(&PyList_Type)), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":32
 *         return self.c_data.x, self.c_data.y
 * 
 *     def asList(self):             # <<<<<<<<<<<<<<
 *         return list(self.c_data.x, self.c_data.y)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.asList", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":35
 *         return list(self.c_data.x, self.c_data.y)
 * 
 *     def __eq__(self, ovrVector2i b):             # <<<<<<<<<<<<<<
 *         return self.c_data.x == b.c_data.x and self.c_data.y == b.c_data.y
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_7__eq__(PyObject *__pyx_v_self, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_7__eq__(PyObject *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__eq__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrVector2i, 1, "b", 0))) __PYX_ERR(0, 35, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_6__eq__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_6__eq__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("__eq__", 0);

  /* "psychxr/ovr/math.pyx":36
 * 
 *     def __eq__(self, ovrVector2i b):
 *         return self.c_data.x == b.c_data.x and self.c_data.y == b.c_data.y             # <<<<<<<<<<<<<<
 * 
 *     def __ne__(self, ovrVector2i b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = (__pyx_v_self->c_data->x == __pyx_v_b->c_data->x);
  if (__pyx_t_2) {
  } else {
    __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 36, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L3_bool_binop_done;
  }
  __pyx_t_2 = (__pyx_v_self->c_data->y == __pyx_v_b->c_data->y);
  __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 36, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __pyx_t_3;
  __pyx_t_3 = 0;
  __pyx_L3_bool_binop_done:;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":35
 *         return list(self.c_data.x, self.c_data.y)
 * 
 *     def __eq__(self, ovrVector2i b):             # <<<<<<<<<<<<<<
 *         return self.c_data.x == b.c_data.x and self.c_data.y == b.c_data.y
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.__eq__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":38
 *         return self.c_data.x == b.c_data.x and self.c_data.y == b.c_data.y
 * 
 *     def __ne__(self, ovrVector2i b):             # <<<<<<<<<<<<<<
 *         return self.c_data.x != b.c_data.x or self.c_data.y != b.c_data.y
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_9__ne__(PyObject *__pyx_v_self, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_9__ne__(PyObject *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__ne__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrVector2i, 1, "b", 0))) __PYX_ERR(0, 38, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_8__ne__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_8__ne__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("__ne__", 0);

  /* "psychxr/ovr/math.pyx":39
 * 
 *     def __ne__(self, ovrVector2i b):
 *         return self.c_data.x != b.c_data.x or self.c_data.y != b.c_data.y             # <<<<<<<<<<<<<<
 * 
 *     def __add__(ovrVector2i a, ovrVector2i b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = (__pyx_v_self->c_data->x != __pyx_v_b->c_data->x);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 39, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L3_bool_binop_done;
  }
  __pyx_t_2 = (__pyx_v_self->c_data->y != __pyx_v_b->c_data->y);
  __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __pyx_t_3;
  __pyx_t_3 = 0;
  __pyx_L3_bool_binop_done:;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":38
 *         return self.c_data.x == b.c_data.x and self.c_data.y == b.c_data.y
 * 
 *     def __ne__(self, ovrVector2i b):             # <<<<<<<<<<<<<<
 *         return self.c_data.x != b.c_data.x or self.c_data.y != b.c_data.y
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.__ne__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":41
 *         return self.c_data.x != b.c_data.x or self.c_data.y != b.c_data.y
 * 
 *     def __add__(ovrVector2i a, ovrVector2i b):             # <<<<<<<<<<<<<<
 *         cdef ovrVector2i to_return = ovrVector2i(
 *             a.c_data.x + b.c_data.x,
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_11__add__(PyObject *__pyx_v_a, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_11__add__(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__add__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_a), __pyx_ptype_7psychxr_3ovr_4math_ovrVector2i, 1, "a", 0))) __PYX_ERR(0, 41, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrVector2i, 1, "b", 0))) __PYX_ERR(0, 41, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_10__add__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_a), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_10__add__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_a, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_b) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("__add__", 0);

  /* "psychxr/ovr/math.pyx":43
 *     def __add__(ovrVector2i a, ovrVector2i b):
 *         cdef ovrVector2i to_return = ovrVector2i(
 *             a.c_data.x + b.c_data.x,             # <<<<<<<<<<<<<<
 *             a.c_data.y + b.c_data.y)
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_int((__pyx_v_a->c_data->x + __pyx_v_b->c_data->x)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 43, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);

  /* "psychxr/ovr/math.pyx":44
 *         cdef ovrVector2i to_return = ovrVector2i(
 *             a.c_data.x + b.c_data.x,
 *             a.c_data.y + b.c_data.y)             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
  __pyx_t_2 = __Pyx_PyInt_From_int((__pyx_v_a->c_data->y + __pyx_v_b->c_data->y)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 44, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "psychxr/ovr/math.pyx":42
 * 
 *     def __add__(ovrVector2i a, ovrVector2i b):
 *         cdef ovrVector2i to_return = ovrVector2i(             # <<<<<<<<<<<<<<
 *             a.c_data.x + b.c_data.x,
 *             a.c_data.y + b.c_data.y)
 */
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 42, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector2i), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 42, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "psychxr/ovr/math.pyx":46
 *             a.c_data.y + b.c_data.y)
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def __iadd__(self, ovrVector2i b):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":41
 *         return self.c_data.x != b.c_data.x or self.c_data.y != b.c_data.y
 * 
 *     def __add__(ovrVector2i a, ovrVector2i b):             # <<<<<<<<<<<<<<
 *         cdef ovrVector2i to_return = ovrVector2i(
 *             a.c_data.x + b.c_data.x,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.__add__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":48
 *         return to_return
 * 
 *     def __iadd__(self, ovrVector2i b):             # <<<<<<<<<<<<<<
 *         self.c_data.x += b.c_data.x
 *         self.c_data.y += b.c_data.y
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_13__iadd__(PyObject *__pyx_v_self, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_13__iadd__(PyObject *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iadd__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrVector2i, 1, "b", 0))) __PYX_ERR(0, 48, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_12__iadd__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_12__iadd__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iadd__", 0);

  /* "psychxr/ovr/math.pyx":49
 * 
 *     def __iadd__(self, ovrVector2i b):
 *         self.c_data.x += b.c_data.x             # <<<<<<<<<<<<<<
 *         self.c_data.y += b.c_data.y
 * 
 */
  __pyx_v_self->c_data->x = (__pyx_v_self->c_data->x + __pyx_v_b->c_data->x);

  /* "psychxr/ovr/math.pyx":50
 *     def __iadd__(self, ovrVector2i b):
 *         self.c_data.x += b.c_data.x
 *         self.c_data.y += b.c_data.y             # <<<<<<<<<<<<<<
 * 
 *         return self
 */
  __pyx_v_self->c_data->y = (__pyx_v_self->c_data->y + __pyx_v_b->c_data->y);

  /* "psychxr/ovr/math.pyx":52
 *         self.c_data.y += b.c_data.y
 * 
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     def __sub__(ovrVector2i a, ovrVector2i b):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":48
 *         return to_return
 * 
 *     def __iadd__(self, ovrVector2i b):             # <<<<<<<<<<<<<<
 *         self.c_data.x += b.c_data.x
 *         self.c_data.y += b.c_data.y
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":54
 *         return self
 * 
 *     def __sub__(ovrVector2i a, ovrVector2i b):             # <<<<<<<<<<<<<<
 *         cdef ovrVector2i to_return = ovrVector2i(
 *             a.c_data.x - b.c_data.x,
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_15__sub__(PyObject *__pyx_v_a, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_15__sub__(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__sub__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_a), __pyx_ptype_7psychxr_3ovr_4math_ovrVector2i, 1, "a", 0))) __PYX_ERR(0, 54, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrVector2i, 1, "b", 0))) __PYX_ERR(0, 54, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_14__sub__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_a), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_14__sub__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_a, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_b) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("__sub__", 0);

  /* "psychxr/ovr/math.pyx":56
 *     def __sub__(ovrVector2i a, ovrVector2i b):
 *         cdef ovrVector2i to_return = ovrVector2i(
 *             a.c_data.x - b.c_data.x,             # <<<<<<<<<<<<<<
 *             a.c_data.y - b.c_data.y)
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_int((__pyx_v_a->c_data->x - __pyx_v_b->c_data->x)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 56, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);

  /* "psychxr/ovr/math.pyx":57
 *         cdef ovrVector2i to_return = ovrVector2i(
 *             a.c_data.x - b.c_data.x,
 *             a.c_data.y - b.c_data.y)             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
  __pyx_t_2 = __Pyx_PyInt_From_int((__pyx_v_a->c_data->y - __pyx_v_b->c_data->y)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 57, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "psychxr/ovr/math.pyx":55
 * 
 *     def __sub__(ovrVector2i a, ovrVector2i b):
 *         cdef ovrVector2i to_return = ovrVector2i(             # <<<<<<<<<<<<<<
 *             a.c_data.x - b.c_data.x,
 *             a.c_data.y - b.c_data.y)
 */
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 55, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector2i), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 55, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "psychxr/ovr/math.pyx":59
 *             a.c_data.y - b.c_data.y)
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def __isub__(self, ovrVector2i b):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":54
 *         return self
 * 
 *     def __sub__(ovrVector2i a, ovrVector2i b):             # <<<<<<<<<<<<<<
 *         cdef ovrVector2i to_return = ovrVector2i(
 *             a.c_data.x - b.c_data.x,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.__sub__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":61
 *         return to_return
 * 
 *     def __isub__(self, ovrVector2i b):             # <<<<<<<<<<<<<<
 *         self.c_data.x -= b.c_data.x
 *         self.c_data.y -= b.c_data.y
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_17__isub__(PyObject *__pyx_v_self, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_17__isub__(PyObject *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__isub__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrVector2i, 1, "b", 0))) __PYX_ERR(0, 61, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_16__isub__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_16__isub__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__isub__", 0);

  /* "psychxr/ovr/math.pyx":62
 * 
 *     def __isub__(self, ovrVector2i b):
 *         self.c_data.x -= b.c_data.x             # <<<<<<<<<<<<<<
 *         self.c_data.y -= b.c_data.y
 * 
 */
  __pyx_v_self->c_data->x = (__pyx_v_self->c_data->x - __pyx_v_b->c_data->x);

  /* "psychxr/ovr/math.pyx":63
 *     def __isub__(self, ovrVector2i b):
 *         self.c_data.x -= b.c_data.x
 *         self.c_data.y -= b.c_data.y             # <<<<<<<<<<<<<<
 * 
 *         return self
 */
  __pyx_v_self->c_data->y = (__pyx_v_self->c_data->y - __pyx_v_b->c_data->y);

  /* "psychxr/ovr/math.pyx":65
 *         self.c_data.y -= b.c_data.y
 * 
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     def __neg__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":61
 *         return to_return
 * 
 *     def __isub__(self, ovrVector2i b):             # <<<<<<<<<<<<<<
 *         self.c_data.x -= b.c_data.x
 *         self.c_data.y -= b.c_data.y
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":67
 *         return self
 * 
 *     def __neg__(self):             # <<<<<<<<<<<<<<
 *         cdef ovrVector2i to_return = ovrVector2i(-self.c_data.x, -self.c_data.y)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_19__neg__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_19__neg__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__neg__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_18__neg__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_18__neg__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("__neg__", 0);

  /* "psychxr/ovr/math.pyx":68
 * 
 *     def __neg__(self):
 *         cdef ovrVector2i to_return = ovrVector2i(-self.c_data.x, -self.c_data.y)             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
  __pyx_t_1 = __Pyx_PyInt_From_int((-__pyx_v_self->c_data->x)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int((-__pyx_v_self->c_data->y)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector2i), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "psychxr/ovr/math.pyx":70
 *         cdef ovrVector2i to_return = ovrVector2i(-self.c_data.x, -self.c_data.y)
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def __mul__(ovrVector2i a, object b):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":67
 *         return self
 * 
 *     def __neg__(self):             # <<<<<<<<<<<<<<
 *         cdef ovrVector2i to_return = ovrVector2i(-self.c_data.x, -self.c_data.y)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.__neg__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":72
 *         return to_return
 * 
 *     def __mul__(ovrVector2i a, object b):             # <<<<<<<<<<<<<<
 *         cdef ovrVector2i to_return
 *         if isinstance(b, ovrVector2i):
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_21__mul__(PyObject *__pyx_v_a, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_21__mul__(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__mul__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_a), __pyx_ptype_7psychxr_3ovr_4math_ovrVector2i, 1, "a", 0))) __PYX_ERR(0, 72, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_20__mul__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_a), ((PyObject *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_20__mul__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_a, PyObject *__pyx_v_b) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  __Pyx_RefNannySetupContext("__mul__", 0);

  /* "psychxr/ovr/math.pyx":74
 *     def __mul__(ovrVector2i a, object b):
 *         cdef ovrVector2i to_return
 *         if isinstance(b, ovrVector2i):             # <<<<<<<<<<<<<<
 *             to_return = ovrVector2i(a.c_data.x * b.c_data.x,
 *                                     a.c_data.y * b.c_data.y)
 */
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_b, __pyx_ptype_7psychxr_3ovr_4math_ovrVector2i); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "psychxr/ovr/math.pyx":75
 *         cdef ovrVector2i to_return
 *         if isinstance(b, ovrVector2i):
 *             to_return = ovrVector2i(a.c_data.x * b.c_data.x,             # <<<<<<<<<<<<<<
 *                                     a.c_data.y * b.c_data.y)
 *         elif isinstance(b, (int, float)):
 */
    __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_a->c_data->x); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 75, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_b, __pyx_n_s_c_data); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 75, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_x); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 75, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = PyNumber_Multiply(__pyx_t_3, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 75, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "psychxr/ovr/math.pyx":76
 *         if isinstance(b, ovrVector2i):
 *             to_return = ovrVector2i(a.c_data.x * b.c_data.x,
 *                                     a.c_data.y * b.c_data.y)             # <<<<<<<<<<<<<<
 *         elif isinstance(b, (int, float)):
 *             to_return = ovrVector2i(a.c_data.x * <int>b,
 */
    __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_a->c_data->y); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 76, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_b, __pyx_n_s_c_data); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 76, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_y); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 76, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyNumber_Multiply(__pyx_t_5, __pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 76, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "psychxr/ovr/math.pyx":75
 *         cdef ovrVector2i to_return
 *         if isinstance(b, ovrVector2i):
 *             to_return = ovrVector2i(a.c_data.x * b.c_data.x,             # <<<<<<<<<<<<<<
 *                                     a.c_data.y * b.c_data.y)
 *         elif isinstance(b, (int, float)):
 */
    __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 75, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_3);
    __pyx_t_4 = 0;
    __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector2i), __pyx_t_6, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 75, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "psychxr/ovr/math.pyx":74
 *     def __mul__(ovrVector2i a, object b):
 *         cdef ovrVector2i to_return
 *         if isinstance(b, ovrVector2i):             # <<<<<<<<<<<<<<
 *             to_return = ovrVector2i(a.c_data.x * b.c_data.x,
 *                                     a.c_data.y * b.c_data.y)
 */
    goto __pyx_L3;
  }

  /* "psychxr/ovr/math.pyx":77
 *             to_return = ovrVector2i(a.c_data.x * b.c_data.x,
 *                                     a.c_data.y * b.c_data.y)
 *         elif isinstance(b, (int, float)):             # <<<<<<<<<<<<<<
 *             to_return = ovrVector2i(a.c_data.x * <int>b,
 *                                     a.c_data.y * <int>b)
 */
  __pyx_t_1 = PyInt_Check(__pyx_v_b); 
  __pyx_t_7 = (__pyx_t_1 != 0);
  if (!__pyx_t_7) {
  } else {
    __pyx_t_2 = __pyx_t_7;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_7 = PyFloat_Check(__pyx_v_b); 
  __pyx_t_1 = (__pyx_t_7 != 0);
  __pyx_t_2 = __pyx_t_1;
  __pyx_L4_bool_binop_done:;
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {

    /* "psychxr/ovr/math.pyx":78
 *                                     a.c_data.y * b.c_data.y)
 *         elif isinstance(b, (int, float)):
 *             to_return = ovrVector2i(a.c_data.x * <int>b,             # <<<<<<<<<<<<<<
 *                                     a.c_data.y * <int>b)
 * 
 */
    __pyx_t_8 = __Pyx_PyInt_As_int(__pyx_v_b); if (unlikely((__pyx_t_8 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 78, __pyx_L1_error)
    __pyx_t_3 = __Pyx_PyInt_From_int((__pyx_v_a->c_data->x * ((int)__pyx_t_8))); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 78, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);

    /* "psychxr/ovr/math.pyx":79
 *         elif isinstance(b, (int, float)):
 *             to_return = ovrVector2i(a.c_data.x * <int>b,
 *                                     a.c_data.y * <int>b)             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
    __pyx_t_8 = __Pyx_PyInt_As_int(__pyx_v_b); if (unlikely((__pyx_t_8 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 79, __pyx_L1_error)
    __pyx_t_6 = __Pyx_PyInt_From_int((__pyx_v_a->c_data->y * ((int)__pyx_t_8))); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 79, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);

    /* "psychxr/ovr/math.pyx":78
 *                                     a.c_data.y * b.c_data.y)
 *         elif isinstance(b, (int, float)):
 *             to_return = ovrVector2i(a.c_data.x * <int>b,             # <<<<<<<<<<<<<<
 *                                     a.c_data.y * <int>b)
 * 
 */
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 78, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_6);
    __pyx_t_3 = 0;
    __pyx_t_6 = 0;
    __pyx_t_6 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector2i), __pyx_t_4, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 78, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_t_6);
    __pyx_t_6 = 0;

    /* "psychxr/ovr/math.pyx":77
 *             to_return = ovrVector2i(a.c_data.x * b.c_data.x,
 *                                     a.c_data.y * b.c_data.y)
 *         elif isinstance(b, (int, float)):             # <<<<<<<<<<<<<<
 *             to_return = ovrVector2i(a.c_data.x * <int>b,
 *                                     a.c_data.y * <int>b)
 */
  }
  __pyx_L3:;

  /* "psychxr/ovr/math.pyx":81
 *                                     a.c_data.y * <int>b)
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def __imul__(self, object b):
 */
  __Pyx_XDECREF(__pyx_r);
  if (unlikely(!__pyx_v_to_return)) { __Pyx_RaiseUnboundLocalError("to_return"); __PYX_ERR(0, 81, __pyx_L1_error) }
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":72
 *         return to_return
 * 
 *     def __mul__(ovrVector2i a, object b):             # <<<<<<<<<<<<<<
 *         cdef ovrVector2i to_return
 *         if isinstance(b, ovrVector2i):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.__mul__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":83
 *         return to_return
 * 
 *     def __imul__(self, object b):             # <<<<<<<<<<<<<<
 *         cdef ovrVector2i to_return
 *         if isinstance(b, ovrVector2i):
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_23__imul__(PyObject *__pyx_v_self, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_23__imul__(PyObject *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__imul__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_22__imul__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_self), ((PyObject *)__pyx_v_b));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_22__imul__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_t_7;
  __Pyx_RefNannySetupContext("__imul__", 0);

  /* "psychxr/ovr/math.pyx":85
 *     def __imul__(self, object b):
 *         cdef ovrVector2i to_return
 *         if isinstance(b, ovrVector2i):             # <<<<<<<<<<<<<<
 *             self.c_data.x *= b.c_data.x
 *             self.c_data.y *= b.c_data.y
 */
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_b, __pyx_ptype_7psychxr_3ovr_4math_ovrVector2i); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "psychxr/ovr/math.pyx":86
 *         cdef ovrVector2i to_return
 *         if isinstance(b, ovrVector2i):
 *             self.c_data.x *= b.c_data.x             # <<<<<<<<<<<<<<
 *             self.c_data.y *= b.c_data.y
 *         elif isinstance(b, (int, float)):
 */
    __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_self->c_data->x); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 86, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_b, __pyx_n_s_c_data); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 86, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_x); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 86, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = PyNumber_InPlaceMultiply(__pyx_t_3, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 86, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_6 = __Pyx_PyInt_As_int(__pyx_t_4); if (unlikely((__pyx_t_6 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 86, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_self->c_data->x = __pyx_t_6;

    /* "psychxr/ovr/math.pyx":87
 *         if isinstance(b, ovrVector2i):
 *             self.c_data.x *= b.c_data.x
 *             self.c_data.y *= b.c_data.y             # <<<<<<<<<<<<<<
 *         elif isinstance(b, (int, float)):
 *             self.c_data.x *= <int>b
 */
    __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_self->c_data->y); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 87, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_b, __pyx_n_s_c_data); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 87, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_y); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 87, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = PyNumber_InPlaceMultiply(__pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 87, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_6 = __Pyx_PyInt_As_int(__pyx_t_5); if (unlikely((__pyx_t_6 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 87, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_v_self->c_data->y = __pyx_t_6;

    /* "psychxr/ovr/math.pyx":85
 *     def __imul__(self, object b):
 *         cdef ovrVector2i to_return
 *         if isinstance(b, ovrVector2i):             # <<<<<<<<<<<<<<
 *             self.c_data.x *= b.c_data.x
 *             self.c_data.y *= b.c_data.y
 */
    goto __pyx_L3;
  }

  /* "psychxr/ovr/math.pyx":88
 *             self.c_data.x *= b.c_data.x
 *             self.c_data.y *= b.c_data.y
 *         elif isinstance(b, (int, float)):             # <<<<<<<<<<<<<<
 *             self.c_data.x *= <int>b
 *             self.c_data.y *= <int>b
 */
  __pyx_t_1 = PyInt_Check(__pyx_v_b); 
  __pyx_t_7 = (__pyx_t_1 != 0);
  if (!__pyx_t_7) {
  } else {
    __pyx_t_2 = __pyx_t_7;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_7 = PyFloat_Check(__pyx_v_b); 
  __pyx_t_1 = (__pyx_t_7 != 0);
  __pyx_t_2 = __pyx_t_1;
  __pyx_L4_bool_binop_done:;
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {

    /* "psychxr/ovr/math.pyx":89
 *             self.c_data.y *= b.c_data.y
 *         elif isinstance(b, (int, float)):
 *             self.c_data.x *= <int>b             # <<<<<<<<<<<<<<
 *             self.c_data.y *= <int>b
 * 
 */
    __pyx_t_6 = __Pyx_PyInt_As_int(__pyx_v_b); if (unlikely((__pyx_t_6 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 89, __pyx_L1_error)
    __pyx_v_self->c_data->x = (__pyx_v_self->c_data->x * ((int)__pyx_t_6));

    /* "psychxr/ovr/math.pyx":90
 *         elif isinstance(b, (int, float)):
 *             self.c_data.x *= <int>b
 *             self.c_data.y *= <int>b             # <<<<<<<<<<<<<<
 * 
 *         return self
 */
    __pyx_t_6 = __Pyx_PyInt_As_int(__pyx_v_b); if (unlikely((__pyx_t_6 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 90, __pyx_L1_error)
    __pyx_v_self->c_data->y = (__pyx_v_self->c_data->y * ((int)__pyx_t_6));

    /* "psychxr/ovr/math.pyx":88
 *             self.c_data.x *= b.c_data.x
 *             self.c_data.y *= b.c_data.y
 *         elif isinstance(b, (int, float)):             # <<<<<<<<<<<<<<
 *             self.c_data.x *= <int>b
 *             self.c_data.y *= <int>b
 */
  }
  __pyx_L3:;

  /* "psychxr/ovr/math.pyx":92
 *             self.c_data.y *= <int>b
 * 
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     def __truediv__(ovrVector2i a, object b):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":83
 *         return to_return
 * 
 *     def __imul__(self, object b):             # <<<<<<<<<<<<<<
 *         cdef ovrVector2i to_return
 *         if isinstance(b, ovrVector2i):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.__imul__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":94
 *         return self
 * 
 *     def __truediv__(ovrVector2i a, object b):             # <<<<<<<<<<<<<<
 *         cdef int rcp = <int>1 / <int>b
 *         cdef ovrVector2i to_return = ovrVector2i(
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_25__truediv__(PyObject *__pyx_v_a, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_25__truediv__(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__truediv__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_a), __pyx_ptype_7psychxr_3ovr_4math_ovrVector2i, 1, "a", 0))) __PYX_ERR(0, 94, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_24__truediv__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_a), ((PyObject *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_24__truediv__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_a, PyObject *__pyx_v_b) {
  int __pyx_v_rcp;
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("__truediv__", 0);

  /* "psychxr/ovr/math.pyx":95
 * 
 *     def __truediv__(ovrVector2i a, object b):
 *         cdef int rcp = <int>1 / <int>b             # <<<<<<<<<<<<<<
 *         cdef ovrVector2i to_return = ovrVector2i(
 *             a.c_data.x * rcp,
 */
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_b); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 95, __pyx_L1_error)
  if (unlikely(((int)__pyx_t_1) == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
    __PYX_ERR(0, 95, __pyx_L1_error)
  }
  else if (sizeof(int) == sizeof(long) && (!(((int)-1) > 0)) && unlikely(((int)__pyx_t_1) == (int)-1)  && unlikely(UNARY_NEG_WOULD_OVERFLOW(((int)1)))) {
    PyErr_SetString(PyExc_OverflowError, "value too large to perform division");
    __PYX_ERR(0, 95, __pyx_L1_error)
  }
  __pyx_v_rcp = __Pyx_div_int(((int)1), ((int)__pyx_t_1));

  /* "psychxr/ovr/math.pyx":97
 *         cdef int rcp = <int>1 / <int>b
 *         cdef ovrVector2i to_return = ovrVector2i(
 *             a.c_data.x * rcp,             # <<<<<<<<<<<<<<
 *             a.c_data.y * rcp)
 * 
 */
  __pyx_t_2 = __Pyx_PyInt_From_int((__pyx_v_a->c_data->x * __pyx_v_rcp)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 97, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "psychxr/ovr/math.pyx":98
 *         cdef ovrVector2i to_return = ovrVector2i(
 *             a.c_data.x * rcp,
 *             a.c_data.y * rcp)             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
  __pyx_t_3 = __Pyx_PyInt_From_int((__pyx_v_a->c_data->y * __pyx_v_rcp)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 98, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "psychxr/ovr/math.pyx":96
 *     def __truediv__(ovrVector2i a, object b):
 *         cdef int rcp = <int>1 / <int>b
 *         cdef ovrVector2i to_return = ovrVector2i(             # <<<<<<<<<<<<<<
 *             a.c_data.x * rcp,
 *             a.c_data.y * rcp)
 */
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 96, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_3);
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector2i), __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 96, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "psychxr/ovr/math.pyx":100
 *             a.c_data.y * rcp)
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def __itruediv__(self, object b):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":94
 *         return self
 * 
 *     def __truediv__(ovrVector2i a, object b):             # <<<<<<<<<<<<<<
 *         cdef int rcp = <int>1 / <int>b
 *         cdef ovrVector2i to_return = ovrVector2i(
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.__truediv__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":102
 *         return to_return
 * 
 *     def __itruediv__(self, object b):             # <<<<<<<<<<<<<<
 *         cdef int rcp = <int>1 / <int>b
 *         self.c_data.x *= rcp
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_27__itruediv__(PyObject *__pyx_v_self, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_27__itruediv__(PyObject *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__itruediv__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_26__itruediv__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_self), ((PyObject *)__pyx_v_b));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_26__itruediv__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, PyObject *__pyx_v_b) {
  int __pyx_v_rcp;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__itruediv__", 0);

  /* "psychxr/ovr/math.pyx":103
 * 
 *     def __itruediv__(self, object b):
 *         cdef int rcp = <int>1 / <int>b             # <<<<<<<<<<<<<<
 *         self.c_data.x *= rcp
 *         self.c_data.y *= rcp
 */
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_b); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 103, __pyx_L1_error)
  if (unlikely(((int)__pyx_t_1) == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
    __PYX_ERR(0, 103, __pyx_L1_error)
  }
  else if (sizeof(int) == sizeof(long) && (!(((int)-1) > 0)) && unlikely(((int)__pyx_t_1) == (int)-1)  && unlikely(UNARY_NEG_WOULD_OVERFLOW(((int)1)))) {
    PyErr_SetString(PyExc_OverflowError, "value too large to perform division");
    __PYX_ERR(0, 103, __pyx_L1_error)
  }
  __pyx_v_rcp = __Pyx_div_int(((int)1), ((int)__pyx_t_1));

  /* "psychxr/ovr/math.pyx":104
 *     def __itruediv__(self, object b):
 *         cdef int rcp = <int>1 / <int>b
 *         self.c_data.x *= rcp             # <<<<<<<<<<<<<<
 *         self.c_data.y *= rcp
 * 
 */
  __pyx_v_self->c_data->x = (__pyx_v_self->c_data->x * __pyx_v_rcp);

  /* "psychxr/ovr/math.pyx":105
 *         cdef int rcp = <int>1 / <int>b
 *         self.c_data.x *= rcp
 *         self.c_data.y *= rcp             # <<<<<<<<<<<<<<
 * 
 *         return self
 */
  __pyx_v_self->c_data->y = (__pyx_v_self->c_data->y * __pyx_v_rcp);

  /* "psychxr/ovr/math.pyx":107
 *         self.c_data.y *= rcp
 * 
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":102
 *         return to_return
 * 
 *     def __itruediv__(self, object b):             # <<<<<<<<<<<<<<
 *         cdef int rcp = <int>1 / <int>b
 *         self.c_data.x *= rcp
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.__itruediv__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":110
 * 
 *     @staticmethod
 *     def min(ovrVector2i a, ovrVector2i b):             # <<<<<<<<<<<<<<
 *         cdef ovrVector2i to_return = ovrVector2i(
 *             a.c_data.x if a.c_data.x < b.c_data.x else b.c_data.x,
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_29min(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_7psychxr_3ovr_4math_11ovrVector2i_29min = {"min", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_29min, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_29min(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_a = 0;
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_b = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("min (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_a,&__pyx_n_s_b,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_a)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_b)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("min", 1, 2, 2, 1); __PYX_ERR(0, 110, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "min") < 0)) __PYX_ERR(0, 110, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_a = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)values[0]);
    __pyx_v_b = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("min", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 110, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.min", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_a), __pyx_ptype_7psychxr_3ovr_4math_ovrVector2i, 1, "a", 0))) __PYX_ERR(0, 110, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrVector2i, 1, "b", 0))) __PYX_ERR(0, 110, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_28min(__pyx_v_a, __pyx_v_b);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_28min(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_a, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_b) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("min", 0);

  /* "psychxr/ovr/math.pyx":112
 *     def min(ovrVector2i a, ovrVector2i b):
 *         cdef ovrVector2i to_return = ovrVector2i(
 *             a.c_data.x if a.c_data.x < b.c_data.x else b.c_data.x,             # <<<<<<<<<<<<<<
 *             a.c_data.y if a.c_data.y < b.c_data.y else b.c_data.y)
 * 
 */
  if (((__pyx_v_a->c_data->x < __pyx_v_b->c_data->x) != 0)) {
    __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_a->c_data->x); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 112, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __pyx_t_2;
    __pyx_t_2 = 0;
  } else {
    __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_b->c_data->x); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 112, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __pyx_t_2;
    __pyx_t_2 = 0;
  }

  /* "psychxr/ovr/math.pyx":113
 *         cdef ovrVector2i to_return = ovrVector2i(
 *             a.c_data.x if a.c_data.x < b.c_data.x else b.c_data.x,
 *             a.c_data.y if a.c_data.y < b.c_data.y else b.c_data.y)             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
  if (((__pyx_v_a->c_data->y < __pyx_v_b->c_data->y) != 0)) {
    __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_a->c_data->y); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 113, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __pyx_t_3;
    __pyx_t_3 = 0;
  } else {
    __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_b->c_data->y); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 113, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __pyx_t_3;
    __pyx_t_3 = 0;
  }

  /* "psychxr/ovr/math.pyx":111
 *     @staticmethod
 *     def min(ovrVector2i a, ovrVector2i b):
 *         cdef ovrVector2i to_return = ovrVector2i(             # <<<<<<<<<<<<<<
 *             a.c_data.x if a.c_data.x < b.c_data.x else b.c_data.x,
 *             a.c_data.y if a.c_data.y < b.c_data.y else b.c_data.y)
 */
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 111, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector2i), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 111, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "psychxr/ovr/math.pyx":115
 *             a.c_data.y if a.c_data.y < b.c_data.y else b.c_data.y)
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":110
 * 
 *     @staticmethod
 *     def min(ovrVector2i a, ovrVector2i b):             # <<<<<<<<<<<<<<
 *         cdef ovrVector2i to_return = ovrVector2i(
 *             a.c_data.x if a.c_data.x < b.c_data.x else b.c_data.x,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.min", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":118
 * 
 *     @staticmethod
 *     def max(ovrVector2i a, ovrVector2i b):             # <<<<<<<<<<<<<<
 *         cdef ovrVector2i to_return = ovrVector2i(
 *             a.c_data.x if a.c_data.x > b.c_data.x else b.c_data.x,
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_31max(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_7psychxr_3ovr_4math_11ovrVector2i_31max = {"max", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_31max, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_31max(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_a = 0;
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_b = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("max (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_a,&__pyx_n_s_b,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_a)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_b)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("max", 1, 2, 2, 1); __PYX_ERR(0, 118, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "max") < 0)) __PYX_ERR(0, 118, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_a = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)values[0]);
    __pyx_v_b = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("max", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 118, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.max", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_a), __pyx_ptype_7psychxr_3ovr_4math_ovrVector2i, 1, "a", 0))) __PYX_ERR(0, 118, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrVector2i, 1, "b", 0))) __PYX_ERR(0, 118, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_30max(__pyx_v_a, __pyx_v_b);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_30max(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_a, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_b) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("max", 0);

  /* "psychxr/ovr/math.pyx":120
 *     def max(ovrVector2i a, ovrVector2i b):
 *         cdef ovrVector2i to_return = ovrVector2i(
 *             a.c_data.x if a.c_data.x > b.c_data.x else b.c_data.x,             # <<<<<<<<<<<<<<
 *             a.c_data.y if a.c_data.y > b.c_data.y else b.c_data.y)
 * 
 */
  if (((__pyx_v_a->c_data->x > __pyx_v_b->c_data->x) != 0)) {
    __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_a->c_data->x); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 120, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __pyx_t_2;
    __pyx_t_2 = 0;
  } else {
    __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_b->c_data->x); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 120, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __pyx_t_2;
    __pyx_t_2 = 0;
  }

  /* "psychxr/ovr/math.pyx":121
 *         cdef ovrVector2i to_return = ovrVector2i(
 *             a.c_data.x if a.c_data.x > b.c_data.x else b.c_data.x,
 *             a.c_data.y if a.c_data.y > b.c_data.y else b.c_data.y)             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
  if (((__pyx_v_a->c_data->y > __pyx_v_b->c_data->y) != 0)) {
    __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_a->c_data->y); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 121, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __pyx_t_3;
    __pyx_t_3 = 0;
  } else {
    __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_b->c_data->y); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 121, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __pyx_t_3;
    __pyx_t_3 = 0;
  }

  /* "psychxr/ovr/math.pyx":119
 *     @staticmethod
 *     def max(ovrVector2i a, ovrVector2i b):
 *         cdef ovrVector2i to_return = ovrVector2i(             # <<<<<<<<<<<<<<
 *             a.c_data.x if a.c_data.x > b.c_data.x else b.c_data.x,
 *             a.c_data.y if a.c_data.y > b.c_data.y else b.c_data.y)
 */
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 119, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector2i), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 119, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "psychxr/ovr/math.pyx":123
 *             a.c_data.y if a.c_data.y > b.c_data.y else b.c_data.y)
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def clamped(self, int max_mag):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":118
 * 
 *     @staticmethod
 *     def max(ovrVector2i a, ovrVector2i b):             # <<<<<<<<<<<<<<
 *         cdef ovrVector2i to_return = ovrVector2i(
 *             a.c_data.x if a.c_data.x > b.c_data.x else b.c_data.x,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.max", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":125
 *         return to_return
 * 
 *     def clamped(self, int max_mag):             # <<<<<<<<<<<<<<
 *         cdef int mag_squared = self.length_sq()
 *         if mag_squared > max_mag * max_mag:
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_33clamped(PyObject *__pyx_v_self, PyObject *__pyx_arg_max_mag); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_33clamped(PyObject *__pyx_v_self, PyObject *__pyx_arg_max_mag) {
  int __pyx_v_max_mag;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("clamped (wrapper)", 0);
  assert(__pyx_arg_max_mag); {
    __pyx_v_max_mag = __Pyx_PyInt_As_int(__pyx_arg_max_mag); if (unlikely((__pyx_v_max_mag == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 125, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.clamped", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_32clamped(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_self), ((int)__pyx_v_max_mag));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_32clamped(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, int __pyx_v_max_mag) {
  int __pyx_v_mag_squared;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  double __pyx_t_6;
  __Pyx_RefNannySetupContext("clamped", 0);

  /* "psychxr/ovr/math.pyx":126
 * 
 *     def clamped(self, int max_mag):
 *         cdef int mag_squared = self.length_sq()             # <<<<<<<<<<<<<<
 *         if mag_squared > max_mag * max_mag:
 *             return self * (max_mag / cmath.sqrt(mag_squared))
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_length_sq); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 126, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 126, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 126, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_4 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 126, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_mag_squared = __pyx_t_4;

  /* "psychxr/ovr/math.pyx":127
 *     def clamped(self, int max_mag):
 *         cdef int mag_squared = self.length_sq()
 *         if mag_squared > max_mag * max_mag:             # <<<<<<<<<<<<<<
 *             return self * (max_mag / cmath.sqrt(mag_squared))
 * 
 */
  __pyx_t_5 = ((__pyx_v_mag_squared > (__pyx_v_max_mag * __pyx_v_max_mag)) != 0);
  if (__pyx_t_5) {

    /* "psychxr/ovr/math.pyx":128
 *         cdef int mag_squared = self.length_sq()
 *         if mag_squared > max_mag * max_mag:
 *             return self * (max_mag / cmath.sqrt(mag_squared))             # <<<<<<<<<<<<<<
 * 
 *         return self
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_6 = sqrt(__pyx_v_mag_squared);
    if (unlikely(__pyx_t_6 == 0)) {
      PyErr_SetString(PyExc_ZeroDivisionError, "float division");
      __PYX_ERR(0, 128, __pyx_L1_error)
    }
    __pyx_t_1 = PyFloat_FromDouble((__pyx_v_max_mag / __pyx_t_6)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 128, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PyNumber_Multiply(((PyObject *)__pyx_v_self), __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 128, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "psychxr/ovr/math.pyx":127
 *     def clamped(self, int max_mag):
 *         cdef int mag_squared = self.length_sq()
 *         if mag_squared > max_mag * max_mag:             # <<<<<<<<<<<<<<
 *             return self * (max_mag / cmath.sqrt(mag_squared))
 * 
 */
  }

  /* "psychxr/ovr/math.pyx":130
 *             return self * (max_mag / cmath.sqrt(mag_squared))
 * 
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     def isEqual(self, ovrVector2i b, int tolerance = 0):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":125
 *         return to_return
 * 
 *     def clamped(self, int max_mag):             # <<<<<<<<<<<<<<
 *         cdef int mag_squared = self.length_sq()
 *         if mag_squared > max_mag * max_mag:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.clamped", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":132
 *         return self
 * 
 *     def isEqual(self, ovrVector2i b, int tolerance = 0):             # <<<<<<<<<<<<<<
 *         return cmath.fabs(b.c_data.x - self.c_data.x) <= tolerance and \
 *             cmath.fabs(b.c_data.y - self.c_data.y) <= tolerance
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_35isEqual(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_35isEqual(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_b = 0;
  int __pyx_v_tolerance;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("isEqual (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_b,&__pyx_n_s_tolerance,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_b)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_tolerance);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "isEqual") < 0)) __PYX_ERR(0, 132, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_b = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)values[0]);
    if (values[1]) {
      __pyx_v_tolerance = __Pyx_PyInt_As_int(values[1]); if (unlikely((__pyx_v_tolerance == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 132, __pyx_L3_error)
    } else {
      __pyx_v_tolerance = ((int)0);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("isEqual", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 132, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.isEqual", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrVector2i, 1, "b", 0))) __PYX_ERR(0, 132, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_34isEqual(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_self), __pyx_v_b, __pyx_v_tolerance);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_34isEqual(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_b, int __pyx_v_tolerance) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("isEqual", 0);

  /* "psychxr/ovr/math.pyx":133
 * 
 *     def isEqual(self, ovrVector2i b, int tolerance = 0):
 *         return cmath.fabs(b.c_data.x - self.c_data.x) <= tolerance and \             # <<<<<<<<<<<<<<
 *             cmath.fabs(b.c_data.y - self.c_data.y) <= tolerance
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = (fabs((__pyx_v_b->c_data->x - __pyx_v_self->c_data->x)) <= __pyx_v_tolerance);
  if (__pyx_t_2) {
  } else {
    __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 133, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L3_bool_binop_done;
  }

  /* "psychxr/ovr/math.pyx":134
 *     def isEqual(self, ovrVector2i b, int tolerance = 0):
 *         return cmath.fabs(b.c_data.x - self.c_data.x) <= tolerance and \
 *             cmath.fabs(b.c_data.y - self.c_data.y) <= tolerance             # <<<<<<<<<<<<<<
 * 
 *     def compare(self, ovrVector2i b, int tolerance = 0):
 */
  __pyx_t_2 = (fabs((__pyx_v_b->c_data->y - __pyx_v_self->c_data->y)) <= __pyx_v_tolerance);
  __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 134, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __pyx_t_3;
  __pyx_t_3 = 0;
  __pyx_L3_bool_binop_done:;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":132
 *         return self
 * 
 *     def isEqual(self, ovrVector2i b, int tolerance = 0):             # <<<<<<<<<<<<<<
 *         return cmath.fabs(b.c_data.x - self.c_data.x) <= tolerance and \
 *             cmath.fabs(b.c_data.y - self.c_data.y) <= tolerance
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.isEqual", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":136
 *             cmath.fabs(b.c_data.y - self.c_data.y) <= tolerance
 * 
 *     def compare(self, ovrVector2i b, int tolerance = 0):             # <<<<<<<<<<<<<<
 *         return self.is_equal(b, tolerance)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_37compare(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_37compare(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_b = 0;
  int __pyx_v_tolerance;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("compare (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_b,&__pyx_n_s_tolerance,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_b)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_tolerance);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "compare") < 0)) __PYX_ERR(0, 136, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_b = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)values[0]);
    if (values[1]) {
      __pyx_v_tolerance = __Pyx_PyInt_As_int(values[1]); if (unlikely((__pyx_v_tolerance == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 136, __pyx_L3_error)
    } else {
      __pyx_v_tolerance = ((int)0);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("compare", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 136, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.compare", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrVector2i, 1, "b", 0))) __PYX_ERR(0, 136, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_36compare(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_self), __pyx_v_b, __pyx_v_tolerance);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_36compare(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_b, int __pyx_v_tolerance) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("compare", 0);

  /* "psychxr/ovr/math.pyx":137
 * 
 *     def compare(self, ovrVector2i b, int tolerance = 0):
 *         return self.is_equal(b, tolerance)             # <<<<<<<<<<<<<<
 * 
 *     def __getitem__(self, int idx):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_is_equal); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 137, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_tolerance); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 137, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, ((PyObject *)__pyx_v_b), __pyx_t_3};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 137, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, ((PyObject *)__pyx_v_b), __pyx_t_3};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 137, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  {
    __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 137, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (__pyx_t_4) {
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_INCREF(((PyObject *)__pyx_v_b));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_b));
    PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, ((PyObject *)__pyx_v_b));
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 137, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":136
 *             cmath.fabs(b.c_data.y - self.c_data.y) <= tolerance
 * 
 *     def compare(self, ovrVector2i b, int tolerance = 0):             # <<<<<<<<<<<<<<
 *         return self.is_equal(b, tolerance)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.compare", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":139
 *         return self.is_equal(b, tolerance)
 * 
 *     def __getitem__(self, int idx):             # <<<<<<<<<<<<<<
 *         assert 0 <= idx < 2
 *         cdef int* ptr_val = &self.c_data.x + idx
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_39__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_arg_idx); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_39__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_arg_idx) {
  int __pyx_v_idx;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getitem__ (wrapper)", 0);
  assert(__pyx_arg_idx); {
    __pyx_v_idx = __Pyx_PyInt_As_int(__pyx_arg_idx); if (unlikely((__pyx_v_idx == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 139, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.__getitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_38__getitem__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_self), ((int)__pyx_v_idx));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_38__getitem__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, int __pyx_v_idx) {
  int *__pyx_v_ptr_val;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__getitem__", 0);

  /* "psychxr/ovr/math.pyx":140
 * 
 *     def __getitem__(self, int idx):
 *         assert 0 <= idx < 2             # <<<<<<<<<<<<<<
 *         cdef int* ptr_val = &self.c_data.x + idx
 * 
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    __pyx_t_1 = (0 <= __pyx_v_idx);
    if (__pyx_t_1) {
      __pyx_t_1 = (__pyx_v_idx < 2);
    }
    if (unlikely(!(__pyx_t_1 != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 140, __pyx_L1_error)
    }
  }
  #endif

  /* "psychxr/ovr/math.pyx":141
 *     def __getitem__(self, int idx):
 *         assert 0 <= idx < 2
 *         cdef int* ptr_val = &self.c_data.x + idx             # <<<<<<<<<<<<<<
 * 
 *         return <int>ptr_val[0]
 */
  __pyx_v_ptr_val = ((&__pyx_v_self->c_data->x) + __pyx_v_idx);

  /* "psychxr/ovr/math.pyx":143
 *         cdef int* ptr_val = &self.c_data.x + idx
 * 
 *         return <int>ptr_val[0]             # <<<<<<<<<<<<<<
 * 
 *     def __setitem__(self, int idx, int val):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_From_int(((int)(__pyx_v_ptr_val[0]))); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 143, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":139
 *         return self.is_equal(b, tolerance)
 * 
 *     def __getitem__(self, int idx):             # <<<<<<<<<<<<<<
 *         assert 0 <= idx < 2
 *         cdef int* ptr_val = &self.c_data.x + idx
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.__getitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":145
 *         return <int>ptr_val[0]
 * 
 *     def __setitem__(self, int idx, int val):             # <<<<<<<<<<<<<<
 *         assert 0 <= idx < 2
 *         cdef int* ptr_val = &self.c_data.x + idx
 */

/* Python wrapper */
static int __pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_41__setitem__(PyObject *__pyx_v_self, PyObject *__pyx_arg_idx, PyObject *__pyx_arg_val); /*proto*/
static int __pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_41__setitem__(PyObject *__pyx_v_self, PyObject *__pyx_arg_idx, PyObject *__pyx_arg_val) {
  int __pyx_v_idx;
  int __pyx_v_val;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setitem__ (wrapper)", 0);
  assert(__pyx_arg_idx); {
    __pyx_v_idx = __Pyx_PyInt_As_int(__pyx_arg_idx); if (unlikely((__pyx_v_idx == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 145, __pyx_L3_error)
  }
  assert(__pyx_arg_val); {
    __pyx_v_val = __Pyx_PyInt_As_int(__pyx_arg_val); if (unlikely((__pyx_v_val == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 145, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.__setitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_40__setitem__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_self), ((int)__pyx_v_idx), ((int)__pyx_v_val));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_40__setitem__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, int __pyx_v_idx, int __pyx_v_val) {
  int *__pyx_v_ptr_val;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__setitem__", 0);

  /* "psychxr/ovr/math.pyx":146
 * 
 *     def __setitem__(self, int idx, int val):
 *         assert 0 <= idx < 2             # <<<<<<<<<<<<<<
 *         cdef int* ptr_val = &self.c_data.x + idx
 *         ptr_val[0] = val
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    __pyx_t_1 = (0 <= __pyx_v_idx);
    if (__pyx_t_1) {
      __pyx_t_1 = (__pyx_v_idx < 2);
    }
    if (unlikely(!(__pyx_t_1 != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 146, __pyx_L1_error)
    }
  }
  #endif

  /* "psychxr/ovr/math.pyx":147
 *     def __setitem__(self, int idx, int val):
 *         assert 0 <= idx < 2
 *         cdef int* ptr_val = &self.c_data.x + idx             # <<<<<<<<<<<<<<
 *         ptr_val[0] = val
 * 
 */
  __pyx_v_ptr_val = ((&__pyx_v_self->c_data->x) + __pyx_v_idx);

  /* "psychxr/ovr/math.pyx":148
 *         assert 0 <= idx < 2
 *         cdef int* ptr_val = &self.c_data.x + idx
 *         ptr_val[0] = val             # <<<<<<<<<<<<<<
 * 
 *     def entrywiseMultiply(self, ovrVector2i b):
 */
  (__pyx_v_ptr_val[0]) = __pyx_v_val;

  /* "psychxr/ovr/math.pyx":145
 *         return <int>ptr_val[0]
 * 
 *     def __setitem__(self, int idx, int val):             # <<<<<<<<<<<<<<
 *         assert 0 <= idx < 2
 *         cdef int* ptr_val = &self.c_data.x + idx
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.__setitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":150
 *         ptr_val[0] = val
 * 
 *     def entrywiseMultiply(self, ovrVector2i b):             # <<<<<<<<<<<<<<
 *         cdef ovrVector2i to_return = ovrVector2i(
 *             self.c_data.x * b.c_data.x,
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_43entrywiseMultiply(PyObject *__pyx_v_self, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_43entrywiseMultiply(PyObject *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("entrywiseMultiply (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrVector2i, 1, "b", 0))) __PYX_ERR(0, 150, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_42entrywiseMultiply(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_42entrywiseMultiply(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_b) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("entrywiseMultiply", 0);

  /* "psychxr/ovr/math.pyx":152
 *     def entrywiseMultiply(self, ovrVector2i b):
 *         cdef ovrVector2i to_return = ovrVector2i(
 *             self.c_data.x * b.c_data.x,             # <<<<<<<<<<<<<<
 *             self.c_data.y * b.c_data.y)
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_int((__pyx_v_self->c_data->x * __pyx_v_b->c_data->x)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 152, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);

  /* "psychxr/ovr/math.pyx":153
 *         cdef ovrVector2i to_return = ovrVector2i(
 *             self.c_data.x * b.c_data.x,
 *             self.c_data.y * b.c_data.y)             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
  __pyx_t_2 = __Pyx_PyInt_From_int((__pyx_v_self->c_data->y * __pyx_v_b->c_data->y)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 153, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "psychxr/ovr/math.pyx":151
 * 
 *     def entrywiseMultiply(self, ovrVector2i b):
 *         cdef ovrVector2i to_return = ovrVector2i(             # <<<<<<<<<<<<<<
 *             self.c_data.x * b.c_data.x,
 *             self.c_data.y * b.c_data.y)
 */
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 151, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector2i), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 151, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "psychxr/ovr/math.pyx":155
 *             self.c_data.y * b.c_data.y)
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def dot(self, ovrVector2i b):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":150
 *         ptr_val[0] = val
 * 
 *     def entrywiseMultiply(self, ovrVector2i b):             # <<<<<<<<<<<<<<
 *         cdef ovrVector2i to_return = ovrVector2i(
 *             self.c_data.x * b.c_data.x,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.entrywiseMultiply", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":157
 *         return to_return
 * 
 *     def dot(self, ovrVector2i b):             # <<<<<<<<<<<<<<
 *         cdef int dot_prod = \
 *             self.c_data.x * b.c_data.x + self.c_data.y * b.c_data.y
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_45dot(PyObject *__pyx_v_self, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_45dot(PyObject *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("dot (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrVector2i, 1, "b", 0))) __PYX_ERR(0, 157, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_44dot(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_44dot(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_b) {
  int __pyx_v_dot_prod;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("dot", 0);

  /* "psychxr/ovr/math.pyx":159
 *     def dot(self, ovrVector2i b):
 *         cdef int dot_prod = \
 *             self.c_data.x * b.c_data.x + self.c_data.y * b.c_data.y             # <<<<<<<<<<<<<<
 * 
 *         return <int>dot_prod
 */
  __pyx_v_dot_prod = ((__pyx_v_self->c_data->x * __pyx_v_b->c_data->x) + (__pyx_v_self->c_data->y * __pyx_v_b->c_data->y));

  /* "psychxr/ovr/math.pyx":161
 *             self.c_data.x * b.c_data.x + self.c_data.y * b.c_data.y
 * 
 *         return <int>dot_prod             # <<<<<<<<<<<<<<
 * 
 *     def angle(self, ovrVector2i b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(((int)__pyx_v_dot_prod)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 161, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":157
 *         return to_return
 * 
 *     def dot(self, ovrVector2i b):             # <<<<<<<<<<<<<<
 *         cdef int dot_prod = \
 *             self.c_data.x * b.c_data.x + self.c_data.y * b.c_data.y
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.dot", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":163
 *         return <int>dot_prod
 * 
 *     def angle(self, ovrVector2i b):             # <<<<<<<<<<<<<<
 *         cdef int div = self.length_sq() * b.length_sq()
 *         assert div != <int>0
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_47angle(PyObject *__pyx_v_self, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_47angle(PyObject *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("angle (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrVector2i, 1, "b", 0))) __PYX_ERR(0, 163, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_46angle(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_46angle(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_b) {
  int __pyx_v_div;
  int __pyx_v_to_return;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  __Pyx_RefNannySetupContext("angle", 0);

  /* "psychxr/ovr/math.pyx":164
 * 
 *     def angle(self, ovrVector2i b):
 *         cdef int div = self.length_sq() * b.length_sq()             # <<<<<<<<<<<<<<
 *         assert div != <int>0
 *         cdef int to_return = self.dot(b) / cmath.sqrt(div)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_length_sq); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 164, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 164, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 164, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_b), __pyx_n_s_length_sq); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 164, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (__pyx_t_4) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 164, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else {
    __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 164, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyNumber_Multiply(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 164, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 164, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_div = __pyx_t_5;

  /* "psychxr/ovr/math.pyx":165
 *     def angle(self, ovrVector2i b):
 *         cdef int div = self.length_sq() * b.length_sq()
 *         assert div != <int>0             # <<<<<<<<<<<<<<
 *         cdef int to_return = self.dot(b) / cmath.sqrt(div)
 * 
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!((__pyx_v_div != ((int)0)) != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 165, __pyx_L1_error)
    }
  }
  #endif

  /* "psychxr/ovr/math.pyx":166
 *         cdef int div = self.length_sq() * b.length_sq()
 *         assert div != <int>0
 *         cdef int to_return = self.dot(b) / cmath.sqrt(div)             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_dot); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 166, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_1) {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_2, ((PyObject *)__pyx_v_b)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 166, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_1, ((PyObject *)__pyx_v_b)};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 166, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_1, ((PyObject *)__pyx_v_b)};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 166, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 166, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1); __pyx_t_1 = NULL;
      __Pyx_INCREF(((PyObject *)__pyx_v_b));
      __Pyx_GIVEREF(((PyObject *)__pyx_v_b));
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, ((PyObject *)__pyx_v_b));
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 166, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyFloat_FromDouble(sqrt(__pyx_v_div)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 166, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyNumber_Divide(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 166, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_t_4); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 166, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_to_return = __pyx_t_5;

  /* "psychxr/ovr/math.pyx":168
 *         cdef int to_return = self.dot(b) / cmath.sqrt(div)
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def lengthSq(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_to_return); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 168, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":163
 *         return <int>dot_prod
 * 
 *     def angle(self, ovrVector2i b):             # <<<<<<<<<<<<<<
 *         cdef int div = self.length_sq() * b.length_sq()
 *         assert div != <int>0
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.angle", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":170
 *         return to_return
 * 
 *     def lengthSq(self):             # <<<<<<<<<<<<<<
 *         return \
 *             <int>(self.c_data.x * self.c_data.x + self.c_data.y * self.c_data.y)
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_49lengthSq(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_49lengthSq(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("lengthSq (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_48lengthSq(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_48lengthSq(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("lengthSq", 0);

  /* "psychxr/ovr/math.pyx":171
 * 
 *     def lengthSq(self):
 *         return \             # <<<<<<<<<<<<<<
 *             <int>(self.c_data.x * self.c_data.x + self.c_data.y * self.c_data.y)
 * 
 */
  __Pyx_XDECREF(__pyx_r);

  /* "psychxr/ovr/math.pyx":172
 *     def lengthSq(self):
 *         return \
 *             <int>(self.c_data.x * self.c_data.x + self.c_data.y * self.c_data.y)             # <<<<<<<<<<<<<<
 * 
 *     def length(self):
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(((int)((__pyx_v_self->c_data->x * __pyx_v_self->c_data->x) + (__pyx_v_self->c_data->y * __pyx_v_self->c_data->y)))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 172, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":170
 *         return to_return
 * 
 *     def lengthSq(self):             # <<<<<<<<<<<<<<
 *         return \
 *             <int>(self.c_data.x * self.c_data.x + self.c_data.y * self.c_data.y)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.lengthSq", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":174
 *             <int>(self.c_data.x * self.c_data.x + self.c_data.y * self.c_data.y)
 * 
 *     def length(self):             # <<<<<<<<<<<<<<
 *         return <int>cmath.sqrt(self.length_sq())
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_51length(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_51length(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("length (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_50length(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_50length(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  double __pyx_t_4;
  __Pyx_RefNannySetupContext("length", 0);

  /* "psychxr/ovr/math.pyx":175
 * 
 *     def length(self):
 *         return <int>cmath.sqrt(self.length_sq())             # <<<<<<<<<<<<<<
 * 
 *     def distanceSq(self, ovrVector2i b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_length_sq); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 175, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 175, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 175, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 175, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyInt_From_int(((int)sqrt(__pyx_t_4))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 175, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":174
 *             <int>(self.c_data.x * self.c_data.x + self.c_data.y * self.c_data.y)
 * 
 *     def length(self):             # <<<<<<<<<<<<<<
 *         return <int>cmath.sqrt(self.length_sq())
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.length", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":177
 *         return <int>cmath.sqrt(self.length_sq())
 * 
 *     def distanceSq(self, ovrVector2i b):             # <<<<<<<<<<<<<<
 *         return (self - b).length_sq()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_53distanceSq(PyObject *__pyx_v_self, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_53distanceSq(PyObject *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("distanceSq (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrVector2i, 1, "b", 0))) __PYX_ERR(0, 177, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_52distanceSq(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_52distanceSq(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("distanceSq", 0);

  /* "psychxr/ovr/math.pyx":178
 * 
 *     def distanceSq(self, ovrVector2i b):
 *         return (self - b).length_sq()             # <<<<<<<<<<<<<<
 * 
 *     def distance(self, ovrVector2i b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = PyNumber_Subtract(((PyObject *)__pyx_v_self), ((PyObject *)__pyx_v_b)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 178, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_length_sq); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 178, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (__pyx_t_2) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 178, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 178, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":177
 *         return <int>cmath.sqrt(self.length_sq())
 * 
 *     def distanceSq(self, ovrVector2i b):             # <<<<<<<<<<<<<<
 *         return (self - b).length_sq()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.distanceSq", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":180
 *         return (self - b).length_sq()
 * 
 *     def distance(self, ovrVector2i b):             # <<<<<<<<<<<<<<
 *         return (self - b).length()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_55distance(PyObject *__pyx_v_self, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_55distance(PyObject *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("distance (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrVector2i, 1, "b", 0))) __PYX_ERR(0, 180, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_54distance(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_54distance(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("distance", 0);

  /* "psychxr/ovr/math.pyx":181
 * 
 *     def distance(self, ovrVector2i b):
 *         return (self - b).length()             # <<<<<<<<<<<<<<
 * 
 *     def isNormalized(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = PyNumber_Subtract(((PyObject *)__pyx_v_self), ((PyObject *)__pyx_v_b)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 181, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_length); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 181, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (__pyx_t_2) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 181, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 181, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":180
 *         return (self - b).length_sq()
 * 
 *     def distance(self, ovrVector2i b):             # <<<<<<<<<<<<<<
 *         return (self - b).length()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.distance", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":183
 *         return (self - b).length()
 * 
 *     def isNormalized(self):             # <<<<<<<<<<<<<<
 *         return cmath.fabs(self.length_sq() - <int>1) < 0
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_57isNormalized(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_57isNormalized(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("isNormalized (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_56isNormalized(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_56isNormalized(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  double __pyx_t_4;
  __Pyx_RefNannySetupContext("isNormalized", 0);

  /* "psychxr/ovr/math.pyx":184
 * 
 *     def isNormalized(self):
 *         return cmath.fabs(self.length_sq() - <int>1) < 0             # <<<<<<<<<<<<<<
 * 
 *     def normalize(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_length_sq); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 184, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 184, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 184, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyInt_From_int(((int)1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 184, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyNumber_Subtract(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 184, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_t_3); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 184, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyBool_FromLong((fabs(__pyx_t_4) < 0.0)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 184, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":183
 *         return (self - b).length()
 * 
 *     def isNormalized(self):             # <<<<<<<<<<<<<<
 *         return cmath.fabs(self.length_sq() - <int>1) < 0
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.isNormalized", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":186
 *         return cmath.fabs(self.length_sq() - <int>1) < 0
 * 
 *     def normalize(self):             # <<<<<<<<<<<<<<
 *         cdef int s = self.length()
 *         if s != <int>0:
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_59normalize(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_59normalize(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("normalize (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_58normalize(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_58normalize(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self) {
  int __pyx_v_s;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  __Pyx_RefNannySetupContext("normalize", 0);
  __Pyx_INCREF((PyObject *)__pyx_v_self);

  /* "psychxr/ovr/math.pyx":187
 * 
 *     def normalize(self):
 *         cdef int s = self.length()             # <<<<<<<<<<<<<<
 *         if s != <int>0:
 *             s = <int>1 / s
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_length); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 187, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 187, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 187, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_4 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 187, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_s = __pyx_t_4;

  /* "psychxr/ovr/math.pyx":188
 *     def normalize(self):
 *         cdef int s = self.length()
 *         if s != <int>0:             # <<<<<<<<<<<<<<
 *             s = <int>1 / s
 * 
 */
  __pyx_t_5 = ((__pyx_v_s != ((int)0)) != 0);
  if (__pyx_t_5) {

    /* "psychxr/ovr/math.pyx":189
 *         cdef int s = self.length()
 *         if s != <int>0:
 *             s = <int>1 / s             # <<<<<<<<<<<<<<
 * 
 *         self *= s
 */
    if (unlikely(__pyx_v_s == 0)) {
      PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
      __PYX_ERR(0, 189, __pyx_L1_error)
    }
    else if (sizeof(int) == sizeof(long) && (!(((int)-1) > 0)) && unlikely(__pyx_v_s == (int)-1)  && unlikely(UNARY_NEG_WOULD_OVERFLOW(((int)1)))) {
      PyErr_SetString(PyExc_OverflowError, "value too large to perform division");
      __PYX_ERR(0, 189, __pyx_L1_error)
    }
    __pyx_v_s = __Pyx_div_int(((int)1), __pyx_v_s);

    /* "psychxr/ovr/math.pyx":188
 *     def normalize(self):
 *         cdef int s = self.length()
 *         if s != <int>0:             # <<<<<<<<<<<<<<
 *             s = <int>1 / s
 * 
 */
  }

  /* "psychxr/ovr/math.pyx":191
 *             s = <int>1 / s
 * 
 *         self *= s             # <<<<<<<<<<<<<<
 * 
 *     def normalized(self):
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_s); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 191, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyNumber_InPlaceMultiply(((PyObject *)__pyx_v_self), __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 191, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_7psychxr_3ovr_4math_ovrVector2i))))) __PYX_ERR(0, 191, __pyx_L1_error)
  __Pyx_DECREF_SET(__pyx_v_self, ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_t_2));
  __pyx_t_2 = 0;

  /* "psychxr/ovr/math.pyx":186
 *         return cmath.fabs(self.length_sq() - <int>1) < 0
 * 
 *     def normalize(self):             # <<<<<<<<<<<<<<
 *         cdef int s = self.length()
 *         if s != <int>0:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.normalize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_self);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":193
 *         self *= s
 * 
 *     def normalized(self):             # <<<<<<<<<<<<<<
 *         cdef int s = self.length()
 *         if s != <int>0:
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_61normalized(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_61normalized(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("normalized (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_60normalized(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_60normalized(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self) {
  int __pyx_v_s;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  __Pyx_RefNannySetupContext("normalized", 0);

  /* "psychxr/ovr/math.pyx":194
 * 
 *     def normalized(self):
 *         cdef int s = self.length()             # <<<<<<<<<<<<<<
 *         if s != <int>0:
 *             s = <int>1 / s
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_length); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 194, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 194, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 194, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_4 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 194, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_s = __pyx_t_4;

  /* "psychxr/ovr/math.pyx":195
 *     def normalized(self):
 *         cdef int s = self.length()
 *         if s != <int>0:             # <<<<<<<<<<<<<<
 *             s = <int>1 / s
 * 
 */
  __pyx_t_5 = ((__pyx_v_s != ((int)0)) != 0);
  if (__pyx_t_5) {

    /* "psychxr/ovr/math.pyx":196
 *         cdef int s = self.length()
 *         if s != <int>0:
 *             s = <int>1 / s             # <<<<<<<<<<<<<<
 * 
 *         return self * s
 */
    if (unlikely(__pyx_v_s == 0)) {
      PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
      __PYX_ERR(0, 196, __pyx_L1_error)
    }
    else if (sizeof(int) == sizeof(long) && (!(((int)-1) > 0)) && unlikely(__pyx_v_s == (int)-1)  && unlikely(UNARY_NEG_WOULD_OVERFLOW(((int)1)))) {
      PyErr_SetString(PyExc_OverflowError, "value too large to perform division");
      __PYX_ERR(0, 196, __pyx_L1_error)
    }
    __pyx_v_s = __Pyx_div_int(((int)1), __pyx_v_s);

    /* "psychxr/ovr/math.pyx":195
 *     def normalized(self):
 *         cdef int s = self.length()
 *         if s != <int>0:             # <<<<<<<<<<<<<<
 *             s = <int>1 / s
 * 
 */
  }

  /* "psychxr/ovr/math.pyx":198
 *             s = <int>1 / s
 * 
 *         return self * s             # <<<<<<<<<<<<<<
 * 
 *     def lerp(self, ovrVector2i b, int f):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_s); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 198, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyNumber_Multiply(((PyObject *)__pyx_v_self), __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 198, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":193
 *         self *= s
 * 
 *     def normalized(self):             # <<<<<<<<<<<<<<
 *         cdef int s = self.length()
 *         if s != <int>0:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.normalized", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":200
 *         return self * s
 * 
 *     def lerp(self, ovrVector2i b, int f):             # <<<<<<<<<<<<<<
 *         return self * (<int>1 - f) + b * f
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_63lerp(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_63lerp(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_b = 0;
  int __pyx_v_f;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("lerp (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_b,&__pyx_n_s_f,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_b)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_f)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("lerp", 1, 2, 2, 1); __PYX_ERR(0, 200, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "lerp") < 0)) __PYX_ERR(0, 200, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_b = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)values[0]);
    __pyx_v_f = __Pyx_PyInt_As_int(values[1]); if (unlikely((__pyx_v_f == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 200, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("lerp", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 200, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.lerp", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrVector2i, 1, "b", 0))) __PYX_ERR(0, 200, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_62lerp(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_self), __pyx_v_b, __pyx_v_f);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_62lerp(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_b, int __pyx_v_f) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("lerp", 0);

  /* "psychxr/ovr/math.pyx":201
 * 
 *     def lerp(self, ovrVector2i b, int f):
 *         return self * (<int>1 - f) + b * f             # <<<<<<<<<<<<<<
 * 
 *     def projectTo(self, ovrVector2i b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int((((int)1) - __pyx_v_f)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 201, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyNumber_Multiply(((PyObject *)__pyx_v_self), __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 201, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_f); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 201, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyNumber_Multiply(((PyObject *)__pyx_v_b), __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 201, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyNumber_Add(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 201, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":200
 *         return self * s
 * 
 *     def lerp(self, ovrVector2i b, int f):             # <<<<<<<<<<<<<<
 *         return self * (<int>1 - f) + b * f
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.lerp", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":203
 *         return self * (<int>1 - f) + b * f
 * 
 *     def projectTo(self, ovrVector2i b):             # <<<<<<<<<<<<<<
 *         cdef int l2 = self.length_sq()
 *         assert l2 != <int>0
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_65projectTo(PyObject *__pyx_v_self, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_65projectTo(PyObject *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("projectTo (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrVector2i, 1, "b", 0))) __PYX_ERR(0, 203, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_64projectTo(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_64projectTo(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_b) {
  int __pyx_v_l2;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("projectTo", 0);

  /* "psychxr/ovr/math.pyx":204
 * 
 *     def projectTo(self, ovrVector2i b):
 *         cdef int l2 = self.length_sq()             # <<<<<<<<<<<<<<
 *         assert l2 != <int>0
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_length_sq); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 204, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 204, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 204, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_4 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 204, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_l2 = __pyx_t_4;

  /* "psychxr/ovr/math.pyx":205
 *     def projectTo(self, ovrVector2i b):
 *         cdef int l2 = self.length_sq()
 *         assert l2 != <int>0             # <<<<<<<<<<<<<<
 * 
 *         return b * (self.dot(b) / l2)
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!((__pyx_v_l2 != ((int)0)) != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 205, __pyx_L1_error)
    }
  }
  #endif

  /* "psychxr/ovr/math.pyx":207
 *         assert l2 != <int>0
 * 
 *         return b * (self.dot(b) / l2)             # <<<<<<<<<<<<<<
 * 
 *     def isClockwise(self, ovrVector2i b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_dot); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 207, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, ((PyObject *)__pyx_v_b)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 207, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, ((PyObject *)__pyx_v_b)};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 207, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, ((PyObject *)__pyx_v_b)};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 207, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 207, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(((PyObject *)__pyx_v_b));
      __Pyx_GIVEREF(((PyObject *)__pyx_v_b));
      PyTuple_SET_ITEM(__pyx_t_5, 0+1, ((PyObject *)__pyx_v_b));
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 207, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_l2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 207, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = __Pyx_PyNumber_Divide(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 207, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyNumber_Multiply(((PyObject *)__pyx_v_b), __pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 207, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":203
 *         return self * (<int>1 - f) + b * f
 * 
 *     def projectTo(self, ovrVector2i b):             # <<<<<<<<<<<<<<
 *         cdef int l2 = self.length_sq()
 *         assert l2 != <int>0
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.projectTo", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":209
 *         return b * (self.dot(b) / l2)
 * 
 *     def isClockwise(self, ovrVector2i b):             # <<<<<<<<<<<<<<
 *         return (self.c_data.x * b.c_data.y - self.c_data.y * b.c_data.x) < 0
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_67isClockwise(PyObject *__pyx_v_self, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_67isClockwise(PyObject *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("isClockwise (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrVector2i, 1, "b", 0))) __PYX_ERR(0, 209, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_66isClockwise(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_66isClockwise(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("isClockwise", 0);

  /* "psychxr/ovr/math.pyx":210
 * 
 *     def isClockwise(self, ovrVector2i b):
 *         return (self.c_data.x * b.c_data.y - self.c_data.y * b.c_data.x) < 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong((((__pyx_v_self->c_data->x * __pyx_v_b->c_data->y) - (__pyx_v_self->c_data->y * __pyx_v_b->c_data->x)) < 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 210, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":209
 *         return b * (self.dot(b) / l2)
 * 
 *     def isClockwise(self, ovrVector2i b):             # <<<<<<<<<<<<<<
 *         return (self.c_data.x * b.c_data.y - self.c_data.y * b.c_data.x) < 0
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.isClockwise", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_69__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_69__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_68__reduce_cython__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_68__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_71__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_71__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_70__setstate_cython__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector2i_70__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector2i.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":214
 * 
 * cdef class ovrSizei:
 *     def __cinit__(self, int w=0, int h=0):             # <<<<<<<<<<<<<<
 *         self.c_data = &self.c_ovrSizei
 * 
 */

/* Python wrapper */
static int __pyx_pw_7psychxr_3ovr_4math_8ovrSizei_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_7psychxr_3ovr_4math_8ovrSizei_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_v_w;
  int __pyx_v_h;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_w,&__pyx_n_s_h,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_w);
          if (value) { values[0] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_h);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(0, 214, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    if (values[0]) {
      __pyx_v_w = __Pyx_PyInt_As_int(values[0]); if (unlikely((__pyx_v_w == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 214, __pyx_L3_error)
    } else {
      __pyx_v_w = ((int)0);
    }
    if (values[1]) {
      __pyx_v_h = __Pyx_PyInt_As_int(values[1]); if (unlikely((__pyx_v_h == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 214, __pyx_L3_error)
    } else {
      __pyx_v_h = ((int)0);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 0, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 214, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrSizei.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrSizei___cinit__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)__pyx_v_self), __pyx_v_w, __pyx_v_h);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7psychxr_3ovr_4math_8ovrSizei___cinit__(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_self, int __pyx_v_w, int __pyx_v_h) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "psychxr/ovr/math.pyx":215
 * cdef class ovrSizei:
 *     def __cinit__(self, int w=0, int h=0):
 *         self.c_data = &self.c_ovrSizei             # <<<<<<<<<<<<<<
 * 
 *         self.c_data.w = w
 */
  __pyx_v_self->c_data = (&__pyx_v_self->c_ovrSizei);

  /* "psychxr/ovr/math.pyx":217
 *         self.c_data = &self.c_ovrSizei
 * 
 *         self.c_data.w = w             # <<<<<<<<<<<<<<
 *         self.c_data.h = h
 * 
 */
  __pyx_v_self->c_data->w = __pyx_v_w;

  /* "psychxr/ovr/math.pyx":218
 * 
 *         self.c_data.w = w
 *         self.c_data.h = h             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __pyx_v_self->c_data->h = __pyx_v_h;

  /* "psychxr/ovr/math.pyx":214
 * 
 * cdef class ovrSizei:
 *     def __cinit__(self, int w=0, int h=0):             # <<<<<<<<<<<<<<
 *         self.c_data = &self.c_ovrSizei
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":221
 * 
 *     @property
 *     def w(self):             # <<<<<<<<<<<<<<
 *         return self.c_data.w
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_1w_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_1w_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrSizei_1w___get__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrSizei_1w___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "psychxr/ovr/math.pyx":222
 *     @property
 *     def w(self):
 *         return self.c_data.w             # <<<<<<<<<<<<<<
 * 
 *     @w.setter
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->c_data->w); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 222, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":221
 * 
 *     @property
 *     def w(self):             # <<<<<<<<<<<<<<
 *         return self.c_data.w
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrSizei.w.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":225
 * 
 *     @w.setter
 *     def w(self, int value):             # <<<<<<<<<<<<<<
 *         self.c_data.w = value
 * 
 */

/* Python wrapper */
static int __pyx_pw_7psychxr_3ovr_4math_8ovrSizei_1w_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_value); /*proto*/
static int __pyx_pw_7psychxr_3ovr_4math_8ovrSizei_1w_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_value) {
  int __pyx_v_value;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_value); {
    __pyx_v_value = __Pyx_PyInt_As_int(__pyx_arg_value); if (unlikely((__pyx_v_value == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 225, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrSizei.w.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrSizei_1w_2__set__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)__pyx_v_self), ((int)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7psychxr_3ovr_4math_8ovrSizei_1w_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_self, int __pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "psychxr/ovr/math.pyx":226
 *     @w.setter
 *     def w(self, int value):
 *         self.c_data.w = value             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __pyx_v_self->c_data->w = __pyx_v_value;

  /* "psychxr/ovr/math.pyx":225
 * 
 *     @w.setter
 *     def w(self, int value):             # <<<<<<<<<<<<<<
 *         self.c_data.w = value
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":229
 * 
 *     @property
 *     def h(self):             # <<<<<<<<<<<<<<
 *         return self.c_data.h
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_1h_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_1h_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrSizei_1h___get__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrSizei_1h___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "psychxr/ovr/math.pyx":230
 *     @property
 *     def h(self):
 *         return self.c_data.h             # <<<<<<<<<<<<<<
 * 
 *     @h.setter
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->c_data->h); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 230, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":229
 * 
 *     @property
 *     def h(self):             # <<<<<<<<<<<<<<
 *         return self.c_data.h
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrSizei.h.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":233
 * 
 *     @h.setter
 *     def h(self, int value):             # <<<<<<<<<<<<<<
 *         self.c_data.h = value
 * 
 */

/* Python wrapper */
static int __pyx_pw_7psychxr_3ovr_4math_8ovrSizei_1h_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_value); /*proto*/
static int __pyx_pw_7psychxr_3ovr_4math_8ovrSizei_1h_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_value) {
  int __pyx_v_value;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_value); {
    __pyx_v_value = __Pyx_PyInt_As_int(__pyx_arg_value); if (unlikely((__pyx_v_value == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 233, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrSizei.h.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrSizei_1h_2__set__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)__pyx_v_self), ((int)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7psychxr_3ovr_4math_8ovrSizei_1h_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_self, int __pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "psychxr/ovr/math.pyx":234
 *     @h.setter
 *     def h(self, int value):
 *         self.c_data.h = value             # <<<<<<<<<<<<<<
 * 
 *     def asTuple(self):
 */
  __pyx_v_self->c_data->h = __pyx_v_value;

  /* "psychxr/ovr/math.pyx":233
 * 
 *     @h.setter
 *     def h(self, int value):             # <<<<<<<<<<<<<<
 *         self.c_data.h = value
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":236
 *         self.c_data.h = value
 * 
 *     def asTuple(self):             # <<<<<<<<<<<<<<
 *         return self.c_data.w, self.c_data.h
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_3asTuple(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_3asTuple(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("asTuple (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrSizei_2asTuple(((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrSizei_2asTuple(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("asTuple", 0);

  /* "psychxr/ovr/math.pyx":237
 * 
 *     def asTuple(self):
 *         return self.c_data.w, self.c_data.h             # <<<<<<<<<<<<<<
 * 
 *     def asList(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->c_data->w); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 237, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->c_data->h); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 237, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 237, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":236
 *         self.c_data.h = value
 * 
 *     def asTuple(self):             # <<<<<<<<<<<<<<
 *         return self.c_data.w, self.c_data.h
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrSizei.asTuple", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":239
 *         return self.c_data.w, self.c_data.h
 * 
 *     def asList(self):             # <<<<<<<<<<<<<<
 *         return [self.c_data.w, self.c_data.h]
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_5asList(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_5asList(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("asList (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrSizei_4asList(((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrSizei_4asList(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("asList", 0);

  /* "psychxr/ovr/math.pyx":240
 * 
 *     def asList(self):
 *         return [self.c_data.w, self.c_data.h]             # <<<<<<<<<<<<<<
 * 
 *     def __eq__(self, ovrSizei b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->c_data->w); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 240, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->c_data->h); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 240, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyList_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 240, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyList_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyList_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":239
 *         return self.c_data.w, self.c_data.h
 * 
 *     def asList(self):             # <<<<<<<<<<<<<<
 *         return [self.c_data.w, self.c_data.h]
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrSizei.asList", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":242
 *         return [self.c_data.w, self.c_data.h]
 * 
 *     def __eq__(self, ovrSizei b):             # <<<<<<<<<<<<<<
 *         return self.c_data.w == b.c_data.w and self.c_data.h == b.c_data.h
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_7__eq__(PyObject *__pyx_v_self, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_7__eq__(PyObject *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__eq__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrSizei, 1, "b", 0))) __PYX_ERR(0, 242, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrSizei_6__eq__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrSizei_6__eq__(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("__eq__", 0);

  /* "psychxr/ovr/math.pyx":243
 * 
 *     def __eq__(self, ovrSizei b):
 *         return self.c_data.w == b.c_data.w and self.c_data.h == b.c_data.h             # <<<<<<<<<<<<<<
 * 
 *     def __ne__(self, ovrSizei b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = (__pyx_v_self->c_data->w == __pyx_v_b->c_data->w);
  if (__pyx_t_2) {
  } else {
    __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 243, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L3_bool_binop_done;
  }
  __pyx_t_2 = (__pyx_v_self->c_data->h == __pyx_v_b->c_data->h);
  __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 243, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __pyx_t_3;
  __pyx_t_3 = 0;
  __pyx_L3_bool_binop_done:;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":242
 *         return [self.c_data.w, self.c_data.h]
 * 
 *     def __eq__(self, ovrSizei b):             # <<<<<<<<<<<<<<
 *         return self.c_data.w == b.c_data.w and self.c_data.h == b.c_data.h
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrSizei.__eq__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":245
 *         return self.c_data.w == b.c_data.w and self.c_data.h == b.c_data.h
 * 
 *     def __ne__(self, ovrSizei b):             # <<<<<<<<<<<<<<
 *         return self.c_data.w != b.c_data.w or self.c_data.h != b.c_data.h
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_9__ne__(PyObject *__pyx_v_self, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_9__ne__(PyObject *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__ne__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrSizei, 1, "b", 0))) __PYX_ERR(0, 245, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrSizei_8__ne__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrSizei_8__ne__(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("__ne__", 0);

  /* "psychxr/ovr/math.pyx":246
 * 
 *     def __ne__(self, ovrSizei b):
 *         return self.c_data.w != b.c_data.w or self.c_data.h != b.c_data.h             # <<<<<<<<<<<<<<
 * 
 *     def __add__(ovrSizei a, ovrSizei b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = (__pyx_v_self->c_data->w != __pyx_v_b->c_data->w);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 246, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L3_bool_binop_done;
  }
  __pyx_t_2 = (__pyx_v_self->c_data->h != __pyx_v_b->c_data->h);
  __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 246, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __pyx_t_3;
  __pyx_t_3 = 0;
  __pyx_L3_bool_binop_done:;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":245
 *         return self.c_data.w == b.c_data.w and self.c_data.h == b.c_data.h
 * 
 *     def __ne__(self, ovrSizei b):             # <<<<<<<<<<<<<<
 *         return self.c_data.w != b.c_data.w or self.c_data.h != b.c_data.h
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrSizei.__ne__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":248
 *         return self.c_data.w != b.c_data.w or self.c_data.h != b.c_data.h
 * 
 *     def __add__(ovrSizei a, ovrSizei b):             # <<<<<<<<<<<<<<
 *         cdef ovrSizei to_return = ovrSizei(
 *             a.c_data.w + b.c_data.w,
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_11__add__(PyObject *__pyx_v_a, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_11__add__(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__add__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_a), __pyx_ptype_7psychxr_3ovr_4math_ovrSizei, 1, "a", 0))) __PYX_ERR(0, 248, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrSizei, 1, "b", 0))) __PYX_ERR(0, 248, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrSizei_10__add__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)__pyx_v_a), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrSizei_10__add__(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_a, struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_b) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("__add__", 0);

  /* "psychxr/ovr/math.pyx":250
 *     def __add__(ovrSizei a, ovrSizei b):
 *         cdef ovrSizei to_return = ovrSizei(
 *             a.c_data.w + b.c_data.w,             # <<<<<<<<<<<<<<
 *             a.c_data.h + b.c_data.h)
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_int((__pyx_v_a->c_data->w + __pyx_v_b->c_data->w)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 250, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);

  /* "psychxr/ovr/math.pyx":251
 *         cdef ovrSizei to_return = ovrSizei(
 *             a.c_data.w + b.c_data.w,
 *             a.c_data.h + b.c_data.h)             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
  __pyx_t_2 = __Pyx_PyInt_From_int((__pyx_v_a->c_data->h + __pyx_v_b->c_data->h)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 251, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "psychxr/ovr/math.pyx":249
 * 
 *     def __add__(ovrSizei a, ovrSizei b):
 *         cdef ovrSizei to_return = ovrSizei(             # <<<<<<<<<<<<<<
 *             a.c_data.w + b.c_data.w,
 *             a.c_data.h + b.c_data.h)
 */
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 249, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrSizei), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 249, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "psychxr/ovr/math.pyx":253
 *             a.c_data.h + b.c_data.h)
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def __iadd__(self, ovrSizei b):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":248
 *         return self.c_data.w != b.c_data.w or self.c_data.h != b.c_data.h
 * 
 *     def __add__(ovrSizei a, ovrSizei b):             # <<<<<<<<<<<<<<
 *         cdef ovrSizei to_return = ovrSizei(
 *             a.c_data.w + b.c_data.w,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrSizei.__add__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":255
 *         return to_return
 * 
 *     def __iadd__(self, ovrSizei b):             # <<<<<<<<<<<<<<
 *         self.c_data.w += b.c_data.w
 *         self.c_data.h += b.c_data.h
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_13__iadd__(PyObject *__pyx_v_self, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_13__iadd__(PyObject *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iadd__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrSizei, 1, "b", 0))) __PYX_ERR(0, 255, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrSizei_12__iadd__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrSizei_12__iadd__(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iadd__", 0);

  /* "psychxr/ovr/math.pyx":256
 * 
 *     def __iadd__(self, ovrSizei b):
 *         self.c_data.w += b.c_data.w             # <<<<<<<<<<<<<<
 *         self.c_data.h += b.c_data.h
 * 
 */
  __pyx_v_self->c_data->w = (__pyx_v_self->c_data->w + __pyx_v_b->c_data->w);

  /* "psychxr/ovr/math.pyx":257
 *     def __iadd__(self, ovrSizei b):
 *         self.c_data.w += b.c_data.w
 *         self.c_data.h += b.c_data.h             # <<<<<<<<<<<<<<
 * 
 *         return self
 */
  __pyx_v_self->c_data->h = (__pyx_v_self->c_data->h + __pyx_v_b->c_data->h);

  /* "psychxr/ovr/math.pyx":259
 *         self.c_data.h += b.c_data.h
 * 
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     def __sub__(ovrSizei a, ovrSizei b):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":255
 *         return to_return
 * 
 *     def __iadd__(self, ovrSizei b):             # <<<<<<<<<<<<<<
 *         self.c_data.w += b.c_data.w
 *         self.c_data.h += b.c_data.h
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":261
 *         return self
 * 
 *     def __sub__(ovrSizei a, ovrSizei b):             # <<<<<<<<<<<<<<
 *         cdef ovrSizei to_return = ovrSizei(
 *             a.c_data.w - b.c_data.w,
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_15__sub__(PyObject *__pyx_v_a, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_15__sub__(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__sub__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_a), __pyx_ptype_7psychxr_3ovr_4math_ovrSizei, 1, "a", 0))) __PYX_ERR(0, 261, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrSizei, 1, "b", 0))) __PYX_ERR(0, 261, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrSizei_14__sub__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)__pyx_v_a), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrSizei_14__sub__(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_a, struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_b) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("__sub__", 0);

  /* "psychxr/ovr/math.pyx":263
 *     def __sub__(ovrSizei a, ovrSizei b):
 *         cdef ovrSizei to_return = ovrSizei(
 *             a.c_data.w - b.c_data.w,             # <<<<<<<<<<<<<<
 *             a.c_data.h - b.c_data.h)
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_int((__pyx_v_a->c_data->w - __pyx_v_b->c_data->w)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 263, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);

  /* "psychxr/ovr/math.pyx":264
 *         cdef ovrSizei to_return = ovrSizei(
 *             a.c_data.w - b.c_data.w,
 *             a.c_data.h - b.c_data.h)             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
  __pyx_t_2 = __Pyx_PyInt_From_int((__pyx_v_a->c_data->h - __pyx_v_b->c_data->h)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 264, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "psychxr/ovr/math.pyx":262
 * 
 *     def __sub__(ovrSizei a, ovrSizei b):
 *         cdef ovrSizei to_return = ovrSizei(             # <<<<<<<<<<<<<<
 *             a.c_data.w - b.c_data.w,
 *             a.c_data.h - b.c_data.h)
 */
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 262, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrSizei), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 262, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "psychxr/ovr/math.pyx":266
 *             a.c_data.h - b.c_data.h)
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def __isub__(self, ovrSizei b):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":261
 *         return self
 * 
 *     def __sub__(ovrSizei a, ovrSizei b):             # <<<<<<<<<<<<<<
 *         cdef ovrSizei to_return = ovrSizei(
 *             a.c_data.w - b.c_data.w,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrSizei.__sub__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":268
 *         return to_return
 * 
 *     def __isub__(self, ovrSizei b):             # <<<<<<<<<<<<<<
 *         self.c_data.w -= b.c_data.w
 *         self.c_data.h -= b.c_data.h
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_17__isub__(PyObject *__pyx_v_self, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_17__isub__(PyObject *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__isub__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrSizei, 1, "b", 0))) __PYX_ERR(0, 268, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrSizei_16__isub__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrSizei_16__isub__(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__isub__", 0);

  /* "psychxr/ovr/math.pyx":269
 * 
 *     def __isub__(self, ovrSizei b):
 *         self.c_data.w -= b.c_data.w             # <<<<<<<<<<<<<<
 *         self.c_data.h -= b.c_data.h
 * 
 */
  __pyx_v_self->c_data->w = (__pyx_v_self->c_data->w - __pyx_v_b->c_data->w);

  /* "psychxr/ovr/math.pyx":270
 *     def __isub__(self, ovrSizei b):
 *         self.c_data.w -= b.c_data.w
 *         self.c_data.h -= b.c_data.h             # <<<<<<<<<<<<<<
 * 
 *         return self
 */
  __pyx_v_self->c_data->h = (__pyx_v_self->c_data->h - __pyx_v_b->c_data->h);

  /* "psychxr/ovr/math.pyx":272
 *         self.c_data.h -= b.c_data.h
 * 
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     def __neg__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":268
 *         return to_return
 * 
 *     def __isub__(self, ovrSizei b):             # <<<<<<<<<<<<<<
 *         self.c_data.w -= b.c_data.w
 *         self.c_data.h -= b.c_data.h
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":274
 *         return self
 * 
 *     def __neg__(self):             # <<<<<<<<<<<<<<
 *         cdef ovrSizei to_return = ovrSizei(-self.c_data.w, -self.c_data.h)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_19__neg__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_19__neg__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__neg__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrSizei_18__neg__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrSizei_18__neg__(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_self) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("__neg__", 0);

  /* "psychxr/ovr/math.pyx":275
 * 
 *     def __neg__(self):
 *         cdef ovrSizei to_return = ovrSizei(-self.c_data.w, -self.c_data.h)             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
  __pyx_t_1 = __Pyx_PyInt_From_int((-__pyx_v_self->c_data->w)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 275, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int((-__pyx_v_self->c_data->h)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 275, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 275, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrSizei), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 275, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "psychxr/ovr/math.pyx":277
 *         cdef ovrSizei to_return = ovrSizei(-self.c_data.w, -self.c_data.h)
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def __mul__(ovrSizei a, object b):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":274
 *         return self
 * 
 *     def __neg__(self):             # <<<<<<<<<<<<<<
 *         cdef ovrSizei to_return = ovrSizei(-self.c_data.w, -self.c_data.h)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrSizei.__neg__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":279
 *         return to_return
 * 
 *     def __mul__(ovrSizei a, object b):             # <<<<<<<<<<<<<<
 *         cdef ovrSizei to_return
 *         if isinstance(b, ovrSizei):
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_21__mul__(PyObject *__pyx_v_a, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_21__mul__(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__mul__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_a), __pyx_ptype_7psychxr_3ovr_4math_ovrSizei, 1, "a", 0))) __PYX_ERR(0, 279, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrSizei_20__mul__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)__pyx_v_a), ((PyObject *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrSizei_20__mul__(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_a, PyObject *__pyx_v_b) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  __Pyx_RefNannySetupContext("__mul__", 0);

  /* "psychxr/ovr/math.pyx":281
 *     def __mul__(ovrSizei a, object b):
 *         cdef ovrSizei to_return
 *         if isinstance(b, ovrSizei):             # <<<<<<<<<<<<<<
 *             to_return = ovrSizei(a.c_data.w * b.c_data.w,
 *                                  a.c_data.h * b.c_data.h)
 */
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_b, __pyx_ptype_7psychxr_3ovr_4math_ovrSizei); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "psychxr/ovr/math.pyx":282
 *         cdef ovrSizei to_return
 *         if isinstance(b, ovrSizei):
 *             to_return = ovrSizei(a.c_data.w * b.c_data.w,             # <<<<<<<<<<<<<<
 *                                  a.c_data.h * b.c_data.h)
 *         elif isinstance(b, (int, float)):
 */
    __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_a->c_data->w); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 282, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_b, __pyx_n_s_c_data); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 282, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_w); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 282, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = PyNumber_Multiply(__pyx_t_3, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 282, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "psychxr/ovr/math.pyx":283
 *         if isinstance(b, ovrSizei):
 *             to_return = ovrSizei(a.c_data.w * b.c_data.w,
 *                                  a.c_data.h * b.c_data.h)             # <<<<<<<<<<<<<<
 *         elif isinstance(b, (int, float)):
 *             to_return = ovrSizei(a.c_data.w * <int>b,
 */
    __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_a->c_data->h); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 283, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_b, __pyx_n_s_c_data); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 283, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_h); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 283, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyNumber_Multiply(__pyx_t_5, __pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 283, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "psychxr/ovr/math.pyx":282
 *         cdef ovrSizei to_return
 *         if isinstance(b, ovrSizei):
 *             to_return = ovrSizei(a.c_data.w * b.c_data.w,             # <<<<<<<<<<<<<<
 *                                  a.c_data.h * b.c_data.h)
 *         elif isinstance(b, (int, float)):
 */
    __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 282, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_3);
    __pyx_t_4 = 0;
    __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrSizei), __pyx_t_6, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 282, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "psychxr/ovr/math.pyx":281
 *     def __mul__(ovrSizei a, object b):
 *         cdef ovrSizei to_return
 *         if isinstance(b, ovrSizei):             # <<<<<<<<<<<<<<
 *             to_return = ovrSizei(a.c_data.w * b.c_data.w,
 *                                  a.c_data.h * b.c_data.h)
 */
    goto __pyx_L3;
  }

  /* "psychxr/ovr/math.pyx":284
 *             to_return = ovrSizei(a.c_data.w * b.c_data.w,
 *                                  a.c_data.h * b.c_data.h)
 *         elif isinstance(b, (int, float)):             # <<<<<<<<<<<<<<
 *             to_return = ovrSizei(a.c_data.w * <int>b,
 *                                  a.c_data.h * <int>b)
 */
  __pyx_t_1 = PyInt_Check(__pyx_v_b); 
  __pyx_t_7 = (__pyx_t_1 != 0);
  if (!__pyx_t_7) {
  } else {
    __pyx_t_2 = __pyx_t_7;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_7 = PyFloat_Check(__pyx_v_b); 
  __pyx_t_1 = (__pyx_t_7 != 0);
  __pyx_t_2 = __pyx_t_1;
  __pyx_L4_bool_binop_done:;
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {

    /* "psychxr/ovr/math.pyx":285
 *                                  a.c_data.h * b.c_data.h)
 *         elif isinstance(b, (int, float)):
 *             to_return = ovrSizei(a.c_data.w * <int>b,             # <<<<<<<<<<<<<<
 *                                  a.c_data.h * <int>b)
 * 
 */
    __pyx_t_8 = __Pyx_PyInt_As_int(__pyx_v_b); if (unlikely((__pyx_t_8 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 285, __pyx_L1_error)
    __pyx_t_3 = __Pyx_PyInt_From_int((__pyx_v_a->c_data->w * ((int)__pyx_t_8))); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 285, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);

    /* "psychxr/ovr/math.pyx":286
 *         elif isinstance(b, (int, float)):
 *             to_return = ovrSizei(a.c_data.w * <int>b,
 *                                  a.c_data.h * <int>b)             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
    __pyx_t_8 = __Pyx_PyInt_As_int(__pyx_v_b); if (unlikely((__pyx_t_8 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 286, __pyx_L1_error)
    __pyx_t_6 = __Pyx_PyInt_From_int((__pyx_v_a->c_data->h * ((int)__pyx_t_8))); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 286, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);

    /* "psychxr/ovr/math.pyx":285
 *                                  a.c_data.h * b.c_data.h)
 *         elif isinstance(b, (int, float)):
 *             to_return = ovrSizei(a.c_data.w * <int>b,             # <<<<<<<<<<<<<<
 *                                  a.c_data.h * <int>b)
 * 
 */
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 285, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_6);
    __pyx_t_3 = 0;
    __pyx_t_6 = 0;
    __pyx_t_6 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrSizei), __pyx_t_4, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 285, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)__pyx_t_6);
    __pyx_t_6 = 0;

    /* "psychxr/ovr/math.pyx":284
 *             to_return = ovrSizei(a.c_data.w * b.c_data.w,
 *                                  a.c_data.h * b.c_data.h)
 *         elif isinstance(b, (int, float)):             # <<<<<<<<<<<<<<
 *             to_return = ovrSizei(a.c_data.w * <int>b,
 *                                  a.c_data.h * <int>b)
 */
  }
  __pyx_L3:;

  /* "psychxr/ovr/math.pyx":288
 *                                  a.c_data.h * <int>b)
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def __imul__(self, object b):
 */
  __Pyx_XDECREF(__pyx_r);
  if (unlikely(!__pyx_v_to_return)) { __Pyx_RaiseUnboundLocalError("to_return"); __PYX_ERR(0, 288, __pyx_L1_error) }
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":279
 *         return to_return
 * 
 *     def __mul__(ovrSizei a, object b):             # <<<<<<<<<<<<<<
 *         cdef ovrSizei to_return
 *         if isinstance(b, ovrSizei):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrSizei.__mul__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":290
 *         return to_return
 * 
 *     def __imul__(self, object b):             # <<<<<<<<<<<<<<
 *         cdef ovrSizei to_return
 *         if isinstance(b, ovrSizei):
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_23__imul__(PyObject *__pyx_v_self, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_23__imul__(PyObject *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__imul__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrSizei_22__imul__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)__pyx_v_self), ((PyObject *)__pyx_v_b));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrSizei_22__imul__(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_t_7;
  __Pyx_RefNannySetupContext("__imul__", 0);

  /* "psychxr/ovr/math.pyx":292
 *     def __imul__(self, object b):
 *         cdef ovrSizei to_return
 *         if isinstance(b, ovrSizei):             # <<<<<<<<<<<<<<
 *             self.c_data.w *= b.c_data.w
 *             self.c_data.h *= b.c_data.h
 */
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_b, __pyx_ptype_7psychxr_3ovr_4math_ovrSizei); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "psychxr/ovr/math.pyx":293
 *         cdef ovrSizei to_return
 *         if isinstance(b, ovrSizei):
 *             self.c_data.w *= b.c_data.w             # <<<<<<<<<<<<<<
 *             self.c_data.h *= b.c_data.h
 *         elif isinstance(b, (int, float)):
 */
    __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_self->c_data->w); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 293, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_b, __pyx_n_s_c_data); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 293, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_w); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 293, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = PyNumber_InPlaceMultiply(__pyx_t_3, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 293, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_6 = __Pyx_PyInt_As_int(__pyx_t_4); if (unlikely((__pyx_t_6 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 293, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_self->c_data->w = __pyx_t_6;

    /* "psychxr/ovr/math.pyx":294
 *         if isinstance(b, ovrSizei):
 *             self.c_data.w *= b.c_data.w
 *             self.c_data.h *= b.c_data.h             # <<<<<<<<<<<<<<
 *         elif isinstance(b, (int, float)):
 *             self.c_data.w *= <int>b
 */
    __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_self->c_data->h); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 294, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_b, __pyx_n_s_c_data); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 294, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_h); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 294, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = PyNumber_InPlaceMultiply(__pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 294, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_6 = __Pyx_PyInt_As_int(__pyx_t_5); if (unlikely((__pyx_t_6 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 294, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_v_self->c_data->h = __pyx_t_6;

    /* "psychxr/ovr/math.pyx":292
 *     def __imul__(self, object b):
 *         cdef ovrSizei to_return
 *         if isinstance(b, ovrSizei):             # <<<<<<<<<<<<<<
 *             self.c_data.w *= b.c_data.w
 *             self.c_data.h *= b.c_data.h
 */
    goto __pyx_L3;
  }

  /* "psychxr/ovr/math.pyx":295
 *             self.c_data.w *= b.c_data.w
 *             self.c_data.h *= b.c_data.h
 *         elif isinstance(b, (int, float)):             # <<<<<<<<<<<<<<
 *             self.c_data.w *= <int>b
 *             self.c_data.h *= <int>b
 */
  __pyx_t_1 = PyInt_Check(__pyx_v_b); 
  __pyx_t_7 = (__pyx_t_1 != 0);
  if (!__pyx_t_7) {
  } else {
    __pyx_t_2 = __pyx_t_7;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_7 = PyFloat_Check(__pyx_v_b); 
  __pyx_t_1 = (__pyx_t_7 != 0);
  __pyx_t_2 = __pyx_t_1;
  __pyx_L4_bool_binop_done:;
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {

    /* "psychxr/ovr/math.pyx":296
 *             self.c_data.h *= b.c_data.h
 *         elif isinstance(b, (int, float)):
 *             self.c_data.w *= <int>b             # <<<<<<<<<<<<<<
 *             self.c_data.h *= <int>b
 * 
 */
    __pyx_t_6 = __Pyx_PyInt_As_int(__pyx_v_b); if (unlikely((__pyx_t_6 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 296, __pyx_L1_error)
    __pyx_v_self->c_data->w = (__pyx_v_self->c_data->w * ((int)__pyx_t_6));

    /* "psychxr/ovr/math.pyx":297
 *         elif isinstance(b, (int, float)):
 *             self.c_data.w *= <int>b
 *             self.c_data.h *= <int>b             # <<<<<<<<<<<<<<
 * 
 *         return self
 */
    __pyx_t_6 = __Pyx_PyInt_As_int(__pyx_v_b); if (unlikely((__pyx_t_6 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 297, __pyx_L1_error)
    __pyx_v_self->c_data->h = (__pyx_v_self->c_data->h * ((int)__pyx_t_6));

    /* "psychxr/ovr/math.pyx":295
 *             self.c_data.w *= b.c_data.w
 *             self.c_data.h *= b.c_data.h
 *         elif isinstance(b, (int, float)):             # <<<<<<<<<<<<<<
 *             self.c_data.w *= <int>b
 *             self.c_data.h *= <int>b
 */
  }
  __pyx_L3:;

  /* "psychxr/ovr/math.pyx":299
 *             self.c_data.h *= <int>b
 * 
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     def __truediv__(ovrSizei a, object b):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":290
 *         return to_return
 * 
 *     def __imul__(self, object b):             # <<<<<<<<<<<<<<
 *         cdef ovrSizei to_return
 *         if isinstance(b, ovrSizei):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrSizei.__imul__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":301
 *         return self
 * 
 *     def __truediv__(ovrSizei a, object b):             # <<<<<<<<<<<<<<
 *         cdef float rcp = <float>1 / <float>b
 *         cdef ovrSizei to_return = ovrSizei(
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_25__truediv__(PyObject *__pyx_v_a, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_25__truediv__(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__truediv__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_a), __pyx_ptype_7psychxr_3ovr_4math_ovrSizei, 1, "a", 0))) __PYX_ERR(0, 301, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrSizei_24__truediv__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)__pyx_v_a), ((PyObject *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrSizei_24__truediv__(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_a, PyObject *__pyx_v_b) {
  float __pyx_v_rcp;
  struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  float __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("__truediv__", 0);

  /* "psychxr/ovr/math.pyx":302
 * 
 *     def __truediv__(ovrSizei a, object b):
 *         cdef float rcp = <float>1 / <float>b             # <<<<<<<<<<<<<<
 *         cdef ovrSizei to_return = ovrSizei(
 *             <int>(<float>a.c_data.w * rcp),
 */
  __pyx_t_1 = __pyx_PyFloat_AsFloat(__pyx_v_b); if (unlikely((__pyx_t_1 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 302, __pyx_L1_error)
  if (unlikely(((float)__pyx_t_1) == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "float division");
    __PYX_ERR(0, 302, __pyx_L1_error)
  }
  __pyx_v_rcp = (((float)1) / ((float)__pyx_t_1));

  /* "psychxr/ovr/math.pyx":304
 *         cdef float rcp = <float>1 / <float>b
 *         cdef ovrSizei to_return = ovrSizei(
 *             <int>(<float>a.c_data.w * rcp),             # <<<<<<<<<<<<<<
 *             <int>(<float>a.c_data.h * rcp))
 * 
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(((int)(((float)__pyx_v_a->c_data->w) * __pyx_v_rcp))); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 304, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "psychxr/ovr/math.pyx":305
 *         cdef ovrSizei to_return = ovrSizei(
 *             <int>(<float>a.c_data.w * rcp),
 *             <int>(<float>a.c_data.h * rcp))             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
  __pyx_t_3 = __Pyx_PyInt_From_int(((int)(((float)__pyx_v_a->c_data->h) * __pyx_v_rcp))); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 305, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "psychxr/ovr/math.pyx":303
 *     def __truediv__(ovrSizei a, object b):
 *         cdef float rcp = <float>1 / <float>b
 *         cdef ovrSizei to_return = ovrSizei(             # <<<<<<<<<<<<<<
 *             <int>(<float>a.c_data.w * rcp),
 *             <int>(<float>a.c_data.h * rcp))
 */
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 303, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_3);
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrSizei), __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 303, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "psychxr/ovr/math.pyx":307
 *             <int>(<float>a.c_data.h * rcp))
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def __itruediv__(self, object b):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":301
 *         return self
 * 
 *     def __truediv__(ovrSizei a, object b):             # <<<<<<<<<<<<<<
 *         cdef float rcp = <float>1 / <float>b
 *         cdef ovrSizei to_return = ovrSizei(
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrSizei.__truediv__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":309
 *         return to_return
 * 
 *     def __itruediv__(self, object b):             # <<<<<<<<<<<<<<
 *         cdef float rcp = <float>1 / <float>b
 *         self.c_data.w = <int>(<float>self.c_data.w * rcp)
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_27__itruediv__(PyObject *__pyx_v_self, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_27__itruediv__(PyObject *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__itruediv__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrSizei_26__itruediv__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)__pyx_v_self), ((PyObject *)__pyx_v_b));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrSizei_26__itruediv__(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_self, PyObject *__pyx_v_b) {
  float __pyx_v_rcp;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  float __pyx_t_1;
  __Pyx_RefNannySetupContext("__itruediv__", 0);

  /* "psychxr/ovr/math.pyx":310
 * 
 *     def __itruediv__(self, object b):
 *         cdef float rcp = <float>1 / <float>b             # <<<<<<<<<<<<<<
 *         self.c_data.w = <int>(<float>self.c_data.w * rcp)
 *         self.c_data.h = <int>(<float>self.c_data.h * rcp)
 */
  __pyx_t_1 = __pyx_PyFloat_AsFloat(__pyx_v_b); if (unlikely((__pyx_t_1 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 310, __pyx_L1_error)
  if (unlikely(((float)__pyx_t_1) == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "float division");
    __PYX_ERR(0, 310, __pyx_L1_error)
  }
  __pyx_v_rcp = (((float)1) / ((float)__pyx_t_1));

  /* "psychxr/ovr/math.pyx":311
 *     def __itruediv__(self, object b):
 *         cdef float rcp = <float>1 / <float>b
 *         self.c_data.w = <int>(<float>self.c_data.w * rcp)             # <<<<<<<<<<<<<<
 *         self.c_data.h = <int>(<float>self.c_data.h * rcp)
 * 
 */
  __pyx_v_self->c_data->w = ((int)(((float)__pyx_v_self->c_data->w) * __pyx_v_rcp));

  /* "psychxr/ovr/math.pyx":312
 *         cdef float rcp = <float>1 / <float>b
 *         self.c_data.w = <int>(<float>self.c_data.w * rcp)
 *         self.c_data.h = <int>(<float>self.c_data.h * rcp)             # <<<<<<<<<<<<<<
 * 
 *         return self
 */
  __pyx_v_self->c_data->h = ((int)(((float)__pyx_v_self->c_data->h) * __pyx_v_rcp));

  /* "psychxr/ovr/math.pyx":314
 *         self.c_data.h = <int>(<float>self.c_data.h * rcp)
 * 
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":309
 *         return to_return
 * 
 *     def __itruediv__(self, object b):             # <<<<<<<<<<<<<<
 *         cdef float rcp = <float>1 / <float>b
 *         self.c_data.w = <int>(<float>self.c_data.w * rcp)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrSizei.__itruediv__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":317
 * 
 *     @staticmethod
 *     def min(ovrSizei a, ovrSizei b):             # <<<<<<<<<<<<<<
 *         cdef ovrSizei to_return = ovrSizei(
 *             a.c_data.w if a.c_data.w < b.c_data.w else b.c_data.w,
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_29min(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_7psychxr_3ovr_4math_8ovrSizei_29min = {"min", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_29min, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_29min(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_a = 0;
  struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_b = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("min (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_a,&__pyx_n_s_b,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_a)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_b)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("min", 1, 2, 2, 1); __PYX_ERR(0, 317, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "min") < 0)) __PYX_ERR(0, 317, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_a = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)values[0]);
    __pyx_v_b = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("min", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 317, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrSizei.min", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_a), __pyx_ptype_7psychxr_3ovr_4math_ovrSizei, 1, "a", 0))) __PYX_ERR(0, 317, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrSizei, 1, "b", 0))) __PYX_ERR(0, 317, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrSizei_28min(__pyx_v_a, __pyx_v_b);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrSizei_28min(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_a, struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_b) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("min", 0);

  /* "psychxr/ovr/math.pyx":319
 *     def min(ovrSizei a, ovrSizei b):
 *         cdef ovrSizei to_return = ovrSizei(
 *             a.c_data.w if a.c_data.w < b.c_data.w else b.c_data.w,             # <<<<<<<<<<<<<<
 *             a.c_data.h if a.c_data.h < b.c_data.h else b.c_data.h)
 * 
 */
  if (((__pyx_v_a->c_data->w < __pyx_v_b->c_data->w) != 0)) {
    __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_a->c_data->w); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 319, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __pyx_t_2;
    __pyx_t_2 = 0;
  } else {
    __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_b->c_data->w); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 319, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __pyx_t_2;
    __pyx_t_2 = 0;
  }

  /* "psychxr/ovr/math.pyx":320
 *         cdef ovrSizei to_return = ovrSizei(
 *             a.c_data.w if a.c_data.w < b.c_data.w else b.c_data.w,
 *             a.c_data.h if a.c_data.h < b.c_data.h else b.c_data.h)             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
  if (((__pyx_v_a->c_data->h < __pyx_v_b->c_data->h) != 0)) {
    __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_a->c_data->h); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 320, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __pyx_t_3;
    __pyx_t_3 = 0;
  } else {
    __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_b->c_data->h); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 320, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __pyx_t_3;
    __pyx_t_3 = 0;
  }

  /* "psychxr/ovr/math.pyx":318
 *     @staticmethod
 *     def min(ovrSizei a, ovrSizei b):
 *         cdef ovrSizei to_return = ovrSizei(             # <<<<<<<<<<<<<<
 *             a.c_data.w if a.c_data.w < b.c_data.w else b.c_data.w,
 *             a.c_data.h if a.c_data.h < b.c_data.h else b.c_data.h)
 */
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 318, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrSizei), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 318, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "psychxr/ovr/math.pyx":322
 *             a.c_data.h if a.c_data.h < b.c_data.h else b.c_data.h)
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":317
 * 
 *     @staticmethod
 *     def min(ovrSizei a, ovrSizei b):             # <<<<<<<<<<<<<<
 *         cdef ovrSizei to_return = ovrSizei(
 *             a.c_data.w if a.c_data.w < b.c_data.w else b.c_data.w,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrSizei.min", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":325
 * 
 *     @staticmethod
 *     def max(ovrSizei a, ovrSizei b):             # <<<<<<<<<<<<<<
 *         cdef ovrSizei to_return = ovrSizei(
 *             a.c_data.w if a.c_data.w > b.c_data.w else b.c_data.w,
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_31max(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_7psychxr_3ovr_4math_8ovrSizei_31max = {"max", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_31max, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_31max(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_a = 0;
  struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_b = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("max (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_a,&__pyx_n_s_b,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_a)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_b)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("max", 1, 2, 2, 1); __PYX_ERR(0, 325, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "max") < 0)) __PYX_ERR(0, 325, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_a = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)values[0]);
    __pyx_v_b = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("max", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 325, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrSizei.max", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_a), __pyx_ptype_7psychxr_3ovr_4math_ovrSizei, 1, "a", 0))) __PYX_ERR(0, 325, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrSizei, 1, "b", 0))) __PYX_ERR(0, 325, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrSizei_30max(__pyx_v_a, __pyx_v_b);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrSizei_30max(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_a, struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_b) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("max", 0);

  /* "psychxr/ovr/math.pyx":327
 *     def max(ovrSizei a, ovrSizei b):
 *         cdef ovrSizei to_return = ovrSizei(
 *             a.c_data.w if a.c_data.w > b.c_data.w else b.c_data.w,             # <<<<<<<<<<<<<<
 *             a.c_data.h if a.c_data.h > b.c_data.h else b.c_data.h)
 * 
 */
  if (((__pyx_v_a->c_data->w > __pyx_v_b->c_data->w) != 0)) {
    __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_a->c_data->w); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 327, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __pyx_t_2;
    __pyx_t_2 = 0;
  } else {
    __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_b->c_data->w); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 327, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __pyx_t_2;
    __pyx_t_2 = 0;
  }

  /* "psychxr/ovr/math.pyx":328
 *         cdef ovrSizei to_return = ovrSizei(
 *             a.c_data.w if a.c_data.w > b.c_data.w else b.c_data.w,
 *             a.c_data.h if a.c_data.h > b.c_data.h else b.c_data.h)             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
  if (((__pyx_v_a->c_data->h > __pyx_v_b->c_data->h) != 0)) {
    __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_a->c_data->h); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 328, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __pyx_t_3;
    __pyx_t_3 = 0;
  } else {
    __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_b->c_data->h); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 328, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __pyx_t_3;
    __pyx_t_3 = 0;
  }

  /* "psychxr/ovr/math.pyx":326
 *     @staticmethod
 *     def max(ovrSizei a, ovrSizei b):
 *         cdef ovrSizei to_return = ovrSizei(             # <<<<<<<<<<<<<<
 *             a.c_data.w if a.c_data.w > b.c_data.w else b.c_data.w,
 *             a.c_data.h if a.c_data.h > b.c_data.h else b.c_data.h)
 */
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 326, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrSizei), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 326, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "psychxr/ovr/math.pyx":330
 *             a.c_data.h if a.c_data.h > b.c_data.h else b.c_data.h)
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def area(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":325
 * 
 *     @staticmethod
 *     def max(ovrSizei a, ovrSizei b):             # <<<<<<<<<<<<<<
 *         cdef ovrSizei to_return = ovrSizei(
 *             a.c_data.w if a.c_data.w > b.c_data.w else b.c_data.w,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrSizei.max", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":332
 *         return to_return
 * 
 *     def area(self):             # <<<<<<<<<<<<<<
 *         return self.c_data.w * self.c_data.h
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_33area(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_33area(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("area (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrSizei_32area(((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrSizei_32area(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("area", 0);

  /* "psychxr/ovr/math.pyx":333
 * 
 *     def area(self):
 *         return self.c_data.w * self.c_data.h             # <<<<<<<<<<<<<<
 * 
 *     def toVector(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int((__pyx_v_self->c_data->w * __pyx_v_self->c_data->h)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 333, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":332
 *         return to_return
 * 
 *     def area(self):             # <<<<<<<<<<<<<<
 *         return self.c_data.w * self.c_data.h
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrSizei.area", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":335
 *         return self.c_data.w * self.c_data.h
 * 
 *     def toVector(self):             # <<<<<<<<<<<<<<
 *         cdef ovrVector2i to_return = ovrVector2i(self.c_data.w, self.c_data.h)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_35toVector(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_35toVector(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("toVector (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrSizei_34toVector(((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrSizei_34toVector(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_self) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("toVector", 0);

  /* "psychxr/ovr/math.pyx":336
 * 
 *     def toVector(self):
 *         cdef ovrVector2i to_return = ovrVector2i(self.c_data.w, self.c_data.h)             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->c_data->w); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 336, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->c_data->h); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 336, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 336, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector2i), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 336, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "psychxr/ovr/math.pyx":338
 *         cdef ovrVector2i to_return = ovrVector2i(self.c_data.w, self.c_data.h)
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def asVector(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":335
 *         return self.c_data.w * self.c_data.h
 * 
 *     def toVector(self):             # <<<<<<<<<<<<<<
 *         cdef ovrVector2i to_return = ovrVector2i(self.c_data.w, self.c_data.h)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrSizei.toVector", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":340
 *         return to_return
 * 
 *     def asVector(self):             # <<<<<<<<<<<<<<
 *         return self.to_vector()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_37asVector(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_37asVector(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("asVector (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrSizei_36asVector(((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrSizei_36asVector(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("asVector", 0);

  /* "psychxr/ovr/math.pyx":341
 * 
 *     def asVector(self):
 *         return self.to_vector()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_to_vector); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 341, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 341, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 341, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":340
 *         return to_return
 * 
 *     def asVector(self):             # <<<<<<<<<<<<<<
 *         return self.to_vector()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrSizei.asVector", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_39__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_39__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrSizei_38__reduce_cython__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrSizei_38__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrSizei.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_41__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_41__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrSizei_40__setstate_cython__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrSizei_40__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrSizei.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":346
 * cdef class ovrRecti:
 * 
 *     def __init__(self, *args, **kwargs):             # <<<<<<<<<<<<<<
 *         pass
 * 
 */

/* Python wrapper */
static int __pyx_pw_7psychxr_3ovr_4math_8ovrRecti_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_7psychxr_3ovr_4math_8ovrRecti_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  CYTHON_UNUSED PyObject *__pyx_v_args = 0;
  CYTHON_UNUSED PyObject *__pyx_v_kwargs = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  if (unlikely(__pyx_kwds) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__init__", 1))) return -1;
  __Pyx_INCREF(__pyx_args);
  __pyx_v_args = __pyx_args;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrRecti___init__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *)__pyx_v_self), __pyx_v_args, __pyx_v_kwargs);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_args);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7psychxr_3ovr_4math_8ovrRecti___init__(CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_args, CYTHON_UNUSED PyObject *__pyx_v_kwargs) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__", 0);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":349
 *         pass
 * 
 *     def __cinit__(self, *args, **kwargs):             # <<<<<<<<<<<<<<
 *         self.c_data = &self.c_ovrRecti
 * 
 */

/* Python wrapper */
static int __pyx_pw_7psychxr_3ovr_4math_8ovrRecti_3__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_7psychxr_3ovr_4math_8ovrRecti_3__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_args = 0;
  CYTHON_UNUSED PyObject *__pyx_v_kwargs = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  if (unlikely(__pyx_kwds) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__cinit__", 1))) return -1;
  __Pyx_INCREF(__pyx_args);
  __pyx_v_args = __pyx_args;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrRecti_2__cinit__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *)__pyx_v_self), __pyx_v_args, __pyx_v_kwargs);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_args);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7psychxr_3ovr_4math_8ovrRecti_2__cinit__(struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_self, PyObject *__pyx_v_args, CYTHON_UNUSED PyObject *__pyx_v_kwargs) {
  int __pyx_v_nargin;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "psychxr/ovr/math.pyx":350
 * 
 *     def __cinit__(self, *args, **kwargs):
 *         self.c_data = &self.c_ovrRecti             # <<<<<<<<<<<<<<
 * 
 *         cdef int nargin = <int>len(args)  # get number of arguments
 */
  __pyx_v_self->c_data = (&__pyx_v_self->c_ovrRecti);

  /* "psychxr/ovr/math.pyx":352
 *         self.c_data = &self.c_ovrRecti
 * 
 *         cdef int nargin = <int>len(args)  # get number of arguments             # <<<<<<<<<<<<<<
 *         if nargin == 0:
 *             self.c_data.Pos.x = 0
 */
  __pyx_t_1 = PyTuple_GET_SIZE(__pyx_v_args); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 352, __pyx_L1_error)
  __pyx_v_nargin = ((int)__pyx_t_1);

  /* "psychxr/ovr/math.pyx":353
 * 
 *         cdef int nargin = <int>len(args)  # get number of arguments
 *         if nargin == 0:             # <<<<<<<<<<<<<<
 *             self.c_data.Pos.x = 0
 *             self.c_data.Pos.y = 0
 */
  __pyx_t_2 = ((__pyx_v_nargin == 0) != 0);
  if (__pyx_t_2) {

    /* "psychxr/ovr/math.pyx":354
 *         cdef int nargin = <int>len(args)  # get number of arguments
 *         if nargin == 0:
 *             self.c_data.Pos.x = 0             # <<<<<<<<<<<<<<
 *             self.c_data.Pos.y = 0
 *             self.c_data.Size.w = 0
 */
    __pyx_v_self->c_data->Pos.x = 0;

    /* "psychxr/ovr/math.pyx":355
 *         if nargin == 0:
 *             self.c_data.Pos.x = 0
 *             self.c_data.Pos.y = 0             # <<<<<<<<<<<<<<
 *             self.c_data.Size.w = 0
 *             self.c_data.Size.h = 0
 */
    __pyx_v_self->c_data->Pos.y = 0;

    /* "psychxr/ovr/math.pyx":356
 *             self.c_data.Pos.x = 0
 *             self.c_data.Pos.y = 0
 *             self.c_data.Size.w = 0             # <<<<<<<<<<<<<<
 *             self.c_data.Size.h = 0
 *         elif nargin == 4:
 */
    __pyx_v_self->c_data->Size.w = 0;

    /* "psychxr/ovr/math.pyx":357
 *             self.c_data.Pos.y = 0
 *             self.c_data.Size.w = 0
 *             self.c_data.Size.h = 0             # <<<<<<<<<<<<<<
 *         elif nargin == 4:
 *             self.c_data.Pos.x = args[0]
 */
    __pyx_v_self->c_data->Size.h = 0;

    /* "psychxr/ovr/math.pyx":353
 * 
 *         cdef int nargin = <int>len(args)  # get number of arguments
 *         if nargin == 0:             # <<<<<<<<<<<<<<
 *             self.c_data.Pos.x = 0
 *             self.c_data.Pos.y = 0
 */
    goto __pyx_L3;
  }

  /* "psychxr/ovr/math.pyx":358
 *             self.c_data.Size.w = 0
 *             self.c_data.Size.h = 0
 *         elif nargin == 4:             # <<<<<<<<<<<<<<
 *             self.c_data.Pos.x = args[0]
 *             self.c_data.Pos.y = args[1]
 */
  __pyx_t_2 = ((__pyx_v_nargin == 4) != 0);
  if (__pyx_t_2) {

    /* "psychxr/ovr/math.pyx":359
 *             self.c_data.Size.h = 0
 *         elif nargin == 4:
 *             self.c_data.Pos.x = args[0]             # <<<<<<<<<<<<<<
 *             self.c_data.Pos.y = args[1]
 *             self.c_data.Size.w = args[2]
 */
    __pyx_t_3 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 359, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_4 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 359, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_self->c_data->Pos.x = __pyx_t_4;

    /* "psychxr/ovr/math.pyx":360
 *         elif nargin == 4:
 *             self.c_data.Pos.x = args[0]
 *             self.c_data.Pos.y = args[1]             # <<<<<<<<<<<<<<
 *             self.c_data.Size.w = args[2]
 *             self.c_data.Size.h = args[3]
 */
    __pyx_t_3 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 360, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_4 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 360, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_self->c_data->Pos.y = __pyx_t_4;

    /* "psychxr/ovr/math.pyx":361
 *             self.c_data.Pos.x = args[0]
 *             self.c_data.Pos.y = args[1]
 *             self.c_data.Size.w = args[2]             # <<<<<<<<<<<<<<
 *             self.c_data.Size.h = args[3]
 *         elif nargin == 2 and \
 */
    __pyx_t_3 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 361, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_4 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 361, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_self->c_data->Size.w = __pyx_t_4;

    /* "psychxr/ovr/math.pyx":362
 *             self.c_data.Pos.y = args[1]
 *             self.c_data.Size.w = args[2]
 *             self.c_data.Size.h = args[3]             # <<<<<<<<<<<<<<
 *         elif nargin == 2 and \
 *                 isinstance(args[0], ovrVector2i) and \
 */
    __pyx_t_3 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 362, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_4 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 362, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_self->c_data->Size.h = __pyx_t_4;

    /* "psychxr/ovr/math.pyx":358
 *             self.c_data.Size.w = 0
 *             self.c_data.Size.h = 0
 *         elif nargin == 4:             # <<<<<<<<<<<<<<
 *             self.c_data.Pos.x = args[0]
 *             self.c_data.Pos.y = args[1]
 */
    goto __pyx_L3;
  }

  /* "psychxr/ovr/math.pyx":363
 *             self.c_data.Size.w = args[2]
 *             self.c_data.Size.h = args[3]
 *         elif nargin == 2 and \             # <<<<<<<<<<<<<<
 *                 isinstance(args[0], ovrVector2i) and \
 *                 isinstance(args[1], ovrSizei):
 */
  __pyx_t_5 = ((__pyx_v_nargin == 2) != 0);
  if (__pyx_t_5) {
  } else {
    __pyx_t_2 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }

  /* "psychxr/ovr/math.pyx":364
 *             self.c_data.Size.h = args[3]
 *         elif nargin == 2 and \
 *                 isinstance(args[0], ovrVector2i) and \             # <<<<<<<<<<<<<<
 *                 isinstance(args[1], ovrSizei):
 *             self.c_data.Pos.x = (<ovrVector2i>args[0]).c_data.x
 */
  __pyx_t_3 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 364, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_TypeCheck(__pyx_t_3, __pyx_ptype_7psychxr_3ovr_4math_ovrVector2i); 
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_2 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }

  /* "psychxr/ovr/math.pyx":365
 *         elif nargin == 2 and \
 *                 isinstance(args[0], ovrVector2i) and \
 *                 isinstance(args[1], ovrSizei):             # <<<<<<<<<<<<<<
 *             self.c_data.Pos.x = (<ovrVector2i>args[0]).c_data.x
 *             self.c_data.Pos.y = (<ovrVector2i>args[0]).c_data.y
 */
  __pyx_t_3 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 365, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_6 = __Pyx_TypeCheck(__pyx_t_3, __pyx_ptype_7psychxr_3ovr_4math_ovrSizei); 
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = (__pyx_t_6 != 0);
  __pyx_t_2 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;

  /* "psychxr/ovr/math.pyx":363
 *             self.c_data.Size.w = args[2]
 *             self.c_data.Size.h = args[3]
 *         elif nargin == 2 and \             # <<<<<<<<<<<<<<
 *                 isinstance(args[0], ovrVector2i) and \
 *                 isinstance(args[1], ovrSizei):
 */
  if (__pyx_t_2) {

    /* "psychxr/ovr/math.pyx":366
 *                 isinstance(args[0], ovrVector2i) and \
 *                 isinstance(args[1], ovrSizei):
 *             self.c_data.Pos.x = (<ovrVector2i>args[0]).c_data.x             # <<<<<<<<<<<<<<
 *             self.c_data.Pos.y = (<ovrVector2i>args[0]).c_data.y
 *             self.c_data.Size.w = (<ovrSizei>args[1]).c_data.w
 */
    __pyx_t_3 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 366, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_t_3)->c_data->x;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_self->c_data->Pos.x = __pyx_t_4;

    /* "psychxr/ovr/math.pyx":367
 *                 isinstance(args[1], ovrSizei):
 *             self.c_data.Pos.x = (<ovrVector2i>args[0]).c_data.x
 *             self.c_data.Pos.y = (<ovrVector2i>args[0]).c_data.y             # <<<<<<<<<<<<<<
 *             self.c_data.Size.w = (<ovrSizei>args[1]).c_data.w
 *             self.c_data.Size.h = (<ovrSizei>args[1]).c_data.h
 */
    __pyx_t_3 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 367, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_t_3)->c_data->y;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_self->c_data->Pos.y = __pyx_t_4;

    /* "psychxr/ovr/math.pyx":368
 *             self.c_data.Pos.x = (<ovrVector2i>args[0]).c_data.x
 *             self.c_data.Pos.y = (<ovrVector2i>args[0]).c_data.y
 *             self.c_data.Size.w = (<ovrSizei>args[1]).c_data.w             # <<<<<<<<<<<<<<
 *             self.c_data.Size.h = (<ovrSizei>args[1]).c_data.h
 * 
 */
    __pyx_t_3 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 368, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)__pyx_t_3)->c_data->w;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_self->c_data->Size.w = __pyx_t_4;

    /* "psychxr/ovr/math.pyx":369
 *             self.c_data.Pos.y = (<ovrVector2i>args[0]).c_data.y
 *             self.c_data.Size.w = (<ovrSizei>args[1]).c_data.w
 *             self.c_data.Size.h = (<ovrSizei>args[1]).c_data.h             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
    __pyx_t_3 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 369, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)__pyx_t_3)->c_data->h;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_self->c_data->Size.h = __pyx_t_4;

    /* "psychxr/ovr/math.pyx":363
 *             self.c_data.Size.w = args[2]
 *             self.c_data.Size.h = args[3]
 *         elif nargin == 2 and \             # <<<<<<<<<<<<<<
 *                 isinstance(args[0], ovrVector2i) and \
 *                 isinstance(args[1], ovrSizei):
 */
  }
  __pyx_L3:;

  /* "psychxr/ovr/math.pyx":349
 *         pass
 * 
 *     def __cinit__(self, *args, **kwargs):             # <<<<<<<<<<<<<<
 *         self.c_data = &self.c_ovrRecti
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrRecti.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":372
 * 
 *     @property
 *     def x(self):             # <<<<<<<<<<<<<<
 *         return self.c_data.Pos.x
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrRecti_1x_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrRecti_1x_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrRecti_1x___get__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrRecti_1x___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "psychxr/ovr/math.pyx":373
 *     @property
 *     def x(self):
 *         return self.c_data.Pos.x             # <<<<<<<<<<<<<<
 * 
 *     @x.setter
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->c_data->Pos.x); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 373, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":372
 * 
 *     @property
 *     def x(self):             # <<<<<<<<<<<<<<
 *         return self.c_data.Pos.x
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrRecti.x.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":376
 * 
 *     @x.setter
 *     def x(self, int value):             # <<<<<<<<<<<<<<
 *         self.c_data.Pos.x = value
 * 
 */

/* Python wrapper */
static int __pyx_pw_7psychxr_3ovr_4math_8ovrRecti_1x_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_value); /*proto*/
static int __pyx_pw_7psychxr_3ovr_4math_8ovrRecti_1x_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_value) {
  int __pyx_v_value;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_value); {
    __pyx_v_value = __Pyx_PyInt_As_int(__pyx_arg_value); if (unlikely((__pyx_v_value == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 376, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrRecti.x.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrRecti_1x_2__set__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *)__pyx_v_self), ((int)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7psychxr_3ovr_4math_8ovrRecti_1x_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_self, int __pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "psychxr/ovr/math.pyx":377
 *     @x.setter
 *     def x(self, int value):
 *         self.c_data.Pos.x = value             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __pyx_v_self->c_data->Pos.x = __pyx_v_value;

  /* "psychxr/ovr/math.pyx":376
 * 
 *     @x.setter
 *     def x(self, int value):             # <<<<<<<<<<<<<<
 *         self.c_data.Pos.x = value
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":380
 * 
 *     @property
 *     def y(self):             # <<<<<<<<<<<<<<
 *         return self.c_data.Pos.y
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrRecti_1y_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrRecti_1y_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrRecti_1y___get__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrRecti_1y___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "psychxr/ovr/math.pyx":381
 *     @property
 *     def y(self):
 *         return self.c_data.Pos.y             # <<<<<<<<<<<<<<
 * 
 *     @y.setter
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->c_data->Pos.y); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 381, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":380
 * 
 *     @property
 *     def y(self):             # <<<<<<<<<<<<<<
 *         return self.c_data.Pos.y
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrRecti.y.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":384
 * 
 *     @y.setter
 *     def y(self, int value):             # <<<<<<<<<<<<<<
 *         self.c_data.Pos.y = value
 * 
 */

/* Python wrapper */
static int __pyx_pw_7psychxr_3ovr_4math_8ovrRecti_1y_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_value); /*proto*/
static int __pyx_pw_7psychxr_3ovr_4math_8ovrRecti_1y_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_value) {
  int __pyx_v_value;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_value); {
    __pyx_v_value = __Pyx_PyInt_As_int(__pyx_arg_value); if (unlikely((__pyx_v_value == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 384, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrRecti.y.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrRecti_1y_2__set__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *)__pyx_v_self), ((int)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7psychxr_3ovr_4math_8ovrRecti_1y_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_self, int __pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "psychxr/ovr/math.pyx":385
 *     @y.setter
 *     def y(self, int value):
 *         self.c_data.Pos.y = value             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __pyx_v_self->c_data->Pos.y = __pyx_v_value;

  /* "psychxr/ovr/math.pyx":384
 * 
 *     @y.setter
 *     def y(self, int value):             # <<<<<<<<<<<<<<
 *         self.c_data.Pos.y = value
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":388
 * 
 *     @property
 *     def w(self):             # <<<<<<<<<<<<<<
 *         return self.c_data.Size.w
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrRecti_1w_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrRecti_1w_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrRecti_1w___get__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrRecti_1w___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "psychxr/ovr/math.pyx":389
 *     @property
 *     def w(self):
 *         return self.c_data.Size.w             # <<<<<<<<<<<<<<
 * 
 *     @w.setter
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->c_data->Size.w); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 389, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":388
 * 
 *     @property
 *     def w(self):             # <<<<<<<<<<<<<<
 *         return self.c_data.Size.w
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrRecti.w.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":392
 * 
 *     @w.setter
 *     def w(self, int value):             # <<<<<<<<<<<<<<
 *         self.c_data.Size.w = value
 * 
 */

/* Python wrapper */
static int __pyx_pw_7psychxr_3ovr_4math_8ovrRecti_1w_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_value); /*proto*/
static int __pyx_pw_7psychxr_3ovr_4math_8ovrRecti_1w_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_value) {
  int __pyx_v_value;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_value); {
    __pyx_v_value = __Pyx_PyInt_As_int(__pyx_arg_value); if (unlikely((__pyx_v_value == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 392, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrRecti.w.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrRecti_1w_2__set__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *)__pyx_v_self), ((int)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7psychxr_3ovr_4math_8ovrRecti_1w_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_self, int __pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "psychxr/ovr/math.pyx":393
 *     @w.setter
 *     def w(self, int value):
 *         self.c_data.Size.w = value             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __pyx_v_self->c_data->Size.w = __pyx_v_value;

  /* "psychxr/ovr/math.pyx":392
 * 
 *     @w.setter
 *     def w(self, int value):             # <<<<<<<<<<<<<<
 *         self.c_data.Size.w = value
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":396
 * 
 *     @property
 *     def h(self):             # <<<<<<<<<<<<<<
 *         return self.c_data.Size.h
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrRecti_1h_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrRecti_1h_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrRecti_1h___get__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrRecti_1h___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "psychxr/ovr/math.pyx":397
 *     @property
 *     def h(self):
 *         return self.c_data.Size.h             # <<<<<<<<<<<<<<
 * 
 *     @h.setter
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->c_data->Size.h); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 397, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":396
 * 
 *     @property
 *     def h(self):             # <<<<<<<<<<<<<<
 *         return self.c_data.Size.h
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrRecti.h.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":400
 * 
 *     @h.setter
 *     def h(self, int value):             # <<<<<<<<<<<<<<
 *         self.c_data.Size.h = value
 * 
 */

/* Python wrapper */
static int __pyx_pw_7psychxr_3ovr_4math_8ovrRecti_1h_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_value); /*proto*/
static int __pyx_pw_7psychxr_3ovr_4math_8ovrRecti_1h_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_value) {
  int __pyx_v_value;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_value); {
    __pyx_v_value = __Pyx_PyInt_As_int(__pyx_arg_value); if (unlikely((__pyx_v_value == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 400, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrRecti.h.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrRecti_1h_2__set__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *)__pyx_v_self), ((int)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7psychxr_3ovr_4math_8ovrRecti_1h_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_self, int __pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "psychxr/ovr/math.pyx":401
 *     @h.setter
 *     def h(self, int value):
 *         self.c_data.Size.h = value             # <<<<<<<<<<<<<<
 * 
 *     def asTuple(self):
 */
  __pyx_v_self->c_data->Size.h = __pyx_v_value;

  /* "psychxr/ovr/math.pyx":400
 * 
 *     @h.setter
 *     def h(self, int value):             # <<<<<<<<<<<<<<
 *         self.c_data.Size.h = value
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":403
 *         self.c_data.Size.h = value
 * 
 *     def asTuple(self):             # <<<<<<<<<<<<<<
 *         return self.c_data.Pos.x, self.c_data.Pos.y, \
 *                self.c_data.Size.w, self.c_data.Size.h
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrRecti_5asTuple(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrRecti_5asTuple(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("asTuple (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrRecti_4asTuple(((struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrRecti_4asTuple(struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("asTuple", 0);

  /* "psychxr/ovr/math.pyx":404
 * 
 *     def asTuple(self):
 *         return self.c_data.Pos.x, self.c_data.Pos.y, \             # <<<<<<<<<<<<<<
 *                self.c_data.Size.w, self.c_data.Size.h
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->c_data->Pos.x); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 404, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->c_data->Pos.y); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 404, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "psychxr/ovr/math.pyx":405
 *     def asTuple(self):
 *         return self.c_data.Pos.x, self.c_data.Pos.y, \
 *                self.c_data.Size.w, self.c_data.Size.h             # <<<<<<<<<<<<<<
 * 
 *     def asList(self):
 */
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_self->c_data->Size.w); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 405, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_self->c_data->Size.h); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 405, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);

  /* "psychxr/ovr/math.pyx":404
 * 
 *     def asTuple(self):
 *         return self.c_data.Pos.x, self.c_data.Pos.y, \             # <<<<<<<<<<<<<<
 *                self.c_data.Size.w, self.c_data.Size.h
 * 
 */
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 404, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":403
 *         self.c_data.Size.h = value
 * 
 *     def asTuple(self):             # <<<<<<<<<<<<<<
 *         return self.c_data.Pos.x, self.c_data.Pos.y, \
 *                self.c_data.Size.w, self.c_data.Size.h
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrRecti.asTuple", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":407
 *                self.c_data.Size.w, self.c_data.Size.h
 * 
 *     def asList(self):             # <<<<<<<<<<<<<<
 *         return [self.c_data.Pos.x, self.c_data.Pos.y,
 *                 self.c_data.Size.w, self.c_data.Size.h]
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrRecti_7asList(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrRecti_7asList(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("asList (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrRecti_6asList(((struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrRecti_6asList(struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("asList", 0);

  /* "psychxr/ovr/math.pyx":408
 * 
 *     def asList(self):
 *         return [self.c_data.Pos.x, self.c_data.Pos.y,             # <<<<<<<<<<<<<<
 *                 self.c_data.Size.w, self.c_data.Size.h]
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->c_data->Pos.x); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 408, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->c_data->Pos.y); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 408, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "psychxr/ovr/math.pyx":409
 *     def asList(self):
 *         return [self.c_data.Pos.x, self.c_data.Pos.y,
 *                 self.c_data.Size.w, self.c_data.Size.h]             # <<<<<<<<<<<<<<
 * 
 *     def __len__(self):
 */
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_self->c_data->Size.w); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 409, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_self->c_data->Size.h); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 409, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);

  /* "psychxr/ovr/math.pyx":408
 * 
 *     def asList(self):
 *         return [self.c_data.Pos.x, self.c_data.Pos.y,             # <<<<<<<<<<<<<<
 *                 self.c_data.Size.w, self.c_data.Size.h]
 * 
 */
  __pyx_t_5 = PyList_New(4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 408, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyList_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyList_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyList_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyList_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":407
 *                self.c_data.Size.w, self.c_data.Size.h
 * 
 *     def asList(self):             # <<<<<<<<<<<<<<
 *         return [self.c_data.Pos.x, self.c_data.Pos.y,
 *                 self.c_data.Size.w, self.c_data.Size.h]
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrRecti.asList", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":411
 *                 self.c_data.Size.w, self.c_data.Size.h]
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         return 4
 * 
 */

/* Python wrapper */
static Py_ssize_t __pyx_pw_7psychxr_3ovr_4math_8ovrRecti_9__len__(PyObject *__pyx_v_self); /*proto*/
static Py_ssize_t __pyx_pw_7psychxr_3ovr_4math_8ovrRecti_9__len__(PyObject *__pyx_v_self) {
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__len__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrRecti_8__len__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static Py_ssize_t __pyx_pf_7psychxr_3ovr_4math_8ovrRecti_8__len__(CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_self) {
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__len__", 0);

  /* "psychxr/ovr/math.pyx":412
 * 
 *     def __len__(self):
 *         return 4             # <<<<<<<<<<<<<<
 * 
 *     def getPos(self):
 */
  __pyx_r = 4;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":411
 *                 self.c_data.Size.w, self.c_data.Size.h]
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         return 4
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":414
 *         return 4
 * 
 *     def getPos(self):             # <<<<<<<<<<<<<<
 *         cdef ovrVector2i to_return = ovrVector2i(self.c_data.Pos.x,
 *                                                  self.c_data.Pos.y)
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrRecti_11getPos(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrRecti_11getPos(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("getPos (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrRecti_10getPos(((struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrRecti_10getPos(struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_self) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("getPos", 0);

  /* "psychxr/ovr/math.pyx":415
 * 
 *     def getPos(self):
 *         cdef ovrVector2i to_return = ovrVector2i(self.c_data.Pos.x,             # <<<<<<<<<<<<<<
 *                                                  self.c_data.Pos.y)
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->c_data->Pos.x); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 415, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);

  /* "psychxr/ovr/math.pyx":416
 *     def getPos(self):
 *         cdef ovrVector2i to_return = ovrVector2i(self.c_data.Pos.x,
 *                                                  self.c_data.Pos.y)             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->c_data->Pos.y); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 416, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "psychxr/ovr/math.pyx":415
 * 
 *     def getPos(self):
 *         cdef ovrVector2i to_return = ovrVector2i(self.c_data.Pos.x,             # <<<<<<<<<<<<<<
 *                                                  self.c_data.Pos.y)
 * 
 */
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 415, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector2i), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 415, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "psychxr/ovr/math.pyx":418
 *                                                  self.c_data.Pos.y)
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def getSize(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":414
 *         return 4
 * 
 *     def getPos(self):             # <<<<<<<<<<<<<<
 *         cdef ovrVector2i to_return = ovrVector2i(self.c_data.Pos.x,
 *                                                  self.c_data.Pos.y)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrRecti.getPos", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":420
 *         return to_return
 * 
 *     def getSize(self):             # <<<<<<<<<<<<<<
 *         cdef ovrSizei to_return = ovrSizei(self.c_data.Size.w,
 *                                            self.c_data.Size.h)
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrRecti_13getSize(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrRecti_13getSize(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("getSize (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrRecti_12getSize(((struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrRecti_12getSize(struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_self) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("getSize", 0);

  /* "psychxr/ovr/math.pyx":421
 * 
 *     def getSize(self):
 *         cdef ovrSizei to_return = ovrSizei(self.c_data.Size.w,             # <<<<<<<<<<<<<<
 *                                            self.c_data.Size.h)
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->c_data->Size.w); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 421, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);

  /* "psychxr/ovr/math.pyx":422
 *     def getSize(self):
 *         cdef ovrSizei to_return = ovrSizei(self.c_data.Size.w,
 *                                            self.c_data.Size.h)             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->c_data->Size.h); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 422, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "psychxr/ovr/math.pyx":421
 * 
 *     def getSize(self):
 *         cdef ovrSizei to_return = ovrSizei(self.c_data.Size.w,             # <<<<<<<<<<<<<<
 *                                            self.c_data.Size.h)
 * 
 */
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 421, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrSizei), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 421, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "psychxr/ovr/math.pyx":424
 *                                            self.c_data.Size.h)
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def setPos(self, ovrVector2i pos):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":420
 *         return to_return
 * 
 *     def getSize(self):             # <<<<<<<<<<<<<<
 *         cdef ovrSizei to_return = ovrSizei(self.c_data.Size.w,
 *                                            self.c_data.Size.h)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrRecti.getSize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":426
 *         return to_return
 * 
 *     def setPos(self, ovrVector2i pos):             # <<<<<<<<<<<<<<
 *         self.c_data.Pos.x = pos.c_data.x
 *         self.c_data.Pos.y = pos.c_data.y
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrRecti_15setPos(PyObject *__pyx_v_self, PyObject *__pyx_v_pos); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrRecti_15setPos(PyObject *__pyx_v_self, PyObject *__pyx_v_pos) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("setPos (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_pos), __pyx_ptype_7psychxr_3ovr_4math_ovrVector2i, 1, "pos", 0))) __PYX_ERR(0, 426, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrRecti_14setPos(((struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *)__pyx_v_pos));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrRecti_14setPos(struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i *__pyx_v_pos) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("setPos", 0);

  /* "psychxr/ovr/math.pyx":427
 * 
 *     def setPos(self, ovrVector2i pos):
 *         self.c_data.Pos.x = pos.c_data.x             # <<<<<<<<<<<<<<
 *         self.c_data.Pos.y = pos.c_data.y
 * 
 */
  __pyx_t_1 = __pyx_v_pos->c_data->x;
  __pyx_v_self->c_data->Pos.x = __pyx_t_1;

  /* "psychxr/ovr/math.pyx":428
 *     def setPos(self, ovrVector2i pos):
 *         self.c_data.Pos.x = pos.c_data.x
 *         self.c_data.Pos.y = pos.c_data.y             # <<<<<<<<<<<<<<
 * 
 *     def setSize(self, ovrSizei size):
 */
  __pyx_t_1 = __pyx_v_pos->c_data->y;
  __pyx_v_self->c_data->Pos.y = __pyx_t_1;

  /* "psychxr/ovr/math.pyx":426
 *         return to_return
 * 
 *     def setPos(self, ovrVector2i pos):             # <<<<<<<<<<<<<<
 *         self.c_data.Pos.x = pos.c_data.x
 *         self.c_data.Pos.y = pos.c_data.y
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":430
 *         self.c_data.Pos.y = pos.c_data.y
 * 
 *     def setSize(self, ovrSizei size):             # <<<<<<<<<<<<<<
 *         self.c_data.Size.w = size.c_data.w
 *         self.c_data.Size.h = size.c_data.h
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrRecti_17setSize(PyObject *__pyx_v_self, PyObject *__pyx_v_size); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrRecti_17setSize(PyObject *__pyx_v_self, PyObject *__pyx_v_size) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("setSize (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_size), __pyx_ptype_7psychxr_3ovr_4math_ovrSizei, 1, "size", 0))) __PYX_ERR(0, 430, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrRecti_16setSize(((struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *)__pyx_v_size));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrRecti_16setSize(struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei *__pyx_v_size) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("setSize", 0);

  /* "psychxr/ovr/math.pyx":431
 * 
 *     def setSize(self, ovrSizei size):
 *         self.c_data.Size.w = size.c_data.w             # <<<<<<<<<<<<<<
 *         self.c_data.Size.h = size.c_data.h
 * 
 */
  __pyx_t_1 = __pyx_v_size->c_data->w;
  __pyx_v_self->c_data->Size.w = __pyx_t_1;

  /* "psychxr/ovr/math.pyx":432
 *     def setSize(self, ovrSizei size):
 *         self.c_data.Size.w = size.c_data.w
 *         self.c_data.Size.h = size.c_data.h             # <<<<<<<<<<<<<<
 * 
 *     def __eq__(self, ovrRecti b):
 */
  __pyx_t_1 = __pyx_v_size->c_data->h;
  __pyx_v_self->c_data->Size.h = __pyx_t_1;

  /* "psychxr/ovr/math.pyx":430
 *         self.c_data.Pos.y = pos.c_data.y
 * 
 *     def setSize(self, ovrSizei size):             # <<<<<<<<<<<<<<
 *         self.c_data.Size.w = size.c_data.w
 *         self.c_data.Size.h = size.c_data.h
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":434
 *         self.c_data.Size.h = size.c_data.h
 * 
 *     def __eq__(self, ovrRecti b):             # <<<<<<<<<<<<<<
 *         return self.c_data.Pos.x == b.c_data.Pos.x and \
 *                self.c_data.Pos.y == b.c_data.Pos.y and \
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrRecti_19__eq__(PyObject *__pyx_v_self, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrRecti_19__eq__(PyObject *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__eq__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrRecti, 1, "b", 0))) __PYX_ERR(0, 434, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrRecti_18__eq__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrRecti_18__eq__(struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("__eq__", 0);

  /* "psychxr/ovr/math.pyx":435
 * 
 *     def __eq__(self, ovrRecti b):
 *         return self.c_data.Pos.x == b.c_data.Pos.x and \             # <<<<<<<<<<<<<<
 *                self.c_data.Pos.y == b.c_data.Pos.y and \
 *                self.c_data.Size.w == b.c_data.Size.w and \
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = (__pyx_v_self->c_data->Pos.x == __pyx_v_b->c_data->Pos.x);
  if (__pyx_t_2) {
  } else {
    __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 435, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L3_bool_binop_done;
  }

  /* "psychxr/ovr/math.pyx":436
 *     def __eq__(self, ovrRecti b):
 *         return self.c_data.Pos.x == b.c_data.Pos.x and \
 *                self.c_data.Pos.y == b.c_data.Pos.y and \             # <<<<<<<<<<<<<<
 *                self.c_data.Size.w == b.c_data.Size.w and \
 *                self.c_data.Size.h == b.c_data.Size.h
 */
  __pyx_t_2 = (__pyx_v_self->c_data->Pos.y == __pyx_v_b->c_data->Pos.y);
  if (__pyx_t_2) {
  } else {
    __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 436, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L3_bool_binop_done;
  }

  /* "psychxr/ovr/math.pyx":437
 *         return self.c_data.Pos.x == b.c_data.Pos.x and \
 *                self.c_data.Pos.y == b.c_data.Pos.y and \
 *                self.c_data.Size.w == b.c_data.Size.w and \             # <<<<<<<<<<<<<<
 *                self.c_data.Size.h == b.c_data.Size.h
 * 
 */
  __pyx_t_2 = (__pyx_v_self->c_data->Size.w == __pyx_v_b->c_data->Size.w);
  if (__pyx_t_2) {
  } else {
    __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 437, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L3_bool_binop_done;
  }

  /* "psychxr/ovr/math.pyx":438
 *                self.c_data.Pos.y == b.c_data.Pos.y and \
 *                self.c_data.Size.w == b.c_data.Size.w and \
 *                self.c_data.Size.h == b.c_data.Size.h             # <<<<<<<<<<<<<<
 * 
 *     def __ne__(self, ovrRecti b):
 */
  __pyx_t_2 = (__pyx_v_self->c_data->Size.h == __pyx_v_b->c_data->Size.h);
  __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 438, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __pyx_t_3;
  __pyx_t_3 = 0;
  __pyx_L3_bool_binop_done:;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":434
 *         self.c_data.Size.h = size.c_data.h
 * 
 *     def __eq__(self, ovrRecti b):             # <<<<<<<<<<<<<<
 *         return self.c_data.Pos.x == b.c_data.Pos.x and \
 *                self.c_data.Pos.y == b.c_data.Pos.y and \
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrRecti.__eq__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":440
 *                self.c_data.Size.h == b.c_data.Size.h
 * 
 *     def __ne__(self, ovrRecti b):             # <<<<<<<<<<<<<<
 *         return not self.__eq__(b)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrRecti_21__ne__(PyObject *__pyx_v_self, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrRecti_21__ne__(PyObject *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__ne__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrRecti, 1, "b", 0))) __PYX_ERR(0, 440, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrRecti_20__ne__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrRecti_20__ne__(struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  __Pyx_RefNannySetupContext("__ne__", 0);

  /* "psychxr/ovr/math.pyx":441
 * 
 *     def __ne__(self, ovrRecti b):
 *         return not self.__eq__(b)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_eq); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 441, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, ((PyObject *)__pyx_v_b)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 441, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, ((PyObject *)__pyx_v_b)};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 441, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, ((PyObject *)__pyx_v_b)};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 441, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 441, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(((PyObject *)__pyx_v_b));
      __Pyx_GIVEREF(((PyObject *)__pyx_v_b));
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, ((PyObject *)__pyx_v_b));
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 441, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 441, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyBool_FromLong((!__pyx_t_5)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 441, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":440
 *                self.c_data.Size.h == b.c_data.Size.h
 * 
 *     def __ne__(self, ovrRecti b):             # <<<<<<<<<<<<<<
 *         return not self.__eq__(b)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrRecti.__ne__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrRecti_23__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrRecti_23__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrRecti_22__reduce_cython__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrRecti_22__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrRecti.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrRecti_25__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrRecti_25__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrRecti_24__setstate_cython__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrRecti_24__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrRecti.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":445
 * 
 * cdef class ovrVector3f:
 *     def __init__(self, *args, **kwargs):             # <<<<<<<<<<<<<<
 *         pass
 * 
 */

/* Python wrapper */
static int __pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  CYTHON_UNUSED PyObject *__pyx_v_args = 0;
  CYTHON_UNUSED PyObject *__pyx_v_kwargs = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  if (unlikely(__pyx_kwds) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__init__", 1))) return -1;
  __Pyx_INCREF(__pyx_args);
  __pyx_v_args = __pyx_args;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f___init__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_self), __pyx_v_args, __pyx_v_kwargs);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_args);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f___init__(CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_args, CYTHON_UNUSED PyObject *__pyx_v_kwargs) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__", 0);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":448
 *         pass
 * 
 *     def __cinit__(self, *args, **kwargs):             # <<<<<<<<<<<<<<
 *         self.c_data = &self.c_Vector3f
 * 
 */

/* Python wrapper */
static int __pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_3__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_3__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_args = 0;
  CYTHON_UNUSED PyObject *__pyx_v_kwargs = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  if (unlikely(__pyx_kwds) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__cinit__", 1))) return -1;
  __Pyx_INCREF(__pyx_args);
  __pyx_v_args = __pyx_args;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_2__cinit__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_self), __pyx_v_args, __pyx_v_kwargs);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_args);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_2__cinit__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, PyObject *__pyx_v_args, CYTHON_UNUSED PyObject *__pyx_v_kwargs) {
  int __pyx_v_nargin;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  OVR::Vector3f __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  float __pyx_t_4;
  float __pyx_t_5;
  float __pyx_t_6;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "psychxr/ovr/math.pyx":449
 * 
 *     def __cinit__(self, *args, **kwargs):
 *         self.c_data = &self.c_Vector3f             # <<<<<<<<<<<<<<
 * 
 *         cdef int nargin = <int>len(args)  # get number of arguments
 */
  __pyx_v_self->c_data = (&__pyx_v_self->c_Vector3f);

  /* "psychxr/ovr/math.pyx":451
 *         self.c_data = &self.c_Vector3f
 * 
 *         cdef int nargin = <int>len(args)  # get number of arguments             # <<<<<<<<<<<<<<
 *         if nargin == 0:
 *             self.c_data[0] = ovr_math.Vector3f.Zero()
 */
  __pyx_t_1 = PyTuple_GET_SIZE(__pyx_v_args); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 451, __pyx_L1_error)
  __pyx_v_nargin = ((int)__pyx_t_1);

  /* "psychxr/ovr/math.pyx":452
 * 
 *         cdef int nargin = <int>len(args)  # get number of arguments
 *         if nargin == 0:             # <<<<<<<<<<<<<<
 *             self.c_data[0] = ovr_math.Vector3f.Zero()
 *         elif nargin == 3:
 */
  switch (__pyx_v_nargin) {
    case 0:

    /* "psychxr/ovr/math.pyx":453
 *         cdef int nargin = <int>len(args)  # get number of arguments
 *         if nargin == 0:
 *             self.c_data[0] = ovr_math.Vector3f.Zero()             # <<<<<<<<<<<<<<
 *         elif nargin == 3:
 *             self.c_data[0] = ovr_math.Vector3f(
 */
    try {
      __pyx_t_2 = OVR::Vector3f::Zero();
    } catch(...) {
      __Pyx_CppExn2PyErr();
      __PYX_ERR(0, 453, __pyx_L1_error)
    }
    (__pyx_v_self->c_data[0]) = __pyx_t_2;

    /* "psychxr/ovr/math.pyx":452
 * 
 *         cdef int nargin = <int>len(args)  # get number of arguments
 *         if nargin == 0:             # <<<<<<<<<<<<<<
 *             self.c_data[0] = ovr_math.Vector3f.Zero()
 *         elif nargin == 3:
 */
    break;

    /* "psychxr/ovr/math.pyx":454
 *         if nargin == 0:
 *             self.c_data[0] = ovr_math.Vector3f.Zero()
 *         elif nargin == 3:             # <<<<<<<<<<<<<<
 *             self.c_data[0] = ovr_math.Vector3f(
 *                 <float>args[0], <float>args[1], <float>args[2])
 */
    case 3:

    /* "psychxr/ovr/math.pyx":456
 *         elif nargin == 3:
 *             self.c_data[0] = ovr_math.Vector3f(
 *                 <float>args[0], <float>args[1], <float>args[2])             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
    __pyx_t_3 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 456, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __pyx_PyFloat_AsFloat(__pyx_t_3); if (unlikely((__pyx_t_4 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 456, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 456, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = __pyx_PyFloat_AsFloat(__pyx_t_3); if (unlikely((__pyx_t_5 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 456, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 456, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_6 = __pyx_PyFloat_AsFloat(__pyx_t_3); if (unlikely((__pyx_t_6 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 456, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "psychxr/ovr/math.pyx":455
 *             self.c_data[0] = ovr_math.Vector3f.Zero()
 *         elif nargin == 3:
 *             self.c_data[0] = ovr_math.Vector3f(             # <<<<<<<<<<<<<<
 *                 <float>args[0], <float>args[1], <float>args[2])
 * 
 */
    try {
      __pyx_t_2 = OVR::Vector3f(((float)__pyx_t_4), ((float)__pyx_t_5), ((float)__pyx_t_6));
    } catch(...) {
      __Pyx_CppExn2PyErr();
      __PYX_ERR(0, 455, __pyx_L1_error)
    }
    (__pyx_v_self->c_data[0]) = __pyx_t_2;

    /* "psychxr/ovr/math.pyx":454
 *         if nargin == 0:
 *             self.c_data[0] = ovr_math.Vector3f.Zero()
 *         elif nargin == 3:             # <<<<<<<<<<<<<<
 *             self.c_data[0] = ovr_math.Vector3f(
 *                 <float>args[0], <float>args[1], <float>args[2])
 */
    break;
    default: break;
  }

  /* "psychxr/ovr/math.pyx":448
 *         pass
 * 
 *     def __cinit__(self, *args, **kwargs):             # <<<<<<<<<<<<<<
 *         self.c_data = &self.c_Vector3f
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":459
 * 
 *     @property
 *     def x(self):             # <<<<<<<<<<<<<<
 *         return self.c_data.x
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_1x_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_1x_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_1x___get__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_1x___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "psychxr/ovr/math.pyx":460
 *     @property
 *     def x(self):
 *         return self.c_data.x             # <<<<<<<<<<<<<<
 * 
 *     @x.setter
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->c_data->x); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 460, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":459
 * 
 *     @property
 *     def x(self):             # <<<<<<<<<<<<<<
 *         return self.c_data.x
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.x.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":463
 * 
 *     @x.setter
 *     def x(self, float value):             # <<<<<<<<<<<<<<
 *         self.c_data.x = value
 * 
 */

/* Python wrapper */
static int __pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_1x_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_value); /*proto*/
static int __pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_1x_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_value) {
  float __pyx_v_value;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_value); {
    __pyx_v_value = __pyx_PyFloat_AsFloat(__pyx_arg_value); if (unlikely((__pyx_v_value == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 463, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.x.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_1x_2__set__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_self), ((float)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_1x_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, float __pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "psychxr/ovr/math.pyx":464
 *     @x.setter
 *     def x(self, float value):
 *         self.c_data.x = value             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __pyx_v_self->c_data->x = __pyx_v_value;

  /* "psychxr/ovr/math.pyx":463
 * 
 *     @x.setter
 *     def x(self, float value):             # <<<<<<<<<<<<<<
 *         self.c_data.x = value
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":467
 * 
 *     @property
 *     def y(self):             # <<<<<<<<<<<<<<
 *         return self.c_data.y
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_1y_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_1y_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_1y___get__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_1y___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "psychxr/ovr/math.pyx":468
 *     @property
 *     def y(self):
 *         return self.c_data.y             # <<<<<<<<<<<<<<
 * 
 *     @y.setter
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->c_data->y); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 468, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":467
 * 
 *     @property
 *     def y(self):             # <<<<<<<<<<<<<<
 *         return self.c_data.y
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.y.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":471
 * 
 *     @y.setter
 *     def y(self, float value):             # <<<<<<<<<<<<<<
 *         self.c_data.y = value
 * 
 */

/* Python wrapper */
static int __pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_1y_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_value); /*proto*/
static int __pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_1y_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_value) {
  float __pyx_v_value;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_value); {
    __pyx_v_value = __pyx_PyFloat_AsFloat(__pyx_arg_value); if (unlikely((__pyx_v_value == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 471, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.y.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_1y_2__set__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_self), ((float)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_1y_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, float __pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "psychxr/ovr/math.pyx":472
 *     @y.setter
 *     def y(self, float value):
 *         self.c_data.y = value             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __pyx_v_self->c_data->y = __pyx_v_value;

  /* "psychxr/ovr/math.pyx":471
 * 
 *     @y.setter
 *     def y(self, float value):             # <<<<<<<<<<<<<<
 *         self.c_data.y = value
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":475
 * 
 *     @property
 *     def z(self):             # <<<<<<<<<<<<<<
 *         return self.c_data.z
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_1z_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_1z_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_1z___get__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_1z___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "psychxr/ovr/math.pyx":476
 *     @property
 *     def z(self):
 *         return self.c_data.z             # <<<<<<<<<<<<<<
 * 
 *     @z.setter
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->c_data->z); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 476, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":475
 * 
 *     @property
 *     def z(self):             # <<<<<<<<<<<<<<
 *         return self.c_data.z
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.z.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":479
 * 
 *     @z.setter
 *     def z(self, float value):             # <<<<<<<<<<<<<<
 *         self.c_data.z = value
 * 
 */

/* Python wrapper */
static int __pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_1z_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_value); /*proto*/
static int __pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_1z_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_value) {
  float __pyx_v_value;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_value); {
    __pyx_v_value = __pyx_PyFloat_AsFloat(__pyx_arg_value); if (unlikely((__pyx_v_value == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 479, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.z.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_1z_2__set__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_self), ((float)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_1z_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, float __pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "psychxr/ovr/math.pyx":480
 *     @z.setter
 *     def z(self, float value):
 *         self.c_data.z = value             # <<<<<<<<<<<<<<
 * 
 *     def as_tuple(self):
 */
  __pyx_v_self->c_data->z = __pyx_v_value;

  /* "psychxr/ovr/math.pyx":479
 * 
 *     @z.setter
 *     def z(self, float value):             # <<<<<<<<<<<<<<
 *         self.c_data.z = value
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":482
 *         self.c_data.z = value
 * 
 *     def as_tuple(self):             # <<<<<<<<<<<<<<
 *         return self.c_data.x, self.c_data.y, self.c_data.z
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_5as_tuple(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_5as_tuple(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("as_tuple (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_4as_tuple(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_4as_tuple(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("as_tuple", 0);

  /* "psychxr/ovr/math.pyx":483
 * 
 *     def as_tuple(self):
 *         return self.c_data.x, self.c_data.y, self.c_data.z             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->c_data->x); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 483, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->c_data->y); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 483, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_self->c_data->z); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 483, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 483, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_3);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":482
 *         self.c_data.z = value
 * 
 *     def as_tuple(self):             # <<<<<<<<<<<<<<
 *         return self.c_data.x, self.c_data.y, self.c_data.z
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.as_tuple", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":486
 * 
 *     @property
 *     def ctypes(self):             # <<<<<<<<<<<<<<
 *         return (GL.GLfloat * 3)(
 *             self.c_data.x,
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_6ctypes_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_6ctypes_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_6ctypes___get__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_6ctypes___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "psychxr/ovr/math.pyx":487
 *     @property
 *     def ctypes(self):
 *         return (GL.GLfloat * 3)(             # <<<<<<<<<<<<<<
 *             self.c_data.x,
 *             self.c_data.y,
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_GL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 487, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_GLfloat); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 487, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyNumber_Multiply(__pyx_t_3, __pyx_int_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 487, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "psychxr/ovr/math.pyx":488
 *     def ctypes(self):
 *         return (GL.GLfloat * 3)(
 *             self.c_data.x,             # <<<<<<<<<<<<<<
 *             self.c_data.y,
 *             self.c_data.z)
 */
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_self->c_data->x); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 488, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "psychxr/ovr/math.pyx":489
 *         return (GL.GLfloat * 3)(
 *             self.c_data.x,
 *             self.c_data.y,             # <<<<<<<<<<<<<<
 *             self.c_data.z)
 * 
 */
  __pyx_t_4 = PyFloat_FromDouble(__pyx_v_self->c_data->y); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 489, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);

  /* "psychxr/ovr/math.pyx":490
 *             self.c_data.x,
 *             self.c_data.y,
 *             self.c_data.z)             # <<<<<<<<<<<<<<
 * 
 *     def __len__(self):
 */
  __pyx_t_5 = PyFloat_FromDouble(__pyx_v_self->c_data->z); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 490, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = NULL;
  __pyx_t_7 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_7 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_6, __pyx_t_3, __pyx_t_4, __pyx_t_5};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 3+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 487, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_6, __pyx_t_3, __pyx_t_4, __pyx_t_5};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 3+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 487, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else
  #endif
  {
    __pyx_t_8 = PyTuple_New(3+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 487, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    if (__pyx_t_6) {
      __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_8, 2+__pyx_t_7, __pyx_t_5);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_5 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 487, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":486
 * 
 *     @property
 *     def ctypes(self):             # <<<<<<<<<<<<<<
 *         return (GL.GLfloat * 3)(
 *             self.c_data.x,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.ctypes.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":492
 *             self.c_data.z)
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         return 3
 * 
 */

/* Python wrapper */
static Py_ssize_t __pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_7__len__(PyObject *__pyx_v_self); /*proto*/
static Py_ssize_t __pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_7__len__(PyObject *__pyx_v_self) {
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__len__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_6__len__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static Py_ssize_t __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_6__len__(CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self) {
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__len__", 0);

  /* "psychxr/ovr/math.pyx":493
 * 
 *     def __len__(self):
 *         return 3             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __pyx_r = 3;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":492
 *             self.c_data.z)
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         return 3
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":496
 * 
 *     @staticmethod
 *     def zero():             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         return to_return
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_9zero(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_7psychxr_3ovr_4math_11ovrVector3f_9zero = {"zero", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_9zero, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_9zero(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("zero (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("zero", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return NULL;}
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "zero", 0))) return NULL;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_8zero();

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_8zero() {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("zero", 0);

  /* "psychxr/ovr/math.pyx":497
 *     @staticmethod
 *     def zero():
 *         cdef ovrVector3f to_return = ovrVector3f()             # <<<<<<<<<<<<<<
 *         return to_return
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector3f), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 497, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":498
 *     def zero():
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def __eq__(self, ovrVector3f b):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":496
 * 
 *     @staticmethod
 *     def zero():             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         return to_return
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.zero", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":500
 *         return to_return
 * 
 *     def __eq__(self, ovrVector3f b):             # <<<<<<<<<<<<<<
 *         return (<ovrVector3f>self).c_data[0] == b.c_data[0]
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_11__eq__(PyObject *__pyx_v_self, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_11__eq__(PyObject *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__eq__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "b", 0))) __PYX_ERR(0, 500, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_10__eq__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_10__eq__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__eq__", 0);

  /* "psychxr/ovr/math.pyx":501
 * 
 *     def __eq__(self, ovrVector3f b):
 *         return (<ovrVector3f>self).c_data[0] == b.c_data[0]             # <<<<<<<<<<<<<<
 * 
 *     def __ne__(self, ovrVector3f b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(((__pyx_v_self->c_data[0]) == (__pyx_v_b->c_data[0]))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 501, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":500
 *         return to_return
 * 
 *     def __eq__(self, ovrVector3f b):             # <<<<<<<<<<<<<<
 *         return (<ovrVector3f>self).c_data[0] == b.c_data[0]
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.__eq__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":503
 *         return (<ovrVector3f>self).c_data[0] == b.c_data[0]
 * 
 *     def __ne__(self, ovrVector3f b):             # <<<<<<<<<<<<<<
 *         return (<ovrVector3f>self).c_data[0] != b.c_data[0]
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_13__ne__(PyObject *__pyx_v_self, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_13__ne__(PyObject *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__ne__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "b", 0))) __PYX_ERR(0, 503, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_12__ne__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_12__ne__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__ne__", 0);

  /* "psychxr/ovr/math.pyx":504
 * 
 *     def __ne__(self, ovrVector3f b):
 *         return (<ovrVector3f>self).c_data[0] != b.c_data[0]             # <<<<<<<<<<<<<<
 * 
 *     def __add__(ovrVector3f a, ovrVector3f b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(((__pyx_v_self->c_data[0]) != (__pyx_v_b->c_data[0]))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 504, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":503
 *         return (<ovrVector3f>self).c_data[0] == b.c_data[0]
 * 
 *     def __ne__(self, ovrVector3f b):             # <<<<<<<<<<<<<<
 *         return (<ovrVector3f>self).c_data[0] != b.c_data[0]
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.__ne__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":506
 *         return (<ovrVector3f>self).c_data[0] != b.c_data[0]
 * 
 *     def __add__(ovrVector3f a, ovrVector3f b):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = a.c_data[0] + b.c_data[0]
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_15__add__(PyObject *__pyx_v_a, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_15__add__(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__add__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_a), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "a", 0))) __PYX_ERR(0, 506, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "b", 0))) __PYX_ERR(0, 506, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_14__add__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_a), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_14__add__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_a, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_b) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__add__", 0);

  /* "psychxr/ovr/math.pyx":507
 * 
 *     def __add__(ovrVector3f a, ovrVector3f b):
 *         cdef ovrVector3f to_return = ovrVector3f()             # <<<<<<<<<<<<<<
 *         (<ovrVector3f>to_return).c_data[0] = a.c_data[0] + b.c_data[0]
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector3f), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 507, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":508
 *     def __add__(ovrVector3f a, ovrVector3f b):
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = a.c_data[0] + b.c_data[0]             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
  (__pyx_v_to_return->c_data[0]) = ((__pyx_v_a->c_data[0]) + (__pyx_v_b->c_data[0]));

  /* "psychxr/ovr/math.pyx":510
 *         (<ovrVector3f>to_return).c_data[0] = a.c_data[0] + b.c_data[0]
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def __iadd__(self, ovrVector3f b):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":506
 *         return (<ovrVector3f>self).c_data[0] != b.c_data[0]
 * 
 *     def __add__(ovrVector3f a, ovrVector3f b):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = a.c_data[0] + b.c_data[0]
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.__add__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":512
 *         return to_return
 * 
 *     def __iadd__(self, ovrVector3f b):             # <<<<<<<<<<<<<<
 *         (<ovrVector3f>self).c_data[0] += b.c_data[0]
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_17__iadd__(PyObject *__pyx_v_self, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_17__iadd__(PyObject *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iadd__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "b", 0))) __PYX_ERR(0, 512, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_16__iadd__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_16__iadd__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iadd__", 0);

  /* "psychxr/ovr/math.pyx":513
 * 
 *     def __iadd__(self, ovrVector3f b):
 *         (<ovrVector3f>self).c_data[0] += b.c_data[0]             # <<<<<<<<<<<<<<
 * 
 *         return self
 */
  (__pyx_v_self->c_data[0]) += (__pyx_v_b->c_data[0]);

  /* "psychxr/ovr/math.pyx":515
 *         (<ovrVector3f>self).c_data[0] += b.c_data[0]
 * 
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     def __sub__(ovrVector3f a, ovrVector3f b):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":512
 *         return to_return
 * 
 *     def __iadd__(self, ovrVector3f b):             # <<<<<<<<<<<<<<
 *         (<ovrVector3f>self).c_data[0] += b.c_data[0]
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":517
 *         return self
 * 
 *     def __sub__(ovrVector3f a, ovrVector3f b):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = a.c_data[0] - b.c_data[0]
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_19__sub__(PyObject *__pyx_v_a, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_19__sub__(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__sub__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_a), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "a", 0))) __PYX_ERR(0, 517, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "b", 0))) __PYX_ERR(0, 517, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_18__sub__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_a), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_18__sub__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_a, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_b) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__sub__", 0);

  /* "psychxr/ovr/math.pyx":518
 * 
 *     def __sub__(ovrVector3f a, ovrVector3f b):
 *         cdef ovrVector3f to_return = ovrVector3f()             # <<<<<<<<<<<<<<
 *         (<ovrVector3f>to_return).c_data[0] = a.c_data[0] - b.c_data[0]
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector3f), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 518, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":519
 *     def __sub__(ovrVector3f a, ovrVector3f b):
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = a.c_data[0] - b.c_data[0]             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
  (__pyx_v_to_return->c_data[0]) = ((__pyx_v_a->c_data[0]) - (__pyx_v_b->c_data[0]));

  /* "psychxr/ovr/math.pyx":521
 *         (<ovrVector3f>to_return).c_data[0] = a.c_data[0] - b.c_data[0]
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def __isub__(self, ovrVector3f b):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":517
 *         return self
 * 
 *     def __sub__(ovrVector3f a, ovrVector3f b):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = a.c_data[0] - b.c_data[0]
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.__sub__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":523
 *         return to_return
 * 
 *     def __isub__(self, ovrVector3f b):             # <<<<<<<<<<<<<<
 *         (<ovrVector3f>self).c_data[0] -= b.c_data[0]
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_21__isub__(PyObject *__pyx_v_self, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_21__isub__(PyObject *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__isub__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "b", 0))) __PYX_ERR(0, 523, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_20__isub__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_20__isub__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__isub__", 0);

  /* "psychxr/ovr/math.pyx":524
 * 
 *     def __isub__(self, ovrVector3f b):
 *         (<ovrVector3f>self).c_data[0] -= b.c_data[0]             # <<<<<<<<<<<<<<
 * 
 *         return self
 */
  (__pyx_v_self->c_data[0]) -= (__pyx_v_b->c_data[0]);

  /* "psychxr/ovr/math.pyx":526
 *         (<ovrVector3f>self).c_data[0] -= b.c_data[0]
 * 
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     def __neg__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":523
 *         return to_return
 * 
 *     def __isub__(self, ovrVector3f b):             # <<<<<<<<<<<<<<
 *         (<ovrVector3f>self).c_data[0] -= b.c_data[0]
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":528
 *         return self
 * 
 *     def __neg__(self):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = -(<ovrVector3f>self).c_data[0]
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_23__neg__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_23__neg__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__neg__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_22__neg__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_22__neg__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__neg__", 0);

  /* "psychxr/ovr/math.pyx":529
 * 
 *     def __neg__(self):
 *         cdef ovrVector3f to_return = ovrVector3f()             # <<<<<<<<<<<<<<
 *         (<ovrVector3f>to_return).c_data[0] = -(<ovrVector3f>self).c_data[0]
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector3f), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 529, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":530
 *     def __neg__(self):
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = -(<ovrVector3f>self).c_data[0]             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
  (__pyx_v_to_return->c_data[0]) = (-(__pyx_v_self->c_data[0]));

  /* "psychxr/ovr/math.pyx":532
 *         (<ovrVector3f>to_return).c_data[0] = -(<ovrVector3f>self).c_data[0]
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def __mul__(ovrVector3f a, object b):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":528
 *         return self
 * 
 *     def __neg__(self):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = -(<ovrVector3f>self).c_data[0]
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.__neg__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":534
 *         return to_return
 * 
 *     def __mul__(ovrVector3f a, object b):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         if isinstance(b, ovrVector3f):
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_25__mul__(PyObject *__pyx_v_a, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_25__mul__(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__mul__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_a), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "a", 0))) __PYX_ERR(0, 534, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_24__mul__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_a), ((PyObject *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_24__mul__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_a, PyObject *__pyx_v_b) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  float __pyx_t_5;
  __Pyx_RefNannySetupContext("__mul__", 0);

  /* "psychxr/ovr/math.pyx":535
 * 
 *     def __mul__(ovrVector3f a, object b):
 *         cdef ovrVector3f to_return = ovrVector3f()             # <<<<<<<<<<<<<<
 *         if isinstance(b, ovrVector3f):
 *             (<ovrVector3f>to_return).c_data[0] = \
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector3f), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 535, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":536
 *     def __mul__(ovrVector3f a, object b):
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         if isinstance(b, ovrVector3f):             # <<<<<<<<<<<<<<
 *             (<ovrVector3f>to_return).c_data[0] = \
 *                 (<ovrVector3f>a).c_data[0] * (<ovrVector3f>b).c_data[0]
 */
  __pyx_t_2 = __Pyx_TypeCheck(__pyx_v_b, __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f); 
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "psychxr/ovr/math.pyx":537
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         if isinstance(b, ovrVector3f):
 *             (<ovrVector3f>to_return).c_data[0] = \             # <<<<<<<<<<<<<<
 *                 (<ovrVector3f>a).c_data[0] * (<ovrVector3f>b).c_data[0]
 *         elif isinstance(b, (int, float)):
 */
    (__pyx_v_to_return->c_data[0]) = ((__pyx_v_a->c_data[0]) * (((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_b)->c_data[0]));

    /* "psychxr/ovr/math.pyx":536
 *     def __mul__(ovrVector3f a, object b):
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         if isinstance(b, ovrVector3f):             # <<<<<<<<<<<<<<
 *             (<ovrVector3f>to_return).c_data[0] = \
 *                 (<ovrVector3f>a).c_data[0] * (<ovrVector3f>b).c_data[0]
 */
    goto __pyx_L3;
  }

  /* "psychxr/ovr/math.pyx":539
 *             (<ovrVector3f>to_return).c_data[0] = \
 *                 (<ovrVector3f>a).c_data[0] * (<ovrVector3f>b).c_data[0]
 *         elif isinstance(b, (int, float)):             # <<<<<<<<<<<<<<
 *             (<ovrVector3f>to_return).c_data[0] = \
 *                 (<ovrVector3f>a).c_data[0] * <float>b
 */
  __pyx_t_2 = PyInt_Check(__pyx_v_b); 
  __pyx_t_4 = (__pyx_t_2 != 0);
  if (!__pyx_t_4) {
  } else {
    __pyx_t_3 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = PyFloat_Check(__pyx_v_b); 
  __pyx_t_2 = (__pyx_t_4 != 0);
  __pyx_t_3 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {

    /* "psychxr/ovr/math.pyx":541
 *         elif isinstance(b, (int, float)):
 *             (<ovrVector3f>to_return).c_data[0] = \
 *                 (<ovrVector3f>a).c_data[0] * <float>b             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
    __pyx_t_5 = __pyx_PyFloat_AsFloat(__pyx_v_b); if (unlikely((__pyx_t_5 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 541, __pyx_L1_error)

    /* "psychxr/ovr/math.pyx":540
 *                 (<ovrVector3f>a).c_data[0] * (<ovrVector3f>b).c_data[0]
 *         elif isinstance(b, (int, float)):
 *             (<ovrVector3f>to_return).c_data[0] = \             # <<<<<<<<<<<<<<
 *                 (<ovrVector3f>a).c_data[0] * <float>b
 * 
 */
    (__pyx_v_to_return->c_data[0]) = ((__pyx_v_a->c_data[0]) * ((float)__pyx_t_5));

    /* "psychxr/ovr/math.pyx":539
 *             (<ovrVector3f>to_return).c_data[0] = \
 *                 (<ovrVector3f>a).c_data[0] * (<ovrVector3f>b).c_data[0]
 *         elif isinstance(b, (int, float)):             # <<<<<<<<<<<<<<
 *             (<ovrVector3f>to_return).c_data[0] = \
 *                 (<ovrVector3f>a).c_data[0] * <float>b
 */
  }
  __pyx_L3:;

  /* "psychxr/ovr/math.pyx":543
 *                 (<ovrVector3f>a).c_data[0] * <float>b
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def __imul__(self, object b):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":534
 *         return to_return
 * 
 *     def __mul__(ovrVector3f a, object b):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         if isinstance(b, ovrVector3f):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.__mul__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":545
 *         return to_return
 * 
 *     def __imul__(self, object b):             # <<<<<<<<<<<<<<
 *         if isinstance(b, (int, float)):
 *             (<ovrVector3f>self).c_data[0] = \
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_27__imul__(PyObject *__pyx_v_self, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_27__imul__(PyObject *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__imul__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_26__imul__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_self), ((PyObject *)__pyx_v_b));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_26__imul__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  float __pyx_t_4;
  __Pyx_RefNannySetupContext("__imul__", 0);

  /* "psychxr/ovr/math.pyx":546
 * 
 *     def __imul__(self, object b):
 *         if isinstance(b, (int, float)):             # <<<<<<<<<<<<<<
 *             (<ovrVector3f>self).c_data[0] = \
 *                 (<ovrVector3f>self).c_data[0] * <float>b
 */
  __pyx_t_2 = PyInt_Check(__pyx_v_b); 
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (!__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = PyFloat_Check(__pyx_v_b); 
  __pyx_t_2 = (__pyx_t_3 != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "psychxr/ovr/math.pyx":548
 *         if isinstance(b, (int, float)):
 *             (<ovrVector3f>self).c_data[0] = \
 *                 (<ovrVector3f>self).c_data[0] * <float>b             # <<<<<<<<<<<<<<
 * 
 *         return self
 */
    __pyx_t_4 = __pyx_PyFloat_AsFloat(__pyx_v_b); if (unlikely((__pyx_t_4 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 548, __pyx_L1_error)

    /* "psychxr/ovr/math.pyx":547
 *     def __imul__(self, object b):
 *         if isinstance(b, (int, float)):
 *             (<ovrVector3f>self).c_data[0] = \             # <<<<<<<<<<<<<<
 *                 (<ovrVector3f>self).c_data[0] * <float>b
 * 
 */
    (__pyx_v_self->c_data[0]) = ((__pyx_v_self->c_data[0]) * ((float)__pyx_t_4));

    /* "psychxr/ovr/math.pyx":546
 * 
 *     def __imul__(self, object b):
 *         if isinstance(b, (int, float)):             # <<<<<<<<<<<<<<
 *             (<ovrVector3f>self).c_data[0] = \
 *                 (<ovrVector3f>self).c_data[0] * <float>b
 */
  }

  /* "psychxr/ovr/math.pyx":550
 *                 (<ovrVector3f>self).c_data[0] * <float>b
 * 
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     def __truediv__(ovrVector3f a, object b):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":545
 *         return to_return
 * 
 *     def __imul__(self, object b):             # <<<<<<<<<<<<<<
 *         if isinstance(b, (int, float)):
 *             (<ovrVector3f>self).c_data[0] = \
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.__imul__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":552
 *         return self
 * 
 *     def __truediv__(ovrVector3f a, object b):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_29__truediv__(PyObject *__pyx_v_a, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_29__truediv__(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__truediv__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_a), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "a", 0))) __PYX_ERR(0, 552, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_28__truediv__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_a), ((PyObject *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_28__truediv__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_a, PyObject *__pyx_v_b) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__truediv__", 0);

  /* "psychxr/ovr/math.pyx":553
 * 
 *     def __truediv__(ovrVector3f a, object b):
 *         cdef ovrVector3f to_return = ovrVector3f()             # <<<<<<<<<<<<<<
 *         (<ovrVector3f>to_return).c_data[0] = \
 *             (<ovrVector3f>a).c_data[0] / (<ovrVector3f>b).c_data[0]
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector3f), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 553, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":554
 *     def __truediv__(ovrVector3f a, object b):
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \             # <<<<<<<<<<<<<<
 *             (<ovrVector3f>a).c_data[0] / (<ovrVector3f>b).c_data[0]
 * 
 */
  (__pyx_v_to_return->c_data[0]) = ((__pyx_v_a->c_data[0]) / (((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_b)->c_data[0]));

  /* "psychxr/ovr/math.pyx":557
 *             (<ovrVector3f>a).c_data[0] / (<ovrVector3f>b).c_data[0]
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def __itruediv__(self, object b):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":552
 *         return self
 * 
 *     def __truediv__(ovrVector3f a, object b):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.__truediv__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":559
 *         return to_return
 * 
 *     def __itruediv__(self, object b):             # <<<<<<<<<<<<<<
 *         if isinstance(b, (int, float)):
 *             (<ovrVector3f>self).c_data[0] = \
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_31__itruediv__(PyObject *__pyx_v_self, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_31__itruediv__(PyObject *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__itruediv__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_30__itruediv__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_self), ((PyObject *)__pyx_v_b));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_30__itruediv__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  float __pyx_t_4;
  __Pyx_RefNannySetupContext("__itruediv__", 0);

  /* "psychxr/ovr/math.pyx":560
 * 
 *     def __itruediv__(self, object b):
 *         if isinstance(b, (int, float)):             # <<<<<<<<<<<<<<
 *             (<ovrVector3f>self).c_data[0] = \
 *                 (<ovrVector3f>self).c_data[0] / <float>b
 */
  __pyx_t_2 = PyInt_Check(__pyx_v_b); 
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (!__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = PyFloat_Check(__pyx_v_b); 
  __pyx_t_2 = (__pyx_t_3 != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "psychxr/ovr/math.pyx":562
 *         if isinstance(b, (int, float)):
 *             (<ovrVector3f>self).c_data[0] = \
 *                 (<ovrVector3f>self).c_data[0] / <float>b             # <<<<<<<<<<<<<<
 * 
 *         return self
 */
    __pyx_t_4 = __pyx_PyFloat_AsFloat(__pyx_v_b); if (unlikely((__pyx_t_4 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 562, __pyx_L1_error)

    /* "psychxr/ovr/math.pyx":561
 *     def __itruediv__(self, object b):
 *         if isinstance(b, (int, float)):
 *             (<ovrVector3f>self).c_data[0] = \             # <<<<<<<<<<<<<<
 *                 (<ovrVector3f>self).c_data[0] / <float>b
 * 
 */
    (__pyx_v_self->c_data[0]) = ((__pyx_v_self->c_data[0]) / ((float)__pyx_t_4));

    /* "psychxr/ovr/math.pyx":560
 * 
 *     def __itruediv__(self, object b):
 *         if isinstance(b, (int, float)):             # <<<<<<<<<<<<<<
 *             (<ovrVector3f>self).c_data[0] = \
 *                 (<ovrVector3f>self).c_data[0] / <float>b
 */
  }

  /* "psychxr/ovr/math.pyx":564
 *                 (<ovrVector3f>self).c_data[0] / <float>b
 * 
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":559
 *         return to_return
 * 
 *     def __itruediv__(self, object b):             # <<<<<<<<<<<<<<
 *         if isinstance(b, (int, float)):
 *             (<ovrVector3f>self).c_data[0] = \
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.__itruediv__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":567
 * 
 *     @staticmethod
 *     def min(ovrVector3f a, ovrVector3f b):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_33min(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_7psychxr_3ovr_4math_11ovrVector3f_33min = {"min", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_33min, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_33min(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_a = 0;
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_b = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("min (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_a,&__pyx_n_s_b,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_a)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_b)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("min", 1, 2, 2, 1); __PYX_ERR(0, 567, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "min") < 0)) __PYX_ERR(0, 567, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_a = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)values[0]);
    __pyx_v_b = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("min", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 567, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.min", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_a), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "a", 0))) __PYX_ERR(0, 567, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "b", 0))) __PYX_ERR(0, 567, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_32min(__pyx_v_a, __pyx_v_b);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_32min(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_a, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_b) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  OVR::Vector3f __pyx_t_2;
  __Pyx_RefNannySetupContext("min", 0);

  /* "psychxr/ovr/math.pyx":568
 *     @staticmethod
 *     def min(ovrVector3f a, ovrVector3f b):
 *         cdef ovrVector3f to_return = ovrVector3f()             # <<<<<<<<<<<<<<
 *         (<ovrVector3f>to_return).c_data[0] = \
 *             ovr_math.Vector3f.Min(a.c_data[0], b.c_data[0])
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector3f), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 568, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":570
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \
 *             ovr_math.Vector3f.Min(a.c_data[0], b.c_data[0])             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
  try {
    __pyx_t_2 = OVR::Vector3f::Min((__pyx_v_a->c_data[0]), (__pyx_v_b->c_data[0]));
  } catch(...) {
    __Pyx_CppExn2PyErr();
    __PYX_ERR(0, 570, __pyx_L1_error)
  }

  /* "psychxr/ovr/math.pyx":569
 *     def min(ovrVector3f a, ovrVector3f b):
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \             # <<<<<<<<<<<<<<
 *             ovr_math.Vector3f.Min(a.c_data[0], b.c_data[0])
 * 
 */
  (__pyx_v_to_return->c_data[0]) = __pyx_t_2;

  /* "psychxr/ovr/math.pyx":572
 *             ovr_math.Vector3f.Min(a.c_data[0], b.c_data[0])
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":567
 * 
 *     @staticmethod
 *     def min(ovrVector3f a, ovrVector3f b):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.min", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":575
 * 
 *     @staticmethod
 *     def max(ovrVector3f a, ovrVector3f b):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_35max(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_7psychxr_3ovr_4math_11ovrVector3f_35max = {"max", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_35max, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_35max(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_a = 0;
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_b = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("max (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_a,&__pyx_n_s_b,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_a)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_b)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("max", 1, 2, 2, 1); __PYX_ERR(0, 575, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "max") < 0)) __PYX_ERR(0, 575, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_a = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)values[0]);
    __pyx_v_b = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("max", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 575, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.max", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_a), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "a", 0))) __PYX_ERR(0, 575, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "b", 0))) __PYX_ERR(0, 575, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_34max(__pyx_v_a, __pyx_v_b);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_34max(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_a, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_b) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  OVR::Vector3f __pyx_t_2;
  __Pyx_RefNannySetupContext("max", 0);

  /* "psychxr/ovr/math.pyx":576
 *     @staticmethod
 *     def max(ovrVector3f a, ovrVector3f b):
 *         cdef ovrVector3f to_return = ovrVector3f()             # <<<<<<<<<<<<<<
 *         (<ovrVector3f>to_return).c_data[0] = \
 *             ovr_math.Vector3f.Max(a.c_data[0], b.c_data[0])
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector3f), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 576, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":578
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \
 *             ovr_math.Vector3f.Max(a.c_data[0], b.c_data[0])             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
  try {
    __pyx_t_2 = OVR::Vector3f::Max((__pyx_v_a->c_data[0]), (__pyx_v_b->c_data[0]));
  } catch(...) {
    __Pyx_CppExn2PyErr();
    __PYX_ERR(0, 578, __pyx_L1_error)
  }

  /* "psychxr/ovr/math.pyx":577
 *     def max(ovrVector3f a, ovrVector3f b):
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \             # <<<<<<<<<<<<<<
 *             ovr_math.Vector3f.Max(a.c_data[0], b.c_data[0])
 * 
 */
  (__pyx_v_to_return->c_data[0]) = __pyx_t_2;

  /* "psychxr/ovr/math.pyx":580
 *             ovr_math.Vector3f.Max(a.c_data[0], b.c_data[0])
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def clamped(self, float max_mag):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":575
 * 
 *     @staticmethod
 *     def max(ovrVector3f a, ovrVector3f b):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.max", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":582
 *         return to_return
 * 
 *     def clamped(self, float max_mag):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_37clamped(PyObject *__pyx_v_self, PyObject *__pyx_arg_max_mag); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_37clamped(PyObject *__pyx_v_self, PyObject *__pyx_arg_max_mag) {
  float __pyx_v_max_mag;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("clamped (wrapper)", 0);
  assert(__pyx_arg_max_mag); {
    __pyx_v_max_mag = __pyx_PyFloat_AsFloat(__pyx_arg_max_mag); if (unlikely((__pyx_v_max_mag == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 582, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.clamped", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_36clamped(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_self), ((float)__pyx_v_max_mag));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_36clamped(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, float __pyx_v_max_mag) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  OVR::Vector3f __pyx_t_2;
  __Pyx_RefNannySetupContext("clamped", 0);

  /* "psychxr/ovr/math.pyx":583
 * 
 *     def clamped(self, float max_mag):
 *         cdef ovrVector3f to_return = ovrVector3f()             # <<<<<<<<<<<<<<
 *         (<ovrVector3f>to_return).c_data[0] = \
 *             (<ovrVector3f>self).c_data[0].Clamped(max_mag)
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector3f), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 583, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":585
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \
 *             (<ovrVector3f>self).c_data[0].Clamped(max_mag)             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
  try {
    __pyx_t_2 = (__pyx_v_self->c_data[0]).Clamped(__pyx_v_max_mag);
  } catch(...) {
    __Pyx_CppExn2PyErr();
    __PYX_ERR(0, 585, __pyx_L1_error)
  }

  /* "psychxr/ovr/math.pyx":584
 *     def clamped(self, float max_mag):
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \             # <<<<<<<<<<<<<<
 *             (<ovrVector3f>self).c_data[0].Clamped(max_mag)
 * 
 */
  (__pyx_v_to_return->c_data[0]) = __pyx_t_2;

  /* "psychxr/ovr/math.pyx":587
 *             (<ovrVector3f>self).c_data[0].Clamped(max_mag)
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def isEqual(self, ovrVector3f b, float tolerance = 0.0):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":582
 *         return to_return
 * 
 *     def clamped(self, float max_mag):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.clamped", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":589
 *         return to_return
 * 
 *     def isEqual(self, ovrVector3f b, float tolerance = 0.0):             # <<<<<<<<<<<<<<
 *         return (<ovrVector3f>self).c_data[0].IsEqual(b.c_data[0], tolerance)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_39isEqual(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_39isEqual(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_b = 0;
  float __pyx_v_tolerance;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("isEqual (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_b,&__pyx_n_s_tolerance,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_b)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_tolerance);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "isEqual") < 0)) __PYX_ERR(0, 589, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_b = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)values[0]);
    if (values[1]) {
      __pyx_v_tolerance = __pyx_PyFloat_AsFloat(values[1]); if (unlikely((__pyx_v_tolerance == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 589, __pyx_L3_error)
    } else {
      __pyx_v_tolerance = ((float)0.0);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("isEqual", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 589, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.isEqual", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "b", 0))) __PYX_ERR(0, 589, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_38isEqual(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_self), __pyx_v_b, __pyx_v_tolerance);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_38isEqual(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_b, float __pyx_v_tolerance) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("isEqual", 0);

  /* "psychxr/ovr/math.pyx":590
 * 
 *     def isEqual(self, ovrVector3f b, float tolerance = 0.0):
 *         return (<ovrVector3f>self).c_data[0].IsEqual(b.c_data[0], tolerance)             # <<<<<<<<<<<<<<
 * 
 *     def compare(self, ovrVector3f b, float tolerance = 0.0):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong((__pyx_v_self->c_data[0]).IsEqual((__pyx_v_b->c_data[0]), __pyx_v_tolerance)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 590, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":589
 *         return to_return
 * 
 *     def isEqual(self, ovrVector3f b, float tolerance = 0.0):             # <<<<<<<<<<<<<<
 *         return (<ovrVector3f>self).c_data[0].IsEqual(b.c_data[0], tolerance)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.isEqual", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":592
 *         return (<ovrVector3f>self).c_data[0].IsEqual(b.c_data[0], tolerance)
 * 
 *     def compare(self, ovrVector3f b, float tolerance = 0.0):             # <<<<<<<<<<<<<<
 *         return self.is_equal(b, tolerance)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_41compare(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_41compare(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_b = 0;
  float __pyx_v_tolerance;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("compare (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_b,&__pyx_n_s_tolerance,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_b)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_tolerance);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "compare") < 0)) __PYX_ERR(0, 592, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_b = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)values[0]);
    if (values[1]) {
      __pyx_v_tolerance = __pyx_PyFloat_AsFloat(values[1]); if (unlikely((__pyx_v_tolerance == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 592, __pyx_L3_error)
    } else {
      __pyx_v_tolerance = ((float)0.0);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("compare", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 592, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.compare", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "b", 0))) __PYX_ERR(0, 592, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_40compare(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_self), __pyx_v_b, __pyx_v_tolerance);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_40compare(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_b, float __pyx_v_tolerance) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("compare", 0);

  /* "psychxr/ovr/math.pyx":593
 * 
 *     def compare(self, ovrVector3f b, float tolerance = 0.0):
 *         return self.is_equal(b, tolerance)             # <<<<<<<<<<<<<<
 * 
 *     def __getitem__(self, int idx):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_is_equal); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 593, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_tolerance); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 593, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, ((PyObject *)__pyx_v_b), __pyx_t_3};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 593, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, ((PyObject *)__pyx_v_b), __pyx_t_3};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 593, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  {
    __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 593, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (__pyx_t_4) {
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_INCREF(((PyObject *)__pyx_v_b));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_b));
    PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, ((PyObject *)__pyx_v_b));
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 593, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":592
 *         return (<ovrVector3f>self).c_data[0].IsEqual(b.c_data[0], tolerance)
 * 
 *     def compare(self, ovrVector3f b, float tolerance = 0.0):             # <<<<<<<<<<<<<<
 *         return self.is_equal(b, tolerance)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.compare", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":595
 *         return self.is_equal(b, tolerance)
 * 
 *     def __getitem__(self, int idx):             # <<<<<<<<<<<<<<
 *         assert 0 <= idx < 3
 *         cdef float* ptr_val = &self.c_data.x + idx
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_43__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_arg_idx); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_43__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_arg_idx) {
  int __pyx_v_idx;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getitem__ (wrapper)", 0);
  assert(__pyx_arg_idx); {
    __pyx_v_idx = __Pyx_PyInt_As_int(__pyx_arg_idx); if (unlikely((__pyx_v_idx == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 595, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.__getitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_42__getitem__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_self), ((int)__pyx_v_idx));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_42__getitem__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, int __pyx_v_idx) {
  float *__pyx_v_ptr_val;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__getitem__", 0);

  /* "psychxr/ovr/math.pyx":596
 * 
 *     def __getitem__(self, int idx):
 *         assert 0 <= idx < 3             # <<<<<<<<<<<<<<
 *         cdef float* ptr_val = &self.c_data.x + idx
 * 
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    __pyx_t_1 = (0 <= __pyx_v_idx);
    if (__pyx_t_1) {
      __pyx_t_1 = (__pyx_v_idx < 3);
    }
    if (unlikely(!(__pyx_t_1 != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 596, __pyx_L1_error)
    }
  }
  #endif

  /* "psychxr/ovr/math.pyx":597
 *     def __getitem__(self, int idx):
 *         assert 0 <= idx < 3
 *         cdef float* ptr_val = &self.c_data.x + idx             # <<<<<<<<<<<<<<
 * 
 *         return <float>ptr_val[0]
 */
  __pyx_v_ptr_val = ((&__pyx_v_self->c_data->x) + __pyx_v_idx);

  /* "psychxr/ovr/math.pyx":599
 *         cdef float* ptr_val = &self.c_data.x + idx
 * 
 *         return <float>ptr_val[0]             # <<<<<<<<<<<<<<
 * 
 *     def __setitem__(self, int idx, float val):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = PyFloat_FromDouble(((float)(__pyx_v_ptr_val[0]))); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 599, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":595
 *         return self.is_equal(b, tolerance)
 * 
 *     def __getitem__(self, int idx):             # <<<<<<<<<<<<<<
 *         assert 0 <= idx < 3
 *         cdef float* ptr_val = &self.c_data.x + idx
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.__getitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":601
 *         return <float>ptr_val[0]
 * 
 *     def __setitem__(self, int idx, float val):             # <<<<<<<<<<<<<<
 *         assert 0 <= idx < 3
 *         cdef float* ptr_val = &self.c_data.x + idx
 */

/* Python wrapper */
static int __pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_45__setitem__(PyObject *__pyx_v_self, PyObject *__pyx_arg_idx, PyObject *__pyx_arg_val); /*proto*/
static int __pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_45__setitem__(PyObject *__pyx_v_self, PyObject *__pyx_arg_idx, PyObject *__pyx_arg_val) {
  int __pyx_v_idx;
  float __pyx_v_val;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setitem__ (wrapper)", 0);
  assert(__pyx_arg_idx); {
    __pyx_v_idx = __Pyx_PyInt_As_int(__pyx_arg_idx); if (unlikely((__pyx_v_idx == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 601, __pyx_L3_error)
  }
  assert(__pyx_arg_val); {
    __pyx_v_val = __pyx_PyFloat_AsFloat(__pyx_arg_val); if (unlikely((__pyx_v_val == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 601, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.__setitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_44__setitem__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_self), ((int)__pyx_v_idx), ((float)__pyx_v_val));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_44__setitem__(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, int __pyx_v_idx, float __pyx_v_val) {
  float *__pyx_v_ptr_val;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__setitem__", 0);

  /* "psychxr/ovr/math.pyx":602
 * 
 *     def __setitem__(self, int idx, float val):
 *         assert 0 <= idx < 3             # <<<<<<<<<<<<<<
 *         cdef float* ptr_val = &self.c_data.x + idx
 *         ptr_val[0] = val
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    __pyx_t_1 = (0 <= __pyx_v_idx);
    if (__pyx_t_1) {
      __pyx_t_1 = (__pyx_v_idx < 3);
    }
    if (unlikely(!(__pyx_t_1 != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 602, __pyx_L1_error)
    }
  }
  #endif

  /* "psychxr/ovr/math.pyx":603
 *     def __setitem__(self, int idx, float val):
 *         assert 0 <= idx < 3
 *         cdef float* ptr_val = &self.c_data.x + idx             # <<<<<<<<<<<<<<
 *         ptr_val[0] = val
 * 
 */
  __pyx_v_ptr_val = ((&__pyx_v_self->c_data->x) + __pyx_v_idx);

  /* "psychxr/ovr/math.pyx":604
 *         assert 0 <= idx < 3
 *         cdef float* ptr_val = &self.c_data.x + idx
 *         ptr_val[0] = val             # <<<<<<<<<<<<<<
 * 
 *     def entrywiseMultiply(self, ovrVector3f b):
 */
  (__pyx_v_ptr_val[0]) = __pyx_v_val;

  /* "psychxr/ovr/math.pyx":601
 *         return <float>ptr_val[0]
 * 
 *     def __setitem__(self, int idx, float val):             # <<<<<<<<<<<<<<
 *         assert 0 <= idx < 3
 *         cdef float* ptr_val = &self.c_data.x + idx
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.__setitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":606
 *         ptr_val[0] = val
 * 
 *     def entrywiseMultiply(self, ovrVector3f b):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_47entrywiseMultiply(PyObject *__pyx_v_self, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_47entrywiseMultiply(PyObject *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("entrywiseMultiply (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "b", 0))) __PYX_ERR(0, 606, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_46entrywiseMultiply(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_46entrywiseMultiply(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_b) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("entrywiseMultiply", 0);

  /* "psychxr/ovr/math.pyx":607
 * 
 *     def entrywiseMultiply(self, ovrVector3f b):
 *         cdef ovrVector3f to_return = ovrVector3f()             # <<<<<<<<<<<<<<
 *         (<ovrVector3f>to_return).c_data[0] = \
 *             (<ovrVector3f>self).c_data[0].EntrywiseMultiply(
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector3f), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 607, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":608
 *     def entrywiseMultiply(self, ovrVector3f b):
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \             # <<<<<<<<<<<<<<
 *             (<ovrVector3f>self).c_data[0].EntrywiseMultiply(
 *             b.c_data[0])
 */
  (__pyx_v_to_return->c_data[0]) = (__pyx_v_self->c_data[0]).EntrywiseMultiply((__pyx_v_b->c_data[0]));

  /* "psychxr/ovr/math.pyx":612
 *             b.c_data[0])
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def dot(self, ovrVector3f b):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":606
 *         ptr_val[0] = val
 * 
 *     def entrywiseMultiply(self, ovrVector3f b):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.entrywiseMultiply", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":614
 *         return to_return
 * 
 *     def dot(self, ovrVector3f b):             # <<<<<<<<<<<<<<
 *         return <float>(<ovrVector3f>self).c_data[0].Dot(b.c_data[0])
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_49dot(PyObject *__pyx_v_self, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_49dot(PyObject *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("dot (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "b", 0))) __PYX_ERR(0, 614, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_48dot(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_48dot(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("dot", 0);

  /* "psychxr/ovr/math.pyx":615
 * 
 *     def dot(self, ovrVector3f b):
 *         return <float>(<ovrVector3f>self).c_data[0].Dot(b.c_data[0])             # <<<<<<<<<<<<<<
 * 
 *     def cross(self, ovrVector3f b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(((float)(__pyx_v_self->c_data[0]).Dot((__pyx_v_b->c_data[0])))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 615, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":614
 *         return to_return
 * 
 *     def dot(self, ovrVector3f b):             # <<<<<<<<<<<<<<
 *         return <float>(<ovrVector3f>self).c_data[0].Dot(b.c_data[0])
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.dot", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":617
 *         return <float>(<ovrVector3f>self).c_data[0].Dot(b.c_data[0])
 * 
 *     def cross(self, ovrVector3f b):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_51cross(PyObject *__pyx_v_self, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_51cross(PyObject *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("cross (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "b", 0))) __PYX_ERR(0, 617, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_50cross(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_50cross(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_b) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("cross", 0);

  /* "psychxr/ovr/math.pyx":618
 * 
 *     def cross(self, ovrVector3f b):
 *         cdef ovrVector3f to_return = ovrVector3f()             # <<<<<<<<<<<<<<
 *         (<ovrVector3f>to_return).c_data[0] = \
 *             (<ovrVector3f>self).c_data[0].Cross(b.c_data[0])
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector3f), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 618, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":619
 *     def cross(self, ovrVector3f b):
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \             # <<<<<<<<<<<<<<
 *             (<ovrVector3f>self).c_data[0].Cross(b.c_data[0])
 * 
 */
  (__pyx_v_to_return->c_data[0]) = (__pyx_v_self->c_data[0]).Cross((__pyx_v_b->c_data[0]));

  /* "psychxr/ovr/math.pyx":622
 *             (<ovrVector3f>self).c_data[0].Cross(b.c_data[0])
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def angle(self, ovrVector3f b):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":617
 *         return <float>(<ovrVector3f>self).c_data[0].Dot(b.c_data[0])
 * 
 *     def cross(self, ovrVector3f b):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.cross", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":624
 *         return to_return
 * 
 *     def angle(self, ovrVector3f b):             # <<<<<<<<<<<<<<
 *         return <float>(<ovrVector3f>self).c_data[0].Angle(b.c_data[0])
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_53angle(PyObject *__pyx_v_self, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_53angle(PyObject *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("angle (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "b", 0))) __PYX_ERR(0, 624, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_52angle(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_52angle(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("angle", 0);

  /* "psychxr/ovr/math.pyx":625
 * 
 *     def angle(self, ovrVector3f b):
 *         return <float>(<ovrVector3f>self).c_data[0].Angle(b.c_data[0])             # <<<<<<<<<<<<<<
 * 
 *     def lengthSq(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(((float)(__pyx_v_self->c_data[0]).Angle((__pyx_v_b->c_data[0])))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 625, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":624
 *         return to_return
 * 
 *     def angle(self, ovrVector3f b):             # <<<<<<<<<<<<<<
 *         return <float>(<ovrVector3f>self).c_data[0].Angle(b.c_data[0])
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.angle", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":627
 *         return <float>(<ovrVector3f>self).c_data[0].Angle(b.c_data[0])
 * 
 *     def lengthSq(self):             # <<<<<<<<<<<<<<
 *         return <float>(<ovrVector3f>self).c_data[0].LengthSq()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_55lengthSq(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_55lengthSq(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("lengthSq (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_54lengthSq(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_54lengthSq(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("lengthSq", 0);

  /* "psychxr/ovr/math.pyx":628
 * 
 *     def lengthSq(self):
 *         return <float>(<ovrVector3f>self).c_data[0].LengthSq()             # <<<<<<<<<<<<<<
 * 
 *     def length(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(((float)(__pyx_v_self->c_data[0]).LengthSq())); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 628, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":627
 *         return <float>(<ovrVector3f>self).c_data[0].Angle(b.c_data[0])
 * 
 *     def lengthSq(self):             # <<<<<<<<<<<<<<
 *         return <float>(<ovrVector3f>self).c_data[0].LengthSq()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.lengthSq", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":630
 *         return <float>(<ovrVector3f>self).c_data[0].LengthSq()
 * 
 *     def length(self):             # <<<<<<<<<<<<<<
 *         return <float>(<ovrVector3f>self).c_data[0].Length()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_57length(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_57length(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("length (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_56length(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_56length(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("length", 0);

  /* "psychxr/ovr/math.pyx":631
 * 
 *     def length(self):
 *         return <float>(<ovrVector3f>self).c_data[0].Length()             # <<<<<<<<<<<<<<
 * 
 *     def distanceSq(self, ovrVector3f b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(((float)(__pyx_v_self->c_data[0]).Length())); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 631, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":630
 *         return <float>(<ovrVector3f>self).c_data[0].LengthSq()
 * 
 *     def length(self):             # <<<<<<<<<<<<<<
 *         return <float>(<ovrVector3f>self).c_data[0].Length()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.length", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":633
 *         return <float>(<ovrVector3f>self).c_data[0].Length()
 * 
 *     def distanceSq(self, ovrVector3f b):             # <<<<<<<<<<<<<<
 *         return <float>(<ovrVector3f>self).c_data[0].DistanceSq(b.c_data[0])
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_59distanceSq(PyObject *__pyx_v_self, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_59distanceSq(PyObject *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("distanceSq (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "b", 0))) __PYX_ERR(0, 633, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_58distanceSq(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_58distanceSq(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("distanceSq", 0);

  /* "psychxr/ovr/math.pyx":634
 * 
 *     def distanceSq(self, ovrVector3f b):
 *         return <float>(<ovrVector3f>self).c_data[0].DistanceSq(b.c_data[0])             # <<<<<<<<<<<<<<
 * 
 *     def distance(self, ovrVector3f b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(((float)(__pyx_v_self->c_data[0]).DistanceSq((__pyx_v_b->c_data[0])))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 634, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":633
 *         return <float>(<ovrVector3f>self).c_data[0].Length()
 * 
 *     def distanceSq(self, ovrVector3f b):             # <<<<<<<<<<<<<<
 *         return <float>(<ovrVector3f>self).c_data[0].DistanceSq(b.c_data[0])
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.distanceSq", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":636
 *         return <float>(<ovrVector3f>self).c_data[0].DistanceSq(b.c_data[0])
 * 
 *     def distance(self, ovrVector3f b):             # <<<<<<<<<<<<<<
 *         return <float>(<ovrVector3f>self).c_data[0].Distance(b.c_data[0])
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_61distance(PyObject *__pyx_v_self, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_61distance(PyObject *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("distance (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "b", 0))) __PYX_ERR(0, 636, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_60distance(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_60distance(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("distance", 0);

  /* "psychxr/ovr/math.pyx":637
 * 
 *     def distance(self, ovrVector3f b):
 *         return <float>(<ovrVector3f>self).c_data[0].Distance(b.c_data[0])             # <<<<<<<<<<<<<<
 * 
 *     def isNormalized(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(((float)(__pyx_v_self->c_data[0]).Distance((__pyx_v_b->c_data[0])))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 637, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":636
 *         return <float>(<ovrVector3f>self).c_data[0].DistanceSq(b.c_data[0])
 * 
 *     def distance(self, ovrVector3f b):             # <<<<<<<<<<<<<<
 *         return <float>(<ovrVector3f>self).c_data[0].Distance(b.c_data[0])
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.distance", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":639
 *         return <float>(<ovrVector3f>self).c_data[0].Distance(b.c_data[0])
 * 
 *     def isNormalized(self):             # <<<<<<<<<<<<<<
 *         return <bint>(<ovrVector3f>self).c_data[0].IsNormalized()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_63isNormalized(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_63isNormalized(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("isNormalized (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_62isNormalized(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_62isNormalized(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("isNormalized", 0);

  /* "psychxr/ovr/math.pyx":640
 * 
 *     def isNormalized(self):
 *         return <bint>(<ovrVector3f>self).c_data[0].IsNormalized()             # <<<<<<<<<<<<<<
 * 
 *     def normalize(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(((__pyx_v_self->c_data[0]).IsNormalized() != 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 640, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":639
 *         return <float>(<ovrVector3f>self).c_data[0].Distance(b.c_data[0])
 * 
 *     def isNormalized(self):             # <<<<<<<<<<<<<<
 *         return <bint>(<ovrVector3f>self).c_data[0].IsNormalized()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.isNormalized", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":642
 *         return <bint>(<ovrVector3f>self).c_data[0].IsNormalized()
 * 
 *     def normalize(self):             # <<<<<<<<<<<<<<
 *         (<ovrVector3f>self).c_data[0].Normalize()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_65normalize(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_65normalize(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("normalize (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_64normalize(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_64normalize(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("normalize", 0);

  /* "psychxr/ovr/math.pyx":643
 * 
 *     def normalize(self):
 *         (<ovrVector3f>self).c_data[0].Normalize()             # <<<<<<<<<<<<<<
 * 
 *     def normalized(self):
 */
  (__pyx_v_self->c_data[0]).Normalize();

  /* "psychxr/ovr/math.pyx":642
 *         return <bint>(<ovrVector3f>self).c_data[0].IsNormalized()
 * 
 *     def normalize(self):             # <<<<<<<<<<<<<<
 *         (<ovrVector3f>self).c_data[0].Normalize()
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":645
 *         (<ovrVector3f>self).c_data[0].Normalize()
 * 
 *     def normalized(self):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_67normalized(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_67normalized(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("normalized (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_66normalized(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_66normalized(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("normalized", 0);

  /* "psychxr/ovr/math.pyx":646
 * 
 *     def normalized(self):
 *         cdef ovrVector3f to_return = ovrVector3f()             # <<<<<<<<<<<<<<
 *         (<ovrVector3f>to_return).c_data[0] = \
 *             (<ovrVector3f>self).c_data[0].Normalized()
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector3f), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 646, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":647
 *     def normalized(self):
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \             # <<<<<<<<<<<<<<
 *             (<ovrVector3f>self).c_data[0].Normalized()
 * 
 */
  (__pyx_v_to_return->c_data[0]) = (__pyx_v_self->c_data[0]).Normalized();

  /* "psychxr/ovr/math.pyx":650
 *             (<ovrVector3f>self).c_data[0].Normalized()
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def lerp(self, ovrVector3f b, float f):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":645
 *         (<ovrVector3f>self).c_data[0].Normalize()
 * 
 *     def normalized(self):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.normalized", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":652
 *         return to_return
 * 
 *     def lerp(self, ovrVector3f b, float f):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_69lerp(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_69lerp(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_b = 0;
  float __pyx_v_f;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("lerp (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_b,&__pyx_n_s_f,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_b)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_f)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("lerp", 1, 2, 2, 1); __PYX_ERR(0, 652, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "lerp") < 0)) __PYX_ERR(0, 652, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_b = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)values[0]);
    __pyx_v_f = __pyx_PyFloat_AsFloat(values[1]); if (unlikely((__pyx_v_f == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 652, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("lerp", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 652, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.lerp", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "b", 0))) __PYX_ERR(0, 652, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_68lerp(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_self), __pyx_v_b, __pyx_v_f);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_68lerp(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_b, float __pyx_v_f) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("lerp", 0);

  /* "psychxr/ovr/math.pyx":653
 * 
 *     def lerp(self, ovrVector3f b, float f):
 *         cdef ovrVector3f to_return = ovrVector3f()             # <<<<<<<<<<<<<<
 *         (<ovrVector3f>to_return).c_data[0] = \
 *             (<ovrVector3f>self).c_data[0].Lerp(b.c_data[0], f)
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector3f), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 653, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":654
 *     def lerp(self, ovrVector3f b, float f):
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \             # <<<<<<<<<<<<<<
 *             (<ovrVector3f>self).c_data[0].Lerp(b.c_data[0], f)
 * 
 */
  (__pyx_v_to_return->c_data[0]) = (__pyx_v_self->c_data[0]).Lerp((__pyx_v_b->c_data[0]), __pyx_v_f);

  /* "psychxr/ovr/math.pyx":657
 *             (<ovrVector3f>self).c_data[0].Lerp(b.c_data[0], f)
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def projectTo(self, ovrVector3f b):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":652
 *         return to_return
 * 
 *     def lerp(self, ovrVector3f b, float f):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.lerp", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":659
 *         return to_return
 * 
 *     def projectTo(self, ovrVector3f b):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_71projectTo(PyObject *__pyx_v_self, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_71projectTo(PyObject *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("projectTo (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "b", 0))) __PYX_ERR(0, 659, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_70projectTo(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_70projectTo(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_b) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("projectTo", 0);

  /* "psychxr/ovr/math.pyx":660
 * 
 *     def projectTo(self, ovrVector3f b):
 *         cdef ovrVector3f to_return = ovrVector3f()             # <<<<<<<<<<<<<<
 *         (<ovrVector3f>to_return).c_data[0] = \
 *             (<ovrVector3f>self).c_data[0].ProjectTo(
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector3f), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 660, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":661
 *     def projectTo(self, ovrVector3f b):
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \             # <<<<<<<<<<<<<<
 *             (<ovrVector3f>self).c_data[0].ProjectTo(
 *             b.c_data[0])
 */
  (__pyx_v_to_return->c_data[0]) = (__pyx_v_self->c_data[0]).ProjectTo((__pyx_v_b->c_data[0]));

  /* "psychxr/ovr/math.pyx":665
 *             b.c_data[0])
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def projectToPlane(self, ovrVector3f normal):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":659
 *         return to_return
 * 
 *     def projectTo(self, ovrVector3f b):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.projectTo", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":667
 *         return to_return
 * 
 *     def projectToPlane(self, ovrVector3f normal):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_73projectToPlane(PyObject *__pyx_v_self, PyObject *__pyx_v_normal); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_73projectToPlane(PyObject *__pyx_v_self, PyObject *__pyx_v_normal) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("projectToPlane (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_normal), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "normal", 0))) __PYX_ERR(0, 667, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_72projectToPlane(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_normal));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_72projectToPlane(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_normal) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("projectToPlane", 0);

  /* "psychxr/ovr/math.pyx":668
 * 
 *     def projectToPlane(self, ovrVector3f normal):
 *         cdef ovrVector3f to_return = ovrVector3f()             # <<<<<<<<<<<<<<
 *         (<ovrVector3f>to_return).c_data[0] = \
 *             (<ovrVector3f>self).c_data[0].ProjectToPlane(
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector3f), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 668, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":669
 *     def projectToPlane(self, ovrVector3f normal):
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \             # <<<<<<<<<<<<<<
 *             (<ovrVector3f>self).c_data[0].ProjectToPlane(
 *                 normal.c_data[0])
 */
  (__pyx_v_to_return->c_data[0]) = (__pyx_v_self->c_data[0]).ProjectToPlane((__pyx_v_normal->c_data[0]));

  /* "psychxr/ovr/math.pyx":673
 *                 normal.c_data[0])
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":667
 *         return to_return
 * 
 *     def projectToPlane(self, ovrVector3f normal):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.projectToPlane", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_75__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_75__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_74__reduce_cython__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_74__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_77__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_77__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_76__setstate_cython__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrVector3f_76__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrVector3f.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":677
 * 
 * cdef class ovrFovPort:
 *     def __cinit__(self,             # <<<<<<<<<<<<<<
 *                   float up_tan=0.0,
 *                   float down_tan=0.0,
 */

/* Python wrapper */
static int __pyx_pw_7psychxr_3ovr_4math_10ovrFovPort_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_7psychxr_3ovr_4math_10ovrFovPort_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  CYTHON_UNUSED float __pyx_v_up_tan;
  CYTHON_UNUSED float __pyx_v_down_tan;
  CYTHON_UNUSED float __pyx_v_left_tan;
  CYTHON_UNUSED float __pyx_v_right_tan;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_up_tan,&__pyx_n_s_down_tan,&__pyx_n_s_left_tan,&__pyx_n_s_right_tan,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_up_tan);
          if (value) { values[0] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_down_tan);
          if (value) { values[1] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_left_tan);
          if (value) { values[2] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_right_tan);
          if (value) { values[3] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(0, 677, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    if (values[0]) {
      __pyx_v_up_tan = __pyx_PyFloat_AsFloat(values[0]); if (unlikely((__pyx_v_up_tan == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 678, __pyx_L3_error)
    } else {
      __pyx_v_up_tan = ((float)0.0);
    }
    if (values[1]) {
      __pyx_v_down_tan = __pyx_PyFloat_AsFloat(values[1]); if (unlikely((__pyx_v_down_tan == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 679, __pyx_L3_error)
    } else {
      __pyx_v_down_tan = ((float)0.0);
    }
    if (values[2]) {
      __pyx_v_left_tan = __pyx_PyFloat_AsFloat(values[2]); if (unlikely((__pyx_v_left_tan == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 680, __pyx_L3_error)
    } else {
      __pyx_v_left_tan = ((float)0.0);
    }
    if (values[3]) {
      __pyx_v_right_tan = __pyx_PyFloat_AsFloat(values[3]); if (unlikely((__pyx_v_right_tan == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 681, __pyx_L3_error)
    } else {
      __pyx_v_right_tan = ((float)0.0);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 0, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 677, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrFovPort.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_10ovrFovPort___cinit__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *)__pyx_v_self), __pyx_v_up_tan, __pyx_v_down_tan, __pyx_v_left_tan, __pyx_v_right_tan);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7psychxr_3ovr_4math_10ovrFovPort___cinit__(struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *__pyx_v_self, CYTHON_UNUSED float __pyx_v_up_tan, CYTHON_UNUSED float __pyx_v_down_tan, CYTHON_UNUSED float __pyx_v_left_tan, CYTHON_UNUSED float __pyx_v_right_tan) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "psychxr/ovr/math.pyx":682
 *                   float left_tan=0.0,
 *                   float right_tan=0.0):
 *         self.c_data = &self.c_ovrFovPort             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __pyx_v_self->c_data = (&__pyx_v_self->c_ovrFovPort);

  /* "psychxr/ovr/math.pyx":677
 * 
 * cdef class ovrFovPort:
 *     def __cinit__(self,             # <<<<<<<<<<<<<<
 *                   float up_tan=0.0,
 *                   float down_tan=0.0,
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":685
 * 
 *     @property
 *     def UpTan(self):             # <<<<<<<<<<<<<<
 *         return <float>self.c_data[0].UpTan
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_10ovrFovPort_5UpTan_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_10ovrFovPort_5UpTan_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_10ovrFovPort_5UpTan___get__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_10ovrFovPort_5UpTan___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "psychxr/ovr/math.pyx":686
 *     @property
 *     def UpTan(self):
 *         return <float>self.c_data[0].UpTan             # <<<<<<<<<<<<<<
 * 
 *     @UpTan.setter
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(((float)(__pyx_v_self->c_data[0]).UpTan)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 686, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":685
 * 
 *     @property
 *     def UpTan(self):             # <<<<<<<<<<<<<<
 *         return <float>self.c_data[0].UpTan
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrFovPort.UpTan.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":689
 * 
 *     @UpTan.setter
 *     def UpTan(self, float value):             # <<<<<<<<<<<<<<
 *         self.c_data[0].UpTan = value
 * 
 */

/* Python wrapper */
static int __pyx_pw_7psychxr_3ovr_4math_10ovrFovPort_5UpTan_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_value); /*proto*/
static int __pyx_pw_7psychxr_3ovr_4math_10ovrFovPort_5UpTan_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_value) {
  float __pyx_v_value;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_value); {
    __pyx_v_value = __pyx_PyFloat_AsFloat(__pyx_arg_value); if (unlikely((__pyx_v_value == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 689, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrFovPort.UpTan.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_10ovrFovPort_5UpTan_2__set__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *)__pyx_v_self), ((float)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7psychxr_3ovr_4math_10ovrFovPort_5UpTan_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *__pyx_v_self, float __pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "psychxr/ovr/math.pyx":690
 *     @UpTan.setter
 *     def UpTan(self, float value):
 *         self.c_data[0].UpTan = value             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  (__pyx_v_self->c_data[0]).UpTan = __pyx_v_value;

  /* "psychxr/ovr/math.pyx":689
 * 
 *     @UpTan.setter
 *     def UpTan(self, float value):             # <<<<<<<<<<<<<<
 *         self.c_data[0].UpTan = value
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":693
 * 
 *     @property
 *     def DownTan(self):             # <<<<<<<<<<<<<<
 *         return <float>self.c_data[0].DownTan
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_10ovrFovPort_7DownTan_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_10ovrFovPort_7DownTan_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_10ovrFovPort_7DownTan___get__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_10ovrFovPort_7DownTan___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "psychxr/ovr/math.pyx":694
 *     @property
 *     def DownTan(self):
 *         return <float>self.c_data[0].DownTan             # <<<<<<<<<<<<<<
 * 
 *     @DownTan.setter
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(((float)(__pyx_v_self->c_data[0]).DownTan)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 694, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":693
 * 
 *     @property
 *     def DownTan(self):             # <<<<<<<<<<<<<<
 *         return <float>self.c_data[0].DownTan
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrFovPort.DownTan.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":697
 * 
 *     @DownTan.setter
 *     def DownTan(self, float value):             # <<<<<<<<<<<<<<
 *         self.c_data[0].DownTan = value
 * 
 */

/* Python wrapper */
static int __pyx_pw_7psychxr_3ovr_4math_10ovrFovPort_7DownTan_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_value); /*proto*/
static int __pyx_pw_7psychxr_3ovr_4math_10ovrFovPort_7DownTan_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_value) {
  float __pyx_v_value;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_value); {
    __pyx_v_value = __pyx_PyFloat_AsFloat(__pyx_arg_value); if (unlikely((__pyx_v_value == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 697, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrFovPort.DownTan.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_10ovrFovPort_7DownTan_2__set__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *)__pyx_v_self), ((float)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7psychxr_3ovr_4math_10ovrFovPort_7DownTan_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *__pyx_v_self, float __pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "psychxr/ovr/math.pyx":698
 *     @DownTan.setter
 *     def DownTan(self, float value):
 *         self.c_data[0].DownTan = value             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  (__pyx_v_self->c_data[0]).DownTan = __pyx_v_value;

  /* "psychxr/ovr/math.pyx":697
 * 
 *     @DownTan.setter
 *     def DownTan(self, float value):             # <<<<<<<<<<<<<<
 *         self.c_data[0].DownTan = value
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":701
 * 
 *     @property
 *     def LeftTan(self):             # <<<<<<<<<<<<<<
 *         return <float>self.c_data[0].LeftTan
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_10ovrFovPort_7LeftTan_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_10ovrFovPort_7LeftTan_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_10ovrFovPort_7LeftTan___get__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_10ovrFovPort_7LeftTan___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "psychxr/ovr/math.pyx":702
 *     @property
 *     def LeftTan(self):
 *         return <float>self.c_data[0].LeftTan             # <<<<<<<<<<<<<<
 * 
 *     @LeftTan.setter
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(((float)(__pyx_v_self->c_data[0]).LeftTan)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 702, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":701
 * 
 *     @property
 *     def LeftTan(self):             # <<<<<<<<<<<<<<
 *         return <float>self.c_data[0].LeftTan
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrFovPort.LeftTan.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":705
 * 
 *     @LeftTan.setter
 *     def LeftTan(self, float value):             # <<<<<<<<<<<<<<
 *         self.c_data[0].LeftTan = value
 * 
 */

/* Python wrapper */
static int __pyx_pw_7psychxr_3ovr_4math_10ovrFovPort_7LeftTan_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_value); /*proto*/
static int __pyx_pw_7psychxr_3ovr_4math_10ovrFovPort_7LeftTan_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_value) {
  float __pyx_v_value;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_value); {
    __pyx_v_value = __pyx_PyFloat_AsFloat(__pyx_arg_value); if (unlikely((__pyx_v_value == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 705, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrFovPort.LeftTan.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_10ovrFovPort_7LeftTan_2__set__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *)__pyx_v_self), ((float)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7psychxr_3ovr_4math_10ovrFovPort_7LeftTan_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *__pyx_v_self, float __pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "psychxr/ovr/math.pyx":706
 *     @LeftTan.setter
 *     def LeftTan(self, float value):
 *         self.c_data[0].LeftTan = value             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  (__pyx_v_self->c_data[0]).LeftTan = __pyx_v_value;

  /* "psychxr/ovr/math.pyx":705
 * 
 *     @LeftTan.setter
 *     def LeftTan(self, float value):             # <<<<<<<<<<<<<<
 *         self.c_data[0].LeftTan = value
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":709
 * 
 *     @property
 *     def RightTan(self):             # <<<<<<<<<<<<<<
 *         return <float>self.c_data[0].RightTan
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_10ovrFovPort_8RightTan_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_10ovrFovPort_8RightTan_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_10ovrFovPort_8RightTan___get__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_10ovrFovPort_8RightTan___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "psychxr/ovr/math.pyx":710
 *     @property
 *     def RightTan(self):
 *         return <float>self.c_data[0].RightTan             # <<<<<<<<<<<<<<
 * 
 *     @RightTan.setter
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(((float)(__pyx_v_self->c_data[0]).RightTan)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 710, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":709
 * 
 *     @property
 *     def RightTan(self):             # <<<<<<<<<<<<<<
 *         return <float>self.c_data[0].RightTan
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrFovPort.RightTan.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":713
 * 
 *     @RightTan.setter
 *     def RightTan(self, float value):             # <<<<<<<<<<<<<<
 *         self.c_data[0].RightTan = value
 * 
 */

/* Python wrapper */
static int __pyx_pw_7psychxr_3ovr_4math_10ovrFovPort_8RightTan_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_value); /*proto*/
static int __pyx_pw_7psychxr_3ovr_4math_10ovrFovPort_8RightTan_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_value) {
  float __pyx_v_value;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_value); {
    __pyx_v_value = __pyx_PyFloat_AsFloat(__pyx_arg_value); if (unlikely((__pyx_v_value == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 713, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrFovPort.RightTan.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_10ovrFovPort_8RightTan_2__set__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *)__pyx_v_self), ((float)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7psychxr_3ovr_4math_10ovrFovPort_8RightTan_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *__pyx_v_self, float __pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "psychxr/ovr/math.pyx":714
 *     @RightTan.setter
 *     def RightTan(self, float value):
 *         self.c_data[0].RightTan = value             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  (__pyx_v_self->c_data[0]).RightTan = __pyx_v_value;

  /* "psychxr/ovr/math.pyx":713
 * 
 *     @RightTan.setter
 *     def RightTan(self, float value):             # <<<<<<<<<<<<<<
 *         self.c_data[0].RightTan = value
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":717
 * 
 *     @staticmethod
 *     def max(ovrFovPort a, ovrFovPort b):             # <<<<<<<<<<<<<<
 *         cdef ovrFovPort to_return = ovrFovPort()
 *         (<ovrFovPort>to_return).c_data[0].UpTan = cmath.fmax(
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_10ovrFovPort_3max(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_7psychxr_3ovr_4math_10ovrFovPort_3max = {"max", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_10ovrFovPort_3max, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7psychxr_3ovr_4math_10ovrFovPort_3max(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *__pyx_v_a = 0;
  struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *__pyx_v_b = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("max (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_a,&__pyx_n_s_b,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_a)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_b)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("max", 1, 2, 2, 1); __PYX_ERR(0, 717, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "max") < 0)) __PYX_ERR(0, 717, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_a = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *)values[0]);
    __pyx_v_b = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("max", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 717, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrFovPort.max", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_a), __pyx_ptype_7psychxr_3ovr_4math_ovrFovPort, 1, "a", 0))) __PYX_ERR(0, 717, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrFovPort, 1, "b", 0))) __PYX_ERR(0, 717, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_10ovrFovPort_2max(__pyx_v_a, __pyx_v_b);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_10ovrFovPort_2max(struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *__pyx_v_a, struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *__pyx_v_b) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("max", 0);

  /* "psychxr/ovr/math.pyx":718
 *     @staticmethod
 *     def max(ovrFovPort a, ovrFovPort b):
 *         cdef ovrFovPort to_return = ovrFovPort()             # <<<<<<<<<<<<<<
 *         (<ovrFovPort>to_return).c_data[0].UpTan = cmath.fmax(
 *             a.c_data[0].UpTan, b.c_data[0].UpTan)
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrFovPort), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 718, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":719
 *     def max(ovrFovPort a, ovrFovPort b):
 *         cdef ovrFovPort to_return = ovrFovPort()
 *         (<ovrFovPort>to_return).c_data[0].UpTan = cmath.fmax(             # <<<<<<<<<<<<<<
 *             a.c_data[0].UpTan, b.c_data[0].UpTan)
 *         (<ovrFovPort>to_return).c_data[0].DownTan = cmath.fmax(
 */
  (__pyx_v_to_return->c_data[0]).UpTan = fmax((__pyx_v_a->c_data[0]).UpTan, (__pyx_v_b->c_data[0]).UpTan);

  /* "psychxr/ovr/math.pyx":721
 *         (<ovrFovPort>to_return).c_data[0].UpTan = cmath.fmax(
 *             a.c_data[0].UpTan, b.c_data[0].UpTan)
 *         (<ovrFovPort>to_return).c_data[0].DownTan = cmath.fmax(             # <<<<<<<<<<<<<<
 *             a.c_data[0].DownTan, b.c_data[0].DownTan)
 *         (<ovrFovPort>to_return).c_data[0].LeftTan = cmath.fmax(
 */
  (__pyx_v_to_return->c_data[0]).DownTan = fmax((__pyx_v_a->c_data[0]).DownTan, (__pyx_v_b->c_data[0]).DownTan);

  /* "psychxr/ovr/math.pyx":723
 *         (<ovrFovPort>to_return).c_data[0].DownTan = cmath.fmax(
 *             a.c_data[0].DownTan, b.c_data[0].DownTan)
 *         (<ovrFovPort>to_return).c_data[0].LeftTan = cmath.fmax(             # <<<<<<<<<<<<<<
 *             a.c_data[0].LeftTan, b.c_data[0].LeftTan)
 *         (<ovrFovPort>to_return).c_data[0].RightTan = cmath.fmax(
 */
  (__pyx_v_to_return->c_data[0]).LeftTan = fmax((__pyx_v_a->c_data[0]).LeftTan, (__pyx_v_b->c_data[0]).LeftTan);

  /* "psychxr/ovr/math.pyx":725
 *         (<ovrFovPort>to_return).c_data[0].LeftTan = cmath.fmax(
 *             a.c_data[0].LeftTan, b.c_data[0].LeftTan)
 *         (<ovrFovPort>to_return).c_data[0].RightTan = cmath.fmax(             # <<<<<<<<<<<<<<
 *             a.c_data[0].RightTan, b.c_data[0].RightTan)
 * 
 */
  (__pyx_v_to_return->c_data[0]).RightTan = fmax((__pyx_v_a->c_data[0]).RightTan, (__pyx_v_b->c_data[0]).RightTan);

  /* "psychxr/ovr/math.pyx":728
 *             a.c_data[0].RightTan, b.c_data[0].RightTan)
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":717
 * 
 *     @staticmethod
 *     def max(ovrFovPort a, ovrFovPort b):             # <<<<<<<<<<<<<<
 *         cdef ovrFovPort to_return = ovrFovPort()
 *         (<ovrFovPort>to_return).c_data[0].UpTan = cmath.fmax(
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrFovPort.max", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":731
 * 
 *     @staticmethod
 *     def min(ovrFovPort a, ovrFovPort b):             # <<<<<<<<<<<<<<
 *         cdef ovrFovPort to_return = ovrFovPort()
 *         (<ovrFovPort>to_return).c_data[0].UpTan = cmath.fmin(
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_10ovrFovPort_5min(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_7psychxr_3ovr_4math_10ovrFovPort_5min = {"min", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_10ovrFovPort_5min, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7psychxr_3ovr_4math_10ovrFovPort_5min(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *__pyx_v_a = 0;
  struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *__pyx_v_b = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("min (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_a,&__pyx_n_s_b,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_a)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_b)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("min", 1, 2, 2, 1); __PYX_ERR(0, 731, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "min") < 0)) __PYX_ERR(0, 731, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_a = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *)values[0]);
    __pyx_v_b = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("min", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 731, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrFovPort.min", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_a), __pyx_ptype_7psychxr_3ovr_4math_ovrFovPort, 1, "a", 0))) __PYX_ERR(0, 731, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrFovPort, 1, "b", 0))) __PYX_ERR(0, 731, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_10ovrFovPort_4min(__pyx_v_a, __pyx_v_b);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_10ovrFovPort_4min(struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *__pyx_v_a, struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *__pyx_v_b) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("min", 0);

  /* "psychxr/ovr/math.pyx":732
 *     @staticmethod
 *     def min(ovrFovPort a, ovrFovPort b):
 *         cdef ovrFovPort to_return = ovrFovPort()             # <<<<<<<<<<<<<<
 *         (<ovrFovPort>to_return).c_data[0].UpTan = cmath.fmin(
 *             a.c_data[0].UpTan, b.c_data[0].UpTan)
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrFovPort), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 732, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":733
 *     def min(ovrFovPort a, ovrFovPort b):
 *         cdef ovrFovPort to_return = ovrFovPort()
 *         (<ovrFovPort>to_return).c_data[0].UpTan = cmath.fmin(             # <<<<<<<<<<<<<<
 *             a.c_data[0].UpTan, b.c_data[0].UpTan)
 *         (<ovrFovPort>to_return).c_data[0].DownTan = cmath.fmin(
 */
  (__pyx_v_to_return->c_data[0]).UpTan = fmin((__pyx_v_a->c_data[0]).UpTan, (__pyx_v_b->c_data[0]).UpTan);

  /* "psychxr/ovr/math.pyx":735
 *         (<ovrFovPort>to_return).c_data[0].UpTan = cmath.fmin(
 *             a.c_data[0].UpTan, b.c_data[0].UpTan)
 *         (<ovrFovPort>to_return).c_data[0].DownTan = cmath.fmin(             # <<<<<<<<<<<<<<
 *             a.c_data[0].DownTan, b.c_data[0].DownTan)
 *         (<ovrFovPort>to_return).c_data[0].LeftTan = cmath.fmin(
 */
  (__pyx_v_to_return->c_data[0]).DownTan = fmin((__pyx_v_a->c_data[0]).DownTan, (__pyx_v_b->c_data[0]).DownTan);

  /* "psychxr/ovr/math.pyx":737
 *         (<ovrFovPort>to_return).c_data[0].DownTan = cmath.fmin(
 *             a.c_data[0].DownTan, b.c_data[0].DownTan)
 *         (<ovrFovPort>to_return).c_data[0].LeftTan = cmath.fmin(             # <<<<<<<<<<<<<<
 *             a.c_data[0].LeftTan, b.c_data[0].LeftTan)
 *         (<ovrFovPort>to_return).c_data[0].RightTan = cmath.fmin(
 */
  (__pyx_v_to_return->c_data[0]).LeftTan = fmin((__pyx_v_a->c_data[0]).LeftTan, (__pyx_v_b->c_data[0]).LeftTan);

  /* "psychxr/ovr/math.pyx":739
 *         (<ovrFovPort>to_return).c_data[0].LeftTan = cmath.fmin(
 *             a.c_data[0].LeftTan, b.c_data[0].LeftTan)
 *         (<ovrFovPort>to_return).c_data[0].RightTan = cmath.fmin(             # <<<<<<<<<<<<<<
 *             a.c_data[0].RightTan, b.c_data[0].RightTan)
 * 
 */
  (__pyx_v_to_return->c_data[0]).RightTan = fmin((__pyx_v_a->c_data[0]).RightTan, (__pyx_v_b->c_data[0]).RightTan);

  /* "psychxr/ovr/math.pyx":742
 *             a.c_data[0].RightTan, b.c_data[0].RightTan)
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 * cdef class ovrQuatf:
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":731
 * 
 *     @staticmethod
 *     def min(ovrFovPort a, ovrFovPort b):             # <<<<<<<<<<<<<<
 *         cdef ovrFovPort to_return = ovrFovPort()
 *         (<ovrFovPort>to_return).c_data[0].UpTan = cmath.fmin(
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrFovPort.min", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_10ovrFovPort_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_10ovrFovPort_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_10ovrFovPort_6__reduce_cython__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_10ovrFovPort_6__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__9, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrFovPort.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_10ovrFovPort_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_10ovrFovPort_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_10ovrFovPort_8__setstate_cython__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_10ovrFovPort_8__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__10, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrFovPort.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":745
 * 
 * cdef class ovrQuatf:
 *     def __cinit__(self, *args):             # <<<<<<<<<<<<<<
 *         self.c_data = &self.c_Quatf
 * 
 */

/* Python wrapper */
static int __pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_args = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__cinit__", 0))) return -1;
  __Pyx_INCREF(__pyx_args);
  __pyx_v_args = __pyx_args;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf___cinit__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_self), __pyx_v_args);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_args);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf___cinit__(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self, PyObject *__pyx_v_args) {
  int __pyx_v_nargin;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  float __pyx_t_8;
  OVR::Quatf __pyx_t_9;
  float __pyx_t_10;
  float __pyx_t_11;
  float __pyx_t_12;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "psychxr/ovr/math.pyx":746
 * cdef class ovrQuatf:
 *     def __cinit__(self, *args):
 *         self.c_data = &self.c_Quatf             # <<<<<<<<<<<<<<
 * 
 *         cdef int nargin = <int>len(args)  # get number of arguments
 */
  __pyx_v_self->c_data = (&__pyx_v_self->c_Quatf);

  /* "psychxr/ovr/math.pyx":748
 *         self.c_data = &self.c_Quatf
 * 
 *         cdef int nargin = <int>len(args)  # get number of arguments             # <<<<<<<<<<<<<<
 *         if nargin == 0:
 *             self.c_data[0] = ovr_math.Quatf.Identity()
 */
  __pyx_t_1 = PyTuple_GET_SIZE(__pyx_v_args); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 748, __pyx_L1_error)
  __pyx_v_nargin = ((int)__pyx_t_1);

  /* "psychxr/ovr/math.pyx":749
 * 
 *         cdef int nargin = <int>len(args)  # get number of arguments
 *         if nargin == 0:             # <<<<<<<<<<<<<<
 *             self.c_data[0] = ovr_math.Quatf.Identity()
 *         elif nargin == 1:
 */
  switch (__pyx_v_nargin) {
    case 0:

    /* "psychxr/ovr/math.pyx":750
 *         cdef int nargin = <int>len(args)  # get number of arguments
 *         if nargin == 0:
 *             self.c_data[0] = ovr_math.Quatf.Identity()             # <<<<<<<<<<<<<<
 *         elif nargin == 1:
 *             if isinstance(ovrMatrix4f, args[0]):
 */
    (__pyx_v_self->c_data[0]) = OVR::Quatf::Identity();

    /* "psychxr/ovr/math.pyx":749
 * 
 *         cdef int nargin = <int>len(args)  # get number of arguments
 *         if nargin == 0:             # <<<<<<<<<<<<<<
 *             self.c_data[0] = ovr_math.Quatf.Identity()
 *         elif nargin == 1:
 */
    break;

    /* "psychxr/ovr/math.pyx":751
 *         if nargin == 0:
 *             self.c_data[0] = ovr_math.Quatf.Identity()
 *         elif nargin == 1:             # <<<<<<<<<<<<<<
 *             if isinstance(ovrMatrix4f, args[0]):
 *                 self.c_data[0] = ovr_math.Quatf(
 */
    case 1:

    /* "psychxr/ovr/math.pyx":752
 *             self.c_data[0] = ovr_math.Quatf.Identity()
 *         elif nargin == 1:
 *             if isinstance(ovrMatrix4f, args[0]):             # <<<<<<<<<<<<<<
 *                 self.c_data[0] = ovr_math.Quatf(
 *                     (<ovrMatrix4f>args[0]).c_data[0])
 */
    __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 752, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyObject_IsInstance(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f), __pyx_t_2); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 752, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_4 = (__pyx_t_3 != 0);
    if (__pyx_t_4) {

      /* "psychxr/ovr/math.pyx":754
 *             if isinstance(ovrMatrix4f, args[0]):
 *                 self.c_data[0] = ovr_math.Quatf(
 *                     (<ovrMatrix4f>args[0]).c_data[0])             # <<<<<<<<<<<<<<
 *             elif isinstance(ovrMatrix4f, args[0]):
 *                 self.c_data[0] = ovr_math.Quatf(
 */
      __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 754, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);

      /* "psychxr/ovr/math.pyx":753
 *         elif nargin == 1:
 *             if isinstance(ovrMatrix4f, args[0]):
 *                 self.c_data[0] = ovr_math.Quatf(             # <<<<<<<<<<<<<<
 *                     (<ovrMatrix4f>args[0]).c_data[0])
 *             elif isinstance(ovrMatrix4f, args[0]):
 */
      (__pyx_v_self->c_data[0]) = OVR::Quatf((((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_t_2)->c_data[0]));
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "psychxr/ovr/math.pyx":752
 *             self.c_data[0] = ovr_math.Quatf.Identity()
 *         elif nargin == 1:
 *             if isinstance(ovrMatrix4f, args[0]):             # <<<<<<<<<<<<<<
 *                 self.c_data[0] = ovr_math.Quatf(
 *                     (<ovrMatrix4f>args[0]).c_data[0])
 */
      goto __pyx_L3;
    }

    /* "psychxr/ovr/math.pyx":755
 *                 self.c_data[0] = ovr_math.Quatf(
 *                     (<ovrMatrix4f>args[0]).c_data[0])
 *             elif isinstance(ovrMatrix4f, args[0]):             # <<<<<<<<<<<<<<
 *                 self.c_data[0] = ovr_math.Quatf(
 *                     (<ovrQuatf>args[0]).c_data[0])
 */
    __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 755, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyObject_IsInstance(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f), __pyx_t_2); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 755, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_3 = (__pyx_t_4 != 0);
    if (__pyx_t_3) {

      /* "psychxr/ovr/math.pyx":757
 *             elif isinstance(ovrMatrix4f, args[0]):
 *                 self.c_data[0] = ovr_math.Quatf(
 *                     (<ovrQuatf>args[0]).c_data[0])             # <<<<<<<<<<<<<<
 *         elif nargin == 2:
 *             # quaternion from axis and angle
 */
      __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 757, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);

      /* "psychxr/ovr/math.pyx":756
 *                     (<ovrMatrix4f>args[0]).c_data[0])
 *             elif isinstance(ovrMatrix4f, args[0]):
 *                 self.c_data[0] = ovr_math.Quatf(             # <<<<<<<<<<<<<<
 *                     (<ovrQuatf>args[0]).c_data[0])
 *         elif nargin == 2:
 */
      (__pyx_v_self->c_data[0]) = OVR::Quatf((((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_t_2)->c_data[0]));
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "psychxr/ovr/math.pyx":755
 *                 self.c_data[0] = ovr_math.Quatf(
 *                     (<ovrMatrix4f>args[0]).c_data[0])
 *             elif isinstance(ovrMatrix4f, args[0]):             # <<<<<<<<<<<<<<
 *                 self.c_data[0] = ovr_math.Quatf(
 *                     (<ovrQuatf>args[0]).c_data[0])
 */
    }
    __pyx_L3:;

    /* "psychxr/ovr/math.pyx":751
 *         if nargin == 0:
 *             self.c_data[0] = ovr_math.Quatf.Identity()
 *         elif nargin == 1:             # <<<<<<<<<<<<<<
 *             if isinstance(ovrMatrix4f, args[0]):
 *                 self.c_data[0] = ovr_math.Quatf(
 */
    break;

    /* "psychxr/ovr/math.pyx":758
 *                 self.c_data[0] = ovr_math.Quatf(
 *                     (<ovrQuatf>args[0]).c_data[0])
 *         elif nargin == 2:             # <<<<<<<<<<<<<<
 *             # quaternion from axis and angle
 *             if isinstance(args[0], ovrVector3f) and \
 */
    case 2:

    /* "psychxr/ovr/math.pyx":760
 *         elif nargin == 2:
 *             # quaternion from axis and angle
 *             if isinstance(args[0], ovrVector3f) and \             # <<<<<<<<<<<<<<
 *                     isinstance(args[1], (int, float)):
 *                 self.c_data[0] = ovr_math.Quatf(
 */
    __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 760, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_TypeCheck(__pyx_t_2, __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f); 
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_5 = (__pyx_t_4 != 0);
    if (__pyx_t_5) {
    } else {
      __pyx_t_3 = __pyx_t_5;
      goto __pyx_L5_bool_binop_done;
    }

    /* "psychxr/ovr/math.pyx":761
 *             # quaternion from axis and angle
 *             if isinstance(args[0], ovrVector3f) and \
 *                     isinstance(args[1], (int, float)):             # <<<<<<<<<<<<<<
 *                 self.c_data[0] = ovr_math.Quatf(
 *                     (<ovrVector3f>args[0]).c_data[0], <float>args[1])
 */
    __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 761, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyInt_Check(__pyx_t_2); 
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_t_6 = PyFloat_Check(__pyx_t_2); 
    __pyx_t_4 = (__pyx_t_6 != 0);
    __pyx_t_5 = __pyx_t_4;
    __pyx_L7_bool_binop_done:;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_4 = (__pyx_t_5 != 0);
    __pyx_t_3 = __pyx_t_4;
    __pyx_L5_bool_binop_done:;

    /* "psychxr/ovr/math.pyx":760
 *         elif nargin == 2:
 *             # quaternion from axis and angle
 *             if isinstance(args[0], ovrVector3f) and \             # <<<<<<<<<<<<<<
 *                     isinstance(args[1], (int, float)):
 *                 self.c_data[0] = ovr_math.Quatf(
 */
    if (__pyx_t_3) {

      /* "psychxr/ovr/math.pyx":763
 *                     isinstance(args[1], (int, float)):
 *                 self.c_data[0] = ovr_math.Quatf(
 *                     (<ovrVector3f>args[0]).c_data[0], <float>args[1])             # <<<<<<<<<<<<<<
 *         elif nargin == 4:
 *             self.c_data[0] = ovr_math.Quatf(
 */
      __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 763, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_7 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 763, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_8 = __pyx_PyFloat_AsFloat(__pyx_t_7); if (unlikely((__pyx_t_8 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 763, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

      /* "psychxr/ovr/math.pyx":762
 *             if isinstance(args[0], ovrVector3f) and \
 *                     isinstance(args[1], (int, float)):
 *                 self.c_data[0] = ovr_math.Quatf(             # <<<<<<<<<<<<<<
 *                     (<ovrVector3f>args[0]).c_data[0], <float>args[1])
 *         elif nargin == 4:
 */
      try {
        __pyx_t_9 = OVR::Quatf((((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_t_2)->c_data[0]), ((float)__pyx_t_8));
      } catch(...) {
        __Pyx_CppExn2PyErr();
        __PYX_ERR(0, 762, __pyx_L1_error)
      }
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      (__pyx_v_self->c_data[0]) = __pyx_t_9;

      /* "psychxr/ovr/math.pyx":760
 *         elif nargin == 2:
 *             # quaternion from axis and angle
 *             if isinstance(args[0], ovrVector3f) and \             # <<<<<<<<<<<<<<
 *                     isinstance(args[1], (int, float)):
 *                 self.c_data[0] = ovr_math.Quatf(
 */
    }

    /* "psychxr/ovr/math.pyx":758
 *                 self.c_data[0] = ovr_math.Quatf(
 *                     (<ovrQuatf>args[0]).c_data[0])
 *         elif nargin == 2:             # <<<<<<<<<<<<<<
 *             # quaternion from axis and angle
 *             if isinstance(args[0], ovrVector3f) and \
 */
    break;

    /* "psychxr/ovr/math.pyx":764
 *                 self.c_data[0] = ovr_math.Quatf(
 *                     (<ovrVector3f>args[0]).c_data[0], <float>args[1])
 *         elif nargin == 4:             # <<<<<<<<<<<<<<
 *             self.c_data[0] = ovr_math.Quatf(
 *                 <float>args[0],
 */
    case 4:

    /* "psychxr/ovr/math.pyx":766
 *         elif nargin == 4:
 *             self.c_data[0] = ovr_math.Quatf(
 *                 <float>args[0],             # <<<<<<<<<<<<<<
 *                 <float>args[1],
 *                 <float>args[2],
 */
    __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 766, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_8 = __pyx_PyFloat_AsFloat(__pyx_t_2); if (unlikely((__pyx_t_8 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 766, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "psychxr/ovr/math.pyx":767
 *             self.c_data[0] = ovr_math.Quatf(
 *                 <float>args[0],
 *                 <float>args[1],             # <<<<<<<<<<<<<<
 *                 <float>args[2],
 *                 <float>args[3])
 */
    __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 767, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_10 = __pyx_PyFloat_AsFloat(__pyx_t_2); if (unlikely((__pyx_t_10 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 767, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "psychxr/ovr/math.pyx":768
 *                 <float>args[0],
 *                 <float>args[1],
 *                 <float>args[2],             # <<<<<<<<<<<<<<
 *                 <float>args[3])
 * 
 */
    __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 768, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_11 = __pyx_PyFloat_AsFloat(__pyx_t_2); if (unlikely((__pyx_t_11 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 768, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "psychxr/ovr/math.pyx":769
 *                 <float>args[1],
 *                 <float>args[2],
 *                 <float>args[3])             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
    __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 769, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_12 = __pyx_PyFloat_AsFloat(__pyx_t_2); if (unlikely((__pyx_t_12 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 769, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "psychxr/ovr/math.pyx":765
 *                     (<ovrVector3f>args[0]).c_data[0], <float>args[1])
 *         elif nargin == 4:
 *             self.c_data[0] = ovr_math.Quatf(             # <<<<<<<<<<<<<<
 *                 <float>args[0],
 *                 <float>args[1],
 */
    try {
      __pyx_t_9 = OVR::Quatf(((float)__pyx_t_8), ((float)__pyx_t_10), ((float)__pyx_t_11), ((float)__pyx_t_12));
    } catch(...) {
      __Pyx_CppExn2PyErr();
      __PYX_ERR(0, 765, __pyx_L1_error)
    }
    (__pyx_v_self->c_data[0]) = __pyx_t_9;

    /* "psychxr/ovr/math.pyx":764
 *                 self.c_data[0] = ovr_math.Quatf(
 *                     (<ovrVector3f>args[0]).c_data[0], <float>args[1])
 *         elif nargin == 4:             # <<<<<<<<<<<<<<
 *             self.c_data[0] = ovr_math.Quatf(
 *                 <float>args[0],
 */
    break;
    default: break;
  }

  /* "psychxr/ovr/math.pyx":745
 * 
 * cdef class ovrQuatf:
 *     def __cinit__(self, *args):             # <<<<<<<<<<<<<<
 *         self.c_data = &self.c_Quatf
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":772
 * 
 *     @property
 *     def x(self):             # <<<<<<<<<<<<<<
 *         return self.c_data.x
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_1x_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_1x_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_1x___get__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_1x___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "psychxr/ovr/math.pyx":773
 *     @property
 *     def x(self):
 *         return self.c_data.x             # <<<<<<<<<<<<<<
 * 
 *     @x.setter
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->c_data->x); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 773, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":772
 * 
 *     @property
 *     def x(self):             # <<<<<<<<<<<<<<
 *         return self.c_data.x
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.x.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":776
 * 
 *     @x.setter
 *     def x(self, float value):             # <<<<<<<<<<<<<<
 *         self.c_data.x = value
 * 
 */

/* Python wrapper */
static int __pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_1x_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_value); /*proto*/
static int __pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_1x_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_value) {
  float __pyx_v_value;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_value); {
    __pyx_v_value = __pyx_PyFloat_AsFloat(__pyx_arg_value); if (unlikely((__pyx_v_value == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 776, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.x.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_1x_2__set__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_self), ((float)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_1x_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self, float __pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "psychxr/ovr/math.pyx":777
 *     @x.setter
 *     def x(self, float value):
 *         self.c_data.x = value             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __pyx_v_self->c_data->x = __pyx_v_value;

  /* "psychxr/ovr/math.pyx":776
 * 
 *     @x.setter
 *     def x(self, float value):             # <<<<<<<<<<<<<<
 *         self.c_data.x = value
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":780
 * 
 *     @property
 *     def y(self):             # <<<<<<<<<<<<<<
 *         return self.c_data.y
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_1y_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_1y_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_1y___get__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_1y___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "psychxr/ovr/math.pyx":781
 *     @property
 *     def y(self):
 *         return self.c_data.y             # <<<<<<<<<<<<<<
 * 
 *     @y.setter
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->c_data->y); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 781, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":780
 * 
 *     @property
 *     def y(self):             # <<<<<<<<<<<<<<
 *         return self.c_data.y
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.y.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":784
 * 
 *     @y.setter
 *     def y(self, float value):             # <<<<<<<<<<<<<<
 *         self.c_data.y = value
 * 
 */

/* Python wrapper */
static int __pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_1y_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_value); /*proto*/
static int __pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_1y_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_value) {
  float __pyx_v_value;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_value); {
    __pyx_v_value = __pyx_PyFloat_AsFloat(__pyx_arg_value); if (unlikely((__pyx_v_value == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 784, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.y.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_1y_2__set__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_self), ((float)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_1y_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self, float __pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "psychxr/ovr/math.pyx":785
 *     @y.setter
 *     def y(self, float value):
 *         self.c_data.y = value             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __pyx_v_self->c_data->y = __pyx_v_value;

  /* "psychxr/ovr/math.pyx":784
 * 
 *     @y.setter
 *     def y(self, float value):             # <<<<<<<<<<<<<<
 *         self.c_data.y = value
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":788
 * 
 *     @property
 *     def z(self):             # <<<<<<<<<<<<<<
 *         return self.c_data.z
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_1z_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_1z_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_1z___get__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_1z___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "psychxr/ovr/math.pyx":789
 *     @property
 *     def z(self):
 *         return self.c_data.z             # <<<<<<<<<<<<<<
 * 
 *     @z.setter
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->c_data->z); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 789, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":788
 * 
 *     @property
 *     def z(self):             # <<<<<<<<<<<<<<
 *         return self.c_data.z
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.z.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":792
 * 
 *     @z.setter
 *     def z(self, float value):             # <<<<<<<<<<<<<<
 *         self.c_data.z = value
 * 
 */

/* Python wrapper */
static int __pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_1z_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_value); /*proto*/
static int __pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_1z_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_value) {
  float __pyx_v_value;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_value); {
    __pyx_v_value = __pyx_PyFloat_AsFloat(__pyx_arg_value); if (unlikely((__pyx_v_value == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 792, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.z.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_1z_2__set__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_self), ((float)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_1z_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self, float __pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "psychxr/ovr/math.pyx":793
 *     @z.setter
 *     def z(self, float value):
 *         self.c_data.z = value             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __pyx_v_self->c_data->z = __pyx_v_value;

  /* "psychxr/ovr/math.pyx":792
 * 
 *     @z.setter
 *     def z(self, float value):             # <<<<<<<<<<<<<<
 *         self.c_data.z = value
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":796
 * 
 *     @property
 *     def w(self):             # <<<<<<<<<<<<<<
 *         return self.c_data.w
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_1w_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_1w_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_1w___get__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_1w___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "psychxr/ovr/math.pyx":797
 *     @property
 *     def w(self):
 *         return self.c_data.w             # <<<<<<<<<<<<<<
 * 
 *     @w.setter
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->c_data->w); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 797, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":796
 * 
 *     @property
 *     def w(self):             # <<<<<<<<<<<<<<
 *         return self.c_data.w
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.w.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":800
 * 
 *     @w.setter
 *     def w(self, float value):             # <<<<<<<<<<<<<<
 *         self.c_data.w = value
 * 
 */

/* Python wrapper */
static int __pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_1w_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_value); /*proto*/
static int __pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_1w_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_arg_value) {
  float __pyx_v_value;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  assert(__pyx_arg_value); {
    __pyx_v_value = __pyx_PyFloat_AsFloat(__pyx_arg_value); if (unlikely((__pyx_v_value == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 800, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.w.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_1w_2__set__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_self), ((float)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_1w_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self, float __pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "psychxr/ovr/math.pyx":801
 *     @w.setter
 *     def w(self, float value):
 *         self.c_data.w = value             # <<<<<<<<<<<<<<
 * 
 *     def asTuple(self):
 */
  __pyx_v_self->c_data->w = __pyx_v_value;

  /* "psychxr/ovr/math.pyx":800
 * 
 *     @w.setter
 *     def w(self, float value):             # <<<<<<<<<<<<<<
 *         self.c_data.w = value
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":803
 *         self.c_data.w = value
 * 
 *     def asTuple(self):             # <<<<<<<<<<<<<<
 *         return self.c_data.x, self.c_data.y, self.c_data.z, self.c_data.w
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_3asTuple(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_3asTuple(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("asTuple (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_2asTuple(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_2asTuple(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("asTuple", 0);

  /* "psychxr/ovr/math.pyx":804
 * 
 *     def asTuple(self):
 *         return self.c_data.x, self.c_data.y, self.c_data.z, self.c_data.w             # <<<<<<<<<<<<<<
 * 
 *     def asList(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->c_data->x); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 804, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->c_data->y); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 804, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_self->c_data->z); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 804, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyFloat_FromDouble(__pyx_v_self->c_data->w); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 804, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 804, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":803
 *         self.c_data.w = value
 * 
 *     def asTuple(self):             # <<<<<<<<<<<<<<
 *         return self.c_data.x, self.c_data.y, self.c_data.z, self.c_data.w
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.asTuple", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":806
 *         return self.c_data.x, self.c_data.y, self.c_data.z, self.c_data.w
 * 
 *     def asList(self):             # <<<<<<<<<<<<<<
 *         return [self.c_data.x, self.c_data.y, self.c_data.z, self.c_data.w]
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_5asList(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_5asList(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("asList (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_4asList(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_4asList(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("asList", 0);

  /* "psychxr/ovr/math.pyx":807
 * 
 *     def asList(self):
 *         return [self.c_data.x, self.c_data.y, self.c_data.z, self.c_data.w]             # <<<<<<<<<<<<<<
 * 
 *     def __len__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->c_data->x); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 807, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->c_data->y); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 807, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_self->c_data->z); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 807, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyFloat_FromDouble(__pyx_v_self->c_data->w); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 807, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyList_New(4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 807, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyList_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyList_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyList_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyList_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":806
 *         return self.c_data.x, self.c_data.y, self.c_data.z, self.c_data.w
 * 
 *     def asList(self):             # <<<<<<<<<<<<<<
 *         return [self.c_data.x, self.c_data.y, self.c_data.z, self.c_data.w]
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.asList", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":809
 *         return [self.c_data.x, self.c_data.y, self.c_data.z, self.c_data.w]
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         return 4
 * 
 */

/* Python wrapper */
static Py_ssize_t __pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_7__len__(PyObject *__pyx_v_self); /*proto*/
static Py_ssize_t __pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_7__len__(PyObject *__pyx_v_self) {
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__len__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_6__len__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static Py_ssize_t __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_6__len__(CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self) {
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__len__", 0);

  /* "psychxr/ovr/math.pyx":810
 * 
 *     def __len__(self):
 *         return 4             # <<<<<<<<<<<<<<
 * 
 *     def __neg__(self):
 */
  __pyx_r = 4;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":809
 *         return [self.c_data.x, self.c_data.y, self.c_data.z, self.c_data.w]
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         return 4
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":812
 *         return 4
 * 
 *     def __neg__(self):             # <<<<<<<<<<<<<<
 *         cdef ovrQuatf to_return = ovrQuatf(
 *             -self.c_data.x, -self.c_data.y, -self.c_data.z, -self.c_data.w)
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_9__neg__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_9__neg__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__neg__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_8__neg__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_8__neg__(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("__neg__", 0);

  /* "psychxr/ovr/math.pyx":814
 *     def __neg__(self):
 *         cdef ovrQuatf to_return = ovrQuatf(
 *             -self.c_data.x, -self.c_data.y, -self.c_data.z, -self.c_data.w)             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
  __pyx_t_1 = PyFloat_FromDouble((-__pyx_v_self->c_data->x)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 814, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyFloat_FromDouble((-__pyx_v_self->c_data->y)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 814, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyFloat_FromDouble((-__pyx_v_self->c_data->z)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 814, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyFloat_FromDouble((-__pyx_v_self->c_data->w)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 814, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);

  /* "psychxr/ovr/math.pyx":813
 * 
 *     def __neg__(self):
 *         cdef ovrQuatf to_return = ovrQuatf(             # <<<<<<<<<<<<<<
 *             -self.c_data.x, -self.c_data.y, -self.c_data.z, -self.c_data.w)
 * 
 */
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 813, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrQuatf), __pyx_t_5, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 813, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "psychxr/ovr/math.pyx":816
 *             -self.c_data.x, -self.c_data.y, -self.c_data.z, -self.c_data.w)
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":812
 *         return 4
 * 
 *     def __neg__(self):             # <<<<<<<<<<<<<<
 *         cdef ovrQuatf to_return = ovrQuatf(
 *             -self.c_data.x, -self.c_data.y, -self.c_data.z, -self.c_data.w)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.__neg__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":819
 * 
 *     @staticmethod
 *     def identity():             # <<<<<<<<<<<<<<
 *         cdef ovrQuatf to_return = ovrQuatf(0.0, 0.0, 0.0, 1.0)
 *         return to_return
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_11identity(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_7psychxr_3ovr_4math_8ovrQuatf_11identity = {"identity", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_11identity, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_11identity(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("identity (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("identity", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return NULL;}
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "identity", 0))) return NULL;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_10identity();

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_10identity() {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("identity", 0);

  /* "psychxr/ovr/math.pyx":820
 *     @staticmethod
 *     def identity():
 *         cdef ovrQuatf to_return = ovrQuatf(0.0, 0.0, 0.0, 1.0)             # <<<<<<<<<<<<<<
 *         return to_return
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrQuatf), __pyx_tuple__11, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 820, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":821
 *     def identity():
 *         cdef ovrQuatf to_return = ovrQuatf(0.0, 0.0, 0.0, 1.0)
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def __eq__(self, ovrQuatf b):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":819
 * 
 *     @staticmethod
 *     def identity():             # <<<<<<<<<<<<<<
 *         cdef ovrQuatf to_return = ovrQuatf(0.0, 0.0, 0.0, 1.0)
 *         return to_return
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.identity", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":823
 *         return to_return
 * 
 *     def __eq__(self, ovrQuatf b):             # <<<<<<<<<<<<<<
 *         return (<ovrQuatf>self).c_data[0] == b.c_data[0]
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_13__eq__(PyObject *__pyx_v_self, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_13__eq__(PyObject *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__eq__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrQuatf, 1, "b", 0))) __PYX_ERR(0, 823, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_12__eq__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_12__eq__(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__eq__", 0);

  /* "psychxr/ovr/math.pyx":824
 * 
 *     def __eq__(self, ovrQuatf b):
 *         return (<ovrQuatf>self).c_data[0] == b.c_data[0]             # <<<<<<<<<<<<<<
 * 
 *     def __ne__(self, ovrQuatf b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(((__pyx_v_self->c_data[0]) == (__pyx_v_b->c_data[0]))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 824, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":823
 *         return to_return
 * 
 *     def __eq__(self, ovrQuatf b):             # <<<<<<<<<<<<<<
 *         return (<ovrQuatf>self).c_data[0] == b.c_data[0]
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.__eq__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":826
 *         return (<ovrQuatf>self).c_data[0] == b.c_data[0]
 * 
 *     def __ne__(self, ovrQuatf b):             # <<<<<<<<<<<<<<
 *         return (<ovrQuatf>self).c_data[0] != b.c_data[0]
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_15__ne__(PyObject *__pyx_v_self, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_15__ne__(PyObject *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__ne__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrQuatf, 1, "b", 0))) __PYX_ERR(0, 826, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_14__ne__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_14__ne__(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__ne__", 0);

  /* "psychxr/ovr/math.pyx":827
 * 
 *     def __ne__(self, ovrQuatf b):
 *         return (<ovrQuatf>self).c_data[0] != b.c_data[0]             # <<<<<<<<<<<<<<
 * 
 *     def __add__(ovrQuatf a, ovrQuatf b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(((__pyx_v_self->c_data[0]) != (__pyx_v_b->c_data[0]))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 827, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":826
 *         return (<ovrQuatf>self).c_data[0] == b.c_data[0]
 * 
 *     def __ne__(self, ovrQuatf b):             # <<<<<<<<<<<<<<
 *         return (<ovrQuatf>self).c_data[0] != b.c_data[0]
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.__ne__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":829
 *         return (<ovrQuatf>self).c_data[0] != b.c_data[0]
 * 
 *     def __add__(ovrQuatf a, ovrQuatf b):             # <<<<<<<<<<<<<<
 *         cdef ovrQuatf to_return = ovrQuatf()
 *         (<ovrQuatf>to_return).c_data[0] = a.c_data[0] + b.c_data[0]
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_17__add__(PyObject *__pyx_v_a, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_17__add__(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__add__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_a), __pyx_ptype_7psychxr_3ovr_4math_ovrQuatf, 1, "a", 0))) __PYX_ERR(0, 829, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrQuatf, 1, "b", 0))) __PYX_ERR(0, 829, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_16__add__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_a), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_16__add__(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_a, struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_b) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__add__", 0);

  /* "psychxr/ovr/math.pyx":830
 * 
 *     def __add__(ovrQuatf a, ovrQuatf b):
 *         cdef ovrQuatf to_return = ovrQuatf()             # <<<<<<<<<<<<<<
 *         (<ovrQuatf>to_return).c_data[0] = a.c_data[0] + b.c_data[0]
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrQuatf), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 830, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":831
 *     def __add__(ovrQuatf a, ovrQuatf b):
 *         cdef ovrQuatf to_return = ovrQuatf()
 *         (<ovrQuatf>to_return).c_data[0] = a.c_data[0] + b.c_data[0]             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
  (__pyx_v_to_return->c_data[0]) = ((__pyx_v_a->c_data[0]) + (__pyx_v_b->c_data[0]));

  /* "psychxr/ovr/math.pyx":833
 *         (<ovrQuatf>to_return).c_data[0] = a.c_data[0] + b.c_data[0]
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def __iadd__(self, ovrQuatf b):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":829
 *         return (<ovrQuatf>self).c_data[0] != b.c_data[0]
 * 
 *     def __add__(ovrQuatf a, ovrQuatf b):             # <<<<<<<<<<<<<<
 *         cdef ovrQuatf to_return = ovrQuatf()
 *         (<ovrQuatf>to_return).c_data[0] = a.c_data[0] + b.c_data[0]
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.__add__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":835
 *         return to_return
 * 
 *     def __iadd__(self, ovrQuatf b):             # <<<<<<<<<<<<<<
 *         self.c_data.x += b.c_data.x
 *         self.c_data.y += b.c_data.y
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_19__iadd__(PyObject *__pyx_v_self, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_19__iadd__(PyObject *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iadd__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrQuatf, 1, "b", 0))) __PYX_ERR(0, 835, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_18__iadd__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_18__iadd__(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iadd__", 0);

  /* "psychxr/ovr/math.pyx":836
 * 
 *     def __iadd__(self, ovrQuatf b):
 *         self.c_data.x += b.c_data.x             # <<<<<<<<<<<<<<
 *         self.c_data.y += b.c_data.y
 *         self.c_data.z += b.c_data.z
 */
  __pyx_v_self->c_data->x = (__pyx_v_self->c_data->x + __pyx_v_b->c_data->x);

  /* "psychxr/ovr/math.pyx":837
 *     def __iadd__(self, ovrQuatf b):
 *         self.c_data.x += b.c_data.x
 *         self.c_data.y += b.c_data.y             # <<<<<<<<<<<<<<
 *         self.c_data.z += b.c_data.z
 *         self.c_data.w += b.c_data.w
 */
  __pyx_v_self->c_data->y = (__pyx_v_self->c_data->y + __pyx_v_b->c_data->y);

  /* "psychxr/ovr/math.pyx":838
 *         self.c_data.x += b.c_data.x
 *         self.c_data.y += b.c_data.y
 *         self.c_data.z += b.c_data.z             # <<<<<<<<<<<<<<
 *         self.c_data.w += b.c_data.w
 * 
 */
  __pyx_v_self->c_data->z = (__pyx_v_self->c_data->z + __pyx_v_b->c_data->z);

  /* "psychxr/ovr/math.pyx":839
 *         self.c_data.y += b.c_data.y
 *         self.c_data.z += b.c_data.z
 *         self.c_data.w += b.c_data.w             # <<<<<<<<<<<<<<
 * 
 *         return self
 */
  __pyx_v_self->c_data->w = (__pyx_v_self->c_data->w + __pyx_v_b->c_data->w);

  /* "psychxr/ovr/math.pyx":841
 *         self.c_data.w += b.c_data.w
 * 
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     def __sub__(ovrQuatf a, ovrQuatf b):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":835
 *         return to_return
 * 
 *     def __iadd__(self, ovrQuatf b):             # <<<<<<<<<<<<<<
 *         self.c_data.x += b.c_data.x
 *         self.c_data.y += b.c_data.y
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":843
 *         return self
 * 
 *     def __sub__(ovrQuatf a, ovrQuatf b):             # <<<<<<<<<<<<<<
 *         cdef ovrQuatf to_return = ovrQuatf()
 *         (<ovrQuatf>to_return).c_data[0] = a.c_data[0] - b.c_data[0]
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_21__sub__(PyObject *__pyx_v_a, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_21__sub__(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__sub__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_a), __pyx_ptype_7psychxr_3ovr_4math_ovrQuatf, 1, "a", 0))) __PYX_ERR(0, 843, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrQuatf, 1, "b", 0))) __PYX_ERR(0, 843, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_20__sub__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_a), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_20__sub__(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_a, struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_b) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__sub__", 0);

  /* "psychxr/ovr/math.pyx":844
 * 
 *     def __sub__(ovrQuatf a, ovrQuatf b):
 *         cdef ovrQuatf to_return = ovrQuatf()             # <<<<<<<<<<<<<<
 *         (<ovrQuatf>to_return).c_data[0] = a.c_data[0] - b.c_data[0]
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrQuatf), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 844, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":845
 *     def __sub__(ovrQuatf a, ovrQuatf b):
 *         cdef ovrQuatf to_return = ovrQuatf()
 *         (<ovrQuatf>to_return).c_data[0] = a.c_data[0] - b.c_data[0]             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
  (__pyx_v_to_return->c_data[0]) = ((__pyx_v_a->c_data[0]) - (__pyx_v_b->c_data[0]));

  /* "psychxr/ovr/math.pyx":847
 *         (<ovrQuatf>to_return).c_data[0] = a.c_data[0] - b.c_data[0]
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def __isub__(self, ovrQuatf b):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":843
 *         return self
 * 
 *     def __sub__(ovrQuatf a, ovrQuatf b):             # <<<<<<<<<<<<<<
 *         cdef ovrQuatf to_return = ovrQuatf()
 *         (<ovrQuatf>to_return).c_data[0] = a.c_data[0] - b.c_data[0]
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.__sub__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":849
 *         return to_return
 * 
 *     def __isub__(self, ovrQuatf b):             # <<<<<<<<<<<<<<
 *         self.c_data.x -= b.c_data.x
 *         self.c_data.y -= b.c_data.y
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_23__isub__(PyObject *__pyx_v_self, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_23__isub__(PyObject *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__isub__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrQuatf, 1, "b", 0))) __PYX_ERR(0, 849, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_22__isub__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_22__isub__(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__isub__", 0);

  /* "psychxr/ovr/math.pyx":850
 * 
 *     def __isub__(self, ovrQuatf b):
 *         self.c_data.x -= b.c_data.x             # <<<<<<<<<<<<<<
 *         self.c_data.y -= b.c_data.y
 *         self.c_data.z -= b.c_data.z
 */
  __pyx_v_self->c_data->x = (__pyx_v_self->c_data->x - __pyx_v_b->c_data->x);

  /* "psychxr/ovr/math.pyx":851
 *     def __isub__(self, ovrQuatf b):
 *         self.c_data.x -= b.c_data.x
 *         self.c_data.y -= b.c_data.y             # <<<<<<<<<<<<<<
 *         self.c_data.z -= b.c_data.z
 *         self.c_data.w -= b.c_data.w
 */
  __pyx_v_self->c_data->y = (__pyx_v_self->c_data->y - __pyx_v_b->c_data->y);

  /* "psychxr/ovr/math.pyx":852
 *         self.c_data.x -= b.c_data.x
 *         self.c_data.y -= b.c_data.y
 *         self.c_data.z -= b.c_data.z             # <<<<<<<<<<<<<<
 *         self.c_data.w -= b.c_data.w
 * 
 */
  __pyx_v_self->c_data->z = (__pyx_v_self->c_data->z - __pyx_v_b->c_data->z);

  /* "psychxr/ovr/math.pyx":853
 *         self.c_data.y -= b.c_data.y
 *         self.c_data.z -= b.c_data.z
 *         self.c_data.w -= b.c_data.w             # <<<<<<<<<<<<<<
 * 
 *         return self
 */
  __pyx_v_self->c_data->w = (__pyx_v_self->c_data->w - __pyx_v_b->c_data->w);

  /* "psychxr/ovr/math.pyx":855
 *         self.c_data.w -= b.c_data.w
 * 
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     def __mul__(ovrQuatf a, object b):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":849
 *         return to_return
 * 
 *     def __isub__(self, ovrQuatf b):             # <<<<<<<<<<<<<<
 *         self.c_data.x -= b.c_data.x
 *         self.c_data.y -= b.c_data.y
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":857
 *         return self
 * 
 *     def __mul__(ovrQuatf a, object b):             # <<<<<<<<<<<<<<
 *         if isinstance(b, ovrVector3f):
 *             return a.rotate(<ovrVector3f>b)
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_25__mul__(PyObject *__pyx_v_a, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_25__mul__(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__mul__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_a), __pyx_ptype_7psychxr_3ovr_4math_ovrQuatf, 1, "a", 0))) __PYX_ERR(0, 857, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_24__mul__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_a), ((PyObject *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_24__mul__(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  float __pyx_t_9;
  __Pyx_RefNannySetupContext("__mul__", 0);

  /* "psychxr/ovr/math.pyx":858
 * 
 *     def __mul__(ovrQuatf a, object b):
 *         if isinstance(b, ovrVector3f):             # <<<<<<<<<<<<<<
 *             return a.rotate(<ovrVector3f>b)
 *         elif isinstance(b, ovrQuatf):
 */
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_b, __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "psychxr/ovr/math.pyx":859
 *     def __mul__(ovrQuatf a, object b):
 *         if isinstance(b, ovrVector3f):
 *             return a.rotate(<ovrVector3f>b)             # <<<<<<<<<<<<<<
 *         elif isinstance(b, ovrQuatf):
 *             # quaternion multiplication
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_a), __pyx_n_s_rotate); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 859, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (!__pyx_t_5) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_v_b); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 859, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_v_b};
        __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 859, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_v_b};
        __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 859, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 859, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
        __Pyx_INCREF(((PyObject *)((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_b)));
        __Pyx_GIVEREF(__pyx_v_b);
        PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_v_b);
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_6, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 859, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "psychxr/ovr/math.pyx":858
 * 
 *     def __mul__(ovrQuatf a, object b):
 *         if isinstance(b, ovrVector3f):             # <<<<<<<<<<<<<<
 *             return a.rotate(<ovrVector3f>b)
 *         elif isinstance(b, ovrQuatf):
 */
  }

  /* "psychxr/ovr/math.pyx":860
 *         if isinstance(b, ovrVector3f):
 *             return a.rotate(<ovrVector3f>b)
 *         elif isinstance(b, ovrQuatf):             # <<<<<<<<<<<<<<
 *             # quaternion multiplication
 *             return ovrQuatf(
 */
  __pyx_t_2 = __Pyx_TypeCheck(__pyx_v_b, __pyx_ptype_7psychxr_3ovr_4math_ovrQuatf); 
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {

    /* "psychxr/ovr/math.pyx":862
 *         elif isinstance(b, ovrQuatf):
 *             # quaternion multiplication
 *             return ovrQuatf(             # <<<<<<<<<<<<<<
 *                 a.c_data.w * (<ovrQuatf>b).c_data.x +
 *                 a.c_data.x * (<ovrQuatf>b).c_data.w +
 */
    __Pyx_XDECREF(__pyx_r);

    /* "psychxr/ovr/math.pyx":865
 *                 a.c_data.w * (<ovrQuatf>b).c_data.x +
 *                 a.c_data.x * (<ovrQuatf>b).c_data.w +
 *                 a.c_data.y * (<ovrQuatf>b).c_data.z -             # <<<<<<<<<<<<<<
 *                 a.c_data.z * (<ovrQuatf>b).c_data.y,
 *                 a.c_data.w * (<ovrQuatf>b).c_data.y -
 */
    __pyx_t_3 = PyFloat_FromDouble(((((__pyx_v_a->c_data->w * ((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_b)->c_data->x) + (__pyx_v_a->c_data->x * ((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_b)->c_data->w)) + (__pyx_v_a->c_data->y * ((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_b)->c_data->z)) - (__pyx_v_a->c_data->z * ((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_b)->c_data->y))); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 865, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);

    /* "psychxr/ovr/math.pyx":869
 *                 a.c_data.w * (<ovrQuatf>b).c_data.y -
 *                 a.c_data.x * (<ovrQuatf>b).c_data.z +
 *                 a.c_data.y * (<ovrQuatf>b).c_data.w +             # <<<<<<<<<<<<<<
 *                 a.c_data.z * (<ovrQuatf>b).c_data.x,
 *                 a.c_data.w * (<ovrQuatf>b).c_data.z +
 */
    __pyx_t_4 = PyFloat_FromDouble(((((__pyx_v_a->c_data->w * ((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_b)->c_data->y) - (__pyx_v_a->c_data->x * ((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_b)->c_data->z)) + (__pyx_v_a->c_data->y * ((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_b)->c_data->w)) + (__pyx_v_a->c_data->z * ((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_b)->c_data->x))); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 869, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);

    /* "psychxr/ovr/math.pyx":873
 *                 a.c_data.w * (<ovrQuatf>b).c_data.z +
 *                 a.c_data.x * (<ovrQuatf>b).c_data.y -
 *                 a.c_data.y * (<ovrQuatf>b).c_data.x +             # <<<<<<<<<<<<<<
 *                 a.c_data.z * (<ovrQuatf>b).c_data.w,
 *                 a.c_data.w * (<ovrQuatf>b).c_data.w -
 */
    __pyx_t_6 = PyFloat_FromDouble(((((__pyx_v_a->c_data->w * ((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_b)->c_data->z) + (__pyx_v_a->c_data->x * ((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_b)->c_data->y)) - (__pyx_v_a->c_data->y * ((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_b)->c_data->x)) + (__pyx_v_a->c_data->z * ((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_b)->c_data->w))); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 873, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);

    /* "psychxr/ovr/math.pyx":877
 *                 a.c_data.w * (<ovrQuatf>b).c_data.w -
 *                 a.c_data.x * (<ovrQuatf>b).c_data.x -
 *                 a.c_data.y * (<ovrQuatf>b).c_data.y -             # <<<<<<<<<<<<<<
 *                 a.c_data.z * (<ovrQuatf>b).c_data.z)
 *         elif isinstance(b, (int, float)):
 */
    __pyx_t_5 = PyFloat_FromDouble(((((__pyx_v_a->c_data->w * ((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_b)->c_data->w) - (__pyx_v_a->c_data->x * ((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_b)->c_data->x)) - (__pyx_v_a->c_data->y * ((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_b)->c_data->y)) - (__pyx_v_a->c_data->z * ((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_b)->c_data->z))); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 877, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);

    /* "psychxr/ovr/math.pyx":862
 *         elif isinstance(b, ovrQuatf):
 *             # quaternion multiplication
 *             return ovrQuatf(             # <<<<<<<<<<<<<<
 *                 a.c_data.w * (<ovrQuatf>b).c_data.x +
 *                 a.c_data.x * (<ovrQuatf>b).c_data.w +
 */
    __pyx_t_7 = PyTuple_New(4); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 862, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_7, 2, __pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_7, 3, __pyx_t_5);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_6 = 0;
    __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrQuatf), __pyx_t_7, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 862, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;

    /* "psychxr/ovr/math.pyx":860
 *         if isinstance(b, ovrVector3f):
 *             return a.rotate(<ovrVector3f>b)
 *         elif isinstance(b, ovrQuatf):             # <<<<<<<<<<<<<<
 *             # quaternion multiplication
 *             return ovrQuatf(
 */
  }

  /* "psychxr/ovr/math.pyx":879
 *                 a.c_data.y * (<ovrQuatf>b).c_data.y -
 *                 a.c_data.z * (<ovrQuatf>b).c_data.z)
 *         elif isinstance(b, (int, float)):             # <<<<<<<<<<<<<<
 *             return ovrQuatf(
 *                 a.c_data.x * <float>b,
 */
  __pyx_t_2 = PyInt_Check(__pyx_v_b); 
  __pyx_t_8 = (__pyx_t_2 != 0);
  if (!__pyx_t_8) {
  } else {
    __pyx_t_1 = __pyx_t_8;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_8 = PyFloat_Check(__pyx_v_b); 
  __pyx_t_2 = (__pyx_t_8 != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "psychxr/ovr/math.pyx":880
 *                 a.c_data.z * (<ovrQuatf>b).c_data.z)
 *         elif isinstance(b, (int, float)):
 *             return ovrQuatf(             # <<<<<<<<<<<<<<
 *                 a.c_data.x * <float>b,
 *                 a.c_data.y * <float>b,
 */
    __Pyx_XDECREF(__pyx_r);

    /* "psychxr/ovr/math.pyx":881
 *         elif isinstance(b, (int, float)):
 *             return ovrQuatf(
 *                 a.c_data.x * <float>b,             # <<<<<<<<<<<<<<
 *                 a.c_data.y * <float>b,
 *                 a.c_data.z * <float>b,
 */
    __pyx_t_9 = __pyx_PyFloat_AsFloat(__pyx_v_b); if (unlikely((__pyx_t_9 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 881, __pyx_L1_error)
    __pyx_t_5 = PyFloat_FromDouble((__pyx_v_a->c_data->x * ((float)__pyx_t_9))); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 881, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);

    /* "psychxr/ovr/math.pyx":882
 *             return ovrQuatf(
 *                 a.c_data.x * <float>b,
 *                 a.c_data.y * <float>b,             # <<<<<<<<<<<<<<
 *                 a.c_data.z * <float>b,
 *                 a.c_data.w * <float>b)
 */
    __pyx_t_9 = __pyx_PyFloat_AsFloat(__pyx_v_b); if (unlikely((__pyx_t_9 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 882, __pyx_L1_error)
    __pyx_t_7 = PyFloat_FromDouble((__pyx_v_a->c_data->y * ((float)__pyx_t_9))); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 882, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);

    /* "psychxr/ovr/math.pyx":883
 *                 a.c_data.x * <float>b,
 *                 a.c_data.y * <float>b,
 *                 a.c_data.z * <float>b,             # <<<<<<<<<<<<<<
 *                 a.c_data.w * <float>b)
 * 
 */
    __pyx_t_9 = __pyx_PyFloat_AsFloat(__pyx_v_b); if (unlikely((__pyx_t_9 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 883, __pyx_L1_error)
    __pyx_t_6 = PyFloat_FromDouble((__pyx_v_a->c_data->z * ((float)__pyx_t_9))); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 883, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);

    /* "psychxr/ovr/math.pyx":884
 *                 a.c_data.y * <float>b,
 *                 a.c_data.z * <float>b,
 *                 a.c_data.w * <float>b)             # <<<<<<<<<<<<<<
 * 
 *     def __imul__(self, object b):
 */
    __pyx_t_9 = __pyx_PyFloat_AsFloat(__pyx_v_b); if (unlikely((__pyx_t_9 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 884, __pyx_L1_error)
    __pyx_t_4 = PyFloat_FromDouble((__pyx_v_a->c_data->w * ((float)__pyx_t_9))); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 884, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);

    /* "psychxr/ovr/math.pyx":880
 *                 a.c_data.z * (<ovrQuatf>b).c_data.z)
 *         elif isinstance(b, (int, float)):
 *             return ovrQuatf(             # <<<<<<<<<<<<<<
 *                 a.c_data.x * <float>b,
 *                 a.c_data.y * <float>b,
 */
    __pyx_t_3 = PyTuple_New(4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 880, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_7);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 3, __pyx_t_4);
    __pyx_t_5 = 0;
    __pyx_t_7 = 0;
    __pyx_t_6 = 0;
    __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrQuatf), __pyx_t_3, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 880, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "psychxr/ovr/math.pyx":879
 *                 a.c_data.y * (<ovrQuatf>b).c_data.y -
 *                 a.c_data.z * (<ovrQuatf>b).c_data.z)
 *         elif isinstance(b, (int, float)):             # <<<<<<<<<<<<<<
 *             return ovrQuatf(
 *                 a.c_data.x * <float>b,
 */
  }

  /* "psychxr/ovr/math.pyx":857
 *         return self
 * 
 *     def __mul__(ovrQuatf a, object b):             # <<<<<<<<<<<<<<
 *         if isinstance(b, ovrVector3f):
 *             return a.rotate(<ovrVector3f>b)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.__mul__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":886
 *                 a.c_data.w * <float>b)
 * 
 *     def __imul__(self, object b):             # <<<<<<<<<<<<<<
 *         if isinstance(b, ovrQuatf):
 *             self.c_data.x *= b.c_data.x
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_27__imul__(PyObject *__pyx_v_self, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_27__imul__(PyObject *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__imul__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_26__imul__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_self), ((PyObject *)__pyx_v_b));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_26__imul__(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  float __pyx_t_6;
  int __pyx_t_7;
  __Pyx_RefNannySetupContext("__imul__", 0);

  /* "psychxr/ovr/math.pyx":887
 * 
 *     def __imul__(self, object b):
 *         if isinstance(b, ovrQuatf):             # <<<<<<<<<<<<<<
 *             self.c_data.x *= b.c_data.x
 *             self.c_data.y *= b.c_data.y
 */
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_b, __pyx_ptype_7psychxr_3ovr_4math_ovrQuatf); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "psychxr/ovr/math.pyx":888
 *     def __imul__(self, object b):
 *         if isinstance(b, ovrQuatf):
 *             self.c_data.x *= b.c_data.x             # <<<<<<<<<<<<<<
 *             self.c_data.y *= b.c_data.y
 *             self.c_data.z *= b.c_data.z
 */
    __pyx_t_3 = PyFloat_FromDouble(__pyx_v_self->c_data->x); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 888, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_b, __pyx_n_s_c_data); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 888, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_x); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 888, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = PyNumber_InPlaceMultiply(__pyx_t_3, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 888, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_6 = __pyx_PyFloat_AsFloat(__pyx_t_4); if (unlikely((__pyx_t_6 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 888, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_self->c_data->x = __pyx_t_6;

    /* "psychxr/ovr/math.pyx":889
 *         if isinstance(b, ovrQuatf):
 *             self.c_data.x *= b.c_data.x
 *             self.c_data.y *= b.c_data.y             # <<<<<<<<<<<<<<
 *             self.c_data.z *= b.c_data.z
 *             self.c_data.w *= b.c_data.w
 */
    __pyx_t_4 = PyFloat_FromDouble(__pyx_v_self->c_data->y); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 889, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_b, __pyx_n_s_c_data); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 889, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_y); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 889, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = PyNumber_InPlaceMultiply(__pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 889, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_6 = __pyx_PyFloat_AsFloat(__pyx_t_5); if (unlikely((__pyx_t_6 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 889, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_v_self->c_data->y = __pyx_t_6;

    /* "psychxr/ovr/math.pyx":890
 *             self.c_data.x *= b.c_data.x
 *             self.c_data.y *= b.c_data.y
 *             self.c_data.z *= b.c_data.z             # <<<<<<<<<<<<<<
 *             self.c_data.w *= b.c_data.w
 *         elif isinstance(b, (int, float)):
 */
    __pyx_t_5 = PyFloat_FromDouble(__pyx_v_self->c_data->z); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 890, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_b, __pyx_n_s_c_data); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 890, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_z); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 890, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyNumber_InPlaceMultiply(__pyx_t_5, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 890, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_6 = __pyx_PyFloat_AsFloat(__pyx_t_3); if (unlikely((__pyx_t_6 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 890, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_self->c_data->z = __pyx_t_6;

    /* "psychxr/ovr/math.pyx":891
 *             self.c_data.y *= b.c_data.y
 *             self.c_data.z *= b.c_data.z
 *             self.c_data.w *= b.c_data.w             # <<<<<<<<<<<<<<
 *         elif isinstance(b, (int, float)):
 *             self.c_data.x *= <float>b
 */
    __pyx_t_3 = PyFloat_FromDouble(__pyx_v_self->c_data->w); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 891, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_b, __pyx_n_s_c_data); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 891, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_w); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 891, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = PyNumber_InPlaceMultiply(__pyx_t_3, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 891, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_6 = __pyx_PyFloat_AsFloat(__pyx_t_4); if (unlikely((__pyx_t_6 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 891, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_self->c_data->w = __pyx_t_6;

    /* "psychxr/ovr/math.pyx":887
 * 
 *     def __imul__(self, object b):
 *         if isinstance(b, ovrQuatf):             # <<<<<<<<<<<<<<
 *             self.c_data.x *= b.c_data.x
 *             self.c_data.y *= b.c_data.y
 */
    goto __pyx_L3;
  }

  /* "psychxr/ovr/math.pyx":892
 *             self.c_data.z *= b.c_data.z
 *             self.c_data.w *= b.c_data.w
 *         elif isinstance(b, (int, float)):             # <<<<<<<<<<<<<<
 *             self.c_data.x *= <float>b
 *             self.c_data.y *= <float>b
 */
  __pyx_t_1 = PyInt_Check(__pyx_v_b); 
  __pyx_t_7 = (__pyx_t_1 != 0);
  if (!__pyx_t_7) {
  } else {
    __pyx_t_2 = __pyx_t_7;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_7 = PyFloat_Check(__pyx_v_b); 
  __pyx_t_1 = (__pyx_t_7 != 0);
  __pyx_t_2 = __pyx_t_1;
  __pyx_L4_bool_binop_done:;
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {

    /* "psychxr/ovr/math.pyx":893
 *             self.c_data.w *= b.c_data.w
 *         elif isinstance(b, (int, float)):
 *             self.c_data.x *= <float>b             # <<<<<<<<<<<<<<
 *             self.c_data.y *= <float>b
 *             self.c_data.z *= <float>b
 */
    __pyx_t_6 = __pyx_PyFloat_AsFloat(__pyx_v_b); if (unlikely((__pyx_t_6 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 893, __pyx_L1_error)
    __pyx_v_self->c_data->x = (__pyx_v_self->c_data->x * ((float)__pyx_t_6));

    /* "psychxr/ovr/math.pyx":894
 *         elif isinstance(b, (int, float)):
 *             self.c_data.x *= <float>b
 *             self.c_data.y *= <float>b             # <<<<<<<<<<<<<<
 *             self.c_data.z *= <float>b
 *             self.c_data.w *= <float>b
 */
    __pyx_t_6 = __pyx_PyFloat_AsFloat(__pyx_v_b); if (unlikely((__pyx_t_6 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 894, __pyx_L1_error)
    __pyx_v_self->c_data->y = (__pyx_v_self->c_data->y * ((float)__pyx_t_6));

    /* "psychxr/ovr/math.pyx":895
 *             self.c_data.x *= <float>b
 *             self.c_data.y *= <float>b
 *             self.c_data.z *= <float>b             # <<<<<<<<<<<<<<
 *             self.c_data.w *= <float>b
 * 
 */
    __pyx_t_6 = __pyx_PyFloat_AsFloat(__pyx_v_b); if (unlikely((__pyx_t_6 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 895, __pyx_L1_error)
    __pyx_v_self->c_data->z = (__pyx_v_self->c_data->z * ((float)__pyx_t_6));

    /* "psychxr/ovr/math.pyx":896
 *             self.c_data.y *= <float>b
 *             self.c_data.z *= <float>b
 *             self.c_data.w *= <float>b             # <<<<<<<<<<<<<<
 * 
 *         return self
 */
    __pyx_t_6 = __pyx_PyFloat_AsFloat(__pyx_v_b); if (unlikely((__pyx_t_6 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 896, __pyx_L1_error)
    __pyx_v_self->c_data->w = (__pyx_v_self->c_data->w * ((float)__pyx_t_6));

    /* "psychxr/ovr/math.pyx":892
 *             self.c_data.z *= b.c_data.z
 *             self.c_data.w *= b.c_data.w
 *         elif isinstance(b, (int, float)):             # <<<<<<<<<<<<<<
 *             self.c_data.x *= <float>b
 *             self.c_data.y *= <float>b
 */
  }
  __pyx_L3:;

  /* "psychxr/ovr/math.pyx":898
 *             self.c_data.w *= <float>b
 * 
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     def __truediv__(ovrQuatf a, float s):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":886
 *                 a.c_data.w * <float>b)
 * 
 *     def __imul__(self, object b):             # <<<<<<<<<<<<<<
 *         if isinstance(b, ovrQuatf):
 *             self.c_data.x *= b.c_data.x
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.__imul__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":900
 *         return self
 * 
 *     def __truediv__(ovrQuatf a, float s):             # <<<<<<<<<<<<<<
 *         cdef ovrQuatf to_return = ovrQuatf()
 *         (<ovrQuatf>to_return).c_data[0] = a.c_data[0] / s
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_29__truediv__(PyObject *__pyx_v_a, PyObject *__pyx_arg_s); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_29__truediv__(PyObject *__pyx_v_a, PyObject *__pyx_arg_s) {
  float __pyx_v_s;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__truediv__ (wrapper)", 0);
  assert(__pyx_arg_s); {
    __pyx_v_s = __pyx_PyFloat_AsFloat(__pyx_arg_s); if (unlikely((__pyx_v_s == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 900, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.__truediv__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_a), __pyx_ptype_7psychxr_3ovr_4math_ovrQuatf, 1, "a", 0))) __PYX_ERR(0, 900, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_28__truediv__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_a), ((float)__pyx_v_s));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_28__truediv__(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_a, float __pyx_v_s) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__truediv__", 0);

  /* "psychxr/ovr/math.pyx":901
 * 
 *     def __truediv__(ovrQuatf a, float s):
 *         cdef ovrQuatf to_return = ovrQuatf()             # <<<<<<<<<<<<<<
 *         (<ovrQuatf>to_return).c_data[0] = a.c_data[0] / s
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrQuatf), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 901, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":902
 *     def __truediv__(ovrQuatf a, float s):
 *         cdef ovrQuatf to_return = ovrQuatf()
 *         (<ovrQuatf>to_return).c_data[0] = a.c_data[0] / s             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
  (__pyx_v_to_return->c_data[0]) = ((__pyx_v_a->c_data[0]) / __pyx_v_s);

  /* "psychxr/ovr/math.pyx":904
 *         (<ovrQuatf>to_return).c_data[0] = a.c_data[0] / s
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def __itruediv__(self, float s):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":900
 *         return self
 * 
 *     def __truediv__(ovrQuatf a, float s):             # <<<<<<<<<<<<<<
 *         cdef ovrQuatf to_return = ovrQuatf()
 *         (<ovrQuatf>to_return).c_data[0] = a.c_data[0] / s
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.__truediv__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":906
 *         return to_return
 * 
 *     def __itruediv__(self, float s):             # <<<<<<<<<<<<<<
 *         (<ovrQuatf>self).c_data[0] = (<ovrQuatf>self).c_data[0] / s
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_31__itruediv__(PyObject *__pyx_v_self, PyObject *__pyx_arg_s); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_31__itruediv__(PyObject *__pyx_v_self, PyObject *__pyx_arg_s) {
  float __pyx_v_s;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__itruediv__ (wrapper)", 0);
  assert(__pyx_arg_s); {
    __pyx_v_s = __pyx_PyFloat_AsFloat(__pyx_arg_s); if (unlikely((__pyx_v_s == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 906, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.__itruediv__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_30__itruediv__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_self), ((float)__pyx_v_s));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_30__itruediv__(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self, float __pyx_v_s) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__itruediv__", 0);

  /* "psychxr/ovr/math.pyx":907
 * 
 *     def __itruediv__(self, float s):
 *         (<ovrQuatf>self).c_data[0] = (<ovrQuatf>self).c_data[0] / s             # <<<<<<<<<<<<<<
 * 
 *         return self
 */
  (__pyx_v_self->c_data[0]) = ((__pyx_v_self->c_data[0]) / __pyx_v_s);

  /* "psychxr/ovr/math.pyx":909
 *         (<ovrQuatf>self).c_data[0] = (<ovrQuatf>self).c_data[0] / s
 * 
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     def isEqual(self, ovrQuatf b, float tolerance = 0.0):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":906
 *         return to_return
 * 
 *     def __itruediv__(self, float s):             # <<<<<<<<<<<<<<
 *         (<ovrQuatf>self).c_data[0] = (<ovrQuatf>self).c_data[0] / s
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":911
 *         return self
 * 
 *     def isEqual(self, ovrQuatf b, float tolerance = 0.0):             # <<<<<<<<<<<<<<
 *         return  self.abs(self.dot(b)) >= <float>1 - tolerance
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_33isEqual(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_33isEqual(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_b = 0;
  float __pyx_v_tolerance;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("isEqual (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_b,&__pyx_n_s_tolerance,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_b)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_tolerance);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "isEqual") < 0)) __PYX_ERR(0, 911, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_b = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)values[0]);
    if (values[1]) {
      __pyx_v_tolerance = __pyx_PyFloat_AsFloat(values[1]); if (unlikely((__pyx_v_tolerance == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 911, __pyx_L3_error)
    } else {
      __pyx_v_tolerance = ((float)0.0);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("isEqual", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 911, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.isEqual", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrQuatf, 1, "b", 0))) __PYX_ERR(0, 911, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_32isEqual(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_self), __pyx_v_b, __pyx_v_tolerance);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_32isEqual(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_b, float __pyx_v_tolerance) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("isEqual", 0);

  /* "psychxr/ovr/math.pyx":912
 * 
 *     def isEqual(self, ovrQuatf b, float tolerance = 0.0):
 *         return  self.abs(self.dot(b)) >= <float>1 - tolerance             # <<<<<<<<<<<<<<
 * 
 *     def abs(self, float v):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_abs); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 912, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_dot); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 912, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (!__pyx_t_5) {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, ((PyObject *)__pyx_v_b)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 912, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[2] = {__pyx_t_5, ((PyObject *)__pyx_v_b)};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 912, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[2] = {__pyx_t_5, ((PyObject *)__pyx_v_b)};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 912, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 912, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
      __Pyx_INCREF(((PyObject *)__pyx_v_b));
      __Pyx_GIVEREF(((PyObject *)__pyx_v_b));
      PyTuple_SET_ITEM(__pyx_t_6, 0+1, ((PyObject *)__pyx_v_b));
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_6, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 912, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 912, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_t_3};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 912, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_t_3};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 912, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else
    #endif
    {
      __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 912, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
      __Pyx_GIVEREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_t_3);
      __pyx_t_3 = 0;
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 912, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyFloat_FromDouble((((float)1) - __pyx_v_tolerance)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 912, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = PyObject_RichCompare(__pyx_t_1, __pyx_t_2, Py_GE); __Pyx_XGOTREF(__pyx_t_6); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 912, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_6;
  __pyx_t_6 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":911
 *         return self
 * 
 *     def isEqual(self, ovrQuatf b, float tolerance = 0.0):             # <<<<<<<<<<<<<<
 *         return  self.abs(self.dot(b)) >= <float>1 - tolerance
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.isEqual", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":914
 *         return  self.abs(self.dot(b)) >= <float>1 - tolerance
 * 
 *     def abs(self, float v):             # <<<<<<<<<<<<<<
 *         return <float>(<ovrQuatf>self).c_data[0].Abs(v)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_35abs(PyObject *__pyx_v_self, PyObject *__pyx_arg_v); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_35abs(PyObject *__pyx_v_self, PyObject *__pyx_arg_v) {
  float __pyx_v_v;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("abs (wrapper)", 0);
  assert(__pyx_arg_v); {
    __pyx_v_v = __pyx_PyFloat_AsFloat(__pyx_arg_v); if (unlikely((__pyx_v_v == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 914, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.abs", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_34abs(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_self), ((float)__pyx_v_v));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_34abs(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self, float __pyx_v_v) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("abs", 0);

  /* "psychxr/ovr/math.pyx":915
 * 
 *     def abs(self, float v):
 *         return <float>(<ovrQuatf>self).c_data[0].Abs(v)             # <<<<<<<<<<<<<<
 * 
 *     def imag(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(((float)(__pyx_v_self->c_data[0]).Abs(__pyx_v_v))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 915, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":914
 *         return  self.abs(self.dot(b)) >= <float>1 - tolerance
 * 
 *     def abs(self, float v):             # <<<<<<<<<<<<<<
 *         return <float>(<ovrQuatf>self).c_data[0].Abs(v)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.abs", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":917
 *         return <float>(<ovrQuatf>self).c_data[0].Abs(v)
 * 
 *     def imag(self):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_37imag(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_37imag(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("imag (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_36imag(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_36imag(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("imag", 0);

  /* "psychxr/ovr/math.pyx":918
 * 
 *     def imag(self):
 *         cdef ovrVector3f to_return = ovrVector3f()             # <<<<<<<<<<<<<<
 *         (<ovrVector3f>to_return).c_data[0] = \
 *             (<ovrQuatf>self).c_data[0].Imag()
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector3f), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 918, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":919
 *     def imag(self):
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \             # <<<<<<<<<<<<<<
 *             (<ovrQuatf>self).c_data[0].Imag()
 * 
 */
  (__pyx_v_to_return->c_data[0]) = (__pyx_v_self->c_data[0]).Imag();

  /* "psychxr/ovr/math.pyx":922
 *             (<ovrQuatf>self).c_data[0].Imag()
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def lengthSq(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":917
 *         return <float>(<ovrQuatf>self).c_data[0].Abs(v)
 * 
 *     def imag(self):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.imag", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":924
 *         return to_return
 * 
 *     def lengthSq(self):             # <<<<<<<<<<<<<<
 *         return <float>(<ovrQuatf>self).c_data[0].LengthSq()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_39lengthSq(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_39lengthSq(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("lengthSq (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_38lengthSq(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_38lengthSq(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("lengthSq", 0);

  /* "psychxr/ovr/math.pyx":925
 * 
 *     def lengthSq(self):
 *         return <float>(<ovrQuatf>self).c_data[0].LengthSq()             # <<<<<<<<<<<<<<
 * 
 *     def length(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(((float)(__pyx_v_self->c_data[0]).LengthSq())); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 925, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":924
 *         return to_return
 * 
 *     def lengthSq(self):             # <<<<<<<<<<<<<<
 *         return <float>(<ovrQuatf>self).c_data[0].LengthSq()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.lengthSq", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":927
 *         return <float>(<ovrQuatf>self).c_data[0].LengthSq()
 * 
 *     def length(self):             # <<<<<<<<<<<<<<
 *         return <float>(<ovrQuatf>self).c_data[0].Length()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_41length(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_41length(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("length (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_40length(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_40length(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("length", 0);

  /* "psychxr/ovr/math.pyx":928
 * 
 *     def length(self):
 *         return <float>(<ovrQuatf>self).c_data[0].Length()             # <<<<<<<<<<<<<<
 * 
 *     def distance(self, ovrQuatf q):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(((float)(__pyx_v_self->c_data[0]).Length())); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 928, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":927
 *         return <float>(<ovrQuatf>self).c_data[0].LengthSq()
 * 
 *     def length(self):             # <<<<<<<<<<<<<<
 *         return <float>(<ovrQuatf>self).c_data[0].Length()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.length", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":930
 *         return <float>(<ovrQuatf>self).c_data[0].Length()
 * 
 *     def distance(self, ovrQuatf q):             # <<<<<<<<<<<<<<
 *         return <float>(<ovrQuatf>self).c_data[0].Distance(q.c_data[0])
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_43distance(PyObject *__pyx_v_self, PyObject *__pyx_v_q); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_43distance(PyObject *__pyx_v_self, PyObject *__pyx_v_q) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("distance (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_q), __pyx_ptype_7psychxr_3ovr_4math_ovrQuatf, 1, "q", 0))) __PYX_ERR(0, 930, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_42distance(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_q));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_42distance(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_q) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("distance", 0);

  /* "psychxr/ovr/math.pyx":931
 * 
 *     def distance(self, ovrQuatf q):
 *         return <float>(<ovrQuatf>self).c_data[0].Distance(q.c_data[0])             # <<<<<<<<<<<<<<
 * 
 *     def distanceSq(self, ovrQuatf q):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(((float)(__pyx_v_self->c_data[0]).Distance((__pyx_v_q->c_data[0])))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 931, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":930
 *         return <float>(<ovrQuatf>self).c_data[0].Length()
 * 
 *     def distance(self, ovrQuatf q):             # <<<<<<<<<<<<<<
 *         return <float>(<ovrQuatf>self).c_data[0].Distance(q.c_data[0])
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.distance", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":933
 *         return <float>(<ovrQuatf>self).c_data[0].Distance(q.c_data[0])
 * 
 *     def distanceSq(self, ovrQuatf q):             # <<<<<<<<<<<<<<
 *         return <float>(<ovrQuatf>self).c_data[0].DistanceSq(q.c_data[0])
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_45distanceSq(PyObject *__pyx_v_self, PyObject *__pyx_v_q); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_45distanceSq(PyObject *__pyx_v_self, PyObject *__pyx_v_q) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("distanceSq (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_q), __pyx_ptype_7psychxr_3ovr_4math_ovrQuatf, 1, "q", 0))) __PYX_ERR(0, 933, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_44distanceSq(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_q));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_44distanceSq(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_q) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("distanceSq", 0);

  /* "psychxr/ovr/math.pyx":934
 * 
 *     def distanceSq(self, ovrQuatf q):
 *         return <float>(<ovrQuatf>self).c_data[0].DistanceSq(q.c_data[0])             # <<<<<<<<<<<<<<
 * 
 *     def dot(self, ovrQuatf q):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(((float)(__pyx_v_self->c_data[0]).DistanceSq((__pyx_v_q->c_data[0])))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 934, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":933
 *         return <float>(<ovrQuatf>self).c_data[0].Distance(q.c_data[0])
 * 
 *     def distanceSq(self, ovrQuatf q):             # <<<<<<<<<<<<<<
 *         return <float>(<ovrQuatf>self).c_data[0].DistanceSq(q.c_data[0])
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.distanceSq", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":936
 *         return <float>(<ovrQuatf>self).c_data[0].DistanceSq(q.c_data[0])
 * 
 *     def dot(self, ovrQuatf q):             # <<<<<<<<<<<<<<
 *         return <float>(<ovrQuatf>self).c_data[0].Dot(q.c_data[0])
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_47dot(PyObject *__pyx_v_self, PyObject *__pyx_v_q); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_47dot(PyObject *__pyx_v_self, PyObject *__pyx_v_q) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("dot (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_q), __pyx_ptype_7psychxr_3ovr_4math_ovrQuatf, 1, "q", 0))) __PYX_ERR(0, 936, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_46dot(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_q));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_46dot(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_q) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("dot", 0);

  /* "psychxr/ovr/math.pyx":937
 * 
 *     def dot(self, ovrQuatf q):
 *         return <float>(<ovrQuatf>self).c_data[0].Dot(q.c_data[0])             # <<<<<<<<<<<<<<
 * 
 *     def angle(self, ovrQuatf q):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(((float)(__pyx_v_self->c_data[0]).Dot((__pyx_v_q->c_data[0])))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 937, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":936
 *         return <float>(<ovrQuatf>self).c_data[0].DistanceSq(q.c_data[0])
 * 
 *     def dot(self, ovrQuatf q):             # <<<<<<<<<<<<<<
 *         return <float>(<ovrQuatf>self).c_data[0].Dot(q.c_data[0])
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.dot", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":939
 *         return <float>(<ovrQuatf>self).c_data[0].Dot(q.c_data[0])
 * 
 *     def angle(self, ovrQuatf q):             # <<<<<<<<<<<<<<
 *         return <float>(<ovrQuatf>self).c_data[0].Angle(q.c_data[0])
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_49angle(PyObject *__pyx_v_self, PyObject *__pyx_v_q); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_49angle(PyObject *__pyx_v_self, PyObject *__pyx_v_q) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("angle (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_q), __pyx_ptype_7psychxr_3ovr_4math_ovrQuatf, 1, "q", 0))) __PYX_ERR(0, 939, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_48angle(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_q));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_48angle(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_q) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("angle", 0);

  /* "psychxr/ovr/math.pyx":940
 * 
 *     def angle(self, ovrQuatf q):
 *         return <float>(<ovrQuatf>self).c_data[0].Angle(q.c_data[0])             # <<<<<<<<<<<<<<
 * 
 *     def isNormalized(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(((float)(__pyx_v_self->c_data[0]).Angle((__pyx_v_q->c_data[0])))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 940, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":939
 *         return <float>(<ovrQuatf>self).c_data[0].Dot(q.c_data[0])
 * 
 *     def angle(self, ovrQuatf q):             # <<<<<<<<<<<<<<
 *         return <float>(<ovrQuatf>self).c_data[0].Angle(q.c_data[0])
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.angle", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":942
 *         return <float>(<ovrQuatf>self).c_data[0].Angle(q.c_data[0])
 * 
 *     def isNormalized(self):             # <<<<<<<<<<<<<<
 *         return <bint>(<ovrQuatf>self).c_data[0].IsNormalized()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_51isNormalized(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_51isNormalized(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("isNormalized (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_50isNormalized(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_50isNormalized(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("isNormalized", 0);

  /* "psychxr/ovr/math.pyx":943
 * 
 *     def isNormalized(self):
 *         return <bint>(<ovrQuatf>self).c_data[0].IsNormalized()             # <<<<<<<<<<<<<<
 * 
 *     def normalize(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(((__pyx_v_self->c_data[0]).IsNormalized() != 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 943, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":942
 *         return <float>(<ovrQuatf>self).c_data[0].Angle(q.c_data[0])
 * 
 *     def isNormalized(self):             # <<<<<<<<<<<<<<
 *         return <bint>(<ovrQuatf>self).c_data[0].IsNormalized()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.isNormalized", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":945
 *         return <bint>(<ovrQuatf>self).c_data[0].IsNormalized()
 * 
 *     def normalize(self):             # <<<<<<<<<<<<<<
 *         (<ovrQuatf>self).c_data[0].Normalize()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_53normalize(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_53normalize(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("normalize (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_52normalize(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_52normalize(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("normalize", 0);

  /* "psychxr/ovr/math.pyx":946
 * 
 *     def normalize(self):
 *         (<ovrQuatf>self).c_data[0].Normalize()             # <<<<<<<<<<<<<<
 * 
 *         return self
 */
  (__pyx_v_self->c_data[0]).Normalize();

  /* "psychxr/ovr/math.pyx":948
 *         (<ovrQuatf>self).c_data[0].Normalize()
 * 
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     def normalized(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":945
 *         return <bint>(<ovrQuatf>self).c_data[0].IsNormalized()
 * 
 *     def normalize(self):             # <<<<<<<<<<<<<<
 *         (<ovrQuatf>self).c_data[0].Normalize()
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":950
 *         return self
 * 
 *     def normalized(self):             # <<<<<<<<<<<<<<
 *         cdef ovrQuatf to_return = ovrQuatf()
 *         (<ovrQuatf>to_return).c_data[0] = \
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_55normalized(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_55normalized(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("normalized (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_54normalized(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_54normalized(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("normalized", 0);

  /* "psychxr/ovr/math.pyx":951
 * 
 *     def normalized(self):
 *         cdef ovrQuatf to_return = ovrQuatf()             # <<<<<<<<<<<<<<
 *         (<ovrQuatf>to_return).c_data[0] = \
 *             (<ovrQuatf>self).c_data[0].Normalized()
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrQuatf), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 951, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":952
 *     def normalized(self):
 *         cdef ovrQuatf to_return = ovrQuatf()
 *         (<ovrQuatf>to_return).c_data[0] = \             # <<<<<<<<<<<<<<
 *             (<ovrQuatf>self).c_data[0].Normalized()
 * 
 */
  (__pyx_v_to_return->c_data[0]) = (__pyx_v_self->c_data[0]).Normalized();

  /* "psychxr/ovr/math.pyx":955
 *             (<ovrQuatf>self).c_data[0].Normalized()
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def conj(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":950
 *         return self
 * 
 *     def normalized(self):             # <<<<<<<<<<<<<<
 *         cdef ovrQuatf to_return = ovrQuatf()
 *         (<ovrQuatf>to_return).c_data[0] = \
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.normalized", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":957
 *         return to_return
 * 
 *     def conj(self):             # <<<<<<<<<<<<<<
 *         cdef ovrQuatf to_return = ovrQuatf()
 *         (<ovrQuatf>to_return).c_data[0] = (<ovrQuatf>self).c_data[0].Conj()
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_57conj(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_57conj(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("conj (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_56conj(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_56conj(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("conj", 0);

  /* "psychxr/ovr/math.pyx":958
 * 
 *     def conj(self):
 *         cdef ovrQuatf to_return = ovrQuatf()             # <<<<<<<<<<<<<<
 *         (<ovrQuatf>to_return).c_data[0] = (<ovrQuatf>self).c_data[0].Conj()
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrQuatf), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 958, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":959
 *     def conj(self):
 *         cdef ovrQuatf to_return = ovrQuatf()
 *         (<ovrQuatf>to_return).c_data[0] = (<ovrQuatf>self).c_data[0].Conj()             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
  (__pyx_v_to_return->c_data[0]) = (__pyx_v_self->c_data[0]).Conj();

  /* "psychxr/ovr/math.pyx":961
 *         (<ovrQuatf>to_return).c_data[0] = (<ovrQuatf>self).c_data[0].Conj()
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":957
 *         return to_return
 * 
 *     def conj(self):             # <<<<<<<<<<<<<<
 *         cdef ovrQuatf to_return = ovrQuatf()
 *         (<ovrQuatf>to_return).c_data[0] = (<ovrQuatf>self).c_data[0].Conj()
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.conj", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":964
 * 
 *     @staticmethod
 *     def align(ovrVector3f align_to, ovrVector3f v):             # <<<<<<<<<<<<<<
 *         cdef ovrQuatf to_return = ovrQuatf()
 *         (<ovrQuatf>to_return).c_data[0] = ovr_math.Quatf.Align(
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_59align(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_7psychxr_3ovr_4math_8ovrQuatf_59align = {"align", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_59align, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_59align(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_align_to = 0;
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_v = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("align (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_align_to,&__pyx_n_s_v,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_align_to)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_v)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("align", 1, 2, 2, 1); __PYX_ERR(0, 964, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "align") < 0)) __PYX_ERR(0, 964, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_align_to = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)values[0]);
    __pyx_v_v = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("align", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 964, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.align", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_align_to), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "align_to", 0))) __PYX_ERR(0, 964, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_v), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "v", 0))) __PYX_ERR(0, 964, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_58align(__pyx_v_align_to, __pyx_v_v);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_58align(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_align_to, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_v) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("align", 0);

  /* "psychxr/ovr/math.pyx":965
 *     @staticmethod
 *     def align(ovrVector3f align_to, ovrVector3f v):
 *         cdef ovrQuatf to_return = ovrQuatf()             # <<<<<<<<<<<<<<
 *         (<ovrQuatf>to_return).c_data[0] = ovr_math.Quatf.Align(
 *             align_to.c_data[0], v.c_data[0])
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrQuatf), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 965, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":966
 *     def align(ovrVector3f align_to, ovrVector3f v):
 *         cdef ovrQuatf to_return = ovrQuatf()
 *         (<ovrQuatf>to_return).c_data[0] = ovr_math.Quatf.Align(             # <<<<<<<<<<<<<<
 *             align_to.c_data[0], v.c_data[0])
 * 
 */
  (__pyx_v_to_return->c_data[0]) = OVR::Quatf::Align((__pyx_v_align_to->c_data[0]), (__pyx_v_v->c_data[0]));

  /* "psychxr/ovr/math.pyx":969
 *             align_to.c_data[0], v.c_data[0])
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def rotate(self, ovrVector3f v):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":964
 * 
 *     @staticmethod
 *     def align(ovrVector3f align_to, ovrVector3f v):             # <<<<<<<<<<<<<<
 *         cdef ovrQuatf to_return = ovrQuatf()
 *         (<ovrQuatf>to_return).c_data[0] = ovr_math.Quatf.Align(
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.align", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":971
 *         return to_return
 * 
 *     def rotate(self, ovrVector3f v):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_61rotate(PyObject *__pyx_v_self, PyObject *__pyx_v_v); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_61rotate(PyObject *__pyx_v_self, PyObject *__pyx_v_v) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("rotate (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_v), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "v", 0))) __PYX_ERR(0, 971, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_60rotate(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_v));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_60rotate(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_v) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("rotate", 0);

  /* "psychxr/ovr/math.pyx":972
 * 
 *     def rotate(self, ovrVector3f v):
 *         cdef ovrVector3f to_return = ovrVector3f()             # <<<<<<<<<<<<<<
 *         (<ovrVector3f>to_return).c_data[0] = \
 *             (<ovrQuatf>self).c_data[0].Rotate(v.c_data[0])
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector3f), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 972, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":973
 *     def rotate(self, ovrVector3f v):
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \             # <<<<<<<<<<<<<<
 *             (<ovrQuatf>self).c_data[0].Rotate(v.c_data[0])
 * 
 */
  (__pyx_v_to_return->c_data[0]) = (__pyx_v_self->c_data[0]).Rotate((__pyx_v_v->c_data[0]));

  /* "psychxr/ovr/math.pyx":976
 *             (<ovrQuatf>self).c_data[0].Rotate(v.c_data[0])
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def inverseRotate(self, ovrVector3f v):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":971
 *         return to_return
 * 
 *     def rotate(self, ovrVector3f v):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.rotate", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":978
 *         return to_return
 * 
 *     def inverseRotate(self, ovrVector3f v):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_63inverseRotate(PyObject *__pyx_v_self, PyObject *__pyx_v_v); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_63inverseRotate(PyObject *__pyx_v_self, PyObject *__pyx_v_v) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("inverseRotate (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_v), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "v", 0))) __PYX_ERR(0, 978, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_62inverseRotate(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_v));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_62inverseRotate(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_v) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("inverseRotate", 0);

  /* "psychxr/ovr/math.pyx":979
 * 
 *     def inverseRotate(self, ovrVector3f v):
 *         cdef ovrVector3f to_return = ovrVector3f()             # <<<<<<<<<<<<<<
 *         (<ovrVector3f>to_return).c_data[0] = \
 *             (<ovrQuatf>self).c_data[0].InverseRotate(v.c_data[0])
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector3f), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 979, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":980
 *     def inverseRotate(self, ovrVector3f v):
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \             # <<<<<<<<<<<<<<
 *             (<ovrQuatf>self).c_data[0].InverseRotate(v.c_data[0])
 * 
 */
  (__pyx_v_to_return->c_data[0]) = (__pyx_v_self->c_data[0]).InverseRotate((__pyx_v_v->c_data[0]));

  /* "psychxr/ovr/math.pyx":983
 *             (<ovrQuatf>self).c_data[0].InverseRotate(v.c_data[0])
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def inverted(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":978
 *         return to_return
 * 
 *     def inverseRotate(self, ovrVector3f v):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.inverseRotate", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":985
 *         return to_return
 * 
 *     def inverted(self):             # <<<<<<<<<<<<<<
 *         cdef ovrQuatf to_return = ovrQuatf()
 *         (<ovrQuatf>to_return).c_data[0] = (<ovrQuatf>self).c_data[0].Inverted()
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_65inverted(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_65inverted(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("inverted (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_64inverted(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_64inverted(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("inverted", 0);

  /* "psychxr/ovr/math.pyx":986
 * 
 *     def inverted(self):
 *         cdef ovrQuatf to_return = ovrQuatf()             # <<<<<<<<<<<<<<
 *         (<ovrQuatf>to_return).c_data[0] = (<ovrQuatf>self).c_data[0].Inverted()
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrQuatf), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 986, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":987
 *     def inverted(self):
 *         cdef ovrQuatf to_return = ovrQuatf()
 *         (<ovrQuatf>to_return).c_data[0] = (<ovrQuatf>self).c_data[0].Inverted()             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
  (__pyx_v_to_return->c_data[0]) = (__pyx_v_self->c_data[0]).Inverted();

  /* "psychxr/ovr/math.pyx":989
 *         (<ovrQuatf>to_return).c_data[0] = (<ovrQuatf>self).c_data[0].Inverted()
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def inverse(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":985
 *         return to_return
 * 
 *     def inverted(self):             # <<<<<<<<<<<<<<
 *         cdef ovrQuatf to_return = ovrQuatf()
 *         (<ovrQuatf>to_return).c_data[0] = (<ovrQuatf>self).c_data[0].Inverted()
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.inverted", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":991
 *         return to_return
 * 
 *     def inverse(self):             # <<<<<<<<<<<<<<
 *         cdef ovrQuatf to_return = ovrQuatf()
 *         (<ovrQuatf>to_return).c_data[0] = (<ovrQuatf>self).c_data[0].Inverse()
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_67inverse(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_67inverse(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("inverse (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_66inverse(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_66inverse(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("inverse", 0);

  /* "psychxr/ovr/math.pyx":992
 * 
 *     def inverse(self):
 *         cdef ovrQuatf to_return = ovrQuatf()             # <<<<<<<<<<<<<<
 *         (<ovrQuatf>to_return).c_data[0] = (<ovrQuatf>self).c_data[0].Inverse()
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrQuatf), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 992, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":993
 *     def inverse(self):
 *         cdef ovrQuatf to_return = ovrQuatf()
 *         (<ovrQuatf>to_return).c_data[0] = (<ovrQuatf>self).c_data[0].Inverse()             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
  (__pyx_v_to_return->c_data[0]) = (__pyx_v_self->c_data[0]).Inverse();

  /* "psychxr/ovr/math.pyx":995
 *         (<ovrQuatf>to_return).c_data[0] = (<ovrQuatf>self).c_data[0].Inverse()
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def invert(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":991
 *         return to_return
 * 
 *     def inverse(self):             # <<<<<<<<<<<<<<
 *         cdef ovrQuatf to_return = ovrQuatf()
 *         (<ovrQuatf>to_return).c_data[0] = (<ovrQuatf>self).c_data[0].Inverse()
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.inverse", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":997
 *         return to_return
 * 
 *     def invert(self):             # <<<<<<<<<<<<<<
 *         (<ovrQuatf>self).c_data[0].Inverse()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_69invert(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_69invert(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("invert (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_68invert(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_68invert(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("invert", 0);

  /* "psychxr/ovr/math.pyx":998
 * 
 *     def invert(self):
 *         (<ovrQuatf>self).c_data[0].Inverse()             # <<<<<<<<<<<<<<
 * 
 *         return self
 */
  (__pyx_v_self->c_data[0]).Inverse();

  /* "psychxr/ovr/math.pyx":1000
 *         (<ovrQuatf>self).c_data[0].Inverse()
 * 
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     def __invert__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":997
 *         return to_return
 * 
 *     def invert(self):             # <<<<<<<<<<<<<<
 *         (<ovrQuatf>self).c_data[0].Inverse()
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1002
 *         return self
 * 
 *     def __invert__(self):             # <<<<<<<<<<<<<<
 *         return self.inverse()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_71__invert__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_71__invert__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__invert__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_70__invert__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_70__invert__(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("__invert__", 0);

  /* "psychxr/ovr/math.pyx":1003
 * 
 *     def __invert__(self):
 *         return self.inverse()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_inverse); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1003, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1003, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1003, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1002
 *         return self
 * 
 *     def __invert__(self):             # <<<<<<<<<<<<<<
 *         return self.inverse()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.__invert__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_73__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_73__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_72__reduce_cython__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_72__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__12, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_75__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_75__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_74__setstate_cython__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrQuatf_74__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__13, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrQuatf.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1012
 * 
 *     """
 *     def __cinit__(self, *args, **kwargs):             # <<<<<<<<<<<<<<
 *         self.c_data = &self.c_Posef
 * 
 */

/* Python wrapper */
static int __pyx_pw_7psychxr_3ovr_4math_8ovrPosef_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_7psychxr_3ovr_4math_8ovrPosef_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_args = 0;
  CYTHON_UNUSED PyObject *__pyx_v_kwargs = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  if (unlikely(__pyx_kwds) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__cinit__", 1))) return -1;
  __Pyx_INCREF(__pyx_args);
  __pyx_v_args = __pyx_args;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrPosef___cinit__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *)__pyx_v_self), __pyx_v_args, __pyx_v_kwargs);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_args);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7psychxr_3ovr_4math_8ovrPosef___cinit__(struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *__pyx_v_self, PyObject *__pyx_v_args, CYTHON_UNUSED PyObject *__pyx_v_kwargs) {
  int __pyx_v_nargin;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  OVR::Vector3f __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "psychxr/ovr/math.pyx":1013
 *     """
 *     def __cinit__(self, *args, **kwargs):
 *         self.c_data = &self.c_Posef             # <<<<<<<<<<<<<<
 * 
 *         cdef int nargin = <int>len(args)  # get number of arguments
 */
  __pyx_v_self->c_data = (&__pyx_v_self->c_Posef);

  /* "psychxr/ovr/math.pyx":1015
 *         self.c_data = &self.c_Posef
 * 
 *         cdef int nargin = <int>len(args)  # get number of arguments             # <<<<<<<<<<<<<<
 *         if nargin == 0:
 *             self.c_data[0].Translation = ovr_math.Vector3f(0.0, 0.0, 0.0)
 */
  __pyx_t_1 = PyTuple_GET_SIZE(__pyx_v_args); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1015, __pyx_L1_error)
  __pyx_v_nargin = ((int)__pyx_t_1);

  /* "psychxr/ovr/math.pyx":1016
 * 
 *         cdef int nargin = <int>len(args)  # get number of arguments
 *         if nargin == 0:             # <<<<<<<<<<<<<<
 *             self.c_data[0].Translation = ovr_math.Vector3f(0.0, 0.0, 0.0)
 *             self.c_data[0].Rotation = ovr_math.Quatf.Identity()
 */
  switch (__pyx_v_nargin) {
    case 0:

    /* "psychxr/ovr/math.pyx":1017
 *         cdef int nargin = <int>len(args)  # get number of arguments
 *         if nargin == 0:
 *             self.c_data[0].Translation = ovr_math.Vector3f(0.0, 0.0, 0.0)             # <<<<<<<<<<<<<<
 *             self.c_data[0].Rotation = ovr_math.Quatf.Identity()
 *         elif nargin == 1:
 */
    try {
      __pyx_t_2 = OVR::Vector3f(0.0, 0.0, 0.0);
    } catch(...) {
      __Pyx_CppExn2PyErr();
      __PYX_ERR(0, 1017, __pyx_L1_error)
    }
    (__pyx_v_self->c_data[0]).Translation = __pyx_t_2;

    /* "psychxr/ovr/math.pyx":1018
 *         if nargin == 0:
 *             self.c_data[0].Translation = ovr_math.Vector3f(0.0, 0.0, 0.0)
 *             self.c_data[0].Rotation = ovr_math.Quatf.Identity()             # <<<<<<<<<<<<<<
 *         elif nargin == 1:
 *             if isinstance(args[0], ovrVector3f):
 */
    (__pyx_v_self->c_data[0]).Rotation = OVR::Quatf::Identity();

    /* "psychxr/ovr/math.pyx":1016
 * 
 *         cdef int nargin = <int>len(args)  # get number of arguments
 *         if nargin == 0:             # <<<<<<<<<<<<<<
 *             self.c_data[0].Translation = ovr_math.Vector3f(0.0, 0.0, 0.0)
 *             self.c_data[0].Rotation = ovr_math.Quatf.Identity()
 */
    break;

    /* "psychxr/ovr/math.pyx":1019
 *             self.c_data[0].Translation = ovr_math.Vector3f(0.0, 0.0, 0.0)
 *             self.c_data[0].Rotation = ovr_math.Quatf.Identity()
 *         elif nargin == 1:             # <<<<<<<<<<<<<<
 *             if isinstance(args[0], ovrVector3f):
 *                 self.c_data[0].Translation = (<ovrVector3f>args[0]).c_data[0]
 */
    case 1:

    /* "psychxr/ovr/math.pyx":1020
 *             self.c_data[0].Rotation = ovr_math.Quatf.Identity()
 *         elif nargin == 1:
 *             if isinstance(args[0], ovrVector3f):             # <<<<<<<<<<<<<<
 *                 self.c_data[0].Translation = (<ovrVector3f>args[0]).c_data[0]
 *             elif isinstance(args[0], ovrQuatf):
 */
    __pyx_t_3 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1020, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_TypeCheck(__pyx_t_3, __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f); 
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_5 = (__pyx_t_4 != 0);
    if (__pyx_t_5) {

      /* "psychxr/ovr/math.pyx":1021
 *         elif nargin == 1:
 *             if isinstance(args[0], ovrVector3f):
 *                 self.c_data[0].Translation = (<ovrVector3f>args[0]).c_data[0]             # <<<<<<<<<<<<<<
 *             elif isinstance(args[0], ovrQuatf):
 *                 self.c_data[0].Rotation = (<ovrQuatf>args[0]).c_data[0]
 */
      __pyx_t_3 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1021, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      (__pyx_v_self->c_data[0]).Translation = (((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_t_3)->c_data[0]);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "psychxr/ovr/math.pyx":1020
 *             self.c_data[0].Rotation = ovr_math.Quatf.Identity()
 *         elif nargin == 1:
 *             if isinstance(args[0], ovrVector3f):             # <<<<<<<<<<<<<<
 *                 self.c_data[0].Translation = (<ovrVector3f>args[0]).c_data[0]
 *             elif isinstance(args[0], ovrQuatf):
 */
      goto __pyx_L3;
    }

    /* "psychxr/ovr/math.pyx":1022
 *             if isinstance(args[0], ovrVector3f):
 *                 self.c_data[0].Translation = (<ovrVector3f>args[0]).c_data[0]
 *             elif isinstance(args[0], ovrQuatf):             # <<<<<<<<<<<<<<
 *                 self.c_data[0].Rotation = (<ovrQuatf>args[0]).c_data[0]
 *         elif nargin == 2:
 */
    __pyx_t_3 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1022, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = __Pyx_TypeCheck(__pyx_t_3, __pyx_ptype_7psychxr_3ovr_4math_ovrQuatf); 
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_4 = (__pyx_t_5 != 0);
    if (__pyx_t_4) {

      /* "psychxr/ovr/math.pyx":1023
 *                 self.c_data[0].Translation = (<ovrVector3f>args[0]).c_data[0]
 *             elif isinstance(args[0], ovrQuatf):
 *                 self.c_data[0].Rotation = (<ovrQuatf>args[0]).c_data[0]             # <<<<<<<<<<<<<<
 *         elif nargin == 2:
 *             if isinstance(args[0], ovrVector3f) and \
 */
      __pyx_t_3 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1023, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      (__pyx_v_self->c_data[0]).Rotation = (((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_t_3)->c_data[0]);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "psychxr/ovr/math.pyx":1022
 *             if isinstance(args[0], ovrVector3f):
 *                 self.c_data[0].Translation = (<ovrVector3f>args[0]).c_data[0]
 *             elif isinstance(args[0], ovrQuatf):             # <<<<<<<<<<<<<<
 *                 self.c_data[0].Rotation = (<ovrQuatf>args[0]).c_data[0]
 *         elif nargin == 2:
 */
    }
    __pyx_L3:;

    /* "psychxr/ovr/math.pyx":1019
 *             self.c_data[0].Translation = ovr_math.Vector3f(0.0, 0.0, 0.0)
 *             self.c_data[0].Rotation = ovr_math.Quatf.Identity()
 *         elif nargin == 1:             # <<<<<<<<<<<<<<
 *             if isinstance(args[0], ovrVector3f):
 *                 self.c_data[0].Translation = (<ovrVector3f>args[0]).c_data[0]
 */
    break;

    /* "psychxr/ovr/math.pyx":1024
 *             elif isinstance(args[0], ovrQuatf):
 *                 self.c_data[0].Rotation = (<ovrQuatf>args[0]).c_data[0]
 *         elif nargin == 2:             # <<<<<<<<<<<<<<
 *             if isinstance(args[0], ovrVector3f) and \
 *                     isinstance(args[1], ovrQuatf):
 */
    case 2:

    /* "psychxr/ovr/math.pyx":1025
 *                 self.c_data[0].Rotation = (<ovrQuatf>args[0]).c_data[0]
 *         elif nargin == 2:
 *             if isinstance(args[0], ovrVector3f) and \             # <<<<<<<<<<<<<<
 *                     isinstance(args[1], ovrQuatf):
 *                 self.c_data[0].Translation = (<ovrVector3f>args[0]).c_data[0]
 */
    __pyx_t_3 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1025, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = __Pyx_TypeCheck(__pyx_t_3, __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f); 
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_6 = (__pyx_t_5 != 0);
    if (__pyx_t_6) {
    } else {
      __pyx_t_4 = __pyx_t_6;
      goto __pyx_L5_bool_binop_done;
    }

    /* "psychxr/ovr/math.pyx":1026
 *         elif nargin == 2:
 *             if isinstance(args[0], ovrVector3f) and \
 *                     isinstance(args[1], ovrQuatf):             # <<<<<<<<<<<<<<
 *                 self.c_data[0].Translation = (<ovrVector3f>args[0]).c_data[0]
 *                 self.c_data[0].Rotation = (<ovrQuatf>args[1]).c_data[0]
 */
    __pyx_t_3 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1026, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_6 = __Pyx_TypeCheck(__pyx_t_3, __pyx_ptype_7psychxr_3ovr_4math_ovrQuatf); 
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_5 = (__pyx_t_6 != 0);
    __pyx_t_4 = __pyx_t_5;
    __pyx_L5_bool_binop_done:;

    /* "psychxr/ovr/math.pyx":1025
 *                 self.c_data[0].Rotation = (<ovrQuatf>args[0]).c_data[0]
 *         elif nargin == 2:
 *             if isinstance(args[0], ovrVector3f) and \             # <<<<<<<<<<<<<<
 *                     isinstance(args[1], ovrQuatf):
 *                 self.c_data[0].Translation = (<ovrVector3f>args[0]).c_data[0]
 */
    if (__pyx_t_4) {

      /* "psychxr/ovr/math.pyx":1027
 *             if isinstance(args[0], ovrVector3f) and \
 *                     isinstance(args[1], ovrQuatf):
 *                 self.c_data[0].Translation = (<ovrVector3f>args[0]).c_data[0]             # <<<<<<<<<<<<<<
 *                 self.c_data[0].Rotation = (<ovrQuatf>args[1]).c_data[0]
 * 
 */
      __pyx_t_3 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1027, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      (__pyx_v_self->c_data[0]).Translation = (((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_t_3)->c_data[0]);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "psychxr/ovr/math.pyx":1028
 *                     isinstance(args[1], ovrQuatf):
 *                 self.c_data[0].Translation = (<ovrVector3f>args[0]).c_data[0]
 *                 self.c_data[0].Rotation = (<ovrQuatf>args[1]).c_data[0]             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
      __pyx_t_3 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1028, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      (__pyx_v_self->c_data[0]).Rotation = (((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_t_3)->c_data[0]);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "psychxr/ovr/math.pyx":1025
 *                 self.c_data[0].Rotation = (<ovrQuatf>args[0]).c_data[0]
 *         elif nargin == 2:
 *             if isinstance(args[0], ovrVector3f) and \             # <<<<<<<<<<<<<<
 *                     isinstance(args[1], ovrQuatf):
 *                 self.c_data[0].Translation = (<ovrVector3f>args[0]).c_data[0]
 */
    }

    /* "psychxr/ovr/math.pyx":1024
 *             elif isinstance(args[0], ovrQuatf):
 *                 self.c_data[0].Rotation = (<ovrQuatf>args[0]).c_data[0]
 *         elif nargin == 2:             # <<<<<<<<<<<<<<
 *             if isinstance(args[0], ovrVector3f) and \
 *                     isinstance(args[1], ovrQuatf):
 */
    break;
    default: break;
  }

  /* "psychxr/ovr/math.pyx":1012
 * 
 *     """
 *     def __cinit__(self, *args, **kwargs):             # <<<<<<<<<<<<<<
 *         self.c_data = &self.c_Posef
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrPosef.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1031
 * 
 *     @property
 *     def rotation(self):             # <<<<<<<<<<<<<<
 *         cdef ovrQuatf to_return = ovrQuatf()
 *         (<ovrQuatf>to_return).c_data[0] = self.c_data[0].Rotation
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_8rotation_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_8rotation_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrPosef_8rotation___get__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrPosef_8rotation___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *__pyx_v_self) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  OVR::Quatf __pyx_t_2;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "psychxr/ovr/math.pyx":1032
 *     @property
 *     def rotation(self):
 *         cdef ovrQuatf to_return = ovrQuatf()             # <<<<<<<<<<<<<<
 *         (<ovrQuatf>to_return).c_data[0] = self.c_data[0].Rotation
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrQuatf), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1032, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":1033
 *     def rotation(self):
 *         cdef ovrQuatf to_return = ovrQuatf()
 *         (<ovrQuatf>to_return).c_data[0] = self.c_data[0].Rotation             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
  __pyx_t_2 = (__pyx_v_self->c_data[0]).Rotation;
  (__pyx_v_to_return->c_data[0]) = __pyx_t_2;

  /* "psychxr/ovr/math.pyx":1035
 *         (<ovrQuatf>to_return).c_data[0] = self.c_data[0].Rotation
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     @rotation.setter
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1031
 * 
 *     @property
 *     def rotation(self):             # <<<<<<<<<<<<<<
 *         cdef ovrQuatf to_return = ovrQuatf()
 *         (<ovrQuatf>to_return).c_data[0] = self.c_data[0].Rotation
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrPosef.rotation.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1038
 * 
 *     @rotation.setter
 *     def rotation(self, ovrQuatf value):             # <<<<<<<<<<<<<<
 *         (<ovrPosef>self).c_data[0].Rotation = value.c_data[0]
 * 
 */

/* Python wrapper */
static int __pyx_pw_7psychxr_3ovr_4math_8ovrPosef_8rotation_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_7psychxr_3ovr_4math_8ovrPosef_8rotation_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_value), __pyx_ptype_7psychxr_3ovr_4math_ovrQuatf, 1, "value", 0))) __PYX_ERR(0, 1038, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrPosef_8rotation_2__set__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_v_value));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7psychxr_3ovr_4math_8ovrPosef_8rotation_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "psychxr/ovr/math.pyx":1039
 *     @rotation.setter
 *     def rotation(self, ovrQuatf value):
 *         (<ovrPosef>self).c_data[0].Rotation = value.c_data[0]             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  (__pyx_v_self->c_data[0]).Rotation = (__pyx_v_value->c_data[0]);

  /* "psychxr/ovr/math.pyx":1038
 * 
 *     @rotation.setter
 *     def rotation(self, ovrQuatf value):             # <<<<<<<<<<<<<<
 *         (<ovrPosef>self).c_data[0].Rotation = value.c_data[0]
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1042
 * 
 *     @property
 *     def translation(self):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = self.c_data[0].Translation
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_11translation_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_11translation_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrPosef_11translation___get__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrPosef_11translation___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *__pyx_v_self) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  OVR::Vector3f __pyx_t_2;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "psychxr/ovr/math.pyx":1043
 *     @property
 *     def translation(self):
 *         cdef ovrVector3f to_return = ovrVector3f()             # <<<<<<<<<<<<<<
 *         (<ovrVector3f>to_return).c_data[0] = self.c_data[0].Translation
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector3f), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1043, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":1044
 *     def translation(self):
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = self.c_data[0].Translation             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
  __pyx_t_2 = (__pyx_v_self->c_data[0]).Translation;
  (__pyx_v_to_return->c_data[0]) = __pyx_t_2;

  /* "psychxr/ovr/math.pyx":1046
 *         (<ovrVector3f>to_return).c_data[0] = self.c_data[0].Translation
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     @translation.setter
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1042
 * 
 *     @property
 *     def translation(self):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = self.c_data[0].Translation
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrPosef.translation.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1049
 * 
 *     @translation.setter
 *     def translation(self, ovrVector3f value):             # <<<<<<<<<<<<<<
 *         (<ovrPosef>self).c_data[0].Translation = value.c_data[0]
 * 
 */

/* Python wrapper */
static int __pyx_pw_7psychxr_3ovr_4math_8ovrPosef_11translation_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_7psychxr_3ovr_4math_8ovrPosef_11translation_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_value), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "value", 0))) __PYX_ERR(0, 1049, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrPosef_11translation_2__set__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_value));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7psychxr_3ovr_4math_8ovrPosef_11translation_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "psychxr/ovr/math.pyx":1050
 *     @translation.setter
 *     def translation(self, ovrVector3f value):
 *         (<ovrPosef>self).c_data[0].Translation = value.c_data[0]             # <<<<<<<<<<<<<<
 * 
 *     def rotate(self, ovrVector3f v):
 */
  (__pyx_v_self->c_data[0]).Translation = (__pyx_v_value->c_data[0]);

  /* "psychxr/ovr/math.pyx":1049
 * 
 *     @translation.setter
 *     def translation(self, ovrVector3f value):             # <<<<<<<<<<<<<<
 *         (<ovrPosef>self).c_data[0].Translation = value.c_data[0]
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1052
 *         (<ovrPosef>self).c_data[0].Translation = value.c_data[0]
 * 
 *     def rotate(self, ovrVector3f v):             # <<<<<<<<<<<<<<
 *         return self.rotation.rotate(v)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_3rotate(PyObject *__pyx_v_self, PyObject *__pyx_v_v); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_3rotate(PyObject *__pyx_v_self, PyObject *__pyx_v_v) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("rotate (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_v), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "v", 0))) __PYX_ERR(0, 1052, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrPosef_2rotate(((struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_v));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrPosef_2rotate(struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_v) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("rotate", 0);

  /* "psychxr/ovr/math.pyx":1053
 * 
 *     def rotate(self, ovrVector3f v):
 *         return self.rotation.rotate(v)             # <<<<<<<<<<<<<<
 * 
 *     def inverseRotate(self, ovrVector3f v):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_rotation); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1053, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_rotate); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1053, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_2) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, ((PyObject *)__pyx_v_v)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1053, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[2] = {__pyx_t_2, ((PyObject *)__pyx_v_v)};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1053, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[2] = {__pyx_t_2, ((PyObject *)__pyx_v_v)};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1053, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1053, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_2); __pyx_t_2 = NULL;
      __Pyx_INCREF(((PyObject *)__pyx_v_v));
      __Pyx_GIVEREF(((PyObject *)__pyx_v_v));
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, ((PyObject *)__pyx_v_v));
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1053, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1052
 *         (<ovrPosef>self).c_data[0].Translation = value.c_data[0]
 * 
 *     def rotate(self, ovrVector3f v):             # <<<<<<<<<<<<<<
 *         return self.rotation.rotate(v)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrPosef.rotate", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1055
 *         return self.rotation.rotate(v)
 * 
 *     def inverseRotate(self, ovrVector3f v):             # <<<<<<<<<<<<<<
 *         return self.rotation.inverseRotate(v)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_5inverseRotate(PyObject *__pyx_v_self, PyObject *__pyx_v_v); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_5inverseRotate(PyObject *__pyx_v_self, PyObject *__pyx_v_v) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("inverseRotate (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_v), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "v", 0))) __PYX_ERR(0, 1055, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrPosef_4inverseRotate(((struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_v));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrPosef_4inverseRotate(struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_v) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("inverseRotate", 0);

  /* "psychxr/ovr/math.pyx":1056
 * 
 *     def inverseRotate(self, ovrVector3f v):
 *         return self.rotation.inverseRotate(v)             # <<<<<<<<<<<<<<
 * 
 *     def translate(self, ovrVector3f v):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_rotation); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1056, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_inverseRotate); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1056, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_2) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, ((PyObject *)__pyx_v_v)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1056, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[2] = {__pyx_t_2, ((PyObject *)__pyx_v_v)};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1056, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[2] = {__pyx_t_2, ((PyObject *)__pyx_v_v)};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1056, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1056, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_2); __pyx_t_2 = NULL;
      __Pyx_INCREF(((PyObject *)__pyx_v_v));
      __Pyx_GIVEREF(((PyObject *)__pyx_v_v));
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, ((PyObject *)__pyx_v_v));
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1056, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1055
 *         return self.rotation.rotate(v)
 * 
 *     def inverseRotate(self, ovrVector3f v):             # <<<<<<<<<<<<<<
 *         return self.rotation.inverseRotate(v)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrPosef.inverseRotate", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1058
 *         return self.rotation.inverseRotate(v)
 * 
 *     def translate(self, ovrVector3f v):             # <<<<<<<<<<<<<<
 *         return v + self.translation
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_7translate(PyObject *__pyx_v_self, PyObject *__pyx_v_v); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_7translate(PyObject *__pyx_v_self, PyObject *__pyx_v_v) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("translate (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_v), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "v", 0))) __PYX_ERR(0, 1058, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrPosef_6translate(((struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_v));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrPosef_6translate(struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_v) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("translate", 0);

  /* "psychxr/ovr/math.pyx":1059
 * 
 *     def translate(self, ovrVector3f v):
 *         return v + self.translation             # <<<<<<<<<<<<<<
 * 
 *     def transform(self, ovrVector3f v):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_translation); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1059, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyNumber_Add(((PyObject *)__pyx_v_v), __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1059, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1058
 *         return self.rotation.inverseRotate(v)
 * 
 *     def translate(self, ovrVector3f v):             # <<<<<<<<<<<<<<
 *         return v + self.translation
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrPosef.translate", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1061
 *         return v + self.translation
 * 
 *     def transform(self, ovrVector3f v):             # <<<<<<<<<<<<<<
 *         return self.rotate(v) + self.translation
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_9transform(PyObject *__pyx_v_self, PyObject *__pyx_v_v); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_9transform(PyObject *__pyx_v_self, PyObject *__pyx_v_v) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("transform (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_v), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "v", 0))) __PYX_ERR(0, 1061, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrPosef_8transform(((struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_v));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrPosef_8transform(struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_v) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("transform", 0);

  /* "psychxr/ovr/math.pyx":1062
 * 
 *     def transform(self, ovrVector3f v):
 *         return self.rotate(v) + self.translation             # <<<<<<<<<<<<<<
 * 
 *     def inverseTransform(self, ovrVector3f v):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_rotate); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1062, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, ((PyObject *)__pyx_v_v)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1062, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, ((PyObject *)__pyx_v_v)};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1062, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, ((PyObject *)__pyx_v_v)};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1062, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1062, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(((PyObject *)__pyx_v_v));
      __Pyx_GIVEREF(((PyObject *)__pyx_v_v));
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, ((PyObject *)__pyx_v_v));
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1062, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_translation); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1062, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = PyNumber_Add(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1062, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1061
 *         return v + self.translation
 * 
 *     def transform(self, ovrVector3f v):             # <<<<<<<<<<<<<<
 *         return self.rotate(v) + self.translation
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrPosef.transform", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1064
 *         return self.rotate(v) + self.translation
 * 
 *     def inverseTransform(self, ovrVector3f v):             # <<<<<<<<<<<<<<
 *         return self.inverseRotate(v - self.translation)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_11inverseTransform(PyObject *__pyx_v_self, PyObject *__pyx_v_v); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_11inverseTransform(PyObject *__pyx_v_self, PyObject *__pyx_v_v) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("inverseTransform (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_v), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "v", 0))) __PYX_ERR(0, 1064, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrPosef_10inverseTransform(((struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_v));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrPosef_10inverseTransform(struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_v) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("inverseTransform", 0);

  /* "psychxr/ovr/math.pyx":1065
 * 
 *     def inverseTransform(self, ovrVector3f v):
 *         return self.inverseRotate(v - self.translation)             # <<<<<<<<<<<<<<
 * 
 *     def transformNormal(self, ovrVector3f v):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_inverseRotate); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1065, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_translation); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1065, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyNumber_Subtract(((PyObject *)__pyx_v_v), __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1065, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1065, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_t_4};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1065, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_t_4};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1065, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else
    #endif
    {
      __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1065, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_GIVEREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_4);
      __pyx_t_4 = 0;
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1065, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1064
 *         return self.rotate(v) + self.translation
 * 
 *     def inverseTransform(self, ovrVector3f v):             # <<<<<<<<<<<<<<
 *         return self.inverseRotate(v - self.translation)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrPosef.inverseTransform", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1067
 *         return self.inverseRotate(v - self.translation)
 * 
 *     def transformNormal(self, ovrVector3f v):             # <<<<<<<<<<<<<<
 *         return self.rotate(v)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_13transformNormal(PyObject *__pyx_v_self, PyObject *__pyx_v_v); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_13transformNormal(PyObject *__pyx_v_self, PyObject *__pyx_v_v) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("transformNormal (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_v), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "v", 0))) __PYX_ERR(0, 1067, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrPosef_12transformNormal(((struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_v));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrPosef_12transformNormal(struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_v) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("transformNormal", 0);

  /* "psychxr/ovr/math.pyx":1068
 * 
 *     def transformNormal(self, ovrVector3f v):
 *         return self.rotate(v)             # <<<<<<<<<<<<<<
 * 
 *     def inverseTransformNormal(self, ovrVector3f v):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_rotate); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1068, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, ((PyObject *)__pyx_v_v)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1068, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, ((PyObject *)__pyx_v_v)};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1068, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, ((PyObject *)__pyx_v_v)};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1068, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1068, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(((PyObject *)__pyx_v_v));
      __Pyx_GIVEREF(((PyObject *)__pyx_v_v));
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, ((PyObject *)__pyx_v_v));
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1068, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1067
 *         return self.inverseRotate(v - self.translation)
 * 
 *     def transformNormal(self, ovrVector3f v):             # <<<<<<<<<<<<<<
 *         return self.rotate(v)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrPosef.transformNormal", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1070
 *         return self.rotate(v)
 * 
 *     def inverseTransformNormal(self, ovrVector3f v):             # <<<<<<<<<<<<<<
 *         return self.inverseRotate(v)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_15inverseTransformNormal(PyObject *__pyx_v_self, PyObject *__pyx_v_v); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_15inverseTransformNormal(PyObject *__pyx_v_self, PyObject *__pyx_v_v) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("inverseTransformNormal (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_v), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "v", 0))) __PYX_ERR(0, 1070, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrPosef_14inverseTransformNormal(((struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_v));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrPosef_14inverseTransformNormal(struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_v) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("inverseTransformNormal", 0);

  /* "psychxr/ovr/math.pyx":1071
 * 
 *     def inverseTransformNormal(self, ovrVector3f v):
 *         return self.inverseRotate(v)             # <<<<<<<<<<<<<<
 * 
 *     def apply(self, ovrVector3f v):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_inverseRotate); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1071, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, ((PyObject *)__pyx_v_v)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1071, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, ((PyObject *)__pyx_v_v)};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1071, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, ((PyObject *)__pyx_v_v)};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1071, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1071, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(((PyObject *)__pyx_v_v));
      __Pyx_GIVEREF(((PyObject *)__pyx_v_v));
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, ((PyObject *)__pyx_v_v));
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1071, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1070
 *         return self.rotate(v)
 * 
 *     def inverseTransformNormal(self, ovrVector3f v):             # <<<<<<<<<<<<<<
 *         return self.inverseRotate(v)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrPosef.inverseTransformNormal", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1073
 *         return self.inverseRotate(v)
 * 
 *     def apply(self, ovrVector3f v):             # <<<<<<<<<<<<<<
 *         return self.transform(v)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_17apply(PyObject *__pyx_v_self, PyObject *__pyx_v_v); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_17apply(PyObject *__pyx_v_self, PyObject *__pyx_v_v) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("apply (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_v), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "v", 0))) __PYX_ERR(0, 1073, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrPosef_16apply(((struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_v));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrPosef_16apply(struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_v) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("apply", 0);

  /* "psychxr/ovr/math.pyx":1074
 * 
 *     def apply(self, ovrVector3f v):
 *         return self.transform(v)             # <<<<<<<<<<<<<<
 * 
 *     def __mul__(ovrPosef a, ovrPosef b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_transform); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1074, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, ((PyObject *)__pyx_v_v)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1074, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, ((PyObject *)__pyx_v_v)};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1074, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, ((PyObject *)__pyx_v_v)};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1074, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1074, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(((PyObject *)__pyx_v_v));
      __Pyx_GIVEREF(((PyObject *)__pyx_v_v));
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, ((PyObject *)__pyx_v_v));
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1074, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1073
 *         return self.inverseRotate(v)
 * 
 *     def apply(self, ovrVector3f v):             # <<<<<<<<<<<<<<
 *         return self.transform(v)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrPosef.apply", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1076
 *         return self.transform(v)
 * 
 *     def __mul__(ovrPosef a, ovrPosef b):             # <<<<<<<<<<<<<<
 *         cdef ovrPosef to_return = ovrPosef(
 *             a.apply(b.translation),
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_19__mul__(PyObject *__pyx_v_a, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_19__mul__(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__mul__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_a), __pyx_ptype_7psychxr_3ovr_4math_ovrPosef, 1, "a", 0))) __PYX_ERR(0, 1076, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrPosef, 1, "b", 0))) __PYX_ERR(0, 1076, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrPosef_18__mul__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *)__pyx_v_a), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrPosef_18__mul__(struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *__pyx_v_a, struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *__pyx_v_b) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("__mul__", 0);

  /* "psychxr/ovr/math.pyx":1078
 *     def __mul__(ovrPosef a, ovrPosef b):
 *         cdef ovrPosef to_return = ovrPosef(
 *             a.apply(b.translation),             # <<<<<<<<<<<<<<
 *             <ovrQuatf>a.rotation * <ovrQuatf>b.rotation)
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_a), __pyx_n_s_apply); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1078, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_b), __pyx_n_s_translation); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1078, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1078, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_t_3};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1078, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_t_3};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1078, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else
    #endif
    {
      __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1078, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
      __Pyx_GIVEREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_3);
      __pyx_t_3 = 0;
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1078, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "psychxr/ovr/math.pyx":1079
 *         cdef ovrPosef to_return = ovrPosef(
 *             a.apply(b.translation),
 *             <ovrQuatf>a.rotation * <ovrQuatf>b.rotation)             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_a), __pyx_n_s_rotation); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1079, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_b), __pyx_n_s_rotation); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1079, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = PyNumber_Multiply(__pyx_t_2, __pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1079, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "psychxr/ovr/math.pyx":1077
 * 
 *     def __mul__(ovrPosef a, ovrPosef b):
 *         cdef ovrPosef to_return = ovrPosef(             # <<<<<<<<<<<<<<
 *             a.apply(b.translation),
 *             <ovrQuatf>a.rotation * <ovrQuatf>b.rotation)
 */
  __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1077, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_3);
  __pyx_t_1 = 0;
  __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrPosef), __pyx_t_5, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1077, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "psychxr/ovr/math.pyx":1081
 *             <ovrQuatf>a.rotation * <ovrQuatf>b.rotation)
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def inverted(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1076
 *         return self.transform(v)
 * 
 *     def __mul__(ovrPosef a, ovrPosef b):             # <<<<<<<<<<<<<<
 *         cdef ovrPosef to_return = ovrPosef(
 *             a.apply(b.translation),
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrPosef.__mul__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1083
 *         return to_return
 * 
 *     def inverted(self):             # <<<<<<<<<<<<<<
 *         cdef ovrQuatf inv = self.rotation.inverted()
 *         cdef ovrPosef to_return = ovrPosef(inv, inv.rotate(-self.translation))
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_21inverted(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_21inverted(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("inverted (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrPosef_20inverted(((struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrPosef_20inverted(struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *__pyx_v_self) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *__pyx_v_inv = 0;
  CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("inverted", 0);

  /* "psychxr/ovr/math.pyx":1084
 * 
 *     def inverted(self):
 *         cdef ovrQuatf inv = self.rotation.inverted()             # <<<<<<<<<<<<<<
 *         cdef ovrPosef to_return = ovrPosef(inv, inv.rotate(-self.translation))
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_rotation); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1084, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_inverted); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1084, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (__pyx_t_2) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1084, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1084, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_7psychxr_3ovr_4math_ovrQuatf))))) __PYX_ERR(0, 1084, __pyx_L1_error)
  __pyx_v_inv = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":1085
 *     def inverted(self):
 *         cdef ovrQuatf inv = self.rotation.inverted()
 *         cdef ovrPosef to_return = ovrPosef(inv, inv.rotate(-self.translation))             # <<<<<<<<<<<<<<
 * 
 *     def normalized(self):
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_inv), __pyx_n_s_rotate); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1085, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_translation); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1085, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = PyNumber_Negative(__pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1085, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_2) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1085, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[2] = {__pyx_t_2, __pyx_t_4};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1085, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[2] = {__pyx_t_2, __pyx_t_4};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1085, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else
    #endif
    {
      __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1085, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_2); __pyx_t_2 = NULL;
      __Pyx_GIVEREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_4);
      __pyx_t_4 = 0;
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1085, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1085, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(((PyObject *)__pyx_v_inv));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_inv));
  PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)__pyx_v_inv));
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrPosef), __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1085, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":1083
 *         return to_return
 * 
 *     def inverted(self):             # <<<<<<<<<<<<<<
 *         cdef ovrQuatf inv = self.rotation.inverted()
 *         cdef ovrPosef to_return = ovrPosef(inv, inv.rotate(-self.translation))
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrPosef.inverted", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_inv);
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1087
 *         cdef ovrPosef to_return = ovrPosef(inv, inv.rotate(-self.translation))
 * 
 *     def normalized(self):             # <<<<<<<<<<<<<<
 *         cdef ovrPosef to_return = ovrPosef(
 *             self.rotation.normalized(),
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_23normalized(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_23normalized(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("normalized (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrPosef_22normalized(((struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrPosef_22normalized(struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *__pyx_v_self) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("normalized", 0);

  /* "psychxr/ovr/math.pyx":1089
 *     def normalized(self):
 *         cdef ovrPosef to_return = ovrPosef(
 *             self.rotation.normalized(),             # <<<<<<<<<<<<<<
 *             self.translation)
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_rotation); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1089, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_normalized); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1089, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (__pyx_t_2) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1089, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1089, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "psychxr/ovr/math.pyx":1090
 *         cdef ovrPosef to_return = ovrPosef(
 *             self.rotation.normalized(),
 *             self.translation)             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_translation); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1090, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "psychxr/ovr/math.pyx":1088
 * 
 *     def normalized(self):
 *         cdef ovrPosef to_return = ovrPosef(             # <<<<<<<<<<<<<<
 *             self.rotation.normalized(),
 *             self.translation)
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1088, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3);
  __pyx_t_1 = 0;
  __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrPosef), __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1088, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "psychxr/ovr/math.pyx":1092
 *             self.translation)
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def normalize(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1087
 *         cdef ovrPosef to_return = ovrPosef(inv, inv.rotate(-self.translation))
 * 
 *     def normalized(self):             # <<<<<<<<<<<<<<
 *         cdef ovrPosef to_return = ovrPosef(
 *             self.rotation.normalized(),
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrPosef.normalized", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1094
 *         return to_return
 * 
 *     def normalize(self):             # <<<<<<<<<<<<<<
 *         self.rotation.normalize()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_25normalize(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_25normalize(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("normalize (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrPosef_24normalize(((struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrPosef_24normalize(struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("normalize", 0);

  /* "psychxr/ovr/math.pyx":1095
 * 
 *     def normalize(self):
 *         self.rotation.normalize()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_rotation); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1095, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_normalize); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1095, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (__pyx_t_2) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1095, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1095, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":1094
 *         return to_return
 * 
 *     def normalize(self):             # <<<<<<<<<<<<<<
 *         self.rotation.normalize()
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrPosef.normalize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_27__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_27__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrPosef_26__reduce_cython__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrPosef_26__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__14, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrPosef.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_29__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_29__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_8ovrPosef_28__setstate_cython__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_8ovrPosef_28__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__15, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrPosef.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1106
 * 
 *     """
 *     def __cinit__(self, *args):             # <<<<<<<<<<<<<<
 *         self.c_data = &self.c_Matrix4f
 * 
 */

/* Python wrapper */
static int __pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_args = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__cinit__", 0))) return -1;
  __Pyx_INCREF(__pyx_args);
  __pyx_v_args = __pyx_args;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f___cinit__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_self), __pyx_v_args);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_args);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f___cinit__(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self, PyObject *__pyx_v_args) {
  int __pyx_v_nargin;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_t_4;
  float __pyx_t_5;
  float __pyx_t_6;
  float __pyx_t_7;
  float __pyx_t_8;
  float __pyx_t_9;
  float __pyx_t_10;
  float __pyx_t_11;
  float __pyx_t_12;
  float __pyx_t_13;
  OVR::Matrix4f __pyx_t_14;
  float __pyx_t_15;
  float __pyx_t_16;
  float __pyx_t_17;
  float __pyx_t_18;
  float __pyx_t_19;
  float __pyx_t_20;
  float __pyx_t_21;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "psychxr/ovr/math.pyx":1107
 *     """
 *     def __cinit__(self, *args):
 *         self.c_data = &self.c_Matrix4f             # <<<<<<<<<<<<<<
 * 
 *         cdef int nargin = <int>len(args)
 */
  __pyx_v_self->c_data = (&__pyx_v_self->c_Matrix4f);

  /* "psychxr/ovr/math.pyx":1109
 *         self.c_data = &self.c_Matrix4f
 * 
 *         cdef int nargin = <int>len(args)             # <<<<<<<<<<<<<<
 *         if nargin == 0:
 *             self.c_data[0] = ovr_math.Matrix4f.Identity()
 */
  __pyx_t_1 = PyTuple_GET_SIZE(__pyx_v_args); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1109, __pyx_L1_error)
  __pyx_v_nargin = ((int)__pyx_t_1);

  /* "psychxr/ovr/math.pyx":1110
 * 
 *         cdef int nargin = <int>len(args)
 *         if nargin == 0:             # <<<<<<<<<<<<<<
 *             self.c_data[0] = ovr_math.Matrix4f.Identity()
 *         elif nargin == 1:
 */
  switch (__pyx_v_nargin) {
    case 0:

    /* "psychxr/ovr/math.pyx":1111
 *         cdef int nargin = <int>len(args)
 *         if nargin == 0:
 *             self.c_data[0] = ovr_math.Matrix4f.Identity()             # <<<<<<<<<<<<<<
 *         elif nargin == 1:
 *             if isinstance(args[0], ovrQuatf):
 */
    (__pyx_v_self->c_data[0]) = OVR::Matrix4f::Identity();

    /* "psychxr/ovr/math.pyx":1110
 * 
 *         cdef int nargin = <int>len(args)
 *         if nargin == 0:             # <<<<<<<<<<<<<<
 *             self.c_data[0] = ovr_math.Matrix4f.Identity()
 *         elif nargin == 1:
 */
    break;

    /* "psychxr/ovr/math.pyx":1112
 *         if nargin == 0:
 *             self.c_data[0] = ovr_math.Matrix4f.Identity()
 *         elif nargin == 1:             # <<<<<<<<<<<<<<
 *             if isinstance(args[0], ovrQuatf):
 *                 self.c_data[0] = ovr_math.Matrix4f(
 */
    case 1:

    /* "psychxr/ovr/math.pyx":1113
 *             self.c_data[0] = ovr_math.Matrix4f.Identity()
 *         elif nargin == 1:
 *             if isinstance(args[0], ovrQuatf):             # <<<<<<<<<<<<<<
 *                 self.c_data[0] = ovr_math.Matrix4f(
 *                     <ovr_math.Quatf>(<ovrQuatf>args[0]).c_data[0])
 */
    __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1113, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_TypeCheck(__pyx_t_2, __pyx_ptype_7psychxr_3ovr_4math_ovrQuatf); 
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_4 = (__pyx_t_3 != 0);
    if (__pyx_t_4) {

      /* "psychxr/ovr/math.pyx":1115
 *             if isinstance(args[0], ovrQuatf):
 *                 self.c_data[0] = ovr_math.Matrix4f(
 *                     <ovr_math.Quatf>(<ovrQuatf>args[0]).c_data[0])             # <<<<<<<<<<<<<<
 *             elif isinstance(args[0], ovrPosef):
 *                 self.c_data[0] = ovr_math.Matrix4f(
 */
      __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1115, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);

      /* "psychxr/ovr/math.pyx":1114
 *         elif nargin == 1:
 *             if isinstance(args[0], ovrQuatf):
 *                 self.c_data[0] = ovr_math.Matrix4f(             # <<<<<<<<<<<<<<
 *                     <ovr_math.Quatf>(<ovrQuatf>args[0]).c_data[0])
 *             elif isinstance(args[0], ovrPosef):
 */
      (__pyx_v_self->c_data[0]) = OVR::Matrix4f(((OVR::Quatf)(((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)__pyx_t_2)->c_data[0])));
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "psychxr/ovr/math.pyx":1113
 *             self.c_data[0] = ovr_math.Matrix4f.Identity()
 *         elif nargin == 1:
 *             if isinstance(args[0], ovrQuatf):             # <<<<<<<<<<<<<<
 *                 self.c_data[0] = ovr_math.Matrix4f(
 *                     <ovr_math.Quatf>(<ovrQuatf>args[0]).c_data[0])
 */
      goto __pyx_L3;
    }

    /* "psychxr/ovr/math.pyx":1116
 *                 self.c_data[0] = ovr_math.Matrix4f(
 *                     <ovr_math.Quatf>(<ovrQuatf>args[0]).c_data[0])
 *             elif isinstance(args[0], ovrPosef):             # <<<<<<<<<<<<<<
 *                 self.c_data[0] = ovr_math.Matrix4f(
 *                     <ovr_math.Posef>(<ovrPosef>args[0]).c_data[0])
 */
    __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1116, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_TypeCheck(__pyx_t_2, __pyx_ptype_7psychxr_3ovr_4math_ovrPosef); 
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_3 = (__pyx_t_4 != 0);
    if (__pyx_t_3) {

      /* "psychxr/ovr/math.pyx":1118
 *             elif isinstance(args[0], ovrPosef):
 *                 self.c_data[0] = ovr_math.Matrix4f(
 *                     <ovr_math.Posef>(<ovrPosef>args[0]).c_data[0])             # <<<<<<<<<<<<<<
 *             elif isinstance(args[0], ovrMatrix4f):
 *                 self.c_data[0] = ovr_math.Matrix4f(
 */
      __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1118, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);

      /* "psychxr/ovr/math.pyx":1117
 *                     <ovr_math.Quatf>(<ovrQuatf>args[0]).c_data[0])
 *             elif isinstance(args[0], ovrPosef):
 *                 self.c_data[0] = ovr_math.Matrix4f(             # <<<<<<<<<<<<<<
 *                     <ovr_math.Posef>(<ovrPosef>args[0]).c_data[0])
 *             elif isinstance(args[0], ovrMatrix4f):
 */
      (__pyx_v_self->c_data[0]) = OVR::Matrix4f(((OVR::Posef)(((struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *)__pyx_t_2)->c_data[0])));
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "psychxr/ovr/math.pyx":1116
 *                 self.c_data[0] = ovr_math.Matrix4f(
 *                     <ovr_math.Quatf>(<ovrQuatf>args[0]).c_data[0])
 *             elif isinstance(args[0], ovrPosef):             # <<<<<<<<<<<<<<
 *                 self.c_data[0] = ovr_math.Matrix4f(
 *                     <ovr_math.Posef>(<ovrPosef>args[0]).c_data[0])
 */
      goto __pyx_L3;
    }

    /* "psychxr/ovr/math.pyx":1119
 *                 self.c_data[0] = ovr_math.Matrix4f(
 *                     <ovr_math.Posef>(<ovrPosef>args[0]).c_data[0])
 *             elif isinstance(args[0], ovrMatrix4f):             # <<<<<<<<<<<<<<
 *                 self.c_data[0] = ovr_math.Matrix4f(
 *                     <ovr_math.Matrix4f>(<ovrMatrix4f>args[0]).c_data[0])
 */
    __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1119, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_TypeCheck(__pyx_t_2, __pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f); 
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_4 = (__pyx_t_3 != 0);
    if (__pyx_t_4) {

      /* "psychxr/ovr/math.pyx":1121
 *             elif isinstance(args[0], ovrMatrix4f):
 *                 self.c_data[0] = ovr_math.Matrix4f(
 *                     <ovr_math.Matrix4f>(<ovrMatrix4f>args[0]).c_data[0])             # <<<<<<<<<<<<<<
 *         elif nargin == 9:
 *             self.c_data[0] = ovr_math.Matrix4f(
 */
      __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1121, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);

      /* "psychxr/ovr/math.pyx":1120
 *                     <ovr_math.Posef>(<ovrPosef>args[0]).c_data[0])
 *             elif isinstance(args[0], ovrMatrix4f):
 *                 self.c_data[0] = ovr_math.Matrix4f(             # <<<<<<<<<<<<<<
 *                     <ovr_math.Matrix4f>(<ovrMatrix4f>args[0]).c_data[0])
 *         elif nargin == 9:
 */
      (__pyx_v_self->c_data[0]) = OVR::Matrix4f(((OVR::Matrix4f)(((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_t_2)->c_data[0])));
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "psychxr/ovr/math.pyx":1119
 *                 self.c_data[0] = ovr_math.Matrix4f(
 *                     <ovr_math.Posef>(<ovrPosef>args[0]).c_data[0])
 *             elif isinstance(args[0], ovrMatrix4f):             # <<<<<<<<<<<<<<
 *                 self.c_data[0] = ovr_math.Matrix4f(
 *                     <ovr_math.Matrix4f>(<ovrMatrix4f>args[0]).c_data[0])
 */
    }
    __pyx_L3:;

    /* "psychxr/ovr/math.pyx":1112
 *         if nargin == 0:
 *             self.c_data[0] = ovr_math.Matrix4f.Identity()
 *         elif nargin == 1:             # <<<<<<<<<<<<<<
 *             if isinstance(args[0], ovrQuatf):
 *                 self.c_data[0] = ovr_math.Matrix4f(
 */
    break;

    /* "psychxr/ovr/math.pyx":1122
 *                 self.c_data[0] = ovr_math.Matrix4f(
 *                     <ovr_math.Matrix4f>(<ovrMatrix4f>args[0]).c_data[0])
 *         elif nargin == 9:             # <<<<<<<<<<<<<<
 *             self.c_data[0] = ovr_math.Matrix4f(
 *                 <float>args[0],
 */
    case 9:

    /* "psychxr/ovr/math.pyx":1124
 *         elif nargin == 9:
 *             self.c_data[0] = ovr_math.Matrix4f(
 *                 <float>args[0],             # <<<<<<<<<<<<<<
 *                 <float>args[1],
 *                 <float>args[2],
 */
    __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1124, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = __pyx_PyFloat_AsFloat(__pyx_t_2); if (unlikely((__pyx_t_5 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1124, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "psychxr/ovr/math.pyx":1125
 *             self.c_data[0] = ovr_math.Matrix4f(
 *                 <float>args[0],
 *                 <float>args[1],             # <<<<<<<<<<<<<<
 *                 <float>args[2],
 *                 <float>args[3],
 */
    __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1125, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_6 = __pyx_PyFloat_AsFloat(__pyx_t_2); if (unlikely((__pyx_t_6 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1125, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "psychxr/ovr/math.pyx":1126
 *                 <float>args[0],
 *                 <float>args[1],
 *                 <float>args[2],             # <<<<<<<<<<<<<<
 *                 <float>args[3],
 *                 <float>args[4],
 */
    __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1126, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_7 = __pyx_PyFloat_AsFloat(__pyx_t_2); if (unlikely((__pyx_t_7 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1126, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "psychxr/ovr/math.pyx":1127
 *                 <float>args[1],
 *                 <float>args[2],
 *                 <float>args[3],             # <<<<<<<<<<<<<<
 *                 <float>args[4],
 *                 <float>args[5],
 */
    __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1127, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_8 = __pyx_PyFloat_AsFloat(__pyx_t_2); if (unlikely((__pyx_t_8 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1127, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "psychxr/ovr/math.pyx":1128
 *                 <float>args[2],
 *                 <float>args[3],
 *                 <float>args[4],             # <<<<<<<<<<<<<<
 *                 <float>args[5],
 *                 <float>args[6],
 */
    __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1128, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __pyx_PyFloat_AsFloat(__pyx_t_2); if (unlikely((__pyx_t_9 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1128, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "psychxr/ovr/math.pyx":1129
 *                 <float>args[3],
 *                 <float>args[4],
 *                 <float>args[5],             # <<<<<<<<<<<<<<
 *                 <float>args[6],
 *                 <float>args[7],
 */
    __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1129, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_10 = __pyx_PyFloat_AsFloat(__pyx_t_2); if (unlikely((__pyx_t_10 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1129, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "psychxr/ovr/math.pyx":1130
 *                 <float>args[4],
 *                 <float>args[5],
 *                 <float>args[6],             # <<<<<<<<<<<<<<
 *                 <float>args[7],
 *                 <float>args[8])
 */
    __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1130, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_11 = __pyx_PyFloat_AsFloat(__pyx_t_2); if (unlikely((__pyx_t_11 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1130, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "psychxr/ovr/math.pyx":1131
 *                 <float>args[5],
 *                 <float>args[6],
 *                 <float>args[7],             # <<<<<<<<<<<<<<
 *                 <float>args[8])
 *         elif nargin == 16:
 */
    __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1131, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_12 = __pyx_PyFloat_AsFloat(__pyx_t_2); if (unlikely((__pyx_t_12 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1131, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "psychxr/ovr/math.pyx":1132
 *                 <float>args[6],
 *                 <float>args[7],
 *                 <float>args[8])             # <<<<<<<<<<<<<<
 *         elif nargin == 16:
 *             self.c_data[0] = ovr_math.Matrix4f(
 */
    __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1132, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_13 = __pyx_PyFloat_AsFloat(__pyx_t_2); if (unlikely((__pyx_t_13 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1132, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "psychxr/ovr/math.pyx":1123
 *                     <ovr_math.Matrix4f>(<ovrMatrix4f>args[0]).c_data[0])
 *         elif nargin == 9:
 *             self.c_data[0] = ovr_math.Matrix4f(             # <<<<<<<<<<<<<<
 *                 <float>args[0],
 *                 <float>args[1],
 */
    try {
      __pyx_t_14 = OVR::Matrix4f(((float)__pyx_t_5), ((float)__pyx_t_6), ((float)__pyx_t_7), ((float)__pyx_t_8), ((float)__pyx_t_9), ((float)__pyx_t_10), ((float)__pyx_t_11), ((float)__pyx_t_12), ((float)__pyx_t_13));
    } catch(...) {
      __Pyx_CppExn2PyErr();
      __PYX_ERR(0, 1123, __pyx_L1_error)
    }
    (__pyx_v_self->c_data[0]) = __pyx_t_14;

    /* "psychxr/ovr/math.pyx":1122
 *                 self.c_data[0] = ovr_math.Matrix4f(
 *                     <ovr_math.Matrix4f>(<ovrMatrix4f>args[0]).c_data[0])
 *         elif nargin == 9:             # <<<<<<<<<<<<<<
 *             self.c_data[0] = ovr_math.Matrix4f(
 *                 <float>args[0],
 */
    break;

    /* "psychxr/ovr/math.pyx":1133
 *                 <float>args[7],
 *                 <float>args[8])
 *         elif nargin == 16:             # <<<<<<<<<<<<<<
 *             self.c_data[0] = ovr_math.Matrix4f(
 *                 <float>args[0],
 */
    case 16:

    /* "psychxr/ovr/math.pyx":1135
 *         elif nargin == 16:
 *             self.c_data[0] = ovr_math.Matrix4f(
 *                 <float>args[0],             # <<<<<<<<<<<<<<
 *                 <float>args[1],
 *                 <float>args[2],
 */
    __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1135, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_13 = __pyx_PyFloat_AsFloat(__pyx_t_2); if (unlikely((__pyx_t_13 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1135, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "psychxr/ovr/math.pyx":1136
 *             self.c_data[0] = ovr_math.Matrix4f(
 *                 <float>args[0],
 *                 <float>args[1],             # <<<<<<<<<<<<<<
 *                 <float>args[2],
 *                 <float>args[3],
 */
    __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1136, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_12 = __pyx_PyFloat_AsFloat(__pyx_t_2); if (unlikely((__pyx_t_12 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1136, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "psychxr/ovr/math.pyx":1137
 *                 <float>args[0],
 *                 <float>args[1],
 *                 <float>args[2],             # <<<<<<<<<<<<<<
 *                 <float>args[3],
 *                 <float>args[4],
 */
    __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1137, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_11 = __pyx_PyFloat_AsFloat(__pyx_t_2); if (unlikely((__pyx_t_11 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1137, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "psychxr/ovr/math.pyx":1138
 *                 <float>args[1],
 *                 <float>args[2],
 *                 <float>args[3],             # <<<<<<<<<<<<<<
 *                 <float>args[4],
 *                 <float>args[5],
 */
    __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1138, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_10 = __pyx_PyFloat_AsFloat(__pyx_t_2); if (unlikely((__pyx_t_10 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1138, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "psychxr/ovr/math.pyx":1139
 *                 <float>args[2],
 *                 <float>args[3],
 *                 <float>args[4],             # <<<<<<<<<<<<<<
 *                 <float>args[5],
 *                 <float>args[6],
 */
    __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1139, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __pyx_PyFloat_AsFloat(__pyx_t_2); if (unlikely((__pyx_t_9 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1139, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "psychxr/ovr/math.pyx":1140
 *                 <float>args[3],
 *                 <float>args[4],
 *                 <float>args[5],             # <<<<<<<<<<<<<<
 *                 <float>args[6],
 *                 <float>args[7],
 */
    __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1140, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_8 = __pyx_PyFloat_AsFloat(__pyx_t_2); if (unlikely((__pyx_t_8 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1140, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "psychxr/ovr/math.pyx":1141
 *                 <float>args[4],
 *                 <float>args[5],
 *                 <float>args[6],             # <<<<<<<<<<<<<<
 *                 <float>args[7],
 *                 <float>args[8],
 */
    __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1141, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_7 = __pyx_PyFloat_AsFloat(__pyx_t_2); if (unlikely((__pyx_t_7 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1141, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "psychxr/ovr/math.pyx":1142
 *                 <float>args[5],
 *                 <float>args[6],
 *                 <float>args[7],             # <<<<<<<<<<<<<<
 *                 <float>args[8],
 *                 <float>args[9],
 */
    __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1142, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_6 = __pyx_PyFloat_AsFloat(__pyx_t_2); if (unlikely((__pyx_t_6 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1142, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "psychxr/ovr/math.pyx":1143
 *                 <float>args[6],
 *                 <float>args[7],
 *                 <float>args[8],             # <<<<<<<<<<<<<<
 *                 <float>args[9],
 *                 <float>args[10],
 */
    __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1143, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = __pyx_PyFloat_AsFloat(__pyx_t_2); if (unlikely((__pyx_t_5 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1143, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "psychxr/ovr/math.pyx":1144
 *                 <float>args[7],
 *                 <float>args[8],
 *                 <float>args[9],             # <<<<<<<<<<<<<<
 *                 <float>args[10],
 *                 <float>args[11],
 */
    __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1144, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_15 = __pyx_PyFloat_AsFloat(__pyx_t_2); if (unlikely((__pyx_t_15 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1144, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "psychxr/ovr/math.pyx":1145
 *                 <float>args[8],
 *                 <float>args[9],
 *                 <float>args[10],             # <<<<<<<<<<<<<<
 *                 <float>args[11],
 *                 <float>args[12],
 */
    __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1145, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_16 = __pyx_PyFloat_AsFloat(__pyx_t_2); if (unlikely((__pyx_t_16 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1145, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "psychxr/ovr/math.pyx":1146
 *                 <float>args[9],
 *                 <float>args[10],
 *                 <float>args[11],             # <<<<<<<<<<<<<<
 *                 <float>args[12],
 *                 <float>args[13],
 */
    __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1146, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_17 = __pyx_PyFloat_AsFloat(__pyx_t_2); if (unlikely((__pyx_t_17 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1146, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "psychxr/ovr/math.pyx":1147
 *                 <float>args[10],
 *                 <float>args[11],
 *                 <float>args[12],             # <<<<<<<<<<<<<<
 *                 <float>args[13],
 *                 <float>args[14],
 */
    __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 12, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1147, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_18 = __pyx_PyFloat_AsFloat(__pyx_t_2); if (unlikely((__pyx_t_18 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1147, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "psychxr/ovr/math.pyx":1148
 *                 <float>args[11],
 *                 <float>args[12],
 *                 <float>args[13],             # <<<<<<<<<<<<<<
 *                 <float>args[14],
 *                 <float>args[15])
 */
    __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 13, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1148, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_19 = __pyx_PyFloat_AsFloat(__pyx_t_2); if (unlikely((__pyx_t_19 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1148, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "psychxr/ovr/math.pyx":1149
 *                 <float>args[12],
 *                 <float>args[13],
 *                 <float>args[14],             # <<<<<<<<<<<<<<
 *                 <float>args[15])
 * 
 */
    __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 14, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1149, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_20 = __pyx_PyFloat_AsFloat(__pyx_t_2); if (unlikely((__pyx_t_20 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1149, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "psychxr/ovr/math.pyx":1150
 *                 <float>args[13],
 *                 <float>args[14],
 *                 <float>args[15])             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
    __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 15, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1150, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_21 = __pyx_PyFloat_AsFloat(__pyx_t_2); if (unlikely((__pyx_t_21 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1150, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "psychxr/ovr/math.pyx":1134
 *                 <float>args[8])
 *         elif nargin == 16:
 *             self.c_data[0] = ovr_math.Matrix4f(             # <<<<<<<<<<<<<<
 *                 <float>args[0],
 *                 <float>args[1],
 */
    try {
      __pyx_t_14 = OVR::Matrix4f(((float)__pyx_t_13), ((float)__pyx_t_12), ((float)__pyx_t_11), ((float)__pyx_t_10), ((float)__pyx_t_9), ((float)__pyx_t_8), ((float)__pyx_t_7), ((float)__pyx_t_6), ((float)__pyx_t_5), ((float)__pyx_t_15), ((float)__pyx_t_16), ((float)__pyx_t_17), ((float)__pyx_t_18), ((float)__pyx_t_19), ((float)__pyx_t_20), ((float)__pyx_t_21));
    } catch(...) {
      __Pyx_CppExn2PyErr();
      __PYX_ERR(0, 1134, __pyx_L1_error)
    }
    (__pyx_v_self->c_data[0]) = __pyx_t_14;

    /* "psychxr/ovr/math.pyx":1133
 *                 <float>args[7],
 *                 <float>args[8])
 *         elif nargin == 16:             # <<<<<<<<<<<<<<
 *             self.c_data[0] = ovr_math.Matrix4f(
 *                 <float>args[0],
 */
    break;
    default: break;
  }

  /* "psychxr/ovr/math.pyx":1106
 * 
 *     """
 *     def __cinit__(self, *args):             # <<<<<<<<<<<<<<
 *         self.c_data = &self.c_Matrix4f
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1153
 * 
 *     @property
 *     def M(self):             # <<<<<<<<<<<<<<
 *         return self.c_data.M
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_1M_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_1M_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_1M___get__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_1M___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "psychxr/ovr/math.pyx":1154
 *     @property
 *     def M(self):
 *         return self.c_data.M             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_carray_to_py_float___5b_4_5d_(__pyx_v_self->c_data->M, 4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1154, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1153
 * 
 *     @property
 *     def M(self):             # <<<<<<<<<<<<<<
 *         return self.c_data.M
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.M.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1157
 * 
 *     @staticmethod
 *     def identity():             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         return to_return
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_3identity(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_7psychxr_3ovr_4math_11ovrMatrix4f_3identity = {"identity", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_3identity, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_3identity(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("identity (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("identity", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return NULL;}
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "identity", 0))) return NULL;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_2identity();

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_2identity() {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("identity", 0);

  /* "psychxr/ovr/math.pyx":1158
 *     @staticmethod
 *     def identity():
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()             # <<<<<<<<<<<<<<
 *         return to_return
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1158, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":1159
 *     def identity():
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def setIdentity(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1157
 * 
 *     @staticmethod
 *     def identity():             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         return to_return
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.identity", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1161
 *         return to_return
 * 
 *     def setIdentity(self):             # <<<<<<<<<<<<<<
 *         self.c_data[0] = ovr_math.Matrix4f()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_5setIdentity(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_5setIdentity(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("setIdentity (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_4setIdentity(((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_4setIdentity(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  OVR::Matrix4f __pyx_t_1;
  __Pyx_RefNannySetupContext("setIdentity", 0);

  /* "psychxr/ovr/math.pyx":1162
 * 
 *     def setIdentity(self):
 *         self.c_data[0] = ovr_math.Matrix4f()             # <<<<<<<<<<<<<<
 * 
 *     def setXBasis(self, ovrVector3f v):
 */
  try {
    __pyx_t_1 = OVR::Matrix4f();
  } catch(...) {
    __Pyx_CppExn2PyErr();
    __PYX_ERR(0, 1162, __pyx_L1_error)
  }
  (__pyx_v_self->c_data[0]) = __pyx_t_1;

  /* "psychxr/ovr/math.pyx":1161
 *         return to_return
 * 
 *     def setIdentity(self):             # <<<<<<<<<<<<<<
 *         self.c_data[0] = ovr_math.Matrix4f()
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.setIdentity", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1164
 *         self.c_data[0] = ovr_math.Matrix4f()
 * 
 *     def setXBasis(self, ovrVector3f v):             # <<<<<<<<<<<<<<
 *         (<ovrMatrix4f>self).c_data[0].SetXBasis(v.c_data[0])
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_7setXBasis(PyObject *__pyx_v_self, PyObject *__pyx_v_v); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_7setXBasis(PyObject *__pyx_v_self, PyObject *__pyx_v_v) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("setXBasis (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_v), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "v", 0))) __PYX_ERR(0, 1164, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_6setXBasis(((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_v));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_6setXBasis(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_v) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("setXBasis", 0);

  /* "psychxr/ovr/math.pyx":1165
 * 
 *     def setXBasis(self, ovrVector3f v):
 *         (<ovrMatrix4f>self).c_data[0].SetXBasis(v.c_data[0])             # <<<<<<<<<<<<<<
 * 
 *     def getXBasis(self):
 */
  (__pyx_v_self->c_data[0]).SetXBasis((__pyx_v_v->c_data[0]));

  /* "psychxr/ovr/math.pyx":1164
 *         self.c_data[0] = ovr_math.Matrix4f()
 * 
 *     def setXBasis(self, ovrVector3f v):             # <<<<<<<<<<<<<<
 *         (<ovrMatrix4f>self).c_data[0].SetXBasis(v.c_data[0])
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1167
 *         (<ovrMatrix4f>self).c_data[0].SetXBasis(v.c_data[0])
 * 
 *     def getXBasis(self):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_9getXBasis(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_9getXBasis(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("getXBasis (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_8getXBasis(((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_8getXBasis(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("getXBasis", 0);

  /* "psychxr/ovr/math.pyx":1168
 * 
 *     def getXBasis(self):
 *         cdef ovrVector3f to_return = ovrVector3f()             # <<<<<<<<<<<<<<
 *         (<ovrVector3f>to_return).c_data[0] = \
 *             (<ovr_math.Matrix4f>self.c_data[0]).GetXBasis()
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector3f), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1168, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":1169
 *     def getXBasis(self):
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \             # <<<<<<<<<<<<<<
 *             (<ovr_math.Matrix4f>self.c_data[0]).GetXBasis()
 * 
 */
  (__pyx_v_to_return->c_data[0]) = ((OVR::Matrix4f)(__pyx_v_self->c_data[0])).GetXBasis();

  /* "psychxr/ovr/math.pyx":1172
 *             (<ovr_math.Matrix4f>self.c_data[0]).GetXBasis()
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1167
 *         (<ovrMatrix4f>self).c_data[0].SetXBasis(v.c_data[0])
 * 
 *     def getXBasis(self):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.getXBasis", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1175
 * 
 *     @property
 *     def xBasis(self):             # <<<<<<<<<<<<<<
 *         return self.getXBasis()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_6xBasis_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_6xBasis_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_6xBasis___get__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_6xBasis___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "psychxr/ovr/math.pyx":1176
 *     @property
 *     def xBasis(self):
 *         return self.getXBasis()             # <<<<<<<<<<<<<<
 * 
 *     @xBasis.setter
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_getXBasis); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1176, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1176, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1176, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1175
 * 
 *     @property
 *     def xBasis(self):             # <<<<<<<<<<<<<<
 *         return self.getXBasis()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.xBasis.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1179
 * 
 *     @xBasis.setter
 *     def xBasis(self, object v):             # <<<<<<<<<<<<<<
 *         if isinstance(v, ovrVector3f):
 *             self.setXBasis(<ovrVector3f>v)
 */

/* Python wrapper */
static int __pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_6xBasis_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_v); /*proto*/
static int __pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_6xBasis_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_v) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_6xBasis_2__set__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_self), ((PyObject *)__pyx_v_v));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_6xBasis_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self, PyObject *__pyx_v_v) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  float __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "psychxr/ovr/math.pyx":1180
 *     @xBasis.setter
 *     def xBasis(self, object v):
 *         if isinstance(v, ovrVector3f):             # <<<<<<<<<<<<<<
 *             self.setXBasis(<ovrVector3f>v)
 *         elif isinstance(v, (list, tuple)):
 */
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_v, __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "psychxr/ovr/math.pyx":1181
 *     def xBasis(self, object v):
 *         if isinstance(v, ovrVector3f):
 *             self.setXBasis(<ovrVector3f>v)             # <<<<<<<<<<<<<<
 *         elif isinstance(v, (list, tuple)):
 *             self.setXBasis(ovrVector3f(<float>v[0], <float>v[1], <float>v[2]))
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_setXBasis); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1181, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (!__pyx_t_5) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_v_v); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1181, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_v_v};
        __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1181, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_v_v};
        __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1181, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1181, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
        __Pyx_INCREF(((PyObject *)((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_v)));
        __Pyx_GIVEREF(__pyx_v_v);
        PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_v_v);
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_6, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1181, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "psychxr/ovr/math.pyx":1180
 *     @xBasis.setter
 *     def xBasis(self, object v):
 *         if isinstance(v, ovrVector3f):             # <<<<<<<<<<<<<<
 *             self.setXBasis(<ovrVector3f>v)
 *         elif isinstance(v, (list, tuple)):
 */
    goto __pyx_L3;
  }

  /* "psychxr/ovr/math.pyx":1182
 *         if isinstance(v, ovrVector3f):
 *             self.setXBasis(<ovrVector3f>v)
 *         elif isinstance(v, (list, tuple)):             # <<<<<<<<<<<<<<
 *             self.setXBasis(ovrVector3f(<float>v[0], <float>v[1], <float>v[2]))
 * 
 */
  __pyx_t_1 = PyList_Check(__pyx_v_v); 
  __pyx_t_7 = (__pyx_t_1 != 0);
  if (!__pyx_t_7) {
  } else {
    __pyx_t_2 = __pyx_t_7;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_7 = PyTuple_Check(__pyx_v_v); 
  __pyx_t_1 = (__pyx_t_7 != 0);
  __pyx_t_2 = __pyx_t_1;
  __pyx_L4_bool_binop_done:;
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {

    /* "psychxr/ovr/math.pyx":1183
 *             self.setXBasis(<ovrVector3f>v)
 *         elif isinstance(v, (list, tuple)):
 *             self.setXBasis(ovrVector3f(<float>v[0], <float>v[1], <float>v[2]))             # <<<<<<<<<<<<<<
 * 
 *     def setYBasis(self, ovrVector3f v):
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_setXBasis); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1183, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = __Pyx_GetItemInt(__pyx_v_v, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1183, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = __pyx_PyFloat_AsFloat(__pyx_t_6); if (unlikely((__pyx_t_8 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1183, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = PyFloat_FromDouble(((float)__pyx_t_8)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1183, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_5 = __Pyx_GetItemInt(__pyx_v_v, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1183, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_8 = __pyx_PyFloat_AsFloat(__pyx_t_5); if (unlikely((__pyx_t_8 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1183, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = PyFloat_FromDouble(((float)__pyx_t_8)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1183, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_9 = __Pyx_GetItemInt(__pyx_v_v, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1183, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_8 = __pyx_PyFloat_AsFloat(__pyx_t_9); if (unlikely((__pyx_t_8 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1183, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyFloat_FromDouble(((float)__pyx_t_8)); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1183, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_10 = PyTuple_New(3); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1183, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_10, 1, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_9);
    PyTuple_SET_ITEM(__pyx_t_10, 2, __pyx_t_9);
    __pyx_t_6 = 0;
    __pyx_t_5 = 0;
    __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector3f), __pyx_t_10, NULL); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1183, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (!__pyx_t_10) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_9); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1183, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_10, __pyx_t_9};
        __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1183, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_10, __pyx_t_9};
        __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1183, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      } else
      #endif
      {
        __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1183, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_10); __pyx_t_10 = NULL;
        __Pyx_GIVEREF(__pyx_t_9);
        PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_9);
        __pyx_t_9 = 0;
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_5, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1183, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "psychxr/ovr/math.pyx":1182
 *         if isinstance(v, ovrVector3f):
 *             self.setXBasis(<ovrVector3f>v)
 *         elif isinstance(v, (list, tuple)):             # <<<<<<<<<<<<<<
 *             self.setXBasis(ovrVector3f(<float>v[0], <float>v[1], <float>v[2]))
 * 
 */
  }
  __pyx_L3:;

  /* "psychxr/ovr/math.pyx":1179
 * 
 *     @xBasis.setter
 *     def xBasis(self, object v):             # <<<<<<<<<<<<<<
 *         if isinstance(v, ovrVector3f):
 *             self.setXBasis(<ovrVector3f>v)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.xBasis.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1185
 *             self.setXBasis(ovrVector3f(<float>v[0], <float>v[1], <float>v[2]))
 * 
 *     def setYBasis(self, ovrVector3f v):             # <<<<<<<<<<<<<<
 *         (<ovrMatrix4f>self).c_data[0].SetYBasis(v.c_data[0])
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_11setYBasis(PyObject *__pyx_v_self, PyObject *__pyx_v_v); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_11setYBasis(PyObject *__pyx_v_self, PyObject *__pyx_v_v) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("setYBasis (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_v), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "v", 0))) __PYX_ERR(0, 1185, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_10setYBasis(((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_v));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_10setYBasis(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_v) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("setYBasis", 0);

  /* "psychxr/ovr/math.pyx":1186
 * 
 *     def setYBasis(self, ovrVector3f v):
 *         (<ovrMatrix4f>self).c_data[0].SetYBasis(v.c_data[0])             # <<<<<<<<<<<<<<
 * 
 *     def getYBasis(self):
 */
  (__pyx_v_self->c_data[0]).SetYBasis((__pyx_v_v->c_data[0]));

  /* "psychxr/ovr/math.pyx":1185
 *             self.setXBasis(ovrVector3f(<float>v[0], <float>v[1], <float>v[2]))
 * 
 *     def setYBasis(self, ovrVector3f v):             # <<<<<<<<<<<<<<
 *         (<ovrMatrix4f>self).c_data[0].SetYBasis(v.c_data[0])
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1188
 *         (<ovrMatrix4f>self).c_data[0].SetYBasis(v.c_data[0])
 * 
 *     def getYBasis(self):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_13getYBasis(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_13getYBasis(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("getYBasis (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_12getYBasis(((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_12getYBasis(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("getYBasis", 0);

  /* "psychxr/ovr/math.pyx":1189
 * 
 *     def getYBasis(self):
 *         cdef ovrVector3f to_return = ovrVector3f()             # <<<<<<<<<<<<<<
 *         (<ovrVector3f>to_return).c_data[0] = \
 *             (<ovr_math.Matrix4f>self.c_data[0]).GetYBasis()
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector3f), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1189, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":1190
 *     def getYBasis(self):
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \             # <<<<<<<<<<<<<<
 *             (<ovr_math.Matrix4f>self.c_data[0]).GetYBasis()
 * 
 */
  (__pyx_v_to_return->c_data[0]) = ((OVR::Matrix4f)(__pyx_v_self->c_data[0])).GetYBasis();

  /* "psychxr/ovr/math.pyx":1193
 *             (<ovr_math.Matrix4f>self.c_data[0]).GetYBasis()
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1188
 *         (<ovrMatrix4f>self).c_data[0].SetYBasis(v.c_data[0])
 * 
 *     def getYBasis(self):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.getYBasis", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1196
 * 
 *     @property
 *     def yBasis(self):             # <<<<<<<<<<<<<<
 *         return self.getYBasis()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_6yBasis_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_6yBasis_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_6yBasis___get__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_6yBasis___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "psychxr/ovr/math.pyx":1197
 *     @property
 *     def yBasis(self):
 *         return self.getYBasis()             # <<<<<<<<<<<<<<
 * 
 *     @yBasis.setter
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_getYBasis); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1197, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1197, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1197, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1196
 * 
 *     @property
 *     def yBasis(self):             # <<<<<<<<<<<<<<
 *         return self.getYBasis()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.yBasis.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1200
 * 
 *     @yBasis.setter
 *     def yBasis(self, object v):             # <<<<<<<<<<<<<<
 *         if isinstance(v, ovrVector3f):
 *             self.setYBasis(<ovrVector3f>v)
 */

/* Python wrapper */
static int __pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_6yBasis_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_v); /*proto*/
static int __pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_6yBasis_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_v) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_6yBasis_2__set__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_self), ((PyObject *)__pyx_v_v));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_6yBasis_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self, PyObject *__pyx_v_v) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  float __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "psychxr/ovr/math.pyx":1201
 *     @yBasis.setter
 *     def yBasis(self, object v):
 *         if isinstance(v, ovrVector3f):             # <<<<<<<<<<<<<<
 *             self.setYBasis(<ovrVector3f>v)
 *         elif isinstance(v, (list, tuple)):
 */
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_v, __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "psychxr/ovr/math.pyx":1202
 *     def yBasis(self, object v):
 *         if isinstance(v, ovrVector3f):
 *             self.setYBasis(<ovrVector3f>v)             # <<<<<<<<<<<<<<
 *         elif isinstance(v, (list, tuple)):
 *             self.setYBasis(ovrVector3f(<float>v[0], <float>v[1], <float>v[2]))
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_setYBasis); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1202, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (!__pyx_t_5) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_v_v); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1202, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_v_v};
        __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1202, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_v_v};
        __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1202, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1202, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
        __Pyx_INCREF(((PyObject *)((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_v)));
        __Pyx_GIVEREF(__pyx_v_v);
        PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_v_v);
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_6, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1202, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "psychxr/ovr/math.pyx":1201
 *     @yBasis.setter
 *     def yBasis(self, object v):
 *         if isinstance(v, ovrVector3f):             # <<<<<<<<<<<<<<
 *             self.setYBasis(<ovrVector3f>v)
 *         elif isinstance(v, (list, tuple)):
 */
    goto __pyx_L3;
  }

  /* "psychxr/ovr/math.pyx":1203
 *         if isinstance(v, ovrVector3f):
 *             self.setYBasis(<ovrVector3f>v)
 *         elif isinstance(v, (list, tuple)):             # <<<<<<<<<<<<<<
 *             self.setYBasis(ovrVector3f(<float>v[0], <float>v[1], <float>v[2]))
 * 
 */
  __pyx_t_1 = PyList_Check(__pyx_v_v); 
  __pyx_t_7 = (__pyx_t_1 != 0);
  if (!__pyx_t_7) {
  } else {
    __pyx_t_2 = __pyx_t_7;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_7 = PyTuple_Check(__pyx_v_v); 
  __pyx_t_1 = (__pyx_t_7 != 0);
  __pyx_t_2 = __pyx_t_1;
  __pyx_L4_bool_binop_done:;
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {

    /* "psychxr/ovr/math.pyx":1204
 *             self.setYBasis(<ovrVector3f>v)
 *         elif isinstance(v, (list, tuple)):
 *             self.setYBasis(ovrVector3f(<float>v[0], <float>v[1], <float>v[2]))             # <<<<<<<<<<<<<<
 * 
 *     def setZBasis(self, ovrVector3f v):
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_setYBasis); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1204, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = __Pyx_GetItemInt(__pyx_v_v, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1204, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = __pyx_PyFloat_AsFloat(__pyx_t_6); if (unlikely((__pyx_t_8 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1204, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = PyFloat_FromDouble(((float)__pyx_t_8)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1204, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_5 = __Pyx_GetItemInt(__pyx_v_v, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1204, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_8 = __pyx_PyFloat_AsFloat(__pyx_t_5); if (unlikely((__pyx_t_8 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1204, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = PyFloat_FromDouble(((float)__pyx_t_8)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1204, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_9 = __Pyx_GetItemInt(__pyx_v_v, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1204, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_8 = __pyx_PyFloat_AsFloat(__pyx_t_9); if (unlikely((__pyx_t_8 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1204, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyFloat_FromDouble(((float)__pyx_t_8)); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1204, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_10 = PyTuple_New(3); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1204, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_10, 1, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_9);
    PyTuple_SET_ITEM(__pyx_t_10, 2, __pyx_t_9);
    __pyx_t_6 = 0;
    __pyx_t_5 = 0;
    __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector3f), __pyx_t_10, NULL); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1204, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (!__pyx_t_10) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_9); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1204, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_10, __pyx_t_9};
        __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1204, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_10, __pyx_t_9};
        __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1204, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      } else
      #endif
      {
        __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1204, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_10); __pyx_t_10 = NULL;
        __Pyx_GIVEREF(__pyx_t_9);
        PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_9);
        __pyx_t_9 = 0;
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_5, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1204, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "psychxr/ovr/math.pyx":1203
 *         if isinstance(v, ovrVector3f):
 *             self.setYBasis(<ovrVector3f>v)
 *         elif isinstance(v, (list, tuple)):             # <<<<<<<<<<<<<<
 *             self.setYBasis(ovrVector3f(<float>v[0], <float>v[1], <float>v[2]))
 * 
 */
  }
  __pyx_L3:;

  /* "psychxr/ovr/math.pyx":1200
 * 
 *     @yBasis.setter
 *     def yBasis(self, object v):             # <<<<<<<<<<<<<<
 *         if isinstance(v, ovrVector3f):
 *             self.setYBasis(<ovrVector3f>v)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.yBasis.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1206
 *             self.setYBasis(ovrVector3f(<float>v[0], <float>v[1], <float>v[2]))
 * 
 *     def setZBasis(self, ovrVector3f v):             # <<<<<<<<<<<<<<
 *         (<ovrMatrix4f>self).c_data[0].SetZBasis(v.c_data[0])
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_15setZBasis(PyObject *__pyx_v_self, PyObject *__pyx_v_v); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_15setZBasis(PyObject *__pyx_v_self, PyObject *__pyx_v_v) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("setZBasis (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_v), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "v", 0))) __PYX_ERR(0, 1206, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_14setZBasis(((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_v));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_14setZBasis(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_v) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("setZBasis", 0);

  /* "psychxr/ovr/math.pyx":1207
 * 
 *     def setZBasis(self, ovrVector3f v):
 *         (<ovrMatrix4f>self).c_data[0].SetZBasis(v.c_data[0])             # <<<<<<<<<<<<<<
 * 
 *     def getZBasis(self):
 */
  (__pyx_v_self->c_data[0]).SetZBasis((__pyx_v_v->c_data[0]));

  /* "psychxr/ovr/math.pyx":1206
 *             self.setYBasis(ovrVector3f(<float>v[0], <float>v[1], <float>v[2]))
 * 
 *     def setZBasis(self, ovrVector3f v):             # <<<<<<<<<<<<<<
 *         (<ovrMatrix4f>self).c_data[0].SetZBasis(v.c_data[0])
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1209
 *         (<ovrMatrix4f>self).c_data[0].SetZBasis(v.c_data[0])
 * 
 *     def getZBasis(self):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_17getZBasis(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_17getZBasis(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("getZBasis (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_16getZBasis(((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_16getZBasis(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("getZBasis", 0);

  /* "psychxr/ovr/math.pyx":1210
 * 
 *     def getZBasis(self):
 *         cdef ovrVector3f to_return = ovrVector3f()             # <<<<<<<<<<<<<<
 *         (<ovrVector3f>to_return).c_data[0] = \
 *             (<ovr_math.Matrix4f>self.c_data[0]).GetZBasis()
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector3f), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1210, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":1211
 *     def getZBasis(self):
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \             # <<<<<<<<<<<<<<
 *             (<ovr_math.Matrix4f>self.c_data[0]).GetZBasis()
 * 
 */
  (__pyx_v_to_return->c_data[0]) = ((OVR::Matrix4f)(__pyx_v_self->c_data[0])).GetZBasis();

  /* "psychxr/ovr/math.pyx":1214
 *             (<ovr_math.Matrix4f>self.c_data[0]).GetZBasis()
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1209
 *         (<ovrMatrix4f>self).c_data[0].SetZBasis(v.c_data[0])
 * 
 *     def getZBasis(self):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.getZBasis", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1217
 * 
 *     @property
 *     def zBasis(self):             # <<<<<<<<<<<<<<
 *         return self.getZBasis()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_6zBasis_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_6zBasis_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_6zBasis___get__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_6zBasis___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "psychxr/ovr/math.pyx":1218
 *     @property
 *     def zBasis(self):
 *         return self.getZBasis()             # <<<<<<<<<<<<<<
 * 
 *     @zBasis.setter
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_getZBasis); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1218, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1218, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1218, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1217
 * 
 *     @property
 *     def zBasis(self):             # <<<<<<<<<<<<<<
 *         return self.getZBasis()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.zBasis.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1221
 * 
 *     @zBasis.setter
 *     def zBasis(self, object v):             # <<<<<<<<<<<<<<
 *         if isinstance(v, ovrVector3f):
 *             self.setZBasis(<ovrVector3f>v)
 */

/* Python wrapper */
static int __pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_6zBasis_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_v); /*proto*/
static int __pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_6zBasis_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_v) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_6zBasis_2__set__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_self), ((PyObject *)__pyx_v_v));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_6zBasis_2__set__(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self, PyObject *__pyx_v_v) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  float __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "psychxr/ovr/math.pyx":1222
 *     @zBasis.setter
 *     def zBasis(self, object v):
 *         if isinstance(v, ovrVector3f):             # <<<<<<<<<<<<<<
 *             self.setZBasis(<ovrVector3f>v)
 *         elif isinstance(v, (list, tuple)):
 */
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_v, __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "psychxr/ovr/math.pyx":1223
 *     def zBasis(self, object v):
 *         if isinstance(v, ovrVector3f):
 *             self.setZBasis(<ovrVector3f>v)             # <<<<<<<<<<<<<<
 *         elif isinstance(v, (list, tuple)):
 *             self.setZBasis(ovrVector3f(<float>v[0], <float>v[1], <float>v[2]))
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_setZBasis); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1223, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (!__pyx_t_5) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_v_v); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1223, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_v_v};
        __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1223, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_v_v};
        __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1223, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1223, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
        __Pyx_INCREF(((PyObject *)((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_v)));
        __Pyx_GIVEREF(__pyx_v_v);
        PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_v_v);
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_6, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1223, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "psychxr/ovr/math.pyx":1222
 *     @zBasis.setter
 *     def zBasis(self, object v):
 *         if isinstance(v, ovrVector3f):             # <<<<<<<<<<<<<<
 *             self.setZBasis(<ovrVector3f>v)
 *         elif isinstance(v, (list, tuple)):
 */
    goto __pyx_L3;
  }

  /* "psychxr/ovr/math.pyx":1224
 *         if isinstance(v, ovrVector3f):
 *             self.setZBasis(<ovrVector3f>v)
 *         elif isinstance(v, (list, tuple)):             # <<<<<<<<<<<<<<
 *             self.setZBasis(ovrVector3f(<float>v[0], <float>v[1], <float>v[2]))
 * 
 */
  __pyx_t_1 = PyList_Check(__pyx_v_v); 
  __pyx_t_7 = (__pyx_t_1 != 0);
  if (!__pyx_t_7) {
  } else {
    __pyx_t_2 = __pyx_t_7;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_7 = PyTuple_Check(__pyx_v_v); 
  __pyx_t_1 = (__pyx_t_7 != 0);
  __pyx_t_2 = __pyx_t_1;
  __pyx_L4_bool_binop_done:;
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {

    /* "psychxr/ovr/math.pyx":1225
 *             self.setZBasis(<ovrVector3f>v)
 *         elif isinstance(v, (list, tuple)):
 *             self.setZBasis(ovrVector3f(<float>v[0], <float>v[1], <float>v[2]))             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_setZBasis); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1225, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = __Pyx_GetItemInt(__pyx_v_v, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1225, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = __pyx_PyFloat_AsFloat(__pyx_t_6); if (unlikely((__pyx_t_8 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1225, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = PyFloat_FromDouble(((float)__pyx_t_8)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1225, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_5 = __Pyx_GetItemInt(__pyx_v_v, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1225, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_8 = __pyx_PyFloat_AsFloat(__pyx_t_5); if (unlikely((__pyx_t_8 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1225, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = PyFloat_FromDouble(((float)__pyx_t_8)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1225, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_9 = __Pyx_GetItemInt(__pyx_v_v, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1225, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_8 = __pyx_PyFloat_AsFloat(__pyx_t_9); if (unlikely((__pyx_t_8 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1225, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyFloat_FromDouble(((float)__pyx_t_8)); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1225, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_10 = PyTuple_New(3); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1225, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_10, 1, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_9);
    PyTuple_SET_ITEM(__pyx_t_10, 2, __pyx_t_9);
    __pyx_t_6 = 0;
    __pyx_t_5 = 0;
    __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector3f), __pyx_t_10, NULL); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1225, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (!__pyx_t_10) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_9); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1225, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_10, __pyx_t_9};
        __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1225, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_10, __pyx_t_9};
        __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1225, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      } else
      #endif
      {
        __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1225, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_10); __pyx_t_10 = NULL;
        __Pyx_GIVEREF(__pyx_t_9);
        PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_9);
        __pyx_t_9 = 0;
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_5, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1225, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "psychxr/ovr/math.pyx":1224
 *         if isinstance(v, ovrVector3f):
 *             self.setZBasis(<ovrVector3f>v)
 *         elif isinstance(v, (list, tuple)):             # <<<<<<<<<<<<<<
 *             self.setZBasis(ovrVector3f(<float>v[0], <float>v[1], <float>v[2]))
 * 
 */
  }
  __pyx_L3:;

  /* "psychxr/ovr/math.pyx":1221
 * 
 *     @zBasis.setter
 *     def zBasis(self, object v):             # <<<<<<<<<<<<<<
 *         if isinstance(v, ovrVector3f):
 *             self.setZBasis(<ovrVector3f>v)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.zBasis.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1228
 * 
 *     @property
 *     def ctypes(self):             # <<<<<<<<<<<<<<
 *         return (GL.GLfloat * 16)(
 *             self.c_data.M[0][0],
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_6ctypes_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_6ctypes_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_6ctypes___get__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_6ctypes___get__(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  int __pyx_t_20;
  PyObject *__pyx_t_21 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "psychxr/ovr/math.pyx":1229
 *     @property
 *     def ctypes(self):
 *         return (GL.GLfloat * 16)(             # <<<<<<<<<<<<<<
 *             self.c_data.M[0][0],
 *             self.c_data.M[1][0],
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_GL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1229, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_GLfloat); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1229, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyNumber_Multiply(__pyx_t_3, __pyx_int_16); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1229, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "psychxr/ovr/math.pyx":1230
 *     def ctypes(self):
 *         return (GL.GLfloat * 16)(
 *             self.c_data.M[0][0],             # <<<<<<<<<<<<<<
 *             self.c_data.M[1][0],
 *             self.c_data.M[2][0],
 */
  __pyx_t_3 = PyFloat_FromDouble(((__pyx_v_self->c_data->M[0])[0])); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1230, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "psychxr/ovr/math.pyx":1231
 *         return (GL.GLfloat * 16)(
 *             self.c_data.M[0][0],
 *             self.c_data.M[1][0],             # <<<<<<<<<<<<<<
 *             self.c_data.M[2][0],
 *             self.c_data.M[3][0],
 */
  __pyx_t_4 = PyFloat_FromDouble(((__pyx_v_self->c_data->M[1])[0])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1231, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);

  /* "psychxr/ovr/math.pyx":1232
 *             self.c_data.M[0][0],
 *             self.c_data.M[1][0],
 *             self.c_data.M[2][0],             # <<<<<<<<<<<<<<
 *             self.c_data.M[3][0],
 *             self.c_data.M[0][1],
 */
  __pyx_t_5 = PyFloat_FromDouble(((__pyx_v_self->c_data->M[2])[0])); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1232, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);

  /* "psychxr/ovr/math.pyx":1233
 *             self.c_data.M[1][0],
 *             self.c_data.M[2][0],
 *             self.c_data.M[3][0],             # <<<<<<<<<<<<<<
 *             self.c_data.M[0][1],
 *             self.c_data.M[1][1],
 */
  __pyx_t_6 = PyFloat_FromDouble(((__pyx_v_self->c_data->M[3])[0])); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1233, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);

  /* "psychxr/ovr/math.pyx":1234
 *             self.c_data.M[2][0],
 *             self.c_data.M[3][0],
 *             self.c_data.M[0][1],             # <<<<<<<<<<<<<<
 *             self.c_data.M[1][1],
 *             self.c_data.M[2][1],
 */
  __pyx_t_7 = PyFloat_FromDouble(((__pyx_v_self->c_data->M[0])[1])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1234, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);

  /* "psychxr/ovr/math.pyx":1235
 *             self.c_data.M[3][0],
 *             self.c_data.M[0][1],
 *             self.c_data.M[1][1],             # <<<<<<<<<<<<<<
 *             self.c_data.M[2][1],
 *             self.c_data.M[3][1],
 */
  __pyx_t_8 = PyFloat_FromDouble(((__pyx_v_self->c_data->M[1])[1])); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1235, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);

  /* "psychxr/ovr/math.pyx":1236
 *             self.c_data.M[0][1],
 *             self.c_data.M[1][1],
 *             self.c_data.M[2][1],             # <<<<<<<<<<<<<<
 *             self.c_data.M[3][1],
 *             self.c_data.M[0][2],
 */
  __pyx_t_9 = PyFloat_FromDouble(((__pyx_v_self->c_data->M[2])[1])); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1236, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);

  /* "psychxr/ovr/math.pyx":1237
 *             self.c_data.M[1][1],
 *             self.c_data.M[2][1],
 *             self.c_data.M[3][1],             # <<<<<<<<<<<<<<
 *             self.c_data.M[0][2],
 *             self.c_data.M[1][2],
 */
  __pyx_t_10 = PyFloat_FromDouble(((__pyx_v_self->c_data->M[3])[1])); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1237, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);

  /* "psychxr/ovr/math.pyx":1238
 *             self.c_data.M[2][1],
 *             self.c_data.M[3][1],
 *             self.c_data.M[0][2],             # <<<<<<<<<<<<<<
 *             self.c_data.M[1][2],
 *             self.c_data.M[2][2],
 */
  __pyx_t_11 = PyFloat_FromDouble(((__pyx_v_self->c_data->M[0])[2])); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1238, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);

  /* "psychxr/ovr/math.pyx":1239
 *             self.c_data.M[3][1],
 *             self.c_data.M[0][2],
 *             self.c_data.M[1][2],             # <<<<<<<<<<<<<<
 *             self.c_data.M[2][2],
 *             self.c_data.M[3][2],
 */
  __pyx_t_12 = PyFloat_FromDouble(((__pyx_v_self->c_data->M[1])[2])); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1239, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);

  /* "psychxr/ovr/math.pyx":1240
 *             self.c_data.M[0][2],
 *             self.c_data.M[1][2],
 *             self.c_data.M[2][2],             # <<<<<<<<<<<<<<
 *             self.c_data.M[3][2],
 *             self.c_data.M[0][3],
 */
  __pyx_t_13 = PyFloat_FromDouble(((__pyx_v_self->c_data->M[2])[2])); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1240, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);

  /* "psychxr/ovr/math.pyx":1241
 *             self.c_data.M[1][2],
 *             self.c_data.M[2][2],
 *             self.c_data.M[3][2],             # <<<<<<<<<<<<<<
 *             self.c_data.M[0][3],
 *             self.c_data.M[1][3],
 */
  __pyx_t_14 = PyFloat_FromDouble(((__pyx_v_self->c_data->M[3])[2])); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1241, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);

  /* "psychxr/ovr/math.pyx":1242
 *             self.c_data.M[2][2],
 *             self.c_data.M[3][2],
 *             self.c_data.M[0][3],             # <<<<<<<<<<<<<<
 *             self.c_data.M[1][3],
 *             self.c_data.M[2][3],
 */
  __pyx_t_15 = PyFloat_FromDouble(((__pyx_v_self->c_data->M[0])[3])); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 1242, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);

  /* "psychxr/ovr/math.pyx":1243
 *             self.c_data.M[3][2],
 *             self.c_data.M[0][3],
 *             self.c_data.M[1][3],             # <<<<<<<<<<<<<<
 *             self.c_data.M[2][3],
 *             self.c_data.M[3][3])
 */
  __pyx_t_16 = PyFloat_FromDouble(((__pyx_v_self->c_data->M[1])[3])); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 1243, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_16);

  /* "psychxr/ovr/math.pyx":1244
 *             self.c_data.M[0][3],
 *             self.c_data.M[1][3],
 *             self.c_data.M[2][3],             # <<<<<<<<<<<<<<
 *             self.c_data.M[3][3])
 * 
 */
  __pyx_t_17 = PyFloat_FromDouble(((__pyx_v_self->c_data->M[2])[3])); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1244, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_17);

  /* "psychxr/ovr/math.pyx":1245
 *             self.c_data.M[1][3],
 *             self.c_data.M[2][3],
 *             self.c_data.M[3][3])             # <<<<<<<<<<<<<<
 * 
 *     def __eq__(self, ovrMatrix4f b):
 */
  __pyx_t_18 = PyFloat_FromDouble(((__pyx_v_self->c_data->M[3])[3])); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 1245, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_18);
  __pyx_t_19 = NULL;
  __pyx_t_20 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_19 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_19)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_19);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_20 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[17] = {__pyx_t_19, __pyx_t_3, __pyx_t_4, __pyx_t_5, __pyx_t_6, __pyx_t_7, __pyx_t_8, __pyx_t_9, __pyx_t_10, __pyx_t_11, __pyx_t_12, __pyx_t_13, __pyx_t_14, __pyx_t_15, __pyx_t_16, __pyx_t_17, __pyx_t_18};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_20, 16+__pyx_t_20); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1229, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_19); __pyx_t_19 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
    __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
    __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
    __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
    __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
    __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[17] = {__pyx_t_19, __pyx_t_3, __pyx_t_4, __pyx_t_5, __pyx_t_6, __pyx_t_7, __pyx_t_8, __pyx_t_9, __pyx_t_10, __pyx_t_11, __pyx_t_12, __pyx_t_13, __pyx_t_14, __pyx_t_15, __pyx_t_16, __pyx_t_17, __pyx_t_18};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_20, 16+__pyx_t_20); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1229, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_19); __pyx_t_19 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
    __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
    __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
    __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
    __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
    __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
  } else
  #endif
  {
    __pyx_t_21 = PyTuple_New(16+__pyx_t_20); if (unlikely(!__pyx_t_21)) __PYX_ERR(0, 1229, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_21);
    if (__pyx_t_19) {
      __Pyx_GIVEREF(__pyx_t_19); PyTuple_SET_ITEM(__pyx_t_21, 0, __pyx_t_19); __pyx_t_19 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_21, 0+__pyx_t_20, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_21, 1+__pyx_t_20, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_21, 2+__pyx_t_20, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_21, 3+__pyx_t_20, __pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_21, 4+__pyx_t_20, __pyx_t_7);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_21, 5+__pyx_t_20, __pyx_t_8);
    __Pyx_GIVEREF(__pyx_t_9);
    PyTuple_SET_ITEM(__pyx_t_21, 6+__pyx_t_20, __pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_10);
    PyTuple_SET_ITEM(__pyx_t_21, 7+__pyx_t_20, __pyx_t_10);
    __Pyx_GIVEREF(__pyx_t_11);
    PyTuple_SET_ITEM(__pyx_t_21, 8+__pyx_t_20, __pyx_t_11);
    __Pyx_GIVEREF(__pyx_t_12);
    PyTuple_SET_ITEM(__pyx_t_21, 9+__pyx_t_20, __pyx_t_12);
    __Pyx_GIVEREF(__pyx_t_13);
    PyTuple_SET_ITEM(__pyx_t_21, 10+__pyx_t_20, __pyx_t_13);
    __Pyx_GIVEREF(__pyx_t_14);
    PyTuple_SET_ITEM(__pyx_t_21, 11+__pyx_t_20, __pyx_t_14);
    __Pyx_GIVEREF(__pyx_t_15);
    PyTuple_SET_ITEM(__pyx_t_21, 12+__pyx_t_20, __pyx_t_15);
    __Pyx_GIVEREF(__pyx_t_16);
    PyTuple_SET_ITEM(__pyx_t_21, 13+__pyx_t_20, __pyx_t_16);
    __Pyx_GIVEREF(__pyx_t_17);
    PyTuple_SET_ITEM(__pyx_t_21, 14+__pyx_t_20, __pyx_t_17);
    __Pyx_GIVEREF(__pyx_t_18);
    PyTuple_SET_ITEM(__pyx_t_21, 15+__pyx_t_20, __pyx_t_18);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_5 = 0;
    __pyx_t_6 = 0;
    __pyx_t_7 = 0;
    __pyx_t_8 = 0;
    __pyx_t_9 = 0;
    __pyx_t_10 = 0;
    __pyx_t_11 = 0;
    __pyx_t_12 = 0;
    __pyx_t_13 = 0;
    __pyx_t_14 = 0;
    __pyx_t_15 = 0;
    __pyx_t_16 = 0;
    __pyx_t_17 = 0;
    __pyx_t_18 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_21, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1229, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_21); __pyx_t_21 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1228
 * 
 *     @property
 *     def ctypes(self):             # <<<<<<<<<<<<<<
 *         return (GL.GLfloat * 16)(
 *             self.c_data.M[0][0],
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_XDECREF(__pyx_t_19);
  __Pyx_XDECREF(__pyx_t_21);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.ctypes.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1247
 *             self.c_data.M[3][3])
 * 
 *     def __eq__(self, ovrMatrix4f b):             # <<<<<<<<<<<<<<
 *         return (<ovrMatrix4f>self).c_data[0] == b.c_data[0]
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_19__eq__(PyObject *__pyx_v_self, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_19__eq__(PyObject *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__eq__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f, 1, "b", 0))) __PYX_ERR(0, 1247, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_18__eq__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_18__eq__(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__eq__", 0);

  /* "psychxr/ovr/math.pyx":1248
 * 
 *     def __eq__(self, ovrMatrix4f b):
 *         return (<ovrMatrix4f>self).c_data[0] == b.c_data[0]             # <<<<<<<<<<<<<<
 * 
 *     def __ne__(self, ovrMatrix4f b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(((__pyx_v_self->c_data[0]) == (__pyx_v_b->c_data[0]))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1248, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1247
 *             self.c_data.M[3][3])
 * 
 *     def __eq__(self, ovrMatrix4f b):             # <<<<<<<<<<<<<<
 *         return (<ovrMatrix4f>self).c_data[0] == b.c_data[0]
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.__eq__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1250
 *         return (<ovrMatrix4f>self).c_data[0] == b.c_data[0]
 * 
 *     def __ne__(self, ovrMatrix4f b):             # <<<<<<<<<<<<<<
 *         return not (<ovrMatrix4f>self).c_data[0] == b.c_data[0]
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_21__ne__(PyObject *__pyx_v_self, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_21__ne__(PyObject *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__ne__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f, 1, "b", 0))) __PYX_ERR(0, 1250, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_20__ne__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_20__ne__(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__ne__", 0);

  /* "psychxr/ovr/math.pyx":1251
 * 
 *     def __ne__(self, ovrMatrix4f b):
 *         return not (<ovrMatrix4f>self).c_data[0] == b.c_data[0]             # <<<<<<<<<<<<<<
 * 
 *     def __add__(ovrMatrix4f a, ovrMatrix4f b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong((!(((__pyx_v_self->c_data[0]) == (__pyx_v_b->c_data[0])) != 0))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1251, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1250
 *         return (<ovrMatrix4f>self).c_data[0] == b.c_data[0]
 * 
 *     def __ne__(self, ovrMatrix4f b):             # <<<<<<<<<<<<<<
 *         return not (<ovrMatrix4f>self).c_data[0] == b.c_data[0]
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.__ne__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1253
 *         return not (<ovrMatrix4f>self).c_data[0] == b.c_data[0]
 * 
 *     def __add__(ovrMatrix4f a, ovrMatrix4f b):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = a.c_data[0] + b.c_data[0]
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_23__add__(PyObject *__pyx_v_a, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_23__add__(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__add__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_a), __pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f, 1, "a", 0))) __PYX_ERR(0, 1253, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f, 1, "b", 0))) __PYX_ERR(0, 1253, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_22__add__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_a), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_22__add__(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_a, struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_b) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__add__", 0);

  /* "psychxr/ovr/math.pyx":1254
 * 
 *     def __add__(ovrMatrix4f a, ovrMatrix4f b):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()             # <<<<<<<<<<<<<<
 *         (<ovrMatrix4f>to_return).c_data[0] = a.c_data[0] + b.c_data[0]
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1254, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":1255
 *     def __add__(ovrMatrix4f a, ovrMatrix4f b):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = a.c_data[0] + b.c_data[0]             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
  (__pyx_v_to_return->c_data[0]) = ((__pyx_v_a->c_data[0]) + (__pyx_v_b->c_data[0]));

  /* "psychxr/ovr/math.pyx":1257
 *         (<ovrMatrix4f>to_return).c_data[0] = a.c_data[0] + b.c_data[0]
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def __iadd__(self, ovrMatrix4f b):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1253
 *         return not (<ovrMatrix4f>self).c_data[0] == b.c_data[0]
 * 
 *     def __add__(ovrMatrix4f a, ovrMatrix4f b):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = a.c_data[0] + b.c_data[0]
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.__add__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1259
 *         return to_return
 * 
 *     def __iadd__(self, ovrMatrix4f b):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>self).c_data[0] = \
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_25__iadd__(PyObject *__pyx_v_self, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_25__iadd__(PyObject *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iadd__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f, 1, "b", 0))) __PYX_ERR(0, 1259, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_24__iadd__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_24__iadd__(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_b) {
  CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__iadd__", 0);

  /* "psychxr/ovr/math.pyx":1260
 * 
 *     def __iadd__(self, ovrMatrix4f b):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()             # <<<<<<<<<<<<<<
 *         (<ovrMatrix4f>self).c_data[0] = \
 *             (<ovrMatrix4f>self).c_data[0] + b.c_data[0]
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1260, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":1261
 *     def __iadd__(self, ovrMatrix4f b):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>self).c_data[0] = \             # <<<<<<<<<<<<<<
 *             (<ovrMatrix4f>self).c_data[0] + b.c_data[0]
 * 
 */
  (__pyx_v_self->c_data[0]) = ((__pyx_v_self->c_data[0]) + (__pyx_v_b->c_data[0]));

  /* "psychxr/ovr/math.pyx":1264
 *             (<ovrMatrix4f>self).c_data[0] + b.c_data[0]
 * 
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     def __sub__(ovrMatrix4f a, ovrMatrix4f b):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1259
 *         return to_return
 * 
 *     def __iadd__(self, ovrMatrix4f b):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>self).c_data[0] = \
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.__iadd__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1266
 *         return self
 * 
 *     def __sub__(ovrMatrix4f a, ovrMatrix4f b):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = a.c_data[0] - b.c_data[0]
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_27__sub__(PyObject *__pyx_v_a, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_27__sub__(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__sub__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_a), __pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f, 1, "a", 0))) __PYX_ERR(0, 1266, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f, 1, "b", 0))) __PYX_ERR(0, 1266, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_26__sub__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_a), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_26__sub__(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_a, struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_b) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__sub__", 0);

  /* "psychxr/ovr/math.pyx":1267
 * 
 *     def __sub__(ovrMatrix4f a, ovrMatrix4f b):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()             # <<<<<<<<<<<<<<
 *         (<ovrMatrix4f>to_return).c_data[0] = a.c_data[0] - b.c_data[0]
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1267, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":1268
 *     def __sub__(ovrMatrix4f a, ovrMatrix4f b):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = a.c_data[0] - b.c_data[0]             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
  (__pyx_v_to_return->c_data[0]) = ((__pyx_v_a->c_data[0]) - (__pyx_v_b->c_data[0]));

  /* "psychxr/ovr/math.pyx":1270
 *         (<ovrMatrix4f>to_return).c_data[0] = a.c_data[0] - b.c_data[0]
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def __isub__(self, ovrMatrix4f b):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1266
 *         return self
 * 
 *     def __sub__(ovrMatrix4f a, ovrMatrix4f b):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = a.c_data[0] - b.c_data[0]
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.__sub__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1272
 *         return to_return
 * 
 *     def __isub__(self, ovrMatrix4f b):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>self).c_data[0] = \
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_29__isub__(PyObject *__pyx_v_self, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_29__isub__(PyObject *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__isub__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f, 1, "b", 0))) __PYX_ERR(0, 1272, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_28__isub__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_28__isub__(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_b) {
  CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__isub__", 0);

  /* "psychxr/ovr/math.pyx":1273
 * 
 *     def __isub__(self, ovrMatrix4f b):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()             # <<<<<<<<<<<<<<
 *         (<ovrMatrix4f>self).c_data[0] = \
 *             (<ovrMatrix4f>self).c_data[0] - b.c_data[0]
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1273, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":1274
 *     def __isub__(self, ovrMatrix4f b):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>self).c_data[0] = \             # <<<<<<<<<<<<<<
 *             (<ovrMatrix4f>self).c_data[0] - b.c_data[0]
 * 
 */
  (__pyx_v_self->c_data[0]) = ((__pyx_v_self->c_data[0]) - (__pyx_v_b->c_data[0]));

  /* "psychxr/ovr/math.pyx":1277
 *             (<ovrMatrix4f>self).c_data[0] - b.c_data[0]
 * 
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1272
 *         return to_return
 * 
 *     def __isub__(self, ovrMatrix4f b):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>self).c_data[0] = \
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.__isub__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1280
 * 
 *     @staticmethod
 *     def multiply(ovrMatrix4f d, ovrMatrix4f a, ovrMatrix4f b):             # <<<<<<<<<<<<<<
 *         ovr_math.Matrix4f.Multiply(&d.c_data[0], a.c_data[0], b.c_data[0])
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_31multiply(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_7psychxr_3ovr_4math_11ovrMatrix4f_31multiply = {"multiply", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_31multiply, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_31multiply(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_d = 0;
  struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_a = 0;
  struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_b = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("multiply (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_d,&__pyx_n_s_a,&__pyx_n_s_b,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_a)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("multiply", 1, 3, 3, 1); __PYX_ERR(0, 1280, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_b)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("multiply", 1, 3, 3, 2); __PYX_ERR(0, 1280, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "multiply") < 0)) __PYX_ERR(0, 1280, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_d = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)values[0]);
    __pyx_v_a = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)values[1]);
    __pyx_v_b = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)values[2]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("multiply", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1280, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.multiply", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_d), __pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f, 1, "d", 0))) __PYX_ERR(0, 1280, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_a), __pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f, 1, "a", 0))) __PYX_ERR(0, 1280, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f, 1, "b", 0))) __PYX_ERR(0, 1280, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_30multiply(__pyx_v_d, __pyx_v_a, __pyx_v_b);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_30multiply(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_d, struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_a, struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("multiply", 0);

  /* "psychxr/ovr/math.pyx":1281
 *     @staticmethod
 *     def multiply(ovrMatrix4f d, ovrMatrix4f a, ovrMatrix4f b):
 *         ovr_math.Matrix4f.Multiply(&d.c_data[0], a.c_data[0], b.c_data[0])             # <<<<<<<<<<<<<<
 * 
 *     def __mul__(ovrMatrix4f a, object b):
 */
  OVR::Matrix4f::Multiply((&(__pyx_v_d->c_data[0])), (__pyx_v_a->c_data[0]), (__pyx_v_b->c_data[0]));

  /* "psychxr/ovr/math.pyx":1280
 * 
 *     @staticmethod
 *     def multiply(ovrMatrix4f d, ovrMatrix4f a, ovrMatrix4f b):             # <<<<<<<<<<<<<<
 *         ovr_math.Matrix4f.Multiply(&d.c_data[0], a.c_data[0], b.c_data[0])
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1283
 *         ovr_math.Matrix4f.Multiply(&d.c_data[0], a.c_data[0], b.c_data[0])
 * 
 *     def __mul__(ovrMatrix4f a, object b):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         if isinstance(b, ovrMatrix4f):
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_33__mul__(PyObject *__pyx_v_a, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_33__mul__(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__mul__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_a), __pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f, 1, "a", 0))) __PYX_ERR(0, 1283, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_32__mul__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_a), ((PyObject *)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_32__mul__(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_a, PyObject *__pyx_v_b) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  float __pyx_t_5;
  __Pyx_RefNannySetupContext("__mul__", 0);

  /* "psychxr/ovr/math.pyx":1284
 * 
 *     def __mul__(ovrMatrix4f a, object b):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()             # <<<<<<<<<<<<<<
 *         if isinstance(b, ovrMatrix4f):
 *             (<ovrMatrix4f>to_return).c_data[0] = \
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1284, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":1285
 *     def __mul__(ovrMatrix4f a, object b):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         if isinstance(b, ovrMatrix4f):             # <<<<<<<<<<<<<<
 *             (<ovrMatrix4f>to_return).c_data[0] = \
 *                 (<ovrMatrix4f>a).c_data[0] * (<ovrMatrix4f>b).c_data[0]
 */
  __pyx_t_2 = __Pyx_TypeCheck(__pyx_v_b, __pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f); 
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "psychxr/ovr/math.pyx":1286
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         if isinstance(b, ovrMatrix4f):
 *             (<ovrMatrix4f>to_return).c_data[0] = \             # <<<<<<<<<<<<<<
 *                 (<ovrMatrix4f>a).c_data[0] * (<ovrMatrix4f>b).c_data[0]
 *         elif isinstance(b, (int, float)):
 */
    (__pyx_v_to_return->c_data[0]) = ((__pyx_v_a->c_data[0]) * (((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_b)->c_data[0]));

    /* "psychxr/ovr/math.pyx":1285
 *     def __mul__(ovrMatrix4f a, object b):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         if isinstance(b, ovrMatrix4f):             # <<<<<<<<<<<<<<
 *             (<ovrMatrix4f>to_return).c_data[0] = \
 *                 (<ovrMatrix4f>a).c_data[0] * (<ovrMatrix4f>b).c_data[0]
 */
    goto __pyx_L3;
  }

  /* "psychxr/ovr/math.pyx":1288
 *             (<ovrMatrix4f>to_return).c_data[0] = \
 *                 (<ovrMatrix4f>a).c_data[0] * (<ovrMatrix4f>b).c_data[0]
 *         elif isinstance(b, (int, float)):             # <<<<<<<<<<<<<<
 *             (<ovrMatrix4f>to_return).c_data[0] = \
 *                 (<ovrMatrix4f>a).c_data[0] * <float>b
 */
  __pyx_t_2 = PyInt_Check(__pyx_v_b); 
  __pyx_t_4 = (__pyx_t_2 != 0);
  if (!__pyx_t_4) {
  } else {
    __pyx_t_3 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = PyFloat_Check(__pyx_v_b); 
  __pyx_t_2 = (__pyx_t_4 != 0);
  __pyx_t_3 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {

    /* "psychxr/ovr/math.pyx":1290
 *         elif isinstance(b, (int, float)):
 *             (<ovrMatrix4f>to_return).c_data[0] = \
 *                 (<ovrMatrix4f>a).c_data[0] * <float>b             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
    __pyx_t_5 = __pyx_PyFloat_AsFloat(__pyx_v_b); if (unlikely((__pyx_t_5 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1290, __pyx_L1_error)

    /* "psychxr/ovr/math.pyx":1289
 *                 (<ovrMatrix4f>a).c_data[0] * (<ovrMatrix4f>b).c_data[0]
 *         elif isinstance(b, (int, float)):
 *             (<ovrMatrix4f>to_return).c_data[0] = \             # <<<<<<<<<<<<<<
 *                 (<ovrMatrix4f>a).c_data[0] * <float>b
 * 
 */
    (__pyx_v_to_return->c_data[0]) = ((__pyx_v_a->c_data[0]) * ((float)__pyx_t_5));

    /* "psychxr/ovr/math.pyx":1288
 *             (<ovrMatrix4f>to_return).c_data[0] = \
 *                 (<ovrMatrix4f>a).c_data[0] * (<ovrMatrix4f>b).c_data[0]
 *         elif isinstance(b, (int, float)):             # <<<<<<<<<<<<<<
 *             (<ovrMatrix4f>to_return).c_data[0] = \
 *                 (<ovrMatrix4f>a).c_data[0] * <float>b
 */
  }
  __pyx_L3:;

  /* "psychxr/ovr/math.pyx":1292
 *                 (<ovrMatrix4f>a).c_data[0] * <float>b
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def __imul__(self, object b):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1283
 *         ovr_math.Matrix4f.Multiply(&d.c_data[0], a.c_data[0], b.c_data[0])
 * 
 *     def __mul__(ovrMatrix4f a, object b):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         if isinstance(b, ovrMatrix4f):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.__mul__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1294
 *         return to_return
 * 
 *     def __imul__(self, object b):             # <<<<<<<<<<<<<<
 *         if isinstance(b, ovrMatrix4f):
 *             (<ovrMatrix4f>self).c_data[0] = \
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_35__imul__(PyObject *__pyx_v_self, PyObject *__pyx_v_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_35__imul__(PyObject *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__imul__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_34__imul__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_self), ((PyObject *)__pyx_v_b));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_34__imul__(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  float __pyx_t_4;
  __Pyx_RefNannySetupContext("__imul__", 0);

  /* "psychxr/ovr/math.pyx":1295
 * 
 *     def __imul__(self, object b):
 *         if isinstance(b, ovrMatrix4f):             # <<<<<<<<<<<<<<
 *             (<ovrMatrix4f>self).c_data[0] = \
 *                 (<ovrMatrix4f>self).c_data[0] * (<ovrMatrix4f>b).c_data[0]
 */
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_b, __pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "psychxr/ovr/math.pyx":1296
 *     def __imul__(self, object b):
 *         if isinstance(b, ovrMatrix4f):
 *             (<ovrMatrix4f>self).c_data[0] = \             # <<<<<<<<<<<<<<
 *                 (<ovrMatrix4f>self).c_data[0] * (<ovrMatrix4f>b).c_data[0]
 *         elif isinstance(b, (int, float)):
 */
    (__pyx_v_self->c_data[0]) = ((__pyx_v_self->c_data[0]) * (((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_b)->c_data[0]));

    /* "psychxr/ovr/math.pyx":1295
 * 
 *     def __imul__(self, object b):
 *         if isinstance(b, ovrMatrix4f):             # <<<<<<<<<<<<<<
 *             (<ovrMatrix4f>self).c_data[0] = \
 *                 (<ovrMatrix4f>self).c_data[0] * (<ovrMatrix4f>b).c_data[0]
 */
    goto __pyx_L3;
  }

  /* "psychxr/ovr/math.pyx":1298
 *             (<ovrMatrix4f>self).c_data[0] = \
 *                 (<ovrMatrix4f>self).c_data[0] * (<ovrMatrix4f>b).c_data[0]
 *         elif isinstance(b, (int, float)):             # <<<<<<<<<<<<<<
 *             (<ovrMatrix4f>self).c_data[0] = \
 *                 (<ovrMatrix4f>self).c_data[0] * <float>b
 */
  __pyx_t_1 = PyInt_Check(__pyx_v_b); 
  __pyx_t_3 = (__pyx_t_1 != 0);
  if (!__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = PyFloat_Check(__pyx_v_b); 
  __pyx_t_1 = (__pyx_t_3 != 0);
  __pyx_t_2 = __pyx_t_1;
  __pyx_L4_bool_binop_done:;
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {

    /* "psychxr/ovr/math.pyx":1300
 *         elif isinstance(b, (int, float)):
 *             (<ovrMatrix4f>self).c_data[0] = \
 *                 (<ovrMatrix4f>self).c_data[0] * <float>b             # <<<<<<<<<<<<<<
 * 
 *         return self
 */
    __pyx_t_4 = __pyx_PyFloat_AsFloat(__pyx_v_b); if (unlikely((__pyx_t_4 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1300, __pyx_L1_error)

    /* "psychxr/ovr/math.pyx":1299
 *                 (<ovrMatrix4f>self).c_data[0] * (<ovrMatrix4f>b).c_data[0]
 *         elif isinstance(b, (int, float)):
 *             (<ovrMatrix4f>self).c_data[0] = \             # <<<<<<<<<<<<<<
 *                 (<ovrMatrix4f>self).c_data[0] * <float>b
 * 
 */
    (__pyx_v_self->c_data[0]) = ((__pyx_v_self->c_data[0]) * ((float)__pyx_t_4));

    /* "psychxr/ovr/math.pyx":1298
 *             (<ovrMatrix4f>self).c_data[0] = \
 *                 (<ovrMatrix4f>self).c_data[0] * (<ovrMatrix4f>b).c_data[0]
 *         elif isinstance(b, (int, float)):             # <<<<<<<<<<<<<<
 *             (<ovrMatrix4f>self).c_data[0] = \
 *                 (<ovrMatrix4f>self).c_data[0] * <float>b
 */
  }
  __pyx_L3:;

  /* "psychxr/ovr/math.pyx":1302
 *                 (<ovrMatrix4f>self).c_data[0] * <float>b
 * 
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     def __truediv__(ovrMatrix4f a, float b):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1294
 *         return to_return
 * 
 *     def __imul__(self, object b):             # <<<<<<<<<<<<<<
 *         if isinstance(b, ovrMatrix4f):
 *             (<ovrMatrix4f>self).c_data[0] = \
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.__imul__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1304
 *         return self
 * 
 *     def __truediv__(ovrMatrix4f a, float b):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = a.c_data[0] * <float>b
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_37__truediv__(PyObject *__pyx_v_a, PyObject *__pyx_arg_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_37__truediv__(PyObject *__pyx_v_a, PyObject *__pyx_arg_b) {
  float __pyx_v_b;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__truediv__ (wrapper)", 0);
  assert(__pyx_arg_b); {
    __pyx_v_b = __pyx_PyFloat_AsFloat(__pyx_arg_b); if (unlikely((__pyx_v_b == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1304, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.__truediv__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_a), __pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f, 1, "a", 0))) __PYX_ERR(0, 1304, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_36__truediv__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_a), ((float)__pyx_v_b));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_36__truediv__(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_a, float __pyx_v_b) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__truediv__", 0);

  /* "psychxr/ovr/math.pyx":1305
 * 
 *     def __truediv__(ovrMatrix4f a, float b):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()             # <<<<<<<<<<<<<<
 *         (<ovrMatrix4f>to_return).c_data[0] = a.c_data[0] * <float>b
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1305, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":1306
 *     def __truediv__(ovrMatrix4f a, float b):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = a.c_data[0] * <float>b             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
  (__pyx_v_to_return->c_data[0]) = ((__pyx_v_a->c_data[0]) * ((float)__pyx_v_b));

  /* "psychxr/ovr/math.pyx":1308
 *         (<ovrMatrix4f>to_return).c_data[0] = a.c_data[0] * <float>b
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def __itruediv__(self, float b):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1304
 *         return self
 * 
 *     def __truediv__(ovrMatrix4f a, float b):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = a.c_data[0] * <float>b
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.__truediv__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1310
 *         return to_return
 * 
 *     def __itruediv__(self, float b):             # <<<<<<<<<<<<<<
 *         (<ovrMatrix4f>self).c_data[0] = (<ovrMatrix4f>self).c_data[0] / <float>b
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_39__itruediv__(PyObject *__pyx_v_self, PyObject *__pyx_arg_b); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_39__itruediv__(PyObject *__pyx_v_self, PyObject *__pyx_arg_b) {
  float __pyx_v_b;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__itruediv__ (wrapper)", 0);
  assert(__pyx_arg_b); {
    __pyx_v_b = __pyx_PyFloat_AsFloat(__pyx_arg_b); if (unlikely((__pyx_v_b == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1310, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.__itruediv__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_38__itruediv__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_self), ((float)__pyx_v_b));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_38__itruediv__(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self, float __pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__itruediv__", 0);

  /* "psychxr/ovr/math.pyx":1311
 * 
 *     def __itruediv__(self, float b):
 *         (<ovrMatrix4f>self).c_data[0] = (<ovrMatrix4f>self).c_data[0] / <float>b             # <<<<<<<<<<<<<<
 * 
 *         return self
 */
  (__pyx_v_self->c_data[0]) = ((__pyx_v_self->c_data[0]) / ((float)__pyx_v_b));

  /* "psychxr/ovr/math.pyx":1313
 *         (<ovrMatrix4f>self).c_data[0] = (<ovrMatrix4f>self).c_data[0] / <float>b
 * 
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     def transform(self, ovrVector3f v):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1310
 *         return to_return
 * 
 *     def __itruediv__(self, float b):             # <<<<<<<<<<<<<<
 *         (<ovrMatrix4f>self).c_data[0] = (<ovrMatrix4f>self).c_data[0] / <float>b
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1315
 *         return self
 * 
 *     def transform(self, ovrVector3f v):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_41transform(PyObject *__pyx_v_self, PyObject *__pyx_v_v); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_41transform(PyObject *__pyx_v_self, PyObject *__pyx_v_v) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("transform (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_v), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "v", 0))) __PYX_ERR(0, 1315, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_40transform(((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_v));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_40transform(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_v) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("transform", 0);

  /* "psychxr/ovr/math.pyx":1316
 * 
 *     def transform(self, ovrVector3f v):
 *         cdef ovrVector3f to_return = ovrVector3f()             # <<<<<<<<<<<<<<
 *         (<ovrVector3f>to_return).c_data[0] = \
 *             (<ovr_math.Matrix4f>self.c_data[0]).Transform(v.c_data[0])
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector3f), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1316, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":1317
 *     def transform(self, ovrVector3f v):
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \             # <<<<<<<<<<<<<<
 *             (<ovr_math.Matrix4f>self.c_data[0]).Transform(v.c_data[0])
 * 
 */
  (__pyx_v_to_return->c_data[0]) = ((OVR::Matrix4f)(__pyx_v_self->c_data[0])).Transform((__pyx_v_v->c_data[0]));

  /* "psychxr/ovr/math.pyx":1320
 *             (<ovr_math.Matrix4f>self.c_data[0]).Transform(v.c_data[0])
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def transposed(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1315
 *         return self
 * 
 *     def transform(self, ovrVector3f v):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.transform", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1322
 *         return to_return
 * 
 *     def transposed(self):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = \
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_43transposed(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_43transposed(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("transposed (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_42transposed(((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_42transposed(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("transposed", 0);

  /* "psychxr/ovr/math.pyx":1323
 * 
 *     def transposed(self):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()             # <<<<<<<<<<<<<<
 *         (<ovrMatrix4f>to_return).c_data[0] = \
 *             (<ovr_math.Matrix4f>self.c_data[0]).Transposed()
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1323, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":1324
 *     def transposed(self):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = \             # <<<<<<<<<<<<<<
 *             (<ovr_math.Matrix4f>self.c_data[0]).Transposed()
 * 
 */
  (__pyx_v_to_return->c_data[0]) = ((OVR::Matrix4f)(__pyx_v_self->c_data[0])).Transposed();

  /* "psychxr/ovr/math.pyx":1327
 *             (<ovr_math.Matrix4f>self.c_data[0]).Transposed()
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def transpose(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1322
 *         return to_return
 * 
 *     def transposed(self):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = \
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.transposed", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1329
 *         return to_return
 * 
 *     def transpose(self):             # <<<<<<<<<<<<<<
 *         (<ovr_math.Matrix4f>self.c_data[0]).Transpose()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_45transpose(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_45transpose(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("transpose (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_44transpose(((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_44transpose(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("transpose", 0);

  /* "psychxr/ovr/math.pyx":1330
 * 
 *     def transpose(self):
 *         (<ovr_math.Matrix4f>self.c_data[0]).Transpose()             # <<<<<<<<<<<<<<
 * 
 *         return self
 */
  ((OVR::Matrix4f)(__pyx_v_self->c_data[0])).Transpose();

  /* "psychxr/ovr/math.pyx":1332
 *         (<ovr_math.Matrix4f>self.c_data[0]).Transpose()
 * 
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     def determinant(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1329
 *         return to_return
 * 
 *     def transpose(self):             # <<<<<<<<<<<<<<
 *         (<ovr_math.Matrix4f>self.c_data[0]).Transpose()
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1334
 *         return self
 * 
 *     def determinant(self):             # <<<<<<<<<<<<<<
 *         cdef float det = (<ovrMatrix4f>self).c_data[0].Determinant()
 *         return det
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_47determinant(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_47determinant(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("determinant (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_46determinant(((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_46determinant(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self) {
  float __pyx_v_det;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("determinant", 0);

  /* "psychxr/ovr/math.pyx":1335
 * 
 *     def determinant(self):
 *         cdef float det = (<ovrMatrix4f>self).c_data[0].Determinant()             # <<<<<<<<<<<<<<
 *         return det
 * 
 */
  __pyx_v_det = (__pyx_v_self->c_data[0]).Determinant();

  /* "psychxr/ovr/math.pyx":1336
 *     def determinant(self):
 *         cdef float det = (<ovrMatrix4f>self).c_data[0].Determinant()
 *         return det             # <<<<<<<<<<<<<<
 * 
 *     def adjugated(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_det); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1336, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1334
 *         return self
 * 
 *     def determinant(self):             # <<<<<<<<<<<<<<
 *         cdef float det = (<ovrMatrix4f>self).c_data[0].Determinant()
 *         return det
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.determinant", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1338
 *         return det
 * 
 *     def adjugated(self):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = \
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_49adjugated(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_49adjugated(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("adjugated (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_48adjugated(((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_48adjugated(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("adjugated", 0);

  /* "psychxr/ovr/math.pyx":1339
 * 
 *     def adjugated(self):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()             # <<<<<<<<<<<<<<
 *         (<ovrMatrix4f>to_return).c_data[0] = \
 *             (<ovr_math.Matrix4f>self.c_data[0]).Adjugated()
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1339, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":1340
 *     def adjugated(self):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = \             # <<<<<<<<<<<<<<
 *             (<ovr_math.Matrix4f>self.c_data[0]).Adjugated()
 * 
 */
  (__pyx_v_to_return->c_data[0]) = ((OVR::Matrix4f)(__pyx_v_self->c_data[0])).Adjugated();

  /* "psychxr/ovr/math.pyx":1343
 *             (<ovr_math.Matrix4f>self.c_data[0]).Adjugated()
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def inverted(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1338
 *         return det
 * 
 *     def adjugated(self):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = \
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.adjugated", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1345
 *         return to_return
 * 
 *     def inverted(self):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = \
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_51inverted(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_51inverted(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("inverted (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_50inverted(((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_50inverted(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("inverted", 0);

  /* "psychxr/ovr/math.pyx":1346
 * 
 *     def inverted(self):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()             # <<<<<<<<<<<<<<
 *         (<ovrMatrix4f>to_return).c_data[0] = \
 *             (<ovr_math.Matrix4f>self.c_data[0]).Inverted()
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1346, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":1347
 *     def inverted(self):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = \             # <<<<<<<<<<<<<<
 *             (<ovr_math.Matrix4f>self.c_data[0]).Inverted()
 * 
 */
  (__pyx_v_to_return->c_data[0]) = ((OVR::Matrix4f)(__pyx_v_self->c_data[0])).Inverted();

  /* "psychxr/ovr/math.pyx":1350
 *             (<ovr_math.Matrix4f>self.c_data[0]).Inverted()
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def invert(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1345
 *         return to_return
 * 
 *     def inverted(self):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = \
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.inverted", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1352
 *         return to_return
 * 
 *     def invert(self):             # <<<<<<<<<<<<<<
 *         (<ovr_math.Matrix4f>self.c_data[0]).Transpose()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_53invert(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_53invert(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("invert (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_52invert(((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_52invert(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("invert", 0);

  /* "psychxr/ovr/math.pyx":1353
 * 
 *     def invert(self):
 *         (<ovr_math.Matrix4f>self.c_data[0]).Transpose()             # <<<<<<<<<<<<<<
 * 
 *         return self
 */
  ((OVR::Matrix4f)(__pyx_v_self->c_data[0])).Transpose();

  /* "psychxr/ovr/math.pyx":1355
 *         (<ovr_math.Matrix4f>self.c_data[0]).Transpose()
 * 
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     def invertedHomogeneousTransform(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1352
 *         return to_return
 * 
 *     def invert(self):             # <<<<<<<<<<<<<<
 *         (<ovr_math.Matrix4f>self.c_data[0]).Transpose()
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1357
 *         return self
 * 
 *     def invertedHomogeneousTransform(self):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = \
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_55invertedHomogeneousTransform(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_55invertedHomogeneousTransform(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("invertedHomogeneousTransform (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_54invertedHomogeneousTransform(((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_54invertedHomogeneousTransform(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("invertedHomogeneousTransform", 0);

  /* "psychxr/ovr/math.pyx":1358
 * 
 *     def invertedHomogeneousTransform(self):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()             # <<<<<<<<<<<<<<
 *         (<ovrMatrix4f>to_return).c_data[0] = \
 *             (<ovr_math.Matrix4f>self.c_data[0]).InvertedHomogeneousTransform()
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1358, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":1359
 *     def invertedHomogeneousTransform(self):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = \             # <<<<<<<<<<<<<<
 *             (<ovr_math.Matrix4f>self.c_data[0]).InvertedHomogeneousTransform()
 * 
 */
  (__pyx_v_to_return->c_data[0]) = ((OVR::Matrix4f)(__pyx_v_self->c_data[0])).InvertedHomogeneousTransform();

  /* "psychxr/ovr/math.pyx":1362
 *             (<ovr_math.Matrix4f>self.c_data[0]).InvertedHomogeneousTransform()
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def invertHomogeneousTransform(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1357
 *         return self
 * 
 *     def invertedHomogeneousTransform(self):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = \
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.invertedHomogeneousTransform", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1364
 *         return to_return
 * 
 *     def invertHomogeneousTransform(self):             # <<<<<<<<<<<<<<
 *         (<ovr_math.Matrix4f>self.c_data[0]).InvertHomogeneousTransform()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_57invertHomogeneousTransform(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_57invertHomogeneousTransform(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("invertHomogeneousTransform (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_56invertHomogeneousTransform(((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_56invertHomogeneousTransform(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("invertHomogeneousTransform", 0);

  /* "psychxr/ovr/math.pyx":1365
 * 
 *     def invertHomogeneousTransform(self):
 *         (<ovr_math.Matrix4f>self.c_data[0]).InvertHomogeneousTransform()             # <<<<<<<<<<<<<<
 * 
 *         return self
 */
  ((OVR::Matrix4f)(__pyx_v_self->c_data[0])).InvertHomogeneousTransform();

  /* "psychxr/ovr/math.pyx":1367
 *         (<ovr_math.Matrix4f>self.c_data[0]).InvertHomogeneousTransform()
 * 
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1364
 *         return to_return
 * 
 *     def invertHomogeneousTransform(self):             # <<<<<<<<<<<<<<
 *         (<ovr_math.Matrix4f>self.c_data[0]).InvertHomogeneousTransform()
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1370
 * 
 *     @staticmethod
 *     def translation(object v):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         if isinstance(v, ovrVector3f):
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_59translation(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_7psychxr_3ovr_4math_11ovrMatrix4f_59translation = {"translation", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_59translation, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_59translation(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_v = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("translation (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_v,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_v)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "translation") < 0)) __PYX_ERR(0, 1370, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_v = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("translation", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1370, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.translation", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_58translation(__pyx_v_v);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_58translation(PyObject *__pyx_v_v) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  float __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  __Pyx_RefNannySetupContext("translation", 0);

  /* "psychxr/ovr/math.pyx":1371
 *     @staticmethod
 *     def translation(object v):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()             # <<<<<<<<<<<<<<
 *         if isinstance(v, ovrVector3f):
 *             (<ovrMatrix4f>to_return).c_data[0] = \
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1371, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":1372
 *     def translation(object v):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         if isinstance(v, ovrVector3f):             # <<<<<<<<<<<<<<
 *             (<ovrMatrix4f>to_return).c_data[0] = \
 *                 ovr_math.Matrix4f.Translation((<ovrVector3f>v).c_data[0])
 */
  __pyx_t_2 = __Pyx_TypeCheck(__pyx_v_v, __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f); 
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "psychxr/ovr/math.pyx":1373
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         if isinstance(v, ovrVector3f):
 *             (<ovrMatrix4f>to_return).c_data[0] = \             # <<<<<<<<<<<<<<
 *                 ovr_math.Matrix4f.Translation((<ovrVector3f>v).c_data[0])
 *         elif isinstance(v, (list, tuple)):
 */
    (__pyx_v_to_return->c_data[0]) = OVR::Matrix4f::Translation((((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_v)->c_data[0]));

    /* "psychxr/ovr/math.pyx":1372
 *     def translation(object v):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         if isinstance(v, ovrVector3f):             # <<<<<<<<<<<<<<
 *             (<ovrMatrix4f>to_return).c_data[0] = \
 *                 ovr_math.Matrix4f.Translation((<ovrVector3f>v).c_data[0])
 */
    goto __pyx_L3;
  }

  /* "psychxr/ovr/math.pyx":1375
 *             (<ovrMatrix4f>to_return).c_data[0] = \
 *                 ovr_math.Matrix4f.Translation((<ovrVector3f>v).c_data[0])
 *         elif isinstance(v, (list, tuple)):             # <<<<<<<<<<<<<<
 *             (<ovrMatrix4f>to_return).c_data[0] = \
 *                 ovr_math.Matrix4f.Translation(
 */
  __pyx_t_2 = PyList_Check(__pyx_v_v); 
  __pyx_t_4 = (__pyx_t_2 != 0);
  if (!__pyx_t_4) {
  } else {
    __pyx_t_3 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = PyTuple_Check(__pyx_v_v); 
  __pyx_t_2 = (__pyx_t_4 != 0);
  __pyx_t_3 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {

    /* "psychxr/ovr/math.pyx":1378
 *             (<ovrMatrix4f>to_return).c_data[0] = \
 *                 ovr_math.Matrix4f.Translation(
 *                     ovrVector3f(<float>v[0], <float>v[1], <float>v[2]).c_data[0])             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
    __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_v, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1378, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __pyx_PyFloat_AsFloat(__pyx_t_1); if (unlikely((__pyx_t_5 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1378, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyFloat_FromDouble(((float)__pyx_t_5)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1378, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = __Pyx_GetItemInt(__pyx_v_v, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1378, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_5 = __pyx_PyFloat_AsFloat(__pyx_t_6); if (unlikely((__pyx_t_5 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1378, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = PyFloat_FromDouble(((float)__pyx_t_5)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1378, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_GetItemInt(__pyx_v_v, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1378, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_5 = __pyx_PyFloat_AsFloat(__pyx_t_7); if (unlikely((__pyx_t_5 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1378, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = PyFloat_FromDouble(((float)__pyx_t_5)); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1378, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = PyTuple_New(3); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1378, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_8, 2, __pyx_t_7);
    __pyx_t_1 = 0;
    __pyx_t_6 = 0;
    __pyx_t_7 = 0;
    __pyx_t_7 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector3f), __pyx_t_8, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1378, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

    /* "psychxr/ovr/math.pyx":1376
 *                 ovr_math.Matrix4f.Translation((<ovrVector3f>v).c_data[0])
 *         elif isinstance(v, (list, tuple)):
 *             (<ovrMatrix4f>to_return).c_data[0] = \             # <<<<<<<<<<<<<<
 *                 ovr_math.Matrix4f.Translation(
 *                     ovrVector3f(<float>v[0], <float>v[1], <float>v[2]).c_data[0])
 */
    (__pyx_v_to_return->c_data[0]) = OVR::Matrix4f::Translation((((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_t_7)->c_data[0]));
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

    /* "psychxr/ovr/math.pyx":1375
 *             (<ovrMatrix4f>to_return).c_data[0] = \
 *                 ovr_math.Matrix4f.Translation((<ovrVector3f>v).c_data[0])
 *         elif isinstance(v, (list, tuple)):             # <<<<<<<<<<<<<<
 *             (<ovrMatrix4f>to_return).c_data[0] = \
 *                 ovr_math.Matrix4f.Translation(
 */
  }
  __pyx_L3:;

  /* "psychxr/ovr/math.pyx":1380
 *                     ovrVector3f(<float>v[0], <float>v[1], <float>v[2]).c_data[0])
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def setTranslation(self, ovrVector3f v):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1370
 * 
 *     @staticmethod
 *     def translation(object v):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         if isinstance(v, ovrVector3f):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.translation", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1382
 *         return to_return
 * 
 *     def setTranslation(self, ovrVector3f v):             # <<<<<<<<<<<<<<
 *         (<ovr_math.Matrix4f>self.c_data[0]).SetTranslation(v.c_data[0])
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_61setTranslation(PyObject *__pyx_v_self, PyObject *__pyx_v_v); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_61setTranslation(PyObject *__pyx_v_self, PyObject *__pyx_v_v) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("setTranslation (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_v), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "v", 0))) __PYX_ERR(0, 1382, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_60setTranslation(((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_v_v));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_60setTranslation(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_v) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("setTranslation", 0);

  /* "psychxr/ovr/math.pyx":1383
 * 
 *     def setTranslation(self, ovrVector3f v):
 *         (<ovr_math.Matrix4f>self.c_data[0]).SetTranslation(v.c_data[0])             # <<<<<<<<<<<<<<
 * 
 *         return self
 */
  ((OVR::Matrix4f)(__pyx_v_self->c_data[0])).SetTranslation((__pyx_v_v->c_data[0]));

  /* "psychxr/ovr/math.pyx":1385
 *         (<ovr_math.Matrix4f>self.c_data[0]).SetTranslation(v.c_data[0])
 * 
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     def getTranslation(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1382
 *         return to_return
 * 
 *     def setTranslation(self, ovrVector3f v):             # <<<<<<<<<<<<<<
 *         (<ovr_math.Matrix4f>self.c_data[0]).SetTranslation(v.c_data[0])
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1387
 *         return self
 * 
 *     def getTranslation(self):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_63getTranslation(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_63getTranslation(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("getTranslation (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_62getTranslation(((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_62getTranslation(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("getTranslation", 0);

  /* "psychxr/ovr/math.pyx":1388
 * 
 *     def getTranslation(self):
 *         cdef ovrVector3f to_return = ovrVector3f()             # <<<<<<<<<<<<<<
 *         (<ovrVector3f>to_return).c_data[0] = \
 *             (<ovr_math.Matrix4f>self.c_data[0]).GetTranslation()
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector3f), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1388, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":1389
 *     def getTranslation(self):
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \             # <<<<<<<<<<<<<<
 *             (<ovr_math.Matrix4f>self.c_data[0]).GetTranslation()
 * 
 */
  (__pyx_v_to_return->c_data[0]) = ((OVR::Matrix4f)(__pyx_v_self->c_data[0])).GetTranslation();

  /* "psychxr/ovr/math.pyx":1392
 *             (<ovr_math.Matrix4f>self.c_data[0]).GetTranslation()
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1387
 *         return self
 * 
 *     def getTranslation(self):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.getTranslation", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1395
 * 
 *     @staticmethod
 *     def scaling(ovrVector3f v):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = \
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_65scaling(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_7psychxr_3ovr_4math_11ovrMatrix4f_65scaling = {"scaling", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_65scaling, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_65scaling(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_v = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("scaling (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_v,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_v)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "scaling") < 0)) __PYX_ERR(0, 1395, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_v = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)values[0]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("scaling", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1395, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.scaling", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_v), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "v", 0))) __PYX_ERR(0, 1395, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_64scaling(__pyx_v_v);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_64scaling(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_v) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("scaling", 0);

  /* "psychxr/ovr/math.pyx":1396
 *     @staticmethod
 *     def scaling(ovrVector3f v):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()             # <<<<<<<<<<<<<<
 *         (<ovrMatrix4f>to_return).c_data[0] = \
 *             ovr_math.Matrix4f.Scaling((<ovrVector3f>v).c_data[0])
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1396, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":1397
 *     def scaling(ovrVector3f v):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = \             # <<<<<<<<<<<<<<
 *             ovr_math.Matrix4f.Scaling((<ovrVector3f>v).c_data[0])
 * 
 */
  (__pyx_v_to_return->c_data[0]) = OVR::Matrix4f::Scaling((__pyx_v_v->c_data[0]));

  /* "psychxr/ovr/math.pyx":1400
 *             ovr_math.Matrix4f.Scaling((<ovrVector3f>v).c_data[0])
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     def distance(self, ovrMatrix4f m2):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1395
 * 
 *     @staticmethod
 *     def scaling(ovrVector3f v):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = \
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.scaling", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1402
 *         return to_return
 * 
 *     def distance(self, ovrMatrix4f m2):             # <<<<<<<<<<<<<<
 *         cdef float distance = \
 *             (<ovr_math.Matrix4f>self.c_data[0]).Distance(m2.c_data[0])
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_67distance(PyObject *__pyx_v_self, PyObject *__pyx_v_m2); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_67distance(PyObject *__pyx_v_self, PyObject *__pyx_v_m2) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("distance (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_m2), __pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f, 1, "m2", 0))) __PYX_ERR(0, 1402, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_66distance(((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_self), ((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_m2));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_66distance(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self, struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_m2) {
  float __pyx_v_distance;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("distance", 0);

  /* "psychxr/ovr/math.pyx":1404
 *     def distance(self, ovrMatrix4f m2):
 *         cdef float distance = \
 *             (<ovr_math.Matrix4f>self.c_data[0]).Distance(m2.c_data[0])             # <<<<<<<<<<<<<<
 * 
 *         return distance
 */
  __pyx_v_distance = ((OVR::Matrix4f)(__pyx_v_self->c_data[0])).Distance((__pyx_v_m2->c_data[0]));

  /* "psychxr/ovr/math.pyx":1406
 *             (<ovr_math.Matrix4f>self.c_data[0]).Distance(m2.c_data[0])
 * 
 *         return distance             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_distance); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1406, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1402
 *         return to_return
 * 
 *     def distance(self, ovrMatrix4f m2):             # <<<<<<<<<<<<<<
 *         cdef float distance = \
 *             (<ovr_math.Matrix4f>self.c_data[0]).Distance(m2.c_data[0])
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.distance", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1409
 * 
 *     @staticmethod
 *     def rotationX(float angle=0.0):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = ovr_math.Matrix4f.RotationX(angle)
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_69rotationX(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_7psychxr_3ovr_4math_11ovrMatrix4f_69rotationX = {"rotationX", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_69rotationX, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_69rotationX(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  float __pyx_v_angle;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("rotationX (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_angle,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_angle);
          if (value) { values[0] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "rotationX") < 0)) __PYX_ERR(0, 1409, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    if (values[0]) {
      __pyx_v_angle = __pyx_PyFloat_AsFloat(values[0]); if (unlikely((__pyx_v_angle == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1409, __pyx_L3_error)
    } else {
      __pyx_v_angle = ((float)0.0);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("rotationX", 0, 0, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1409, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.rotationX", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_68rotationX(__pyx_v_angle);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_68rotationX(float __pyx_v_angle) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("rotationX", 0);

  /* "psychxr/ovr/math.pyx":1410
 *     @staticmethod
 *     def rotationX(float angle=0.0):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()             # <<<<<<<<<<<<<<
 *         (<ovrMatrix4f>to_return).c_data[0] = ovr_math.Matrix4f.RotationX(angle)
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1410, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":1411
 *     def rotationX(float angle=0.0):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = ovr_math.Matrix4f.RotationX(angle)             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
  (__pyx_v_to_return->c_data[0]) = OVR::Matrix4f::RotationX(__pyx_v_angle);

  /* "psychxr/ovr/math.pyx":1413
 *         (<ovrMatrix4f>to_return).c_data[0] = ovr_math.Matrix4f.RotationX(angle)
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1409
 * 
 *     @staticmethod
 *     def rotationX(float angle=0.0):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = ovr_math.Matrix4f.RotationX(angle)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.rotationX", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1416
 * 
 *     @staticmethod
 *     def rotationY(float angle=0.0):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = ovr_math.Matrix4f.RotationY(angle)
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_71rotationY(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_7psychxr_3ovr_4math_11ovrMatrix4f_71rotationY = {"rotationY", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_71rotationY, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_71rotationY(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  float __pyx_v_angle;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("rotationY (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_angle,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_angle);
          if (value) { values[0] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "rotationY") < 0)) __PYX_ERR(0, 1416, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    if (values[0]) {
      __pyx_v_angle = __pyx_PyFloat_AsFloat(values[0]); if (unlikely((__pyx_v_angle == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1416, __pyx_L3_error)
    } else {
      __pyx_v_angle = ((float)0.0);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("rotationY", 0, 0, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1416, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.rotationY", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_70rotationY(__pyx_v_angle);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_70rotationY(float __pyx_v_angle) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("rotationY", 0);

  /* "psychxr/ovr/math.pyx":1417
 *     @staticmethod
 *     def rotationY(float angle=0.0):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()             # <<<<<<<<<<<<<<
 *         (<ovrMatrix4f>to_return).c_data[0] = ovr_math.Matrix4f.RotationY(angle)
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1417, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":1418
 *     def rotationY(float angle=0.0):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = ovr_math.Matrix4f.RotationY(angle)             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
  (__pyx_v_to_return->c_data[0]) = OVR::Matrix4f::RotationY(__pyx_v_angle);

  /* "psychxr/ovr/math.pyx":1420
 *         (<ovrMatrix4f>to_return).c_data[0] = ovr_math.Matrix4f.RotationY(angle)
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1416
 * 
 *     @staticmethod
 *     def rotationY(float angle=0.0):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = ovr_math.Matrix4f.RotationY(angle)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.rotationY", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1423
 * 
 *     @staticmethod
 *     def rotationZ(float angle=0.0):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = ovr_math.Matrix4f.RotationZ(angle)
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_73rotationZ(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_7psychxr_3ovr_4math_11ovrMatrix4f_73rotationZ = {"rotationZ", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_73rotationZ, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_73rotationZ(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  float __pyx_v_angle;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("rotationZ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_angle,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_angle);
          if (value) { values[0] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "rotationZ") < 0)) __PYX_ERR(0, 1423, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    if (values[0]) {
      __pyx_v_angle = __pyx_PyFloat_AsFloat(values[0]); if (unlikely((__pyx_v_angle == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1423, __pyx_L3_error)
    } else {
      __pyx_v_angle = ((float)0.0);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("rotationZ", 0, 0, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1423, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.rotationZ", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_72rotationZ(__pyx_v_angle);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_72rotationZ(float __pyx_v_angle) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("rotationZ", 0);

  /* "psychxr/ovr/math.pyx":1424
 *     @staticmethod
 *     def rotationZ(float angle=0.0):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()             # <<<<<<<<<<<<<<
 *         (<ovrMatrix4f>to_return).c_data[0] = ovr_math.Matrix4f.RotationZ(angle)
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1424, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":1425
 *     def rotationZ(float angle=0.0):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = ovr_math.Matrix4f.RotationZ(angle)             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
  (__pyx_v_to_return->c_data[0]) = OVR::Matrix4f::RotationZ(__pyx_v_angle);

  /* "psychxr/ovr/math.pyx":1427
 *         (<ovrMatrix4f>to_return).c_data[0] = ovr_math.Matrix4f.RotationZ(angle)
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1423
 * 
 *     @staticmethod
 *     def rotationZ(float angle=0.0):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = ovr_math.Matrix4f.RotationZ(angle)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.rotationZ", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1430
 * 
 *     @staticmethod
 *     def lookAt(ovrVector3f eye, ovrVector3f at, ovrVector3f up):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = ovr_math.Matrix4f.LookAtRH(
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_75lookAt(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_7psychxr_3ovr_4math_11ovrMatrix4f_75lookAt = {"lookAt", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_75lookAt, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_75lookAt(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_eye = 0;
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_at = 0;
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_up = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("lookAt (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_eye,&__pyx_n_s_at,&__pyx_n_s_up,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_eye)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_at)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("lookAt", 1, 3, 3, 1); __PYX_ERR(0, 1430, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_up)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("lookAt", 1, 3, 3, 2); __PYX_ERR(0, 1430, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "lookAt") < 0)) __PYX_ERR(0, 1430, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_eye = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)values[0]);
    __pyx_v_at = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)values[1]);
    __pyx_v_up = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)values[2]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("lookAt", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1430, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.lookAt", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_eye), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "eye", 0))) __PYX_ERR(0, 1430, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_at), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "at", 0))) __PYX_ERR(0, 1430, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_up), __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, 1, "up", 0))) __PYX_ERR(0, 1430, __pyx_L1_error)
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_74lookAt(__pyx_v_eye, __pyx_v_at, __pyx_v_up);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_74lookAt(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_eye, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_at, struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *__pyx_v_up) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("lookAt", 0);

  /* "psychxr/ovr/math.pyx":1431
 *     @staticmethod
 *     def lookAt(ovrVector3f eye, ovrVector3f at, ovrVector3f up):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()             # <<<<<<<<<<<<<<
 *         (<ovrMatrix4f>to_return).c_data[0] = ovr_math.Matrix4f.LookAtRH(
 *             eye.c_data[0], at.c_data[0], up.c_data[0])
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1431, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":1432
 *     def lookAt(ovrVector3f eye, ovrVector3f at, ovrVector3f up):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = ovr_math.Matrix4f.LookAtRH(             # <<<<<<<<<<<<<<
 *             eye.c_data[0], at.c_data[0], up.c_data[0])
 * 
 */
  (__pyx_v_to_return->c_data[0]) = OVR::Matrix4f::LookAtRH((__pyx_v_eye->c_data[0]), (__pyx_v_at->c_data[0]), (__pyx_v_up->c_data[0]));

  /* "psychxr/ovr/math.pyx":1435
 *             eye.c_data[0], at.c_data[0], up.c_data[0])
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1430
 * 
 *     @staticmethod
 *     def lookAt(ovrVector3f eye, ovrVector3f at, ovrVector3f up):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = ovr_math.Matrix4f.LookAtRH(
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.lookAt", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1438
 * 
 *     @staticmethod
 *     def perspective(float yfov, float aspect, float znear, float zfar):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = ovr_math.Matrix4f.PerspectiveRH(
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_77perspective(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_7psychxr_3ovr_4math_11ovrMatrix4f_77perspective = {"perspective", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_77perspective, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_77perspective(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  float __pyx_v_yfov;
  float __pyx_v_aspect;
  float __pyx_v_znear;
  float __pyx_v_zfar;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("perspective (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_yfov,&__pyx_n_s_aspect,&__pyx_n_s_znear,&__pyx_n_s_zfar,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_yfov)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_aspect)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("perspective", 1, 4, 4, 1); __PYX_ERR(0, 1438, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_znear)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("perspective", 1, 4, 4, 2); __PYX_ERR(0, 1438, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_zfar)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("perspective", 1, 4, 4, 3); __PYX_ERR(0, 1438, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "perspective") < 0)) __PYX_ERR(0, 1438, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_yfov = __pyx_PyFloat_AsFloat(values[0]); if (unlikely((__pyx_v_yfov == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1438, __pyx_L3_error)
    __pyx_v_aspect = __pyx_PyFloat_AsFloat(values[1]); if (unlikely((__pyx_v_aspect == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1438, __pyx_L3_error)
    __pyx_v_znear = __pyx_PyFloat_AsFloat(values[2]); if (unlikely((__pyx_v_znear == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1438, __pyx_L3_error)
    __pyx_v_zfar = __pyx_PyFloat_AsFloat(values[3]); if (unlikely((__pyx_v_zfar == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1438, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("perspective", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1438, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.perspective", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_76perspective(__pyx_v_yfov, __pyx_v_aspect, __pyx_v_znear, __pyx_v_zfar);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_76perspective(float __pyx_v_yfov, float __pyx_v_aspect, float __pyx_v_znear, float __pyx_v_zfar) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("perspective", 0);

  /* "psychxr/ovr/math.pyx":1439
 *     @staticmethod
 *     def perspective(float yfov, float aspect, float znear, float zfar):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()             # <<<<<<<<<<<<<<
 *         (<ovrMatrix4f>to_return).c_data[0] = ovr_math.Matrix4f.PerspectiveRH(
 *             yfov, aspect, znear, zfar)
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1439, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":1440
 *     def perspective(float yfov, float aspect, float znear, float zfar):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = ovr_math.Matrix4f.PerspectiveRH(             # <<<<<<<<<<<<<<
 *             yfov, aspect, znear, zfar)
 * 
 */
  (__pyx_v_to_return->c_data[0]) = OVR::Matrix4f::PerspectiveRH(__pyx_v_yfov, __pyx_v_aspect, __pyx_v_znear, __pyx_v_zfar);

  /* "psychxr/ovr/math.pyx":1443
 *             yfov, aspect, znear, zfar)
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1438
 * 
 *     @staticmethod
 *     def perspective(float yfov, float aspect, float znear, float zfar):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = ovr_math.Matrix4f.PerspectiveRH(
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.perspective", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "psychxr/ovr/math.pyx":1446
 * 
 *     @staticmethod
 *     def ortho2d(float w, float h):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = ovr_math.Matrix4f.Ortho2D(w, h)
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_79ortho2d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_7psychxr_3ovr_4math_11ovrMatrix4f_79ortho2d = {"ortho2d", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_79ortho2d, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_79ortho2d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  float __pyx_v_w;
  float __pyx_v_h;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("ortho2d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_w,&__pyx_n_s_h,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_w)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_h)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("ortho2d", 1, 2, 2, 1); __PYX_ERR(0, 1446, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "ortho2d") < 0)) __PYX_ERR(0, 1446, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_w = __pyx_PyFloat_AsFloat(values[0]); if (unlikely((__pyx_v_w == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1446, __pyx_L3_error)
    __pyx_v_h = __pyx_PyFloat_AsFloat(values[1]); if (unlikely((__pyx_v_h == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1446, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("ortho2d", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1446, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.ortho2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_78ortho2d(__pyx_v_w, __pyx_v_h);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_78ortho2d(float __pyx_v_w, float __pyx_v_h) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_to_return = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("ortho2d", 0);

  /* "psychxr/ovr/math.pyx":1447
 *     @staticmethod
 *     def ortho2d(float w, float h):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()             # <<<<<<<<<<<<<<
 *         (<ovrMatrix4f>to_return).c_data[0] = ovr_math.Matrix4f.Ortho2D(w, h)
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1447, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_return = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "psychxr/ovr/math.pyx":1448
 *     def ortho2d(float w, float h):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = ovr_math.Matrix4f.Ortho2D(w, h)             # <<<<<<<<<<<<<<
 * 
 *         return to_return
 */
  (__pyx_v_to_return->c_data[0]) = OVR::Matrix4f::Ortho2D(__pyx_v_w, __pyx_v_h);

  /* "psychxr/ovr/math.pyx":1450
 *         (<ovrMatrix4f>to_return).c_data[0] = ovr_math.Matrix4f.Ortho2D(w, h)
 * 
 *         return to_return             # <<<<<<<<<<<<<<
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_to_return));
  __pyx_r = ((PyObject *)__pyx_v_to_return);
  goto __pyx_L0;

  /* "psychxr/ovr/math.pyx":1446
 * 
 *     @staticmethod
 *     def ortho2d(float w, float h):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = ovr_math.Matrix4f.Ortho2D(w, h)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.ortho2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_to_return);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_81__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_81__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_80__reduce_cython__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_80__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__16, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_83__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_83__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_82__setstate_cython__(((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7psychxr_3ovr_4math_11ovrMatrix4f_82__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__17, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("psychxr.ovr.math.ovrMatrix4f.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ovr_errorcode.pxd":120
 *         char[512] ErrorString
 * 
 * cdef inline int OVR_SUCCESS(ovrResult result):             # <<<<<<<<<<<<<<
 *     return result >= ovrSuccessType.ovrSuccess
 * 
 */

static CYTHON_INLINE int __pyx_f_7psychxr_3ovr_13ovr_errorcode_OVR_SUCCESS(ovrResult __pyx_v_result) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("OVR_SUCCESS", 0);

  /* "ovr_errorcode.pxd":121
 * 
 * cdef inline int OVR_SUCCESS(ovrResult result):
 *     return result >= ovrSuccessType.ovrSuccess             # <<<<<<<<<<<<<<
 * 
 * cdef inline int OVR_UNQUALIFIED_SUCCESS(ovrResult result):
 */
  __pyx_r = (__pyx_v_result >= ovrSuccess);
  goto __pyx_L0;

  /* "ovr_errorcode.pxd":120
 *         char[512] ErrorString
 * 
 * cdef inline int OVR_SUCCESS(ovrResult result):             # <<<<<<<<<<<<<<
 *     return result >= ovrSuccessType.ovrSuccess
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ovr_errorcode.pxd":123
 *     return result >= ovrSuccessType.ovrSuccess
 * 
 * cdef inline int OVR_UNQUALIFIED_SUCCESS(ovrResult result):             # <<<<<<<<<<<<<<
 *     return result == ovrSuccessType.ovrSuccess
 * 
 */

static CYTHON_INLINE int __pyx_f_7psychxr_3ovr_13ovr_errorcode_OVR_UNQUALIFIED_SUCCESS(ovrResult __pyx_v_result) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("OVR_UNQUALIFIED_SUCCESS", 0);

  /* "ovr_errorcode.pxd":124
 * 
 * cdef inline int OVR_UNQUALIFIED_SUCCESS(ovrResult result):
 *     return result == ovrSuccessType.ovrSuccess             # <<<<<<<<<<<<<<
 * 
 * cdef inline int OVR_FAILURE(ovrResult result):
 */
  __pyx_r = (__pyx_v_result == ovrSuccess);
  goto __pyx_L0;

  /* "ovr_errorcode.pxd":123
 *     return result >= ovrSuccessType.ovrSuccess
 * 
 * cdef inline int OVR_UNQUALIFIED_SUCCESS(ovrResult result):             # <<<<<<<<<<<<<<
 *     return result == ovrSuccessType.ovrSuccess
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ovr_errorcode.pxd":126
 *     return result == ovrSuccessType.ovrSuccess
 * 
 * cdef inline int OVR_FAILURE(ovrResult result):             # <<<<<<<<<<<<<<
 *     return not OVR_SUCCESS(result)
 */

static CYTHON_INLINE int __pyx_f_7psychxr_3ovr_13ovr_errorcode_OVR_FAILURE(ovrResult __pyx_v_result) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("OVR_FAILURE", 0);

  /* "ovr_errorcode.pxd":127
 * 
 * cdef inline int OVR_FAILURE(ovrResult result):
 *     return not OVR_SUCCESS(result)             # <<<<<<<<<<<<<<
 */
  __pyx_r = (!(__pyx_f_7psychxr_3ovr_13ovr_errorcode_OVR_SUCCESS(__pyx_v_result) != 0));
  goto __pyx_L0;

  /* "ovr_errorcode.pxd":126
 *     return result == ovrSuccessType.ovrSuccess
 * 
 * cdef inline int OVR_FAILURE(ovrResult result):             # <<<<<<<<<<<<<<
 *     return not OVR_SUCCESS(result)
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "carray.to_py":112
 * 
 * @cname("__Pyx_carray_to_py_float")
 * cdef inline list __Pyx_carray_to_py_float(base_type *v, Py_ssize_t length):             # <<<<<<<<<<<<<<
 *     cdef size_t i
 *     cdef object value
 */

static CYTHON_INLINE PyObject *__Pyx_carray_to_py_float(float *__pyx_v_v, Py_ssize_t __pyx_v_length) {
  size_t __pyx_v_i;
  PyObject *__pyx_v_value = 0;
  PyObject *__pyx_v_l = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  size_t __pyx_t_2;
  size_t __pyx_t_3;
  __Pyx_RefNannySetupContext("__Pyx_carray_to_py_float", 0);

  /* "carray.to_py":115
 *     cdef size_t i
 *     cdef object value
 *     l = PyList_New(length)             # <<<<<<<<<<<<<<
 *     for i in range(<size_t>length):
 *         value = v[i]
 */
  __pyx_t_1 = PyList_New(__pyx_v_length); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 115, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_l = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "carray.to_py":116
 *     cdef object value
 *     l = PyList_New(length)
 *     for i in range(<size_t>length):             # <<<<<<<<<<<<<<
 *         value = v[i]
 *         Py_INCREF(value)
 */
  __pyx_t_2 = ((size_t)__pyx_v_length);
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "carray.to_py":117
 *     l = PyList_New(length)
 *     for i in range(<size_t>length):
 *         value = v[i]             # <<<<<<<<<<<<<<
 *         Py_INCREF(value)
 *         PyList_SET_ITEM(l, i, value)
 */
    __pyx_t_1 = PyFloat_FromDouble((__pyx_v_v[__pyx_v_i])); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 117, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_value, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "carray.to_py":118
 *     for i in range(<size_t>length):
 *         value = v[i]
 *         Py_INCREF(value)             # <<<<<<<<<<<<<<
 *         PyList_SET_ITEM(l, i, value)
 *     return l
 */
    Py_INCREF(__pyx_v_value);

    /* "carray.to_py":119
 *         value = v[i]
 *         Py_INCREF(value)
 *         PyList_SET_ITEM(l, i, value)             # <<<<<<<<<<<<<<
 *     return l
 * 
 */
    PyList_SET_ITEM(__pyx_v_l, __pyx_v_i, __pyx_v_value);
  }

  /* "carray.to_py":120
 *         Py_INCREF(value)
 *         PyList_SET_ITEM(l, i, value)
 *     return l             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_l);
  __pyx_r = __pyx_v_l;
  goto __pyx_L0;

  /* "carray.to_py":112
 * 
 * @cname("__Pyx_carray_to_py_float")
 * cdef inline list __Pyx_carray_to_py_float(base_type *v, Py_ssize_t length):             # <<<<<<<<<<<<<<
 *     cdef size_t i
 *     cdef object value
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("carray.to_py.__Pyx_carray_to_py_float", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_value);
  __Pyx_XDECREF(__pyx_v_l);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "carray.to_py":124
 * 
 * @cname("__Pyx_carray_to_tuple_float")
 * cdef inline tuple __Pyx_carray_to_tuple_float(base_type *v, Py_ssize_t length):             # <<<<<<<<<<<<<<
 *     cdef size_t i
 *     cdef object value
 */

static CYTHON_INLINE PyObject *__Pyx_carray_to_tuple_float(float *__pyx_v_v, Py_ssize_t __pyx_v_length) {
  size_t __pyx_v_i;
  PyObject *__pyx_v_value = 0;
  PyObject *__pyx_v_t = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  size_t __pyx_t_2;
  size_t __pyx_t_3;
  __Pyx_RefNannySetupContext("__Pyx_carray_to_tuple_float", 0);

  /* "carray.to_py":127
 *     cdef size_t i
 *     cdef object value
 *     t = PyTuple_New(length)             # <<<<<<<<<<<<<<
 *     for i in range(<size_t>length):
 *         value = v[i]
 */
  __pyx_t_1 = PyTuple_New(__pyx_v_length); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 127, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_t = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "carray.to_py":128
 *     cdef object value
 *     t = PyTuple_New(length)
 *     for i in range(<size_t>length):             # <<<<<<<<<<<<<<
 *         value = v[i]
 *         Py_INCREF(value)
 */
  __pyx_t_2 = ((size_t)__pyx_v_length);
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "carray.to_py":129
 *     t = PyTuple_New(length)
 *     for i in range(<size_t>length):
 *         value = v[i]             # <<<<<<<<<<<<<<
 *         Py_INCREF(value)
 *         PyTuple_SET_ITEM(t, i, value)
 */
    __pyx_t_1 = PyFloat_FromDouble((__pyx_v_v[__pyx_v_i])); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 129, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_value, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "carray.to_py":130
 *     for i in range(<size_t>length):
 *         value = v[i]
 *         Py_INCREF(value)             # <<<<<<<<<<<<<<
 *         PyTuple_SET_ITEM(t, i, value)
 *     return t
 */
    Py_INCREF(__pyx_v_value);

    /* "carray.to_py":131
 *         value = v[i]
 *         Py_INCREF(value)
 *         PyTuple_SET_ITEM(t, i, value)             # <<<<<<<<<<<<<<
 *     return t
 */
    PyTuple_SET_ITEM(__pyx_v_t, __pyx_v_i, __pyx_v_value);
  }

  /* "carray.to_py":132
 *         Py_INCREF(value)
 *         PyTuple_SET_ITEM(t, i, value)
 *     return t             # <<<<<<<<<<<<<<
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_t);
  __pyx_r = __pyx_v_t;
  goto __pyx_L0;

  /* "carray.to_py":124
 * 
 * @cname("__Pyx_carray_to_tuple_float")
 * cdef inline tuple __Pyx_carray_to_tuple_float(base_type *v, Py_ssize_t length):             # <<<<<<<<<<<<<<
 *     cdef size_t i
 *     cdef object value
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("carray.to_py.__Pyx_carray_to_tuple_float", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_value);
  __Pyx_XDECREF(__pyx_v_t);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "carray.to_py":112
 * 
 * @cname("__Pyx_carray_to_py_float___5b_4_5d_")
 * cdef inline list __Pyx_carray_to_py_float___5b_4_5d_(base_type *v, Py_ssize_t length):             # <<<<<<<<<<<<<<
 *     cdef size_t i
 *     cdef object value
 */

static CYTHON_INLINE PyObject *__Pyx_carray_to_py_float___5b_4_5d_(float (*__pyx_v_v)[4], Py_ssize_t __pyx_v_length) {
  size_t __pyx_v_i;
  PyObject *__pyx_v_value = 0;
  PyObject *__pyx_v_l = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  size_t __pyx_t_2;
  size_t __pyx_t_3;
  __Pyx_RefNannySetupContext("__Pyx_carray_to_py_float___5b_4_5d_", 0);

  /* "carray.to_py":115
 *     cdef size_t i
 *     cdef object value
 *     l = PyList_New(length)             # <<<<<<<<<<<<<<
 *     for i in range(<size_t>length):
 *         value = v[i]
 */
  __pyx_t_1 = PyList_New(__pyx_v_length); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 115, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_l = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "carray.to_py":116
 *     cdef object value
 *     l = PyList_New(length)
 *     for i in range(<size_t>length):             # <<<<<<<<<<<<<<
 *         value = v[i]
 *         Py_INCREF(value)
 */
  __pyx_t_2 = ((size_t)__pyx_v_length);
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "carray.to_py":117
 *     l = PyList_New(length)
 *     for i in range(<size_t>length):
 *         value = v[i]             # <<<<<<<<<<<<<<
 *         Py_INCREF(value)
 *         PyList_SET_ITEM(l, i, value)
 */
    __pyx_t_1 = __Pyx_carray_to_py_float((__pyx_v_v[__pyx_v_i]), 4); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 117, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_value, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "carray.to_py":118
 *     for i in range(<size_t>length):
 *         value = v[i]
 *         Py_INCREF(value)             # <<<<<<<<<<<<<<
 *         PyList_SET_ITEM(l, i, value)
 *     return l
 */
    Py_INCREF(__pyx_v_value);

    /* "carray.to_py":119
 *         value = v[i]
 *         Py_INCREF(value)
 *         PyList_SET_ITEM(l, i, value)             # <<<<<<<<<<<<<<
 *     return l
 * 
 */
    PyList_SET_ITEM(__pyx_v_l, __pyx_v_i, __pyx_v_value);
  }

  /* "carray.to_py":120
 *         Py_INCREF(value)
 *         PyList_SET_ITEM(l, i, value)
 *     return l             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_l);
  __pyx_r = __pyx_v_l;
  goto __pyx_L0;

  /* "carray.to_py":112
 * 
 * @cname("__Pyx_carray_to_py_float___5b_4_5d_")
 * cdef inline list __Pyx_carray_to_py_float___5b_4_5d_(base_type *v, Py_ssize_t length):             # <<<<<<<<<<<<<<
 *     cdef size_t i
 *     cdef object value
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("carray.to_py.__Pyx_carray_to_py_float___5b_4_5d_", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_value);
  __Pyx_XDECREF(__pyx_v_l);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "carray.to_py":124
 * 
 * @cname("__Pyx_carray_to_tuple_float___5b_4_5d_")
 * cdef inline tuple __Pyx_carray_to_tuple_float___5b_4_5d_(base_type *v, Py_ssize_t length):             # <<<<<<<<<<<<<<
 *     cdef size_t i
 *     cdef object value
 */

static CYTHON_INLINE PyObject *__Pyx_carray_to_tuple_float___5b_4_5d_(float (*__pyx_v_v)[4], Py_ssize_t __pyx_v_length) {
  size_t __pyx_v_i;
  PyObject *__pyx_v_value = 0;
  PyObject *__pyx_v_t = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  size_t __pyx_t_2;
  size_t __pyx_t_3;
  __Pyx_RefNannySetupContext("__Pyx_carray_to_tuple_float___5b_4_5d_", 0);

  /* "carray.to_py":127
 *     cdef size_t i
 *     cdef object value
 *     t = PyTuple_New(length)             # <<<<<<<<<<<<<<
 *     for i in range(<size_t>length):
 *         value = v[i]
 */
  __pyx_t_1 = PyTuple_New(__pyx_v_length); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 127, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_t = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "carray.to_py":128
 *     cdef object value
 *     t = PyTuple_New(length)
 *     for i in range(<size_t>length):             # <<<<<<<<<<<<<<
 *         value = v[i]
 *         Py_INCREF(value)
 */
  __pyx_t_2 = ((size_t)__pyx_v_length);
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "carray.to_py":129
 *     t = PyTuple_New(length)
 *     for i in range(<size_t>length):
 *         value = v[i]             # <<<<<<<<<<<<<<
 *         Py_INCREF(value)
 *         PyTuple_SET_ITEM(t, i, value)
 */
    __pyx_t_1 = __Pyx_carray_to_py_float((__pyx_v_v[__pyx_v_i]), 4); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 129, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_value, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "carray.to_py":130
 *     for i in range(<size_t>length):
 *         value = v[i]
 *         Py_INCREF(value)             # <<<<<<<<<<<<<<
 *         PyTuple_SET_ITEM(t, i, value)
 *     return t
 */
    Py_INCREF(__pyx_v_value);

    /* "carray.to_py":131
 *         value = v[i]
 *         Py_INCREF(value)
 *         PyTuple_SET_ITEM(t, i, value)             # <<<<<<<<<<<<<<
 *     return t
 */
    PyTuple_SET_ITEM(__pyx_v_t, __pyx_v_i, __pyx_v_value);
  }

  /* "carray.to_py":132
 *         Py_INCREF(value)
 *         PyTuple_SET_ITEM(t, i, value)
 *     return t             # <<<<<<<<<<<<<<
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_t);
  __pyx_r = __pyx_v_t;
  goto __pyx_L0;

  /* "carray.to_py":124
 * 
 * @cname("__Pyx_carray_to_tuple_float___5b_4_5d_")
 * cdef inline tuple __Pyx_carray_to_tuple_float___5b_4_5d_(base_type *v, Py_ssize_t length):             # <<<<<<<<<<<<<<
 *     cdef size_t i
 *     cdef object value
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("carray.to_py.__Pyx_carray_to_tuple_float___5b_4_5d_", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_value);
  __Pyx_XDECREF(__pyx_v_t);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_tp_new_7psychxr_3ovr_4math_ovrVector2i(PyTypeObject *t, PyObject *a, PyObject *k) {
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  if (unlikely(__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_7psychxr_3ovr_4math_ovrVector2i(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  (*Py_TYPE(o)->tp_free)(o);
}
static PyObject *__pyx_sq_item_7psychxr_3ovr_4math_ovrVector2i(PyObject *o, Py_ssize_t i) {
  PyObject *r;
  PyObject *x = PyInt_FromSsize_t(i); if(!x) return 0;
  r = Py_TYPE(o)->tp_as_mapping->mp_subscript(o, x);
  Py_DECREF(x);
  return r;
}

static int __pyx_mp_ass_subscript_7psychxr_3ovr_4math_ovrVector2i(PyObject *o, PyObject *i, PyObject *v) {
  if (v) {
    return __pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_41__setitem__(o, i, v);
  }
  else {
    PyErr_Format(PyExc_NotImplementedError,
      "Subscript deletion not supported by %.200s", Py_TYPE(o)->tp_name);
    return -1;
  }
}

static PyObject *__pyx_tp_richcompare_7psychxr_3ovr_4math_ovrVector2i(PyObject *o1, PyObject *o2, int op) {
  switch (op) {
    case Py_EQ: {
      return __pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_7__eq__(o1, o2);
    }
    case Py_NE: {
      return __pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_9__ne__(o1, o2);
    }
    default: {
      return __Pyx_NewRef(Py_NotImplemented);
    }
  }
}

static PyObject *__pyx_getprop_7psychxr_3ovr_4math_11ovrVector2i_x(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_1x_1__get__(o);
}

static int __pyx_setprop_7psychxr_3ovr_4math_11ovrVector2i_x(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_1x_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_7psychxr_3ovr_4math_11ovrVector2i_y(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_1y_1__get__(o);
}

static int __pyx_setprop_7psychxr_3ovr_4math_11ovrVector2i_y(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_1y_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_7psychxr_3ovr_4math_ovrVector2i[] = {
  {"asTuple", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_3asTuple, METH_NOARGS, 0},
  {"asList", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_5asList, METH_NOARGS, 0},
  {"min", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_29min, METH_VARARGS|METH_KEYWORDS, 0},
  {"max", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_31max, METH_VARARGS|METH_KEYWORDS, 0},
  {"clamped", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_33clamped, METH_O, 0},
  {"isEqual", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_35isEqual, METH_VARARGS|METH_KEYWORDS, 0},
  {"compare", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_37compare, METH_VARARGS|METH_KEYWORDS, 0},
  {"entrywiseMultiply", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_43entrywiseMultiply, METH_O, 0},
  {"dot", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_45dot, METH_O, 0},
  {"angle", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_47angle, METH_O, 0},
  {"lengthSq", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_49lengthSq, METH_NOARGS, 0},
  {"length", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_51length, METH_NOARGS, 0},
  {"distanceSq", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_53distanceSq, METH_O, 0},
  {"distance", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_55distance, METH_O, 0},
  {"isNormalized", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_57isNormalized, METH_NOARGS, 0},
  {"normalize", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_59normalize, METH_NOARGS, 0},
  {"normalized", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_61normalized, METH_NOARGS, 0},
  {"lerp", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_63lerp, METH_VARARGS|METH_KEYWORDS, 0},
  {"projectTo", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_65projectTo, METH_O, 0},
  {"isClockwise", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_67isClockwise, METH_O, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_69__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_71__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_7psychxr_3ovr_4math_ovrVector2i[] = {
  {(char *)"x", __pyx_getprop_7psychxr_3ovr_4math_11ovrVector2i_x, __pyx_setprop_7psychxr_3ovr_4math_11ovrVector2i_x, (char *)0, 0},
  {(char *)"y", __pyx_getprop_7psychxr_3ovr_4math_11ovrVector2i_y, __pyx_setprop_7psychxr_3ovr_4math_11ovrVector2i_y, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyNumberMethods __pyx_tp_as_number_ovrVector2i = {
  __pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_11__add__, /*nb_add*/
  __pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_15__sub__, /*nb_subtract*/
  __pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_21__mul__, /*nb_multiply*/
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_divide*/
  #endif
  0, /*nb_remainder*/
  0, /*nb_divmod*/
  0, /*nb_power*/
  __pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_19__neg__, /*nb_negative*/
  0, /*nb_positive*/
  0, /*nb_absolute*/
  0, /*nb_nonzero*/
  0, /*nb_invert*/
  0, /*nb_lshift*/
  0, /*nb_rshift*/
  0, /*nb_and*/
  0, /*nb_xor*/
  0, /*nb_or*/
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_coerce*/
  #endif
  0, /*nb_int*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_long*/
  #else
  0, /*reserved*/
  #endif
  0, /*nb_float*/
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_oct*/
  #endif
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_hex*/
  #endif
  __pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_13__iadd__, /*nb_inplace_add*/
  __pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_17__isub__, /*nb_inplace_subtract*/
  __pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_23__imul__, /*nb_inplace_multiply*/
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_inplace_divide*/
  #endif
  0, /*nb_inplace_remainder*/
  0, /*nb_inplace_power*/
  0, /*nb_inplace_lshift*/
  0, /*nb_inplace_rshift*/
  0, /*nb_inplace_and*/
  0, /*nb_inplace_xor*/
  0, /*nb_inplace_or*/
  0, /*nb_floor_divide*/
  __pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_25__truediv__, /*nb_true_divide*/
  0, /*nb_inplace_floor_divide*/
  __pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_27__itruediv__, /*nb_inplace_true_divide*/
  0, /*nb_index*/
  #if PY_VERSION_HEX >= 0x03050000
  0, /*nb_matrix_multiply*/
  #endif
  #if PY_VERSION_HEX >= 0x03050000
  0, /*nb_inplace_matrix_multiply*/
  #endif
};

static PySequenceMethods __pyx_tp_as_sequence_ovrVector2i = {
  0, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  __pyx_sq_item_7psychxr_3ovr_4math_ovrVector2i, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_ovrVector2i = {
  0, /*mp_length*/
  __pyx_pw_7psychxr_3ovr_4math_11ovrVector2i_39__getitem__, /*mp_subscript*/
  __pyx_mp_ass_subscript_7psychxr_3ovr_4math_ovrVector2i, /*mp_ass_subscript*/
};

static PyTypeObject __pyx_type_7psychxr_3ovr_4math_ovrVector2i = {
  PyVarObject_HEAD_INIT(0, 0)
  "psychxr.ovr.math.ovrVector2i", /*tp_name*/
  sizeof(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector2i), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7psychxr_3ovr_4math_ovrVector2i, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  &__pyx_tp_as_number_ovrVector2i, /*tp_as_number*/
  &__pyx_tp_as_sequence_ovrVector2i, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_ovrVector2i, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  __pyx_tp_richcompare_7psychxr_3ovr_4math_ovrVector2i, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_7psychxr_3ovr_4math_ovrVector2i, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_7psychxr_3ovr_4math_ovrVector2i, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7psychxr_3ovr_4math_ovrVector2i, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyObject *__pyx_tp_new_7psychxr_3ovr_4math_ovrSizei(PyTypeObject *t, PyObject *a, PyObject *k) {
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  if (unlikely(__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_7psychxr_3ovr_4math_ovrSizei(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  (*Py_TYPE(o)->tp_free)(o);
}

static PyObject *__pyx_tp_richcompare_7psychxr_3ovr_4math_ovrSizei(PyObject *o1, PyObject *o2, int op) {
  switch (op) {
    case Py_EQ: {
      return __pyx_pw_7psychxr_3ovr_4math_8ovrSizei_7__eq__(o1, o2);
    }
    case Py_NE: {
      return __pyx_pw_7psychxr_3ovr_4math_8ovrSizei_9__ne__(o1, o2);
    }
    default: {
      return __Pyx_NewRef(Py_NotImplemented);
    }
  }
}

static PyObject *__pyx_getprop_7psychxr_3ovr_4math_8ovrSizei_w(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7psychxr_3ovr_4math_8ovrSizei_1w_1__get__(o);
}

static int __pyx_setprop_7psychxr_3ovr_4math_8ovrSizei_w(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7psychxr_3ovr_4math_8ovrSizei_1w_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_7psychxr_3ovr_4math_8ovrSizei_h(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7psychxr_3ovr_4math_8ovrSizei_1h_1__get__(o);
}

static int __pyx_setprop_7psychxr_3ovr_4math_8ovrSizei_h(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7psychxr_3ovr_4math_8ovrSizei_1h_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_7psychxr_3ovr_4math_ovrSizei[] = {
  {"asTuple", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_3asTuple, METH_NOARGS, 0},
  {"asList", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_5asList, METH_NOARGS, 0},
  {"min", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_29min, METH_VARARGS|METH_KEYWORDS, 0},
  {"max", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_31max, METH_VARARGS|METH_KEYWORDS, 0},
  {"area", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_33area, METH_NOARGS, 0},
  {"toVector", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_35toVector, METH_NOARGS, 0},
  {"asVector", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_37asVector, METH_NOARGS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_39__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrSizei_41__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_7psychxr_3ovr_4math_ovrSizei[] = {
  {(char *)"w", __pyx_getprop_7psychxr_3ovr_4math_8ovrSizei_w, __pyx_setprop_7psychxr_3ovr_4math_8ovrSizei_w, (char *)0, 0},
  {(char *)"h", __pyx_getprop_7psychxr_3ovr_4math_8ovrSizei_h, __pyx_setprop_7psychxr_3ovr_4math_8ovrSizei_h, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyNumberMethods __pyx_tp_as_number_ovrSizei = {
  __pyx_pw_7psychxr_3ovr_4math_8ovrSizei_11__add__, /*nb_add*/
  __pyx_pw_7psychxr_3ovr_4math_8ovrSizei_15__sub__, /*nb_subtract*/
  __pyx_pw_7psychxr_3ovr_4math_8ovrSizei_21__mul__, /*nb_multiply*/
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_divide*/
  #endif
  0, /*nb_remainder*/
  0, /*nb_divmod*/
  0, /*nb_power*/
  __pyx_pw_7psychxr_3ovr_4math_8ovrSizei_19__neg__, /*nb_negative*/
  0, /*nb_positive*/
  0, /*nb_absolute*/
  0, /*nb_nonzero*/
  0, /*nb_invert*/
  0, /*nb_lshift*/
  0, /*nb_rshift*/
  0, /*nb_and*/
  0, /*nb_xor*/
  0, /*nb_or*/
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_coerce*/
  #endif
  0, /*nb_int*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_long*/
  #else
  0, /*reserved*/
  #endif
  0, /*nb_float*/
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_oct*/
  #endif
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_hex*/
  #endif
  __pyx_pw_7psychxr_3ovr_4math_8ovrSizei_13__iadd__, /*nb_inplace_add*/
  __pyx_pw_7psychxr_3ovr_4math_8ovrSizei_17__isub__, /*nb_inplace_subtract*/
  __pyx_pw_7psychxr_3ovr_4math_8ovrSizei_23__imul__, /*nb_inplace_multiply*/
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_inplace_divide*/
  #endif
  0, /*nb_inplace_remainder*/
  0, /*nb_inplace_power*/
  0, /*nb_inplace_lshift*/
  0, /*nb_inplace_rshift*/
  0, /*nb_inplace_and*/
  0, /*nb_inplace_xor*/
  0, /*nb_inplace_or*/
  0, /*nb_floor_divide*/
  __pyx_pw_7psychxr_3ovr_4math_8ovrSizei_25__truediv__, /*nb_true_divide*/
  0, /*nb_inplace_floor_divide*/
  __pyx_pw_7psychxr_3ovr_4math_8ovrSizei_27__itruediv__, /*nb_inplace_true_divide*/
  0, /*nb_index*/
  #if PY_VERSION_HEX >= 0x03050000
  0, /*nb_matrix_multiply*/
  #endif
  #if PY_VERSION_HEX >= 0x03050000
  0, /*nb_inplace_matrix_multiply*/
  #endif
};

static PyTypeObject __pyx_type_7psychxr_3ovr_4math_ovrSizei = {
  PyVarObject_HEAD_INIT(0, 0)
  "psychxr.ovr.math.ovrSizei", /*tp_name*/
  sizeof(struct __pyx_obj_7psychxr_3ovr_4math_ovrSizei), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7psychxr_3ovr_4math_ovrSizei, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  &__pyx_tp_as_number_ovrSizei, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  __pyx_tp_richcompare_7psychxr_3ovr_4math_ovrSizei, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_7psychxr_3ovr_4math_ovrSizei, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_7psychxr_3ovr_4math_ovrSizei, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7psychxr_3ovr_4math_ovrSizei, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyObject *__pyx_tp_new_7psychxr_3ovr_4math_ovrRecti(PyTypeObject *t, PyObject *a, PyObject *k) {
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  if (unlikely(__pyx_pw_7psychxr_3ovr_4math_8ovrRecti_3__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_7psychxr_3ovr_4math_ovrRecti(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  (*Py_TYPE(o)->tp_free)(o);
}

static PyObject *__pyx_tp_richcompare_7psychxr_3ovr_4math_ovrRecti(PyObject *o1, PyObject *o2, int op) {
  switch (op) {
    case Py_EQ: {
      return __pyx_pw_7psychxr_3ovr_4math_8ovrRecti_19__eq__(o1, o2);
    }
    case Py_NE: {
      return __pyx_pw_7psychxr_3ovr_4math_8ovrRecti_21__ne__(o1, o2);
    }
    default: {
      return __Pyx_NewRef(Py_NotImplemented);
    }
  }
}

static PyObject *__pyx_getprop_7psychxr_3ovr_4math_8ovrRecti_x(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7psychxr_3ovr_4math_8ovrRecti_1x_1__get__(o);
}

static int __pyx_setprop_7psychxr_3ovr_4math_8ovrRecti_x(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7psychxr_3ovr_4math_8ovrRecti_1x_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_7psychxr_3ovr_4math_8ovrRecti_y(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7psychxr_3ovr_4math_8ovrRecti_1y_1__get__(o);
}

static int __pyx_setprop_7psychxr_3ovr_4math_8ovrRecti_y(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7psychxr_3ovr_4math_8ovrRecti_1y_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_7psychxr_3ovr_4math_8ovrRecti_w(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7psychxr_3ovr_4math_8ovrRecti_1w_1__get__(o);
}

static int __pyx_setprop_7psychxr_3ovr_4math_8ovrRecti_w(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7psychxr_3ovr_4math_8ovrRecti_1w_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_7psychxr_3ovr_4math_8ovrRecti_h(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7psychxr_3ovr_4math_8ovrRecti_1h_1__get__(o);
}

static int __pyx_setprop_7psychxr_3ovr_4math_8ovrRecti_h(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7psychxr_3ovr_4math_8ovrRecti_1h_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_7psychxr_3ovr_4math_ovrRecti[] = {
  {"asTuple", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrRecti_5asTuple, METH_NOARGS, 0},
  {"asList", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrRecti_7asList, METH_NOARGS, 0},
  {"getPos", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrRecti_11getPos, METH_NOARGS, 0},
  {"getSize", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrRecti_13getSize, METH_NOARGS, 0},
  {"setPos", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrRecti_15setPos, METH_O, 0},
  {"setSize", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrRecti_17setSize, METH_O, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrRecti_23__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrRecti_25__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_7psychxr_3ovr_4math_ovrRecti[] = {
  {(char *)"x", __pyx_getprop_7psychxr_3ovr_4math_8ovrRecti_x, __pyx_setprop_7psychxr_3ovr_4math_8ovrRecti_x, (char *)0, 0},
  {(char *)"y", __pyx_getprop_7psychxr_3ovr_4math_8ovrRecti_y, __pyx_setprop_7psychxr_3ovr_4math_8ovrRecti_y, (char *)0, 0},
  {(char *)"w", __pyx_getprop_7psychxr_3ovr_4math_8ovrRecti_w, __pyx_setprop_7psychxr_3ovr_4math_8ovrRecti_w, (char *)0, 0},
  {(char *)"h", __pyx_getprop_7psychxr_3ovr_4math_8ovrRecti_h, __pyx_setprop_7psychxr_3ovr_4math_8ovrRecti_h, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PySequenceMethods __pyx_tp_as_sequence_ovrRecti = {
  __pyx_pw_7psychxr_3ovr_4math_8ovrRecti_9__len__, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  0, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_ovrRecti = {
  __pyx_pw_7psychxr_3ovr_4math_8ovrRecti_9__len__, /*mp_length*/
  0, /*mp_subscript*/
  0, /*mp_ass_subscript*/
};

static PyTypeObject __pyx_type_7psychxr_3ovr_4math_ovrRecti = {
  PyVarObject_HEAD_INIT(0, 0)
  "psychxr.ovr.math.ovrRecti", /*tp_name*/
  sizeof(struct __pyx_obj_7psychxr_3ovr_4math_ovrRecti), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7psychxr_3ovr_4math_ovrRecti, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  &__pyx_tp_as_sequence_ovrRecti, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_ovrRecti, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  __pyx_tp_richcompare_7psychxr_3ovr_4math_ovrRecti, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_7psychxr_3ovr_4math_ovrRecti, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_7psychxr_3ovr_4math_ovrRecti, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_7psychxr_3ovr_4math_8ovrRecti_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7psychxr_3ovr_4math_ovrRecti, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyObject *__pyx_tp_new_7psychxr_3ovr_4math_ovrVector3f(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)o);
  new((void*)&(p->c_Vector3f)) OVR::Vector3f();
  if (unlikely(__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_3__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_7psychxr_3ovr_4math_ovrVector3f(PyObject *o) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *p = (struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  __Pyx_call_destructor(p->c_Vector3f);
  (*Py_TYPE(o)->tp_free)(o);
}
static PyObject *__pyx_sq_item_7psychxr_3ovr_4math_ovrVector3f(PyObject *o, Py_ssize_t i) {
  PyObject *r;
  PyObject *x = PyInt_FromSsize_t(i); if(!x) return 0;
  r = Py_TYPE(o)->tp_as_mapping->mp_subscript(o, x);
  Py_DECREF(x);
  return r;
}

static int __pyx_mp_ass_subscript_7psychxr_3ovr_4math_ovrVector3f(PyObject *o, PyObject *i, PyObject *v) {
  if (v) {
    return __pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_45__setitem__(o, i, v);
  }
  else {
    PyErr_Format(PyExc_NotImplementedError,
      "Subscript deletion not supported by %.200s", Py_TYPE(o)->tp_name);
    return -1;
  }
}

static PyObject *__pyx_tp_richcompare_7psychxr_3ovr_4math_ovrVector3f(PyObject *o1, PyObject *o2, int op) {
  switch (op) {
    case Py_EQ: {
      return __pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_11__eq__(o1, o2);
    }
    case Py_NE: {
      return __pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_13__ne__(o1, o2);
    }
    default: {
      return __Pyx_NewRef(Py_NotImplemented);
    }
  }
}

static PyObject *__pyx_getprop_7psychxr_3ovr_4math_11ovrVector3f_x(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_1x_1__get__(o);
}

static int __pyx_setprop_7psychxr_3ovr_4math_11ovrVector3f_x(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_1x_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_7psychxr_3ovr_4math_11ovrVector3f_y(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_1y_1__get__(o);
}

static int __pyx_setprop_7psychxr_3ovr_4math_11ovrVector3f_y(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_1y_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_7psychxr_3ovr_4math_11ovrVector3f_z(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_1z_1__get__(o);
}

static int __pyx_setprop_7psychxr_3ovr_4math_11ovrVector3f_z(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_1z_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_7psychxr_3ovr_4math_11ovrVector3f_ctypes(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_6ctypes_1__get__(o);
}

static PyMethodDef __pyx_methods_7psychxr_3ovr_4math_ovrVector3f[] = {
  {"as_tuple", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_5as_tuple, METH_NOARGS, 0},
  {"zero", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_9zero, METH_VARARGS|METH_KEYWORDS, 0},
  {"min", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_33min, METH_VARARGS|METH_KEYWORDS, 0},
  {"max", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_35max, METH_VARARGS|METH_KEYWORDS, 0},
  {"clamped", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_37clamped, METH_O, 0},
  {"isEqual", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_39isEqual, METH_VARARGS|METH_KEYWORDS, 0},
  {"compare", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_41compare, METH_VARARGS|METH_KEYWORDS, 0},
  {"entrywiseMultiply", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_47entrywiseMultiply, METH_O, 0},
  {"dot", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_49dot, METH_O, 0},
  {"cross", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_51cross, METH_O, 0},
  {"angle", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_53angle, METH_O, 0},
  {"lengthSq", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_55lengthSq, METH_NOARGS, 0},
  {"length", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_57length, METH_NOARGS, 0},
  {"distanceSq", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_59distanceSq, METH_O, 0},
  {"distance", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_61distance, METH_O, 0},
  {"isNormalized", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_63isNormalized, METH_NOARGS, 0},
  {"normalize", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_65normalize, METH_NOARGS, 0},
  {"normalized", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_67normalized, METH_NOARGS, 0},
  {"lerp", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_69lerp, METH_VARARGS|METH_KEYWORDS, 0},
  {"projectTo", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_71projectTo, METH_O, 0},
  {"projectToPlane", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_73projectToPlane, METH_O, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_75__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_77__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_7psychxr_3ovr_4math_ovrVector3f[] = {
  {(char *)"x", __pyx_getprop_7psychxr_3ovr_4math_11ovrVector3f_x, __pyx_setprop_7psychxr_3ovr_4math_11ovrVector3f_x, (char *)0, 0},
  {(char *)"y", __pyx_getprop_7psychxr_3ovr_4math_11ovrVector3f_y, __pyx_setprop_7psychxr_3ovr_4math_11ovrVector3f_y, (char *)0, 0},
  {(char *)"z", __pyx_getprop_7psychxr_3ovr_4math_11ovrVector3f_z, __pyx_setprop_7psychxr_3ovr_4math_11ovrVector3f_z, (char *)0, 0},
  {(char *)"ctypes", __pyx_getprop_7psychxr_3ovr_4math_11ovrVector3f_ctypes, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyNumberMethods __pyx_tp_as_number_ovrVector3f = {
  __pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_15__add__, /*nb_add*/
  __pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_19__sub__, /*nb_subtract*/
  __pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_25__mul__, /*nb_multiply*/
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_divide*/
  #endif
  0, /*nb_remainder*/
  0, /*nb_divmod*/
  0, /*nb_power*/
  __pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_23__neg__, /*nb_negative*/
  0, /*nb_positive*/
  0, /*nb_absolute*/
  0, /*nb_nonzero*/
  0, /*nb_invert*/
  0, /*nb_lshift*/
  0, /*nb_rshift*/
  0, /*nb_and*/
  0, /*nb_xor*/
  0, /*nb_or*/
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_coerce*/
  #endif
  0, /*nb_int*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_long*/
  #else
  0, /*reserved*/
  #endif
  0, /*nb_float*/
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_oct*/
  #endif
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_hex*/
  #endif
  __pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_17__iadd__, /*nb_inplace_add*/
  __pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_21__isub__, /*nb_inplace_subtract*/
  __pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_27__imul__, /*nb_inplace_multiply*/
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_inplace_divide*/
  #endif
  0, /*nb_inplace_remainder*/
  0, /*nb_inplace_power*/
  0, /*nb_inplace_lshift*/
  0, /*nb_inplace_rshift*/
  0, /*nb_inplace_and*/
  0, /*nb_inplace_xor*/
  0, /*nb_inplace_or*/
  0, /*nb_floor_divide*/
  __pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_29__truediv__, /*nb_true_divide*/
  0, /*nb_inplace_floor_divide*/
  __pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_31__itruediv__, /*nb_inplace_true_divide*/
  0, /*nb_index*/
  #if PY_VERSION_HEX >= 0x03050000
  0, /*nb_matrix_multiply*/
  #endif
  #if PY_VERSION_HEX >= 0x03050000
  0, /*nb_inplace_matrix_multiply*/
  #endif
};

static PySequenceMethods __pyx_tp_as_sequence_ovrVector3f = {
  __pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_7__len__, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  __pyx_sq_item_7psychxr_3ovr_4math_ovrVector3f, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_ovrVector3f = {
  __pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_7__len__, /*mp_length*/
  __pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_43__getitem__, /*mp_subscript*/
  __pyx_mp_ass_subscript_7psychxr_3ovr_4math_ovrVector3f, /*mp_ass_subscript*/
};

static PyTypeObject __pyx_type_7psychxr_3ovr_4math_ovrVector3f = {
  PyVarObject_HEAD_INIT(0, 0)
  "psychxr.ovr.math.ovrVector3f", /*tp_name*/
  sizeof(struct __pyx_obj_7psychxr_3ovr_4math_ovrVector3f), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7psychxr_3ovr_4math_ovrVector3f, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  &__pyx_tp_as_number_ovrVector3f, /*tp_as_number*/
  &__pyx_tp_as_sequence_ovrVector3f, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_ovrVector3f, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  __pyx_tp_richcompare_7psychxr_3ovr_4math_ovrVector3f, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_7psychxr_3ovr_4math_ovrVector3f, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_7psychxr_3ovr_4math_ovrVector3f, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_7psychxr_3ovr_4math_11ovrVector3f_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7psychxr_3ovr_4math_ovrVector3f, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyObject *__pyx_tp_new_7psychxr_3ovr_4math_ovrFovPort(PyTypeObject *t, PyObject *a, PyObject *k) {
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  if (unlikely(__pyx_pw_7psychxr_3ovr_4math_10ovrFovPort_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_7psychxr_3ovr_4math_ovrFovPort(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  (*Py_TYPE(o)->tp_free)(o);
}

static PyObject *__pyx_getprop_7psychxr_3ovr_4math_10ovrFovPort_UpTan(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7psychxr_3ovr_4math_10ovrFovPort_5UpTan_1__get__(o);
}

static int __pyx_setprop_7psychxr_3ovr_4math_10ovrFovPort_UpTan(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7psychxr_3ovr_4math_10ovrFovPort_5UpTan_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_7psychxr_3ovr_4math_10ovrFovPort_DownTan(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7psychxr_3ovr_4math_10ovrFovPort_7DownTan_1__get__(o);
}

static int __pyx_setprop_7psychxr_3ovr_4math_10ovrFovPort_DownTan(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7psychxr_3ovr_4math_10ovrFovPort_7DownTan_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_7psychxr_3ovr_4math_10ovrFovPort_LeftTan(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7psychxr_3ovr_4math_10ovrFovPort_7LeftTan_1__get__(o);
}

static int __pyx_setprop_7psychxr_3ovr_4math_10ovrFovPort_LeftTan(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7psychxr_3ovr_4math_10ovrFovPort_7LeftTan_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_7psychxr_3ovr_4math_10ovrFovPort_RightTan(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7psychxr_3ovr_4math_10ovrFovPort_8RightTan_1__get__(o);
}

static int __pyx_setprop_7psychxr_3ovr_4math_10ovrFovPort_RightTan(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7psychxr_3ovr_4math_10ovrFovPort_8RightTan_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_7psychxr_3ovr_4math_ovrFovPort[] = {
  {"max", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_10ovrFovPort_3max, METH_VARARGS|METH_KEYWORDS, 0},
  {"min", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_10ovrFovPort_5min, METH_VARARGS|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_10ovrFovPort_7__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_10ovrFovPort_9__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_7psychxr_3ovr_4math_ovrFovPort[] = {
  {(char *)"UpTan", __pyx_getprop_7psychxr_3ovr_4math_10ovrFovPort_UpTan, __pyx_setprop_7psychxr_3ovr_4math_10ovrFovPort_UpTan, (char *)0, 0},
  {(char *)"DownTan", __pyx_getprop_7psychxr_3ovr_4math_10ovrFovPort_DownTan, __pyx_setprop_7psychxr_3ovr_4math_10ovrFovPort_DownTan, (char *)0, 0},
  {(char *)"LeftTan", __pyx_getprop_7psychxr_3ovr_4math_10ovrFovPort_LeftTan, __pyx_setprop_7psychxr_3ovr_4math_10ovrFovPort_LeftTan, (char *)0, 0},
  {(char *)"RightTan", __pyx_getprop_7psychxr_3ovr_4math_10ovrFovPort_RightTan, __pyx_setprop_7psychxr_3ovr_4math_10ovrFovPort_RightTan, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_7psychxr_3ovr_4math_ovrFovPort = {
  PyVarObject_HEAD_INIT(0, 0)
  "psychxr.ovr.math.ovrFovPort", /*tp_name*/
  sizeof(struct __pyx_obj_7psychxr_3ovr_4math_ovrFovPort), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7psychxr_3ovr_4math_ovrFovPort, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_7psychxr_3ovr_4math_ovrFovPort, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_7psychxr_3ovr_4math_ovrFovPort, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7psychxr_3ovr_4math_ovrFovPort, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyObject *__pyx_tp_new_7psychxr_3ovr_4math_ovrQuatf(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)o);
  new((void*)&(p->c_Quatf)) OVR::Quatf();
  if (unlikely(__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_7psychxr_3ovr_4math_ovrQuatf(PyObject *o) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *p = (struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  __Pyx_call_destructor(p->c_Quatf);
  (*Py_TYPE(o)->tp_free)(o);
}

static PyObject *__pyx_tp_richcompare_7psychxr_3ovr_4math_ovrQuatf(PyObject *o1, PyObject *o2, int op) {
  switch (op) {
    case Py_EQ: {
      return __pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_13__eq__(o1, o2);
    }
    case Py_NE: {
      return __pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_15__ne__(o1, o2);
    }
    default: {
      return __Pyx_NewRef(Py_NotImplemented);
    }
  }
}

static PyObject *__pyx_getprop_7psychxr_3ovr_4math_8ovrQuatf_x(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_1x_1__get__(o);
}

static int __pyx_setprop_7psychxr_3ovr_4math_8ovrQuatf_x(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_1x_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_7psychxr_3ovr_4math_8ovrQuatf_y(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_1y_1__get__(o);
}

static int __pyx_setprop_7psychxr_3ovr_4math_8ovrQuatf_y(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_1y_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_7psychxr_3ovr_4math_8ovrQuatf_z(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_1z_1__get__(o);
}

static int __pyx_setprop_7psychxr_3ovr_4math_8ovrQuatf_z(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_1z_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_7psychxr_3ovr_4math_8ovrQuatf_w(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_1w_1__get__(o);
}

static int __pyx_setprop_7psychxr_3ovr_4math_8ovrQuatf_w(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_1w_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_7psychxr_3ovr_4math_ovrQuatf[] = {
  {"asTuple", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_3asTuple, METH_NOARGS, 0},
  {"asList", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_5asList, METH_NOARGS, 0},
  {"identity", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_11identity, METH_VARARGS|METH_KEYWORDS, 0},
  {"isEqual", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_33isEqual, METH_VARARGS|METH_KEYWORDS, 0},
  {"abs", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_35abs, METH_O, 0},
  {"imag", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_37imag, METH_NOARGS, 0},
  {"lengthSq", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_39lengthSq, METH_NOARGS, 0},
  {"length", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_41length, METH_NOARGS, 0},
  {"distance", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_43distance, METH_O, 0},
  {"distanceSq", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_45distanceSq, METH_O, 0},
  {"dot", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_47dot, METH_O, 0},
  {"angle", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_49angle, METH_O, 0},
  {"isNormalized", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_51isNormalized, METH_NOARGS, 0},
  {"normalize", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_53normalize, METH_NOARGS, 0},
  {"normalized", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_55normalized, METH_NOARGS, 0},
  {"conj", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_57conj, METH_NOARGS, 0},
  {"align", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_59align, METH_VARARGS|METH_KEYWORDS, 0},
  {"rotate", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_61rotate, METH_O, 0},
  {"inverseRotate", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_63inverseRotate, METH_O, 0},
  {"inverted", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_65inverted, METH_NOARGS, 0},
  {"inverse", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_67inverse, METH_NOARGS, 0},
  {"invert", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_69invert, METH_NOARGS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_73__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_75__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_7psychxr_3ovr_4math_ovrQuatf[] = {
  {(char *)"x", __pyx_getprop_7psychxr_3ovr_4math_8ovrQuatf_x, __pyx_setprop_7psychxr_3ovr_4math_8ovrQuatf_x, (char *)0, 0},
  {(char *)"y", __pyx_getprop_7psychxr_3ovr_4math_8ovrQuatf_y, __pyx_setprop_7psychxr_3ovr_4math_8ovrQuatf_y, (char *)0, 0},
  {(char *)"z", __pyx_getprop_7psychxr_3ovr_4math_8ovrQuatf_z, __pyx_setprop_7psychxr_3ovr_4math_8ovrQuatf_z, (char *)0, 0},
  {(char *)"w", __pyx_getprop_7psychxr_3ovr_4math_8ovrQuatf_w, __pyx_setprop_7psychxr_3ovr_4math_8ovrQuatf_w, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyNumberMethods __pyx_tp_as_number_ovrQuatf = {
  __pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_17__add__, /*nb_add*/
  __pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_21__sub__, /*nb_subtract*/
  __pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_25__mul__, /*nb_multiply*/
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_divide*/
  #endif
  0, /*nb_remainder*/
  0, /*nb_divmod*/
  0, /*nb_power*/
  __pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_9__neg__, /*nb_negative*/
  0, /*nb_positive*/
  0, /*nb_absolute*/
  0, /*nb_nonzero*/
  __pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_71__invert__, /*nb_invert*/
  0, /*nb_lshift*/
  0, /*nb_rshift*/
  0, /*nb_and*/
  0, /*nb_xor*/
  0, /*nb_or*/
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_coerce*/
  #endif
  0, /*nb_int*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_long*/
  #else
  0, /*reserved*/
  #endif
  0, /*nb_float*/
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_oct*/
  #endif
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_hex*/
  #endif
  __pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_19__iadd__, /*nb_inplace_add*/
  __pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_23__isub__, /*nb_inplace_subtract*/
  __pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_27__imul__, /*nb_inplace_multiply*/
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_inplace_divide*/
  #endif
  0, /*nb_inplace_remainder*/
  0, /*nb_inplace_power*/
  0, /*nb_inplace_lshift*/
  0, /*nb_inplace_rshift*/
  0, /*nb_inplace_and*/
  0, /*nb_inplace_xor*/
  0, /*nb_inplace_or*/
  0, /*nb_floor_divide*/
  __pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_29__truediv__, /*nb_true_divide*/
  0, /*nb_inplace_floor_divide*/
  __pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_31__itruediv__, /*nb_inplace_true_divide*/
  0, /*nb_index*/
  #if PY_VERSION_HEX >= 0x03050000
  0, /*nb_matrix_multiply*/
  #endif
  #if PY_VERSION_HEX >= 0x03050000
  0, /*nb_inplace_matrix_multiply*/
  #endif
};

static PySequenceMethods __pyx_tp_as_sequence_ovrQuatf = {
  __pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_7__len__, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  0, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_ovrQuatf = {
  __pyx_pw_7psychxr_3ovr_4math_8ovrQuatf_7__len__, /*mp_length*/
  0, /*mp_subscript*/
  0, /*mp_ass_subscript*/
};

static PyTypeObject __pyx_type_7psychxr_3ovr_4math_ovrQuatf = {
  PyVarObject_HEAD_INIT(0, 0)
  "psychxr.ovr.math.ovrQuatf", /*tp_name*/
  sizeof(struct __pyx_obj_7psychxr_3ovr_4math_ovrQuatf), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7psychxr_3ovr_4math_ovrQuatf, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  &__pyx_tp_as_number_ovrQuatf, /*tp_as_number*/
  &__pyx_tp_as_sequence_ovrQuatf, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_ovrQuatf, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  __pyx_tp_richcompare_7psychxr_3ovr_4math_ovrQuatf, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_7psychxr_3ovr_4math_ovrQuatf, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_7psychxr_3ovr_4math_ovrQuatf, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7psychxr_3ovr_4math_ovrQuatf, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyObject *__pyx_tp_new_7psychxr_3ovr_4math_ovrPosef(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *)o);
  new((void*)&(p->c_Posef)) OVR::Posef();
  if (unlikely(__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_7psychxr_3ovr_4math_ovrPosef(PyObject *o) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *p = (struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  __Pyx_call_destructor(p->c_Posef);
  (*Py_TYPE(o)->tp_free)(o);
}

static PyObject *__pyx_getprop_7psychxr_3ovr_4math_8ovrPosef_rotation(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7psychxr_3ovr_4math_8ovrPosef_8rotation_1__get__(o);
}

static int __pyx_setprop_7psychxr_3ovr_4math_8ovrPosef_rotation(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7psychxr_3ovr_4math_8ovrPosef_8rotation_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_7psychxr_3ovr_4math_8ovrPosef_translation(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7psychxr_3ovr_4math_8ovrPosef_11translation_1__get__(o);
}

static int __pyx_setprop_7psychxr_3ovr_4math_8ovrPosef_translation(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7psychxr_3ovr_4math_8ovrPosef_11translation_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_7psychxr_3ovr_4math_ovrPosef[] = {
  {"rotate", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_3rotate, METH_O, 0},
  {"inverseRotate", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_5inverseRotate, METH_O, 0},
  {"translate", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_7translate, METH_O, 0},
  {"transform", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_9transform, METH_O, 0},
  {"inverseTransform", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_11inverseTransform, METH_O, 0},
  {"transformNormal", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_13transformNormal, METH_O, 0},
  {"inverseTransformNormal", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_15inverseTransformNormal, METH_O, 0},
  {"apply", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_17apply, METH_O, 0},
  {"inverted", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_21inverted, METH_NOARGS, 0},
  {"normalized", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_23normalized, METH_NOARGS, 0},
  {"normalize", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_25normalize, METH_NOARGS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_27__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_8ovrPosef_29__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_7psychxr_3ovr_4math_ovrPosef[] = {
  {(char *)"rotation", __pyx_getprop_7psychxr_3ovr_4math_8ovrPosef_rotation, __pyx_setprop_7psychxr_3ovr_4math_8ovrPosef_rotation, (char *)0, 0},
  {(char *)"translation", __pyx_getprop_7psychxr_3ovr_4math_8ovrPosef_translation, __pyx_setprop_7psychxr_3ovr_4math_8ovrPosef_translation, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyNumberMethods __pyx_tp_as_number_ovrPosef = {
  0, /*nb_add*/
  0, /*nb_subtract*/
  __pyx_pw_7psychxr_3ovr_4math_8ovrPosef_19__mul__, /*nb_multiply*/
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_divide*/
  #endif
  0, /*nb_remainder*/
  0, /*nb_divmod*/
  0, /*nb_power*/
  0, /*nb_negative*/
  0, /*nb_positive*/
  0, /*nb_absolute*/
  0, /*nb_nonzero*/
  0, /*nb_invert*/
  0, /*nb_lshift*/
  0, /*nb_rshift*/
  0, /*nb_and*/
  0, /*nb_xor*/
  0, /*nb_or*/
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_coerce*/
  #endif
  0, /*nb_int*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_long*/
  #else
  0, /*reserved*/
  #endif
  0, /*nb_float*/
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_oct*/
  #endif
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_hex*/
  #endif
  0, /*nb_inplace_add*/
  0, /*nb_inplace_subtract*/
  0, /*nb_inplace_multiply*/
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_inplace_divide*/
  #endif
  0, /*nb_inplace_remainder*/
  0, /*nb_inplace_power*/
  0, /*nb_inplace_lshift*/
  0, /*nb_inplace_rshift*/
  0, /*nb_inplace_and*/
  0, /*nb_inplace_xor*/
  0, /*nb_inplace_or*/
  0, /*nb_floor_divide*/
  0, /*nb_true_divide*/
  0, /*nb_inplace_floor_divide*/
  0, /*nb_inplace_true_divide*/
  0, /*nb_index*/
  #if PY_VERSION_HEX >= 0x03050000
  0, /*nb_matrix_multiply*/
  #endif
  #if PY_VERSION_HEX >= 0x03050000
  0, /*nb_inplace_matrix_multiply*/
  #endif
};

static PyTypeObject __pyx_type_7psychxr_3ovr_4math_ovrPosef = {
  PyVarObject_HEAD_INIT(0, 0)
  "psychxr.ovr.math.ovrPosef", /*tp_name*/
  sizeof(struct __pyx_obj_7psychxr_3ovr_4math_ovrPosef), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7psychxr_3ovr_4math_ovrPosef, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  &__pyx_tp_as_number_ovrPosef, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  "ovrPosef\n\n    Class to represent a pose using a vector and quaternion.\n\n    ", /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_7psychxr_3ovr_4math_ovrPosef, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_7psychxr_3ovr_4math_ovrPosef, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7psychxr_3ovr_4math_ovrPosef, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyObject *__pyx_tp_new_7psychxr_3ovr_4math_ovrMatrix4f(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)o);
  new((void*)&(p->c_Matrix4f)) OVR::Matrix4f();
  if (unlikely(__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_7psychxr_3ovr_4math_ovrMatrix4f(PyObject *o) {
  struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *p = (struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  __Pyx_call_destructor(p->c_Matrix4f);
  (*Py_TYPE(o)->tp_free)(o);
}

static PyObject *__pyx_tp_richcompare_7psychxr_3ovr_4math_ovrMatrix4f(PyObject *o1, PyObject *o2, int op) {
  switch (op) {
    case Py_EQ: {
      return __pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_19__eq__(o1, o2);
    }
    case Py_NE: {
      return __pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_21__ne__(o1, o2);
    }
    default: {
      return __Pyx_NewRef(Py_NotImplemented);
    }
  }
}

static PyObject *__pyx_getprop_7psychxr_3ovr_4math_11ovrMatrix4f_M(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_1M_1__get__(o);
}

static PyObject *__pyx_getprop_7psychxr_3ovr_4math_11ovrMatrix4f_xBasis(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_6xBasis_1__get__(o);
}

static int __pyx_setprop_7psychxr_3ovr_4math_11ovrMatrix4f_xBasis(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_6xBasis_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_7psychxr_3ovr_4math_11ovrMatrix4f_yBasis(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_6yBasis_1__get__(o);
}

static int __pyx_setprop_7psychxr_3ovr_4math_11ovrMatrix4f_yBasis(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_6yBasis_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_7psychxr_3ovr_4math_11ovrMatrix4f_zBasis(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_6zBasis_1__get__(o);
}

static int __pyx_setprop_7psychxr_3ovr_4math_11ovrMatrix4f_zBasis(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_6zBasis_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_7psychxr_3ovr_4math_11ovrMatrix4f_ctypes(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_6ctypes_1__get__(o);
}

static PyMethodDef __pyx_methods_7psychxr_3ovr_4math_ovrMatrix4f[] = {
  {"identity", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_3identity, METH_VARARGS|METH_KEYWORDS, 0},
  {"setIdentity", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_5setIdentity, METH_NOARGS, 0},
  {"setXBasis", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_7setXBasis, METH_O, 0},
  {"getXBasis", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_9getXBasis, METH_NOARGS, 0},
  {"setYBasis", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_11setYBasis, METH_O, 0},
  {"getYBasis", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_13getYBasis, METH_NOARGS, 0},
  {"setZBasis", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_15setZBasis, METH_O, 0},
  {"getZBasis", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_17getZBasis, METH_NOARGS, 0},
  {"multiply", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_31multiply, METH_VARARGS|METH_KEYWORDS, 0},
  {"transform", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_41transform, METH_O, 0},
  {"transposed", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_43transposed, METH_NOARGS, 0},
  {"transpose", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_45transpose, METH_NOARGS, 0},
  {"determinant", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_47determinant, METH_NOARGS, 0},
  {"adjugated", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_49adjugated, METH_NOARGS, 0},
  {"inverted", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_51inverted, METH_NOARGS, 0},
  {"invert", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_53invert, METH_NOARGS, 0},
  {"invertedHomogeneousTransform", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_55invertedHomogeneousTransform, METH_NOARGS, 0},
  {"invertHomogeneousTransform", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_57invertHomogeneousTransform, METH_NOARGS, 0},
  {"translation", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_59translation, METH_VARARGS|METH_KEYWORDS, 0},
  {"setTranslation", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_61setTranslation, METH_O, 0},
  {"getTranslation", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_63getTranslation, METH_NOARGS, 0},
  {"scaling", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_65scaling, METH_VARARGS|METH_KEYWORDS, 0},
  {"distance", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_67distance, METH_O, 0},
  {"rotationX", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_69rotationX, METH_VARARGS|METH_KEYWORDS, 0},
  {"rotationY", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_71rotationY, METH_VARARGS|METH_KEYWORDS, 0},
  {"rotationZ", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_73rotationZ, METH_VARARGS|METH_KEYWORDS, 0},
  {"lookAt", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_75lookAt, METH_VARARGS|METH_KEYWORDS, 0},
  {"perspective", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_77perspective, METH_VARARGS|METH_KEYWORDS, 0},
  {"ortho2d", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_79ortho2d, METH_VARARGS|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_81__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_83__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_7psychxr_3ovr_4math_ovrMatrix4f[] = {
  {(char *)"M", __pyx_getprop_7psychxr_3ovr_4math_11ovrMatrix4f_M, 0, (char *)0, 0},
  {(char *)"xBasis", __pyx_getprop_7psychxr_3ovr_4math_11ovrMatrix4f_xBasis, __pyx_setprop_7psychxr_3ovr_4math_11ovrMatrix4f_xBasis, (char *)0, 0},
  {(char *)"yBasis", __pyx_getprop_7psychxr_3ovr_4math_11ovrMatrix4f_yBasis, __pyx_setprop_7psychxr_3ovr_4math_11ovrMatrix4f_yBasis, (char *)0, 0},
  {(char *)"zBasis", __pyx_getprop_7psychxr_3ovr_4math_11ovrMatrix4f_zBasis, __pyx_setprop_7psychxr_3ovr_4math_11ovrMatrix4f_zBasis, (char *)0, 0},
  {(char *)"ctypes", __pyx_getprop_7psychxr_3ovr_4math_11ovrMatrix4f_ctypes, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyNumberMethods __pyx_tp_as_number_ovrMatrix4f = {
  __pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_23__add__, /*nb_add*/
  __pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_27__sub__, /*nb_subtract*/
  __pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_33__mul__, /*nb_multiply*/
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_divide*/
  #endif
  0, /*nb_remainder*/
  0, /*nb_divmod*/
  0, /*nb_power*/
  0, /*nb_negative*/
  0, /*nb_positive*/
  0, /*nb_absolute*/
  0, /*nb_nonzero*/
  0, /*nb_invert*/
  0, /*nb_lshift*/
  0, /*nb_rshift*/
  0, /*nb_and*/
  0, /*nb_xor*/
  0, /*nb_or*/
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_coerce*/
  #endif
  0, /*nb_int*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_long*/
  #else
  0, /*reserved*/
  #endif
  0, /*nb_float*/
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_oct*/
  #endif
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_hex*/
  #endif
  __pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_25__iadd__, /*nb_inplace_add*/
  __pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_29__isub__, /*nb_inplace_subtract*/
  __pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_35__imul__, /*nb_inplace_multiply*/
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_inplace_divide*/
  #endif
  0, /*nb_inplace_remainder*/
  0, /*nb_inplace_power*/
  0, /*nb_inplace_lshift*/
  0, /*nb_inplace_rshift*/
  0, /*nb_inplace_and*/
  0, /*nb_inplace_xor*/
  0, /*nb_inplace_or*/
  0, /*nb_floor_divide*/
  __pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_37__truediv__, /*nb_true_divide*/
  0, /*nb_inplace_floor_divide*/
  __pyx_pw_7psychxr_3ovr_4math_11ovrMatrix4f_39__itruediv__, /*nb_inplace_true_divide*/
  0, /*nb_index*/
  #if PY_VERSION_HEX >= 0x03050000
  0, /*nb_matrix_multiply*/
  #endif
  #if PY_VERSION_HEX >= 0x03050000
  0, /*nb_inplace_matrix_multiply*/
  #endif
};

static PyTypeObject __pyx_type_7psychxr_3ovr_4math_ovrMatrix4f = {
  PyVarObject_HEAD_INIT(0, 0)
  "psychxr.ovr.math.ovrMatrix4f", /*tp_name*/
  sizeof(struct __pyx_obj_7psychxr_3ovr_4math_ovrMatrix4f), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7psychxr_3ovr_4math_ovrMatrix4f, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  &__pyx_tp_as_number_ovrMatrix4f, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  "ovrMatrix4f\n\n    4x4 Matrix typically used for 3D transformations. By default, all matrices\n    are right handed. Values are stored in row-major order. Transformations\n    are applied left-to-right.\n\n    ", /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  __pyx_tp_richcompare_7psychxr_3ovr_4math_ovrMatrix4f, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_7psychxr_3ovr_4math_ovrMatrix4f, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_7psychxr_3ovr_4math_ovrMatrix4f, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7psychxr_3ovr_4math_ovrMatrix4f, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_math(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_math},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "math",
    0, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_n_s_GL, __pyx_k_GL, sizeof(__pyx_k_GL), 0, 0, 1, 1},
  {&__pyx_n_s_GLfloat, __pyx_k_GLfloat, sizeof(__pyx_k_GLfloat), 0, 0, 1, 1},
  {&__pyx_n_s_OpenGL_GL, __pyx_k_OpenGL_GL, sizeof(__pyx_k_OpenGL_GL), 0, 0, 1, 1},
  {&__pyx_n_s_TypeError, __pyx_k_TypeError, sizeof(__pyx_k_TypeError), 0, 0, 1, 1},
  {&__pyx_n_s__18, __pyx_k__18, sizeof(__pyx_k__18), 0, 0, 1, 1},
  {&__pyx_n_s_a, __pyx_k_a, sizeof(__pyx_k_a), 0, 0, 1, 1},
  {&__pyx_n_s_abs, __pyx_k_abs, sizeof(__pyx_k_abs), 0, 0, 1, 1},
  {&__pyx_n_s_align, __pyx_k_align, sizeof(__pyx_k_align), 0, 0, 1, 1},
  {&__pyx_n_s_align_to, __pyx_k_align_to, sizeof(__pyx_k_align_to), 0, 0, 1, 1},
  {&__pyx_n_s_angle, __pyx_k_angle, sizeof(__pyx_k_angle), 0, 0, 1, 1},
  {&__pyx_n_s_apply, __pyx_k_apply, sizeof(__pyx_k_apply), 0, 0, 1, 1},
  {&__pyx_n_s_aspect, __pyx_k_aspect, sizeof(__pyx_k_aspect), 0, 0, 1, 1},
  {&__pyx_n_s_at, __pyx_k_at, sizeof(__pyx_k_at), 0, 0, 1, 1},
  {&__pyx_n_s_b, __pyx_k_b, sizeof(__pyx_k_b), 0, 0, 1, 1},
  {&__pyx_n_s_c_data, __pyx_k_c_data, sizeof(__pyx_k_c_data), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_d, __pyx_k_d, sizeof(__pyx_k_d), 0, 0, 1, 1},
  {&__pyx_n_s_dot, __pyx_k_dot, sizeof(__pyx_k_dot), 0, 0, 1, 1},
  {&__pyx_n_s_down_tan, __pyx_k_down_tan, sizeof(__pyx_k_down_tan), 0, 0, 1, 1},
  {&__pyx_n_s_eq, __pyx_k_eq, sizeof(__pyx_k_eq), 0, 0, 1, 1},
  {&__pyx_n_s_eye, __pyx_k_eye, sizeof(__pyx_k_eye), 0, 0, 1, 1},
  {&__pyx_n_s_f, __pyx_k_f, sizeof(__pyx_k_f), 0, 0, 1, 1},
  {&__pyx_n_s_getXBasis, __pyx_k_getXBasis, sizeof(__pyx_k_getXBasis), 0, 0, 1, 1},
  {&__pyx_n_s_getYBasis, __pyx_k_getYBasis, sizeof(__pyx_k_getYBasis), 0, 0, 1, 1},
  {&__pyx_n_s_getZBasis, __pyx_k_getZBasis, sizeof(__pyx_k_getZBasis), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_n_s_h, __pyx_k_h, sizeof(__pyx_k_h), 0, 0, 1, 1},
  {&__pyx_n_s_identity, __pyx_k_identity, sizeof(__pyx_k_identity), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_inverse, __pyx_k_inverse, sizeof(__pyx_k_inverse), 0, 0, 1, 1},
  {&__pyx_n_s_inverseRotate, __pyx_k_inverseRotate, sizeof(__pyx_k_inverseRotate), 0, 0, 1, 1},
  {&__pyx_n_s_inverted, __pyx_k_inverted, sizeof(__pyx_k_inverted), 0, 0, 1, 1},
  {&__pyx_n_s_is_equal, __pyx_k_is_equal, sizeof(__pyx_k_is_equal), 0, 0, 1, 1},
  {&__pyx_n_s_left_tan, __pyx_k_left_tan, sizeof(__pyx_k_left_tan), 0, 0, 1, 1},
  {&__pyx_n_s_length, __pyx_k_length, sizeof(__pyx_k_length), 0, 0, 1, 1},
  {&__pyx_n_s_length_sq, __pyx_k_length_sq, sizeof(__pyx_k_length_sq), 0, 0, 1, 1},
  {&__pyx_n_s_lookAt, __pyx_k_lookAt, sizeof(__pyx_k_lookAt), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_max, __pyx_k_max, sizeof(__pyx_k_max), 0, 0, 1, 1},
  {&__pyx_n_s_min, __pyx_k_min, sizeof(__pyx_k_min), 0, 0, 1, 1},
  {&__pyx_n_s_multiply, __pyx_k_multiply, sizeof(__pyx_k_multiply), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_kp_s_no_default___reduce___due_to_non, __pyx_k_no_default___reduce___due_to_non, sizeof(__pyx_k_no_default___reduce___due_to_non), 0, 0, 1, 0},
  {&__pyx_n_s_normalize, __pyx_k_normalize, sizeof(__pyx_k_normalize), 0, 0, 1, 1},
  {&__pyx_n_s_normalized, __pyx_k_normalized, sizeof(__pyx_k_normalized), 0, 0, 1, 1},
  {&__pyx_n_s_ortho2d, __pyx_k_ortho2d, sizeof(__pyx_k_ortho2d), 0, 0, 1, 1},
  {&__pyx_n_s_perspective, __pyx_k_perspective, sizeof(__pyx_k_perspective), 0, 0, 1, 1},
  {&__pyx_n_s_psychxr_ovr_math, __pyx_k_psychxr_ovr_math, sizeof(__pyx_k_psychxr_ovr_math), 0, 0, 1, 1},
  {&__pyx_kp_s_psychxr_ovr_math_pyx, __pyx_k_psychxr_ovr_math_pyx, sizeof(__pyx_k_psychxr_ovr_math_pyx), 0, 0, 1, 0},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_n_s_right_tan, __pyx_k_right_tan, sizeof(__pyx_k_right_tan), 0, 0, 1, 1},
  {&__pyx_n_s_rotate, __pyx_k_rotate, sizeof(__pyx_k_rotate), 0, 0, 1, 1},
  {&__pyx_n_s_rotation, __pyx_k_rotation, sizeof(__pyx_k_rotation), 0, 0, 1, 1},
  {&__pyx_n_s_rotationX, __pyx_k_rotationX, sizeof(__pyx_k_rotationX), 0, 0, 1, 1},
  {&__pyx_n_s_rotationY, __pyx_k_rotationY, sizeof(__pyx_k_rotationY), 0, 0, 1, 1},
  {&__pyx_n_s_rotationZ, __pyx_k_rotationZ, sizeof(__pyx_k_rotationZ), 0, 0, 1, 1},
  {&__pyx_n_s_scaling, __pyx_k_scaling, sizeof(__pyx_k_scaling), 0, 0, 1, 1},
  {&__pyx_n_s_setXBasis, __pyx_k_setXBasis, sizeof(__pyx_k_setXBasis), 0, 0, 1, 1},
  {&__pyx_n_s_setYBasis, __pyx_k_setYBasis, sizeof(__pyx_k_setYBasis), 0, 0, 1, 1},
  {&__pyx_n_s_setZBasis, __pyx_k_setZBasis, sizeof(__pyx_k_setZBasis), 0, 0, 1, 1},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_n_s_staticmethod, __pyx_k_staticmethod, sizeof(__pyx_k_staticmethod), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_to_return, __pyx_k_to_return, sizeof(__pyx_k_to_return), 0, 0, 1, 1},
  {&__pyx_n_s_to_vector, __pyx_k_to_vector, sizeof(__pyx_k_to_vector), 0, 0, 1, 1},
  {&__pyx_n_s_tolerance, __pyx_k_tolerance, sizeof(__pyx_k_tolerance), 0, 0, 1, 1},
  {&__pyx_n_s_transform, __pyx_k_transform, sizeof(__pyx_k_transform), 0, 0, 1, 1},
  {&__pyx_n_s_translation, __pyx_k_translation, sizeof(__pyx_k_translation), 0, 0, 1, 1},
  {&__pyx_n_s_up, __pyx_k_up, sizeof(__pyx_k_up), 0, 0, 1, 1},
  {&__pyx_n_s_up_tan, __pyx_k_up_tan, sizeof(__pyx_k_up_tan), 0, 0, 1, 1},
  {&__pyx_n_s_v, __pyx_k_v, sizeof(__pyx_k_v), 0, 0, 1, 1},
  {&__pyx_n_s_w, __pyx_k_w, sizeof(__pyx_k_w), 0, 0, 1, 1},
  {&__pyx_n_s_x, __pyx_k_x, sizeof(__pyx_k_x), 0, 0, 1, 1},
  {&__pyx_n_s_y, __pyx_k_y, sizeof(__pyx_k_y), 0, 0, 1, 1},
  {&__pyx_n_s_yfov, __pyx_k_yfov, sizeof(__pyx_k_yfov), 0, 0, 1, 1},
  {&__pyx_n_s_z, __pyx_k_z, sizeof(__pyx_k_z), 0, 0, 1, 1},
  {&__pyx_n_s_zero, __pyx_k_zero, sizeof(__pyx_k_zero), 0, 0, 1, 1},
  {&__pyx_n_s_zfar, __pyx_k_zfar, sizeof(__pyx_k_zfar), 0, 0, 1, 1},
  {&__pyx_n_s_znear, __pyx_k_znear, sizeof(__pyx_k_znear), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_staticmethod = __Pyx_GetBuiltinName(__pyx_n_s_staticmethod); if (!__pyx_builtin_staticmethod) __PYX_ERR(0, 109, __pyx_L1_error)
  __pyx_builtin_TypeError = __Pyx_GetBuiltinName(__pyx_n_s_TypeError); if (!__pyx_builtin_TypeError) __PYX_ERR(1, 2, __pyx_L1_error)
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) __PYX_ERR(1, 116, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple_ = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple_)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__2 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__2)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__3 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__3)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__4 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__5 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__5)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__5);
  __Pyx_GIVEREF(__pyx_tuple__5);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__6 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__7 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__7)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__7);
  __Pyx_GIVEREF(__pyx_tuple__7);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__8 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__8)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__8);
  __Pyx_GIVEREF(__pyx_tuple__8);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__9 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__9)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__9);
  __Pyx_GIVEREF(__pyx_tuple__9);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__10 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__10)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__10);
  __Pyx_GIVEREF(__pyx_tuple__10);

  /* "psychxr/ovr/math.pyx":820
 *     @staticmethod
 *     def identity():
 *         cdef ovrQuatf to_return = ovrQuatf(0.0, 0.0, 0.0, 1.0)             # <<<<<<<<<<<<<<
 *         return to_return
 * 
 */
  __pyx_tuple__11 = PyTuple_Pack(4, __pyx_float_0_0, __pyx_float_0_0, __pyx_float_0_0, __pyx_float_1_0); if (unlikely(!__pyx_tuple__11)) __PYX_ERR(0, 820, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__11);
  __Pyx_GIVEREF(__pyx_tuple__11);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__12 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__12)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__12);
  __Pyx_GIVEREF(__pyx_tuple__12);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__13 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__13)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__13);
  __Pyx_GIVEREF(__pyx_tuple__13);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__14 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__14)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__14);
  __Pyx_GIVEREF(__pyx_tuple__14);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__15 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__15)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__15);
  __Pyx_GIVEREF(__pyx_tuple__15);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__16 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__16)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__16);
  __Pyx_GIVEREF(__pyx_tuple__16);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__17 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__17)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__17);
  __Pyx_GIVEREF(__pyx_tuple__17);

  /* "psychxr/ovr/math.pyx":110
 * 
 *     @staticmethod
 *     def min(ovrVector2i a, ovrVector2i b):             # <<<<<<<<<<<<<<
 *         cdef ovrVector2i to_return = ovrVector2i(
 *             a.c_data.x if a.c_data.x < b.c_data.x else b.c_data.x,
 */
  __pyx_tuple__19 = PyTuple_Pack(3, __pyx_n_s_a, __pyx_n_s_b, __pyx_n_s_to_return); if (unlikely(!__pyx_tuple__19)) __PYX_ERR(0, 110, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__19);
  __Pyx_GIVEREF(__pyx_tuple__19);
  __pyx_codeobj__20 = (PyObject*)__Pyx_PyCode_New(2, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__19, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_psychxr_ovr_math_pyx, __pyx_n_s_min, 110, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__20)) __PYX_ERR(0, 110, __pyx_L1_error)

  /* "psychxr/ovr/math.pyx":118
 * 
 *     @staticmethod
 *     def max(ovrVector2i a, ovrVector2i b):             # <<<<<<<<<<<<<<
 *         cdef ovrVector2i to_return = ovrVector2i(
 *             a.c_data.x if a.c_data.x > b.c_data.x else b.c_data.x,
 */
  __pyx_tuple__21 = PyTuple_Pack(3, __pyx_n_s_a, __pyx_n_s_b, __pyx_n_s_to_return); if (unlikely(!__pyx_tuple__21)) __PYX_ERR(0, 118, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__21);
  __Pyx_GIVEREF(__pyx_tuple__21);
  __pyx_codeobj__22 = (PyObject*)__Pyx_PyCode_New(2, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__21, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_psychxr_ovr_math_pyx, __pyx_n_s_max, 118, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__22)) __PYX_ERR(0, 118, __pyx_L1_error)

  /* "psychxr/ovr/math.pyx":317
 * 
 *     @staticmethod
 *     def min(ovrSizei a, ovrSizei b):             # <<<<<<<<<<<<<<
 *         cdef ovrSizei to_return = ovrSizei(
 *             a.c_data.w if a.c_data.w < b.c_data.w else b.c_data.w,
 */
  __pyx_tuple__23 = PyTuple_Pack(3, __pyx_n_s_a, __pyx_n_s_b, __pyx_n_s_to_return); if (unlikely(!__pyx_tuple__23)) __PYX_ERR(0, 317, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__23);
  __Pyx_GIVEREF(__pyx_tuple__23);
  __pyx_codeobj__24 = (PyObject*)__Pyx_PyCode_New(2, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__23, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_psychxr_ovr_math_pyx, __pyx_n_s_min, 317, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__24)) __PYX_ERR(0, 317, __pyx_L1_error)

  /* "psychxr/ovr/math.pyx":325
 * 
 *     @staticmethod
 *     def max(ovrSizei a, ovrSizei b):             # <<<<<<<<<<<<<<
 *         cdef ovrSizei to_return = ovrSizei(
 *             a.c_data.w if a.c_data.w > b.c_data.w else b.c_data.w,
 */
  __pyx_tuple__25 = PyTuple_Pack(3, __pyx_n_s_a, __pyx_n_s_b, __pyx_n_s_to_return); if (unlikely(!__pyx_tuple__25)) __PYX_ERR(0, 325, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__25);
  __Pyx_GIVEREF(__pyx_tuple__25);
  __pyx_codeobj__26 = (PyObject*)__Pyx_PyCode_New(2, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__25, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_psychxr_ovr_math_pyx, __pyx_n_s_max, 325, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__26)) __PYX_ERR(0, 325, __pyx_L1_error)

  /* "psychxr/ovr/math.pyx":496
 * 
 *     @staticmethod
 *     def zero():             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         return to_return
 */
  __pyx_tuple__27 = PyTuple_Pack(1, __pyx_n_s_to_return); if (unlikely(!__pyx_tuple__27)) __PYX_ERR(0, 496, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__27);
  __Pyx_GIVEREF(__pyx_tuple__27);
  __pyx_codeobj__28 = (PyObject*)__Pyx_PyCode_New(0, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__27, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_psychxr_ovr_math_pyx, __pyx_n_s_zero, 496, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__28)) __PYX_ERR(0, 496, __pyx_L1_error)

  /* "psychxr/ovr/math.pyx":567
 * 
 *     @staticmethod
 *     def min(ovrVector3f a, ovrVector3f b):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \
 */
  __pyx_tuple__29 = PyTuple_Pack(3, __pyx_n_s_a, __pyx_n_s_b, __pyx_n_s_to_return); if (unlikely(!__pyx_tuple__29)) __PYX_ERR(0, 567, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__29);
  __Pyx_GIVEREF(__pyx_tuple__29);
  __pyx_codeobj__30 = (PyObject*)__Pyx_PyCode_New(2, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__29, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_psychxr_ovr_math_pyx, __pyx_n_s_min, 567, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__30)) __PYX_ERR(0, 567, __pyx_L1_error)

  /* "psychxr/ovr/math.pyx":575
 * 
 *     @staticmethod
 *     def max(ovrVector3f a, ovrVector3f b):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \
 */
  __pyx_tuple__31 = PyTuple_Pack(3, __pyx_n_s_a, __pyx_n_s_b, __pyx_n_s_to_return); if (unlikely(!__pyx_tuple__31)) __PYX_ERR(0, 575, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__31);
  __Pyx_GIVEREF(__pyx_tuple__31);
  __pyx_codeobj__32 = (PyObject*)__Pyx_PyCode_New(2, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__31, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_psychxr_ovr_math_pyx, __pyx_n_s_max, 575, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__32)) __PYX_ERR(0, 575, __pyx_L1_error)

  /* "psychxr/ovr/math.pyx":717
 * 
 *     @staticmethod
 *     def max(ovrFovPort a, ovrFovPort b):             # <<<<<<<<<<<<<<
 *         cdef ovrFovPort to_return = ovrFovPort()
 *         (<ovrFovPort>to_return).c_data[0].UpTan = cmath.fmax(
 */
  __pyx_tuple__33 = PyTuple_Pack(3, __pyx_n_s_a, __pyx_n_s_b, __pyx_n_s_to_return); if (unlikely(!__pyx_tuple__33)) __PYX_ERR(0, 717, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__33);
  __Pyx_GIVEREF(__pyx_tuple__33);
  __pyx_codeobj__34 = (PyObject*)__Pyx_PyCode_New(2, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__33, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_psychxr_ovr_math_pyx, __pyx_n_s_max, 717, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__34)) __PYX_ERR(0, 717, __pyx_L1_error)

  /* "psychxr/ovr/math.pyx":731
 * 
 *     @staticmethod
 *     def min(ovrFovPort a, ovrFovPort b):             # <<<<<<<<<<<<<<
 *         cdef ovrFovPort to_return = ovrFovPort()
 *         (<ovrFovPort>to_return).c_data[0].UpTan = cmath.fmin(
 */
  __pyx_tuple__35 = PyTuple_Pack(3, __pyx_n_s_a, __pyx_n_s_b, __pyx_n_s_to_return); if (unlikely(!__pyx_tuple__35)) __PYX_ERR(0, 731, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__35);
  __Pyx_GIVEREF(__pyx_tuple__35);
  __pyx_codeobj__36 = (PyObject*)__Pyx_PyCode_New(2, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__35, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_psychxr_ovr_math_pyx, __pyx_n_s_min, 731, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__36)) __PYX_ERR(0, 731, __pyx_L1_error)

  /* "psychxr/ovr/math.pyx":819
 * 
 *     @staticmethod
 *     def identity():             # <<<<<<<<<<<<<<
 *         cdef ovrQuatf to_return = ovrQuatf(0.0, 0.0, 0.0, 1.0)
 *         return to_return
 */
  __pyx_tuple__37 = PyTuple_Pack(1, __pyx_n_s_to_return); if (unlikely(!__pyx_tuple__37)) __PYX_ERR(0, 819, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__37);
  __Pyx_GIVEREF(__pyx_tuple__37);
  __pyx_codeobj__38 = (PyObject*)__Pyx_PyCode_New(0, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__37, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_psychxr_ovr_math_pyx, __pyx_n_s_identity, 819, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__38)) __PYX_ERR(0, 819, __pyx_L1_error)

  /* "psychxr/ovr/math.pyx":964
 * 
 *     @staticmethod
 *     def align(ovrVector3f align_to, ovrVector3f v):             # <<<<<<<<<<<<<<
 *         cdef ovrQuatf to_return = ovrQuatf()
 *         (<ovrQuatf>to_return).c_data[0] = ovr_math.Quatf.Align(
 */
  __pyx_tuple__39 = PyTuple_Pack(3, __pyx_n_s_align_to, __pyx_n_s_v, __pyx_n_s_to_return); if (unlikely(!__pyx_tuple__39)) __PYX_ERR(0, 964, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__39);
  __Pyx_GIVEREF(__pyx_tuple__39);
  __pyx_codeobj__40 = (PyObject*)__Pyx_PyCode_New(2, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__39, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_psychxr_ovr_math_pyx, __pyx_n_s_align, 964, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__40)) __PYX_ERR(0, 964, __pyx_L1_error)

  /* "psychxr/ovr/math.pyx":1157
 * 
 *     @staticmethod
 *     def identity():             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         return to_return
 */
  __pyx_tuple__41 = PyTuple_Pack(1, __pyx_n_s_to_return); if (unlikely(!__pyx_tuple__41)) __PYX_ERR(0, 1157, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__41);
  __Pyx_GIVEREF(__pyx_tuple__41);
  __pyx_codeobj__42 = (PyObject*)__Pyx_PyCode_New(0, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__41, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_psychxr_ovr_math_pyx, __pyx_n_s_identity, 1157, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__42)) __PYX_ERR(0, 1157, __pyx_L1_error)

  /* "psychxr/ovr/math.pyx":1280
 * 
 *     @staticmethod
 *     def multiply(ovrMatrix4f d, ovrMatrix4f a, ovrMatrix4f b):             # <<<<<<<<<<<<<<
 *         ovr_math.Matrix4f.Multiply(&d.c_data[0], a.c_data[0], b.c_data[0])
 * 
 */
  __pyx_tuple__43 = PyTuple_Pack(3, __pyx_n_s_d, __pyx_n_s_a, __pyx_n_s_b); if (unlikely(!__pyx_tuple__43)) __PYX_ERR(0, 1280, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__43);
  __Pyx_GIVEREF(__pyx_tuple__43);
  __pyx_codeobj__44 = (PyObject*)__Pyx_PyCode_New(3, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__43, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_psychxr_ovr_math_pyx, __pyx_n_s_multiply, 1280, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__44)) __PYX_ERR(0, 1280, __pyx_L1_error)

  /* "psychxr/ovr/math.pyx":1370
 * 
 *     @staticmethod
 *     def translation(object v):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         if isinstance(v, ovrVector3f):
 */
  __pyx_tuple__45 = PyTuple_Pack(2, __pyx_n_s_v, __pyx_n_s_to_return); if (unlikely(!__pyx_tuple__45)) __PYX_ERR(0, 1370, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__45);
  __Pyx_GIVEREF(__pyx_tuple__45);
  __pyx_codeobj__46 = (PyObject*)__Pyx_PyCode_New(1, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__45, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_psychxr_ovr_math_pyx, __pyx_n_s_translation, 1370, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__46)) __PYX_ERR(0, 1370, __pyx_L1_error)

  /* "psychxr/ovr/math.pyx":1395
 * 
 *     @staticmethod
 *     def scaling(ovrVector3f v):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = \
 */
  __pyx_tuple__47 = PyTuple_Pack(2, __pyx_n_s_v, __pyx_n_s_to_return); if (unlikely(!__pyx_tuple__47)) __PYX_ERR(0, 1395, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__47);
  __Pyx_GIVEREF(__pyx_tuple__47);
  __pyx_codeobj__48 = (PyObject*)__Pyx_PyCode_New(1, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__47, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_psychxr_ovr_math_pyx, __pyx_n_s_scaling, 1395, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__48)) __PYX_ERR(0, 1395, __pyx_L1_error)

  /* "psychxr/ovr/math.pyx":1409
 * 
 *     @staticmethod
 *     def rotationX(float angle=0.0):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = ovr_math.Matrix4f.RotationX(angle)
 */
  __pyx_tuple__49 = PyTuple_Pack(2, __pyx_n_s_angle, __pyx_n_s_to_return); if (unlikely(!__pyx_tuple__49)) __PYX_ERR(0, 1409, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__49);
  __Pyx_GIVEREF(__pyx_tuple__49);
  __pyx_codeobj__50 = (PyObject*)__Pyx_PyCode_New(1, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__49, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_psychxr_ovr_math_pyx, __pyx_n_s_rotationX, 1409, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__50)) __PYX_ERR(0, 1409, __pyx_L1_error)

  /* "psychxr/ovr/math.pyx":1416
 * 
 *     @staticmethod
 *     def rotationY(float angle=0.0):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = ovr_math.Matrix4f.RotationY(angle)
 */
  __pyx_tuple__51 = PyTuple_Pack(2, __pyx_n_s_angle, __pyx_n_s_to_return); if (unlikely(!__pyx_tuple__51)) __PYX_ERR(0, 1416, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__51);
  __Pyx_GIVEREF(__pyx_tuple__51);
  __pyx_codeobj__52 = (PyObject*)__Pyx_PyCode_New(1, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__51, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_psychxr_ovr_math_pyx, __pyx_n_s_rotationY, 1416, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__52)) __PYX_ERR(0, 1416, __pyx_L1_error)

  /* "psychxr/ovr/math.pyx":1423
 * 
 *     @staticmethod
 *     def rotationZ(float angle=0.0):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = ovr_math.Matrix4f.RotationZ(angle)
 */
  __pyx_tuple__53 = PyTuple_Pack(2, __pyx_n_s_angle, __pyx_n_s_to_return); if (unlikely(!__pyx_tuple__53)) __PYX_ERR(0, 1423, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__53);
  __Pyx_GIVEREF(__pyx_tuple__53);
  __pyx_codeobj__54 = (PyObject*)__Pyx_PyCode_New(1, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__53, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_psychxr_ovr_math_pyx, __pyx_n_s_rotationZ, 1423, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__54)) __PYX_ERR(0, 1423, __pyx_L1_error)

  /* "psychxr/ovr/math.pyx":1430
 * 
 *     @staticmethod
 *     def lookAt(ovrVector3f eye, ovrVector3f at, ovrVector3f up):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = ovr_math.Matrix4f.LookAtRH(
 */
  __pyx_tuple__55 = PyTuple_Pack(4, __pyx_n_s_eye, __pyx_n_s_at, __pyx_n_s_up, __pyx_n_s_to_return); if (unlikely(!__pyx_tuple__55)) __PYX_ERR(0, 1430, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__55);
  __Pyx_GIVEREF(__pyx_tuple__55);
  __pyx_codeobj__56 = (PyObject*)__Pyx_PyCode_New(3, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__55, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_psychxr_ovr_math_pyx, __pyx_n_s_lookAt, 1430, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__56)) __PYX_ERR(0, 1430, __pyx_L1_error)

  /* "psychxr/ovr/math.pyx":1438
 * 
 *     @staticmethod
 *     def perspective(float yfov, float aspect, float znear, float zfar):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = ovr_math.Matrix4f.PerspectiveRH(
 */
  __pyx_tuple__57 = PyTuple_Pack(5, __pyx_n_s_yfov, __pyx_n_s_aspect, __pyx_n_s_znear, __pyx_n_s_zfar, __pyx_n_s_to_return); if (unlikely(!__pyx_tuple__57)) __PYX_ERR(0, 1438, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__57);
  __Pyx_GIVEREF(__pyx_tuple__57);
  __pyx_codeobj__58 = (PyObject*)__Pyx_PyCode_New(4, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__57, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_psychxr_ovr_math_pyx, __pyx_n_s_perspective, 1438, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__58)) __PYX_ERR(0, 1438, __pyx_L1_error)

  /* "psychxr/ovr/math.pyx":1446
 * 
 *     @staticmethod
 *     def ortho2d(float w, float h):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = ovr_math.Matrix4f.Ortho2D(w, h)
 */
  __pyx_tuple__59 = PyTuple_Pack(3, __pyx_n_s_w, __pyx_n_s_h, __pyx_n_s_to_return); if (unlikely(!__pyx_tuple__59)) __PYX_ERR(0, 1446, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__59);
  __Pyx_GIVEREF(__pyx_tuple__59);
  __pyx_codeobj__60 = (PyObject*)__Pyx_PyCode_New(2, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__59, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_psychxr_ovr_math_pyx, __pyx_n_s_ortho2d, 1446, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__60)) __PYX_ERR(0, 1446, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_float_0_0 = PyFloat_FromDouble(0.0); if (unlikely(!__pyx_float_0_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_1_0 = PyFloat_FromDouble(1.0); if (unlikely(!__pyx_float_1_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_3 = PyInt_FromLong(3); if (unlikely(!__pyx_int_3)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_16 = PyInt_FromLong(16); if (unlikely(!__pyx_int_16)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

#if PY_MAJOR_VERSION < 3
PyMODINIT_FUNC initmath(void); /*proto*/
PyMODINIT_FUNC initmath(void)
#else
PyMODINIT_FUNC PyInit_math(void); /*proto*/
PyMODINIT_FUNC PyInit_math(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        result = PyDict_SetItemString(moddict, to_name, value);
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__") < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static int __pyx_pymod_exec_math(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m && __pyx_m == __pyx_pyinit_module) return 0;
  #endif
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
      PyErr_Clear();
      __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
      if (!__Pyx_RefNanny)
          Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  __Pyx_RefNannySetupContext("PyMODINIT_FUNC PyInit_math(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("math", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_psychxr__ovr__math) {
    if (PyObject_SetAttrString(__pyx_m, "__name__", __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "psychxr.ovr.math")) {
      if (unlikely(PyDict_SetItemString(modules, "psychxr.ovr.math", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global init code ---*/
  /*--- Variable export code ---*/
  /*--- Function export code ---*/
  /*--- Type init code ---*/
  if (PyType_Ready(&__pyx_type_7psychxr_3ovr_4math_ovrVector2i) < 0) __PYX_ERR(0, 6, __pyx_L1_error)
  __pyx_type_7psychxr_3ovr_4math_ovrVector2i.tp_print = 0;
  if (PyObject_SetAttrString(__pyx_m, "ovrVector2i", (PyObject *)&__pyx_type_7psychxr_3ovr_4math_ovrVector2i) < 0) __PYX_ERR(0, 6, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_7psychxr_3ovr_4math_ovrVector2i) < 0) __PYX_ERR(0, 6, __pyx_L1_error)
  __pyx_ptype_7psychxr_3ovr_4math_ovrVector2i = &__pyx_type_7psychxr_3ovr_4math_ovrVector2i;
  if (PyType_Ready(&__pyx_type_7psychxr_3ovr_4math_ovrSizei) < 0) __PYX_ERR(0, 213, __pyx_L1_error)
  __pyx_type_7psychxr_3ovr_4math_ovrSizei.tp_print = 0;
  if (PyObject_SetAttrString(__pyx_m, "ovrSizei", (PyObject *)&__pyx_type_7psychxr_3ovr_4math_ovrSizei) < 0) __PYX_ERR(0, 213, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_7psychxr_3ovr_4math_ovrSizei) < 0) __PYX_ERR(0, 213, __pyx_L1_error)
  __pyx_ptype_7psychxr_3ovr_4math_ovrSizei = &__pyx_type_7psychxr_3ovr_4math_ovrSizei;
  if (PyType_Ready(&__pyx_type_7psychxr_3ovr_4math_ovrRecti) < 0) __PYX_ERR(0, 344, __pyx_L1_error)
  __pyx_type_7psychxr_3ovr_4math_ovrRecti.tp_print = 0;
  if (PyObject_SetAttrString(__pyx_m, "ovrRecti", (PyObject *)&__pyx_type_7psychxr_3ovr_4math_ovrRecti) < 0) __PYX_ERR(0, 344, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_7psychxr_3ovr_4math_ovrRecti) < 0) __PYX_ERR(0, 344, __pyx_L1_error)
  __pyx_ptype_7psychxr_3ovr_4math_ovrRecti = &__pyx_type_7psychxr_3ovr_4math_ovrRecti;
  if (PyType_Ready(&__pyx_type_7psychxr_3ovr_4math_ovrVector3f) < 0) __PYX_ERR(0, 444, __pyx_L1_error)
  __pyx_type_7psychxr_3ovr_4math_ovrVector3f.tp_print = 0;
  if (PyObject_SetAttrString(__pyx_m, "ovrVector3f", (PyObject *)&__pyx_type_7psychxr_3ovr_4math_ovrVector3f) < 0) __PYX_ERR(0, 444, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_7psychxr_3ovr_4math_ovrVector3f) < 0) __PYX_ERR(0, 444, __pyx_L1_error)
  __pyx_ptype_7psychxr_3ovr_4math_ovrVector3f = &__pyx_type_7psychxr_3ovr_4math_ovrVector3f;
  if (PyType_Ready(&__pyx_type_7psychxr_3ovr_4math_ovrFovPort) < 0) __PYX_ERR(0, 676, __pyx_L1_error)
  __pyx_type_7psychxr_3ovr_4math_ovrFovPort.tp_print = 0;
  if (PyObject_SetAttrString(__pyx_m, "ovrFovPort", (PyObject *)&__pyx_type_7psychxr_3ovr_4math_ovrFovPort) < 0) __PYX_ERR(0, 676, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_7psychxr_3ovr_4math_ovrFovPort) < 0) __PYX_ERR(0, 676, __pyx_L1_error)
  __pyx_ptype_7psychxr_3ovr_4math_ovrFovPort = &__pyx_type_7psychxr_3ovr_4math_ovrFovPort;
  if (PyType_Ready(&__pyx_type_7psychxr_3ovr_4math_ovrQuatf) < 0) __PYX_ERR(0, 744, __pyx_L1_error)
  __pyx_type_7psychxr_3ovr_4math_ovrQuatf.tp_print = 0;
  if (PyObject_SetAttrString(__pyx_m, "ovrQuatf", (PyObject *)&__pyx_type_7psychxr_3ovr_4math_ovrQuatf) < 0) __PYX_ERR(0, 744, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_7psychxr_3ovr_4math_ovrQuatf) < 0) __PYX_ERR(0, 744, __pyx_L1_error)
  __pyx_ptype_7psychxr_3ovr_4math_ovrQuatf = &__pyx_type_7psychxr_3ovr_4math_ovrQuatf;
  if (PyType_Ready(&__pyx_type_7psychxr_3ovr_4math_ovrPosef) < 0) __PYX_ERR(0, 1006, __pyx_L1_error)
  __pyx_type_7psychxr_3ovr_4math_ovrPosef.tp_print = 0;
  if (PyObject_SetAttrString(__pyx_m, "ovrPosef", (PyObject *)&__pyx_type_7psychxr_3ovr_4math_ovrPosef) < 0) __PYX_ERR(0, 1006, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_7psychxr_3ovr_4math_ovrPosef) < 0) __PYX_ERR(0, 1006, __pyx_L1_error)
  __pyx_ptype_7psychxr_3ovr_4math_ovrPosef = &__pyx_type_7psychxr_3ovr_4math_ovrPosef;
  if (PyType_Ready(&__pyx_type_7psychxr_3ovr_4math_ovrMatrix4f) < 0) __PYX_ERR(0, 1098, __pyx_L1_error)
  __pyx_type_7psychxr_3ovr_4math_ovrMatrix4f.tp_print = 0;
  if (PyObject_SetAttrString(__pyx_m, "ovrMatrix4f", (PyObject *)&__pyx_type_7psychxr_3ovr_4math_ovrMatrix4f) < 0) __PYX_ERR(0, 1098, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_7psychxr_3ovr_4math_ovrMatrix4f) < 0) __PYX_ERR(0, 1098, __pyx_L1_error)
  __pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f = &__pyx_type_7psychxr_3ovr_4math_ovrMatrix4f;
  /*--- Type import code ---*/
  /*--- Variable import code ---*/
  /*--- Function import code ---*/
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "psychxr/ovr/math.pyx":4
 * from math cimport *
 * cimport libc.math as cmath
 * import OpenGL.GL as GL             # <<<<<<<<<<<<<<
 * 
 * cdef class ovrVector2i:
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s__18);
  __Pyx_GIVEREF(__pyx_n_s__18);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s__18);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_OpenGL_GL, __pyx_t_1, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL, __pyx_t_2) < 0) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "psychxr/ovr/math.pyx":110
 * 
 *     @staticmethod
 *     def min(ovrVector2i a, ovrVector2i b):             # <<<<<<<<<<<<<<
 *         cdef ovrVector2i to_return = ovrVector2i(
 *             a.c_data.x if a.c_data.x < b.c_data.x else b.c_data.x,
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_7psychxr_3ovr_4math_11ovrVector2i_29min, NULL, __pyx_n_s_psychxr_ovr_math); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 110, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector2i->tp_dict, __pyx_n_s_min, __pyx_t_2) < 0) __PYX_ERR(0, 110, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7psychxr_3ovr_4math_ovrVector2i);

  /* "psychxr/ovr/math.pyx":109
 *         return self
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def min(ovrVector2i a, ovrVector2i b):
 *         cdef ovrVector2i to_return = ovrVector2i(
 */
  __pyx_t_2 = __Pyx_GetNameInClass((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector2i, __pyx_n_s_min); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 110, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 109, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_staticmethod, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 109, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector2i->tp_dict, __pyx_n_s_min, __pyx_t_2) < 0) __PYX_ERR(0, 110, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7psychxr_3ovr_4math_ovrVector2i);

  /* "psychxr/ovr/math.pyx":118
 * 
 *     @staticmethod
 *     def max(ovrVector2i a, ovrVector2i b):             # <<<<<<<<<<<<<<
 *         cdef ovrVector2i to_return = ovrVector2i(
 *             a.c_data.x if a.c_data.x > b.c_data.x else b.c_data.x,
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_7psychxr_3ovr_4math_11ovrVector2i_31max, NULL, __pyx_n_s_psychxr_ovr_math); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 118, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector2i->tp_dict, __pyx_n_s_max, __pyx_t_2) < 0) __PYX_ERR(0, 118, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7psychxr_3ovr_4math_ovrVector2i);

  /* "psychxr/ovr/math.pyx":117
 *         return to_return
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def max(ovrVector2i a, ovrVector2i b):
 *         cdef ovrVector2i to_return = ovrVector2i(
 */
  __pyx_t_2 = __Pyx_GetNameInClass((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector2i, __pyx_n_s_max); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 118, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 117, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_staticmethod, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 117, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector2i->tp_dict, __pyx_n_s_max, __pyx_t_2) < 0) __PYX_ERR(0, 118, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7psychxr_3ovr_4math_ovrVector2i);

  /* "psychxr/ovr/math.pyx":317
 * 
 *     @staticmethod
 *     def min(ovrSizei a, ovrSizei b):             # <<<<<<<<<<<<<<
 *         cdef ovrSizei to_return = ovrSizei(
 *             a.c_data.w if a.c_data.w < b.c_data.w else b.c_data.w,
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_7psychxr_3ovr_4math_8ovrSizei_29min, NULL, __pyx_n_s_psychxr_ovr_math); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 317, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrSizei->tp_dict, __pyx_n_s_min, __pyx_t_2) < 0) __PYX_ERR(0, 317, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7psychxr_3ovr_4math_ovrSizei);

  /* "psychxr/ovr/math.pyx":316
 *         return self
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def min(ovrSizei a, ovrSizei b):
 *         cdef ovrSizei to_return = ovrSizei(
 */
  __pyx_t_2 = __Pyx_GetNameInClass((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrSizei, __pyx_n_s_min); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 317, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 316, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_staticmethod, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 316, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrSizei->tp_dict, __pyx_n_s_min, __pyx_t_2) < 0) __PYX_ERR(0, 317, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7psychxr_3ovr_4math_ovrSizei);

  /* "psychxr/ovr/math.pyx":325
 * 
 *     @staticmethod
 *     def max(ovrSizei a, ovrSizei b):             # <<<<<<<<<<<<<<
 *         cdef ovrSizei to_return = ovrSizei(
 *             a.c_data.w if a.c_data.w > b.c_data.w else b.c_data.w,
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_7psychxr_3ovr_4math_8ovrSizei_31max, NULL, __pyx_n_s_psychxr_ovr_math); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 325, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrSizei->tp_dict, __pyx_n_s_max, __pyx_t_2) < 0) __PYX_ERR(0, 325, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7psychxr_3ovr_4math_ovrSizei);

  /* "psychxr/ovr/math.pyx":324
 *         return to_return
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def max(ovrSizei a, ovrSizei b):
 *         cdef ovrSizei to_return = ovrSizei(
 */
  __pyx_t_2 = __Pyx_GetNameInClass((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrSizei, __pyx_n_s_max); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 325, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 324, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_staticmethod, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 324, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrSizei->tp_dict, __pyx_n_s_max, __pyx_t_2) < 0) __PYX_ERR(0, 325, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7psychxr_3ovr_4math_ovrSizei);

  /* "psychxr/ovr/math.pyx":496
 * 
 *     @staticmethod
 *     def zero():             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         return to_return
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_7psychxr_3ovr_4math_11ovrVector3f_9zero, NULL, __pyx_n_s_psychxr_ovr_math); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 496, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector3f->tp_dict, __pyx_n_s_zero, __pyx_t_2) < 0) __PYX_ERR(0, 496, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7psychxr_3ovr_4math_ovrVector3f);

  /* "psychxr/ovr/math.pyx":495
 *         return 3
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def zero():
 *         cdef ovrVector3f to_return = ovrVector3f()
 */
  __pyx_t_2 = __Pyx_GetNameInClass((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, __pyx_n_s_zero); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 496, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 495, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_staticmethod, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 495, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector3f->tp_dict, __pyx_n_s_zero, __pyx_t_2) < 0) __PYX_ERR(0, 496, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7psychxr_3ovr_4math_ovrVector3f);

  /* "psychxr/ovr/math.pyx":567
 * 
 *     @staticmethod
 *     def min(ovrVector3f a, ovrVector3f b):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_7psychxr_3ovr_4math_11ovrVector3f_33min, NULL, __pyx_n_s_psychxr_ovr_math); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 567, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector3f->tp_dict, __pyx_n_s_min, __pyx_t_2) < 0) __PYX_ERR(0, 567, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7psychxr_3ovr_4math_ovrVector3f);

  /* "psychxr/ovr/math.pyx":566
 *         return self
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def min(ovrVector3f a, ovrVector3f b):
 *         cdef ovrVector3f to_return = ovrVector3f()
 */
  __pyx_t_2 = __Pyx_GetNameInClass((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, __pyx_n_s_min); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 567, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 566, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_staticmethod, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 566, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector3f->tp_dict, __pyx_n_s_min, __pyx_t_2) < 0) __PYX_ERR(0, 567, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7psychxr_3ovr_4math_ovrVector3f);

  /* "psychxr/ovr/math.pyx":575
 * 
 *     @staticmethod
 *     def max(ovrVector3f a, ovrVector3f b):             # <<<<<<<<<<<<<<
 *         cdef ovrVector3f to_return = ovrVector3f()
 *         (<ovrVector3f>to_return).c_data[0] = \
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_7psychxr_3ovr_4math_11ovrVector3f_35max, NULL, __pyx_n_s_psychxr_ovr_math); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 575, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector3f->tp_dict, __pyx_n_s_max, __pyx_t_2) < 0) __PYX_ERR(0, 575, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7psychxr_3ovr_4math_ovrVector3f);

  /* "psychxr/ovr/math.pyx":574
 *         return to_return
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def max(ovrVector3f a, ovrVector3f b):
 *         cdef ovrVector3f to_return = ovrVector3f()
 */
  __pyx_t_2 = __Pyx_GetNameInClass((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector3f, __pyx_n_s_max); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 575, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 574, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_staticmethod, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 574, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrVector3f->tp_dict, __pyx_n_s_max, __pyx_t_2) < 0) __PYX_ERR(0, 575, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7psychxr_3ovr_4math_ovrVector3f);

  /* "psychxr/ovr/math.pyx":717
 * 
 *     @staticmethod
 *     def max(ovrFovPort a, ovrFovPort b):             # <<<<<<<<<<<<<<
 *         cdef ovrFovPort to_return = ovrFovPort()
 *         (<ovrFovPort>to_return).c_data[0].UpTan = cmath.fmax(
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_7psychxr_3ovr_4math_10ovrFovPort_3max, NULL, __pyx_n_s_psychxr_ovr_math); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 717, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrFovPort->tp_dict, __pyx_n_s_max, __pyx_t_2) < 0) __PYX_ERR(0, 717, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7psychxr_3ovr_4math_ovrFovPort);

  /* "psychxr/ovr/math.pyx":716
 *         self.c_data[0].RightTan = value
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def max(ovrFovPort a, ovrFovPort b):
 *         cdef ovrFovPort to_return = ovrFovPort()
 */
  __pyx_t_2 = __Pyx_GetNameInClass((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrFovPort, __pyx_n_s_max); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 717, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 716, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_staticmethod, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 716, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrFovPort->tp_dict, __pyx_n_s_max, __pyx_t_2) < 0) __PYX_ERR(0, 717, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7psychxr_3ovr_4math_ovrFovPort);

  /* "psychxr/ovr/math.pyx":731
 * 
 *     @staticmethod
 *     def min(ovrFovPort a, ovrFovPort b):             # <<<<<<<<<<<<<<
 *         cdef ovrFovPort to_return = ovrFovPort()
 *         (<ovrFovPort>to_return).c_data[0].UpTan = cmath.fmin(
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_7psychxr_3ovr_4math_10ovrFovPort_5min, NULL, __pyx_n_s_psychxr_ovr_math); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 731, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrFovPort->tp_dict, __pyx_n_s_min, __pyx_t_2) < 0) __PYX_ERR(0, 731, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7psychxr_3ovr_4math_ovrFovPort);

  /* "psychxr/ovr/math.pyx":730
 *         return to_return
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def min(ovrFovPort a, ovrFovPort b):
 *         cdef ovrFovPort to_return = ovrFovPort()
 */
  __pyx_t_2 = __Pyx_GetNameInClass((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrFovPort, __pyx_n_s_min); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 731, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 730, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_staticmethod, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 730, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrFovPort->tp_dict, __pyx_n_s_min, __pyx_t_2) < 0) __PYX_ERR(0, 731, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7psychxr_3ovr_4math_ovrFovPort);

  /* "psychxr/ovr/math.pyx":819
 * 
 *     @staticmethod
 *     def identity():             # <<<<<<<<<<<<<<
 *         cdef ovrQuatf to_return = ovrQuatf(0.0, 0.0, 0.0, 1.0)
 *         return to_return
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_7psychxr_3ovr_4math_8ovrQuatf_11identity, NULL, __pyx_n_s_psychxr_ovr_math); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 819, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrQuatf->tp_dict, __pyx_n_s_identity, __pyx_t_2) < 0) __PYX_ERR(0, 819, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7psychxr_3ovr_4math_ovrQuatf);

  /* "psychxr/ovr/math.pyx":818
 *         return to_return
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def identity():
 *         cdef ovrQuatf to_return = ovrQuatf(0.0, 0.0, 0.0, 1.0)
 */
  __pyx_t_2 = __Pyx_GetNameInClass((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrQuatf, __pyx_n_s_identity); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 819, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 818, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_staticmethod, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 818, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrQuatf->tp_dict, __pyx_n_s_identity, __pyx_t_2) < 0) __PYX_ERR(0, 819, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7psychxr_3ovr_4math_ovrQuatf);

  /* "psychxr/ovr/math.pyx":964
 * 
 *     @staticmethod
 *     def align(ovrVector3f align_to, ovrVector3f v):             # <<<<<<<<<<<<<<
 *         cdef ovrQuatf to_return = ovrQuatf()
 *         (<ovrQuatf>to_return).c_data[0] = ovr_math.Quatf.Align(
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_7psychxr_3ovr_4math_8ovrQuatf_59align, NULL, __pyx_n_s_psychxr_ovr_math); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 964, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrQuatf->tp_dict, __pyx_n_s_align, __pyx_t_2) < 0) __PYX_ERR(0, 964, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7psychxr_3ovr_4math_ovrQuatf);

  /* "psychxr/ovr/math.pyx":963
 *         return to_return
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def align(ovrVector3f align_to, ovrVector3f v):
 *         cdef ovrQuatf to_return = ovrQuatf()
 */
  __pyx_t_2 = __Pyx_GetNameInClass((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrQuatf, __pyx_n_s_align); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 964, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 963, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_staticmethod, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 963, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrQuatf->tp_dict, __pyx_n_s_align, __pyx_t_2) < 0) __PYX_ERR(0, 964, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7psychxr_3ovr_4math_ovrQuatf);

  /* "psychxr/ovr/math.pyx":1157
 * 
 *     @staticmethod
 *     def identity():             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         return to_return
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_7psychxr_3ovr_4math_11ovrMatrix4f_3identity, NULL, __pyx_n_s_psychxr_ovr_math); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1157, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f->tp_dict, __pyx_n_s_identity, __pyx_t_2) < 0) __PYX_ERR(0, 1157, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f);

  /* "psychxr/ovr/math.pyx":1156
 *         return self.c_data.M
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def identity():
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 */
  __pyx_t_2 = __Pyx_GetNameInClass((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f, __pyx_n_s_identity); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1157, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1156, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_staticmethod, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1156, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f->tp_dict, __pyx_n_s_identity, __pyx_t_2) < 0) __PYX_ERR(0, 1157, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f);

  /* "psychxr/ovr/math.pyx":1280
 * 
 *     @staticmethod
 *     def multiply(ovrMatrix4f d, ovrMatrix4f a, ovrMatrix4f b):             # <<<<<<<<<<<<<<
 *         ovr_math.Matrix4f.Multiply(&d.c_data[0], a.c_data[0], b.c_data[0])
 * 
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_7psychxr_3ovr_4math_11ovrMatrix4f_31multiply, NULL, __pyx_n_s_psychxr_ovr_math); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1280, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f->tp_dict, __pyx_n_s_multiply, __pyx_t_2) < 0) __PYX_ERR(0, 1280, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f);

  /* "psychxr/ovr/math.pyx":1279
 *         return self
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def multiply(ovrMatrix4f d, ovrMatrix4f a, ovrMatrix4f b):
 *         ovr_math.Matrix4f.Multiply(&d.c_data[0], a.c_data[0], b.c_data[0])
 */
  __pyx_t_2 = __Pyx_GetNameInClass((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f, __pyx_n_s_multiply); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1280, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1279, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_staticmethod, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1279, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f->tp_dict, __pyx_n_s_multiply, __pyx_t_2) < 0) __PYX_ERR(0, 1280, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f);

  /* "psychxr/ovr/math.pyx":1370
 * 
 *     @staticmethod
 *     def translation(object v):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         if isinstance(v, ovrVector3f):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_7psychxr_3ovr_4math_11ovrMatrix4f_59translation, NULL, __pyx_n_s_psychxr_ovr_math); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1370, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f->tp_dict, __pyx_n_s_translation, __pyx_t_2) < 0) __PYX_ERR(0, 1370, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f);

  /* "psychxr/ovr/math.pyx":1369
 *         return self
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def translation(object v):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 */
  __pyx_t_2 = __Pyx_GetNameInClass((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f, __pyx_n_s_translation); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1370, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1369, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_staticmethod, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1369, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f->tp_dict, __pyx_n_s_translation, __pyx_t_2) < 0) __PYX_ERR(0, 1370, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f);

  /* "psychxr/ovr/math.pyx":1395
 * 
 *     @staticmethod
 *     def scaling(ovrVector3f v):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = \
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_7psychxr_3ovr_4math_11ovrMatrix4f_65scaling, NULL, __pyx_n_s_psychxr_ovr_math); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1395, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f->tp_dict, __pyx_n_s_scaling, __pyx_t_2) < 0) __PYX_ERR(0, 1395, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f);

  /* "psychxr/ovr/math.pyx":1394
 *         return to_return
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def scaling(ovrVector3f v):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 */
  __pyx_t_2 = __Pyx_GetNameInClass((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f, __pyx_n_s_scaling); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1395, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1394, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_staticmethod, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1394, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f->tp_dict, __pyx_n_s_scaling, __pyx_t_2) < 0) __PYX_ERR(0, 1395, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f);

  /* "psychxr/ovr/math.pyx":1409
 * 
 *     @staticmethod
 *     def rotationX(float angle=0.0):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = ovr_math.Matrix4f.RotationX(angle)
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_7psychxr_3ovr_4math_11ovrMatrix4f_69rotationX, NULL, __pyx_n_s_psychxr_ovr_math); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1409, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f->tp_dict, __pyx_n_s_rotationX, __pyx_t_2) < 0) __PYX_ERR(0, 1409, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f);

  /* "psychxr/ovr/math.pyx":1408
 *         return distance
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def rotationX(float angle=0.0):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 */
  __pyx_t_2 = __Pyx_GetNameInClass((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f, __pyx_n_s_rotationX); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1409, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1408, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_staticmethod, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1408, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f->tp_dict, __pyx_n_s_rotationX, __pyx_t_2) < 0) __PYX_ERR(0, 1409, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f);

  /* "psychxr/ovr/math.pyx":1416
 * 
 *     @staticmethod
 *     def rotationY(float angle=0.0):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = ovr_math.Matrix4f.RotationY(angle)
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_7psychxr_3ovr_4math_11ovrMatrix4f_71rotationY, NULL, __pyx_n_s_psychxr_ovr_math); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1416, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f->tp_dict, __pyx_n_s_rotationY, __pyx_t_2) < 0) __PYX_ERR(0, 1416, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f);

  /* "psychxr/ovr/math.pyx":1415
 *         return to_return
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def rotationY(float angle=0.0):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 */
  __pyx_t_2 = __Pyx_GetNameInClass((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f, __pyx_n_s_rotationY); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1416, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1415, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_staticmethod, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1415, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f->tp_dict, __pyx_n_s_rotationY, __pyx_t_2) < 0) __PYX_ERR(0, 1416, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f);

  /* "psychxr/ovr/math.pyx":1423
 * 
 *     @staticmethod
 *     def rotationZ(float angle=0.0):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = ovr_math.Matrix4f.RotationZ(angle)
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_7psychxr_3ovr_4math_11ovrMatrix4f_73rotationZ, NULL, __pyx_n_s_psychxr_ovr_math); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1423, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f->tp_dict, __pyx_n_s_rotationZ, __pyx_t_2) < 0) __PYX_ERR(0, 1423, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f);

  /* "psychxr/ovr/math.pyx":1422
 *         return to_return
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def rotationZ(float angle=0.0):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 */
  __pyx_t_2 = __Pyx_GetNameInClass((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f, __pyx_n_s_rotationZ); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1423, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1422, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_staticmethod, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1422, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f->tp_dict, __pyx_n_s_rotationZ, __pyx_t_2) < 0) __PYX_ERR(0, 1423, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f);

  /* "psychxr/ovr/math.pyx":1430
 * 
 *     @staticmethod
 *     def lookAt(ovrVector3f eye, ovrVector3f at, ovrVector3f up):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = ovr_math.Matrix4f.LookAtRH(
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_7psychxr_3ovr_4math_11ovrMatrix4f_75lookAt, NULL, __pyx_n_s_psychxr_ovr_math); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1430, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f->tp_dict, __pyx_n_s_lookAt, __pyx_t_2) < 0) __PYX_ERR(0, 1430, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f);

  /* "psychxr/ovr/math.pyx":1429
 *         return to_return
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def lookAt(ovrVector3f eye, ovrVector3f at, ovrVector3f up):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 */
  __pyx_t_2 = __Pyx_GetNameInClass((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f, __pyx_n_s_lookAt); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1430, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1429, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_staticmethod, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1429, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f->tp_dict, __pyx_n_s_lookAt, __pyx_t_2) < 0) __PYX_ERR(0, 1430, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f);

  /* "psychxr/ovr/math.pyx":1438
 * 
 *     @staticmethod
 *     def perspective(float yfov, float aspect, float znear, float zfar):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = ovr_math.Matrix4f.PerspectiveRH(
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_7psychxr_3ovr_4math_11ovrMatrix4f_77perspective, NULL, __pyx_n_s_psychxr_ovr_math); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1438, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f->tp_dict, __pyx_n_s_perspective, __pyx_t_2) < 0) __PYX_ERR(0, 1438, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f);

  /* "psychxr/ovr/math.pyx":1437
 *         return to_return
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def perspective(float yfov, float aspect, float znear, float zfar):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 */
  __pyx_t_2 = __Pyx_GetNameInClass((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f, __pyx_n_s_perspective); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1438, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1437, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_staticmethod, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1437, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f->tp_dict, __pyx_n_s_perspective, __pyx_t_2) < 0) __PYX_ERR(0, 1438, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f);

  /* "psychxr/ovr/math.pyx":1446
 * 
 *     @staticmethod
 *     def ortho2d(float w, float h):             # <<<<<<<<<<<<<<
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 *         (<ovrMatrix4f>to_return).c_data[0] = ovr_math.Matrix4f.Ortho2D(w, h)
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_7psychxr_3ovr_4math_11ovrMatrix4f_79ortho2d, NULL, __pyx_n_s_psychxr_ovr_math); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1446, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f->tp_dict, __pyx_n_s_ortho2d, __pyx_t_2) < 0) __PYX_ERR(0, 1446, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f);

  /* "psychxr/ovr/math.pyx":1445
 *         return to_return
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def ortho2d(float w, float h):
 *         cdef ovrMatrix4f to_return = ovrMatrix4f()
 */
  __pyx_t_2 = __Pyx_GetNameInClass((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f, __pyx_n_s_ortho2d); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1446, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1445, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_staticmethod, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1445, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f->tp_dict, __pyx_n_s_ortho2d, __pyx_t_2) < 0) __PYX_ERR(0, 1446, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7psychxr_3ovr_4math_ovrMatrix4f);

  /* "psychxr/ovr/math.pyx":1
 * # distutils: language=c++             # <<<<<<<<<<<<<<
 * from math cimport *
 * cimport libc.math as cmath
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_2) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "carray.to_py":124
 * 
 * @cname("__Pyx_carray_to_tuple_float___5b_4_5d_")
 * cdef inline tuple __Pyx_carray_to_tuple_float___5b_4_5d_(base_type *v, Py_ssize_t length):             # <<<<<<<<<<<<<<
 *     cdef size_t i
 *     cdef object value
 */

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init psychxr.ovr.math", 0, __pyx_lineno, __pyx_filename);
    }
    Py_DECREF(__pyx_m); __pyx_m = 0;
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init psychxr.ovr.math");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule((char *)modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, (char *)"RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_CheckExact(key)) || likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (PyUnicode_GET_SIZE(**name) != PyUnicode_GET_SIZE(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (PyUnicode_GET_SIZE(**argname) != PyUnicode_GET_SIZE(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* ArgTypeTest */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (exact) {
        #if PY_MAJOR_VERSION == 2
        if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    }
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
    return 0;
}

/* None */
static CYTHON_INLINE void __Pyx_RaiseUnboundLocalError(const char *varname) {
    PyErr_Format(PyExc_UnboundLocalError, "local variable '%s' referenced before assignment", varname);
}

/* None */
static CYTHON_INLINE int __Pyx_div_int(int a, int b) {
    int q = a / b;
    int r = a - q*b;
    q -= ((r != 0) & ((r ^ b) < 0));
    return q;
}

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)meth)) (self, args, nargs);
    }
}
#endif

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
#include "frameobject.h"
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = f->f_localsplus;
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallOneArg */
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (PyCFunction_GET_FLAGS(func) & METH_FASTCALL) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* PyObjectCallNoArg */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_TypeCheck(func, __pyx_CyFunctionType))) {
#else
    if (likely(PyCFunction_Check(func))) {
#endif
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* ExtTypeTest */
  static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

/* PyErrFetchRestore */
  #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* RaiseException */
  #if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* KeywordStringCheck */
  static int __Pyx_CheckKeywordStrings(
    PyObject *kwdict,
    const char* function_name,
    int kw_allowed)
{
    PyObject* key = 0;
    Py_ssize_t pos = 0;
#if CYTHON_COMPILING_IN_PYPY
    if (!kw_allowed && PyDict_Next(kwdict, &pos, &key, 0))
        goto invalid_keyword;
    return 1;
#else
    while (PyDict_Next(kwdict, &pos, &key, 0)) {
        #if PY_MAJOR_VERSION < 3
        if (unlikely(!PyString_Check(key)))
        #endif
            if (unlikely(!PyUnicode_Check(key)))
                goto invalid_keyword_type;
    }
    if ((!kw_allowed) && unlikely(key))
        goto invalid_keyword;
    return 1;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    return 0;
#endif
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
    return 0;
}

/* GetItemInt */
  static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely((0 <= wrapped_i) & (wrapped_i < PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely((0 <= wrapped_i) & (wrapped_i < PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely((n >= 0) & (n < PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely((n >= 0) & (n < PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* GetModuleGlobalName */
  static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name) {
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
    result = PyDict_GetItem(__pyx_d, name);
    if (likely(result)) {
        Py_INCREF(result);
    } else {
#else
    result = PyObject_GetItem(__pyx_d, name);
    if (!result) {
        PyErr_Clear();
#endif
        result = __Pyx_GetBuiltinName(name);
    }
    return result;
}

/* SetupReduce */
    static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStr(meth, __pyx_n_s_name);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    if (_PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate)) goto GOOD;
#else
    if (PyObject_HasAttr(type_obj, __pyx_n_s_getstate)) goto GOOD;
#endif
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_cython); if (unlikely(!reduce_cython)) goto BAD;
            ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto BAD;
            ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto BAD;
            setstate = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate_cython); if (unlikely(!setstate_cython)) goto BAD;
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto BAD;
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto GOOD;
BAD:
    if (!PyErr_Occurred())
        PyErr_Format(PyExc_RuntimeError, "Unable to initialize pickling for %s", ((PyTypeObject*)type_obj)->tp_name);
    ret = -1;
GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* Import */
    static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.')) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* PyErrExceptionMatches */
    #if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    PyObject *exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
    if (unlikely(PyTuple_Check(err)))
        return __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    return __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
}
#endif

/* GetNameInClass */
    static PyObject *__Pyx_GetGlobalNameAfterAttributeLookup(PyObject *name) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    return __Pyx_GetModuleGlobalName(name);
}
static PyObject *__Pyx_GetNameInClass(PyObject *nmspace, PyObject *name) {
    PyObject *result;
    result = __Pyx_PyObject_GetAttrStr(nmspace, name);
    if (!result) {
        result = __Pyx_GetGlobalNameAfterAttributeLookup(name);
    }
    return result;
}

/* CLineInTraceback */
    #ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
      use_cline = PyDict_GetItem(*cython_runtime_dict, __pyx_n_s_cline_in_traceback);
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (PyObject_Not(use_cline) != 0) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
    static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, (size_t)new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
    #include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntFromPyVerify */
    #define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntToPy */
    static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
    static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
    static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntFromPy */
    static CYTHON_INLINE size_t __Pyx_PyInt_As_size_t(PyObject *x) {
    const size_t neg_one = (size_t) -1, const_zero = (size_t) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(size_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(size_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (size_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (size_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(size_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(size_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 2 * PyLong_SHIFT) {
                            return (size_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(size_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 3 * PyLong_SHIFT) {
                            return (size_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(size_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 4 * PyLong_SHIFT) {
                            return (size_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (size_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(size_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(size_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (size_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(size_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(size_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(size_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(size_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                            return (size_t) ((((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(size_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                            return (size_t) ((((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(size_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT) {
                            return (size_t) ((((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(size_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(size_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            size_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (size_t) -1;
        }
    } else {
        size_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (size_t) -1;
        val = __Pyx_PyInt_As_size_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to size_t");
    return (size_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to size_t");
    return (size_t) -1;
}

/* CIntFromPy */
    static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* FastTypeChecks */
    #if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
    static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* InitStrings */
    static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            PyErr_Clear();
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(x);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
